Package: cvcqv
Type: Package
Title: Coefficient of Variation (CV) with Confidence Intervals (CI)
Version: 1.0.3
Date: 2026-05-08
Authors@R: person("Maani", "Beigy", email = "manibeygi@gmail.com",
                  role = c("aut", "cre"))
Maintainer: Maani Beigy <manibeygi@gmail.com>
Description: Provides some easy-to-use functions and classes to calculate
    variability measures such as coefficient of variation with confidence
    intervals provided with all available methods. References are
    'Panichkitkosolkul' (2013) <doi:10.1155/2013/324940>,
    'Altunkaynak' & 'Gamgam' (2018) <doi:10.1080/03610918.2018.1435800>,
    'Albatineh', 'Kibria', Wilcox & 'Zogheib' (2014) <doi:10.1080/02664763.2013.847405>.
Depends: R (>= 3.5.0), dplyr (>= 0.8.0.1)
Imports: R6, SciViews, boot, MBESS
Suggests: testthat, knitr, rmarkdown, covr
VignetteBuilder: knitr
URL: https://github.com/MaaniBeigy/cvcqv
BugReports: https://github.com/MaaniBeigy/cvcqv/issues
License: GPL-3
Encoding: UTF-8
RoxygenNote: 8.0.0
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-07 22:19:14 UTC; 20240700
Author: Maani Beigy [aut, cre]
Repository: CRAN
Date/Publication: 2026-05-21 09:30:07 UTC
Built: R 4.5.2; ; 2026-05-25 02:38:25 UTC; unix

Package: cvcqv
Type: Package
Title: Coefficient of Variation (CV) with Confidence Intervals (CI)
Version: 1.0.4
Date: 2026-07-06
Authors@R: person("Maani", "Beigy", email = "manibeygi@gmail.com",
                  role = c("aut", "cre"))
Maintainer: Maani Beigy <manibeygi@gmail.com>
Description: Provides some easy-to-use functions and classes to calculate
    variability measures such as coefficient of variation with confidence
    intervals provided with all available methods. References are
    'Panichkitkosolkul' (2013) <doi:10.1155/2013/324940>,
    'Altunkaynak' & 'Gamgam' (2018) <doi:10.1080/03610918.2018.1435800>,
    'Albatineh', 'Kibria', Wilcox & 'Zogheib' (2014) <doi:10.1080/02664763.2013.847405>.
Depends: R (>= 4.1.0), dplyr (>= 0.8.0.1)
Imports: R6, SciViews, boot, MBESS
Suggests: testthat, knitr, rmarkdown, covr
VignetteBuilder: knitr
URL: https://github.com/MaaniBeigy/cvcqv
BugReports: https://github.com/MaaniBeigy/cvcqv/issues
License: GPL-3
Encoding: UTF-8
RoxygenNote: 8.0.0
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-06 11:41:29 UTC; 20240700
Author: Maani Beigy [aut, cre]
Repository: CRAN
Date/Publication: 2026-07-06 12:00:02 UTC

# Generated by roxygen2: do not edit by hand

export(BootCoefQuartVar)
export(BootCoefVar)
export(CoefQuartVar)
export(CoefQuartVarCI)
export(CoefVar)
export(CoefVarCI)
export(SampleQuantiles)
export(cqv_versatile)
export(cv_versatile)
import(R6)
import(SciViews)
import(boot)
import(dplyr)
import(utils)
importFrom(MBESS,conf.limits.nct)
importFrom(stats,qchisq)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,sd)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# cvcqv <img src="./man/figures/sticker.svg" align="right" width="120" />

<!-- badges: start -->

[![R-CMD-check](https://github.com/MaaniBeigy/cvcqv/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/MaaniBeigy/cvcqv/actions/workflows/R-CMD-check.yaml)
[![test-coverage](https://github.com/MaaniBeigy/cvcqv/actions/workflows/test-coverage.yaml/badge.svg)](https://github.com/MaaniBeigy/cvcqv/actions/workflows/test-coverage.yaml)
[![lint](https://github.com/MaaniBeigy/cvcqv/actions/workflows/lint.yaml/badge.svg)](https://github.com/MaaniBeigy/cvcqv/actions/workflows/lint.yaml)
[![CRAN
status](https://www.r-pkg.org/badges/version/cvcqv)](https://CRAN.R-project.org/package=cvcqv)
[![License: GPL
v3](https://img.shields.io/badge/License-GPL%20v3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0)
[![minimal R
version](https://img.shields.io/badge/R%3E%3D-3.5.0-6666ff.svg)](https://cran.r-project.org/)
[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![Project Status: Active – The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![metacran
downloads](https://cranlogs.r-pkg.org/badges/grand-total/cvcqv)](https://CRAN.R-project.org/package=cvcqv)
<!-- badges: end -->

`cvcqv` provides some easy-to-use functions and classes to calculate
Coefficient of Variation (`cv`) and Coefficient of Quartile Variation
(`cqv`) with confidence intervals provided with all available methods.

# Overview

### Background

There are abundant methods available for the calculation of **confidence
intervals** of a dispersion measure like coefficient of variation (`cv`)
or coefficient of quartile variation (`cqv`), which have not yet been
implemented in R. Also, `cqv` is quite useful in conditions where the
distribution of variables does not follow normal distribution.

#### Coefficient of Variation

`cv` is a measure of relative dispersion representing the degree of
variability relative to the mean (Albatineh et al, 2014). Since cv is
unitless, it is useful for comparison of variables with different units.
It is also a measure of homogeneity (Albatineh et al, 2014).

#### Coefficient of Quartile Variation

`cqv` is a measure of relative dispersion that is based on interquartile
range (`IQR`). Since `cqv` is unitless, it is also useful for comparison
of variables with different units. It is also a measure of homogeneity
(Bonett, 2006; Altunkaynak, 2018).

#### Confidence Interval

Since the measurements of `cv` and `cqv` are obtained from samples, we
cannot easily generalize them and decide based upon them. Confidence
Intervals (CI) help us to make a probabilistic interval around the
estimation of calculated `cv` and `cqv`. For example, 95% CI indicates
that it is 95% probable that the measurement for a population lies
between the lower and upper bounds of that CI.

## Installation

If you are an ubuntu user, you are going to need these non-R packages:

``` linux
sudo apt install libcurl4-openssl-dev libssl-dev libxml2-dev libgsl-dev   
```

The `cvcqv` package is available on CRAN. To install:

``` r
install.packages("cvcqv")
```

The `cvcqv` package is also available on github. To install it in `R`,
use:

``` r
devtools::install_github('MaaniBeigy/cvcqv')  
```

\* Currently, these tools are available:

| name             | is.R6.. | Description                             |
|:-----------------|:--------|:----------------------------------------|
| CoefVar          | TRUE    | Coefficient of Variation (cv)           |
| CoefQuartVar     | TRUE    | Coefficient of Quartile Variation (cqv) |
| CoefVarCI        | TRUE    | Confidence Intervals for cv             |
| CoefQuartVarCI   | TRUE    | Confidence Intervals for cqv            |
| SampleQuantiles  | TRUE    | Sample Quantiles                        |
| cv_versatile     | FALSE   | Coefficient of Variation                |
| cqv_versatile    | FALSE   | Coefficient of Quartile Variation       |
| BootCoefVar      | TRUE    | Bootstrap Resampling for cv             |
| BootCoefQuartVar | TRUE    | Bootstrap Resampling for cqv            |

\* This package is inspired by `dplyr`, `R6`, `SciView`, `boot`, and
`MBESS`.

## Example

Here, we want to observe **all** available *confidence intervals* for
the `cv` of variable *x*:

``` r
x <- c(
    0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
    4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
)
results <- CoefVarCI$new(x, digits = 3)$all_ci()  # R6 class
# or alternatively: 
results <- cv_versatile(x, digits = 3, method = "all")  # functional programming
```

The `results` will be:

|  | est | lower | upper | description |
|:---|---:|---:|---:|:---|
| kelley | 57.774 | 41.287 | 97.894 | cv with Kelley 95% CI |
| mckay | 57.774 | 41.441 | 108.483 | cv with McKay 95% CI |
| miller | 57.774 | 34.053 | 81.495 | cv with Miller 95% CI |
| vangel | 57.774 | 41.264 | 105.426 | cv with Vangel 95% CI |
| mahmoudvand_hassani | 57.774 | 43.476 | 82.857 | cv with Mahmoudvand-Hassani 95% CI |
| equal_tailed | 57.774 | 43.937 | 84.383 | cv with Equal-Tailed 95% CI |
| shortest_length | 57.774 | 42.015 | 81.013 | cv with Shortest-Length 95% CI |
| normal_approximation | 57.774 | 44.533 | 85.272 | cv with Normal Approximation 95% CI |
| norm | 57.774 | 38.799 | 78.937 | cv with Normal Approximation Bootstrap 95% CI |
| basic | 57.774 | 35.055 | 78.167 | cv with Basic Bootstrap 95% CI |
| perc | 57.774 | 38.879 | 79.174 | cv with Bootstrap Percentile 95% CI |
| bca | 57.774 | 40.807 | 82.297 | cv with Adjusted Bootstrap Percentile (BCa) 95% CI |
| aak_adj | 57.774 | 48.029 | 72.516 | cv with Abu-Shawiesh-Akyuz-Kibria ADJ 95% CI |
| aak_ls | 57.774 | 46.310 | 72.075 | cv with Abu-Shawiesh-Akyuz-Kibria LS 95% CI |
| aak_als | 57.774 | 45.839 | 75.092 | cv with Abu-Shawiesh-Akyuz-Kibria ALS 95% CI |

Next, we want to find **all** of the available *confidence intervals*
for the `cqv` of variable *x*:

``` r
results <- CoefQuartVarCI$new(x, digits = 3)$all_ci()  # R6 class
# or alternatively:
results <- cqv_versatile(x, , digits = 3, method = "all")  # functional programming
```

The `results` will be:

|         |    est |  lower |  upper | description                                     |
|:--------|-------:|-------:|-------:|:------------------------------------------------|
| bonett  | 45.625 | 24.785 | 77.329 | cqv with Bonett CI                              |
| norm    | 45.625 | 19.957 | 70.840 | cqv with normal approximation CI                |
| basic   | 45.625 | 18.992 | 73.917 | cqv with basic bootstrap CI                     |
| percent | 45.625 | 17.122 | 68.683 | cqv with bootstrap percentile CI                |
| bca     | 45.625 | 24.273 | 83.264 | cqv with adjusted bootstrap percentile (BCa) CI |

## Documentation

Once installed, browse the package vignettes:

``` r
browseVignettes("cvcqv")
```

## References

Albatineh, AN., Kibria, BM., Wilcox, ML., & Zogheib, B, 2014, Confidence
interval estimation for the population coefficient of variation using
ranked set sampling: A simulation study, Journal of Applied Statistics,
41(4), 733–751, DOI: <https://doi.org/10.1080/02664763.2013.847405>

Bonett, DG., 2006, Confidence interval for a coefficient of quartile
variation, Computational Statistics & Data Analysis, 50(11), 2953-7,
DOI: <https://doi.org/10.1016/j.csda.2005.05.007>

Altunkaynak, B., Gamgam, H., 2018, Bootstrap confidence intervals for
the coefficient of quartile variation, Simulation and Computation, 1-9,
DOI: <https://doi.org/10.1080/03610918.2018.1435800>

Abu-Shawiesh, M. O. A., Akyuz, H. E., & Kibria, B. M. G., 2019,
Performance of Some Confidence Intervals for Estimating the Population
Coefficient of Variation under both Symmetric and Skewed Distributions,
Statistics, Optimization & Information Computing, 7(2), 277-290, DOI:
<https://doi.org/10.19139/soic.v7i2.630>

<!-- README.md is generated from README.Rmd. Please edit that file -->

# cvcqv <img src="./man/figures/sticker.svg" align="right" width="120" />

<!-- badges: start -->

[![R-CMD-check](https://github.com/MaaniBeigy/cvcqv/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/MaaniBeigy/cvcqv/actions/workflows/R-CMD-check.yaml)
[![test-coverage](https://github.com/MaaniBeigy/cvcqv/actions/workflows/test-coverage.yaml/badge.svg)](https://github.com/MaaniBeigy/cvcqv/actions/workflows/test-coverage.yaml)
[![lint](https://github.com/MaaniBeigy/cvcqv/actions/workflows/lint.yaml/badge.svg)](https://github.com/MaaniBeigy/cvcqv/actions/workflows/lint.yaml)
[![CRAN
status](https://www.r-pkg.org/badges/version/cvcqv)](https://CRAN.R-project.org/package=cvcqv)
[![License: GPL
v3](https://img.shields.io/badge/License-GPL%20v3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0)
[![minimal R
version](https://img.shields.io/badge/R%3E%3D-3.5.0-6666ff.svg)](https://cran.r-project.org/)
[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![Project Status: Active – The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![metacran
downloads](https://cranlogs.r-pkg.org/badges/grand-total/cvcqv)](https://CRAN.R-project.org/package=cvcqv)
<!-- badges: end -->

`cvcqv` provides some easy-to-use functions and classes to calculate
Coefficient of Variation (`cv`) and Coefficient of Quartile Variation
(`cqv`) with confidence intervals provided with all available methods.

# Overview

### Background

There are abundant methods available for the calculation of **confidence
intervals** of a dispersion measure like coefficient of variation (`cv`)
or coefficient of quartile variation (`cqv`), which have not yet been
implemented in R. Also, `cqv` is quite useful in conditions where the
distribution of variables does not follow normal distribution.

#### Coefficient of Variation

`cv` is a measure of relative dispersion representing the degree of
variability relative to the mean (Albatineh et al, 2014). Since cv is
unitless, it is useful for comparison of variables with different units.
It is also a measure of homogeneity (Albatineh et al, 2014).

#### Coefficient of Quartile Variation

`cqv` is a measure of relative dispersion that is based on interquartile
range (`IQR`). Since `cqv` is unitless, it is also useful for comparison
of variables with different units. It is also a measure of homogeneity
(Bonett, 2006; Altunkaynak, 2018).

#### Confidence Interval

Since the measurements of `cv` and `cqv` are obtained from samples, we
cannot easily generalize them and decide based upon them. Confidence
Intervals (CI) help us to make a probabilistic interval around the
estimation of calculated `cv` and `cqv`. For example, 95% CI indicates
that it is 95% probable that the measurement for a population lies
between the lower and upper bounds of that CI.

## Installation

If you are an ubuntu user, you are going to need these non-R packages:

``` linux
sudo apt install libcurl4-openssl-dev libssl-dev libxml2-dev libgsl-dev   
```

The `cvcqv` package is available on CRAN. To install:

``` r
install.packages("cvcqv")
```

The `cvcqv` package is also available on github. To install it in `R`,
use:

``` r
devtools::install_github('MaaniBeigy/cvcqv')  
```

\* Currently, these tools are available:

| name             | is.R6.. | Description                             |
|:-----------------|:--------|:----------------------------------------|
| CoefVar          | TRUE    | Coefficient of Variation (cv)           |
| CoefQuartVar     | TRUE    | Coefficient of Quartile Variation (cqv) |
| CoefVarCI        | TRUE    | Confidence Intervals for cv             |
| CoefQuartVarCI   | TRUE    | Confidence Intervals for cqv            |
| SampleQuantiles  | TRUE    | Sample Quantiles                        |
| cv_versatile     | FALSE   | Coefficient of Variation                |
| cqv_versatile    | FALSE   | Coefficient of Quartile Variation       |
| BootCoefVar      | TRUE    | Bootstrap Resampling for cv             |
| BootCoefQuartVar | TRUE    | Bootstrap Resampling for cqv            |

\* This package is inspired by `dplyr`, `R6`, `SciView`, `boot`, and
`MBESS`.

## Example

Here, we want to observe **all** available *confidence intervals* for
the `cv` of variable *x*:

``` r
x <- c(
    0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
    4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
)
results <- CoefVarCI$new(x, digits = 3)$all_ci()  # R6 class
# or alternatively: 
results <- cv_versatile(x, digits = 3, method = "all")  # functional programming
```

The `results` will be:

|  | est | lower | upper | description |
|:---|---:|---:|---:|:---|
| kelley | 57.774 | 41.287 | 97.894 | cv with Kelley 95% CI |
| mckay | 57.774 | 41.441 | 108.483 | cv with McKay 95% CI |
| miller | 57.774 | 34.053 | 81.495 | cv with Miller 95% CI |
| vangel | 57.774 | 41.264 | 105.426 | cv with Vangel 95% CI |
| mahmoudvand_hassani | 57.774 | 43.476 | 82.857 | cv with Mahmoudvand-Hassani 95% CI |
| equal_tailed | 57.774 | 43.937 | 84.383 | cv with Equal-Tailed 95% CI |
| shortest_length | 57.774 | 42.015 | 81.013 | cv with Shortest-Length 95% CI |
| normal_approximation | 57.774 | 44.533 | 85.272 | cv with Normal Approximation 95% CI |
| norm | 57.774 | 38.799 | 78.937 | cv with Normal Approximation Bootstrap 95% CI |
| basic | 57.774 | 35.055 | 78.167 | cv with Basic Bootstrap 95% CI |
| perc | 57.774 | 38.879 | 79.174 | cv with Bootstrap Percentile 95% CI |
| bca | 57.774 | 40.807 | 82.297 | cv with Adjusted Bootstrap Percentile (BCa) 95% CI |

Next, we want to find **all** of the available *confidence intervals*
for the `cqv` of variable *x*:

``` r
results <- CoefQuartVarCI$new(x, digits = 3)$all_ci()  # R6 class
# or alternatively:
results <- cqv_versatile(x, , digits = 3, method = "all")  # functional programming
```

The `results` will be:

|         |    est |  lower |  upper | description                                     |
|:--------|-------:|-------:|-------:|:------------------------------------------------|
| bonett  | 45.625 | 24.785 | 77.329 | cqv with Bonett CI                              |
| norm    | 45.625 | 19.957 | 70.840 | cqv with normal approximation CI                |
| basic   | 45.625 | 18.992 | 73.917 | cqv with basic bootstrap CI                     |
| percent | 45.625 | 17.122 | 68.683 | cqv with bootstrap percentile CI                |
| bca     | 45.625 | 24.273 | 83.264 | cqv with adjusted bootstrap percentile (BCa) CI |

## Documentation

Once installed, browse the package vignettes:

``` r
browseVignettes("cvcqv")
```

## References

Albatineh, AN., Kibria, BM., Wilcox, ML., & Zogheib, B, 2014, Confidence
interval estimation for the population coefficient of variation using
ranked set sampling: A simulation study, Journal of Applied Statistics,
41(4), 733–751, DOI: <https://doi.org/10.1080/02664763.2013.847405>

Bonett, DG., 2006, Confidence interval for a coefficient of quartile
variation, Computational Statistics & Data Analysis, 50(11), 2953-7,
DOI: <https://doi.org/10.1016/j.csda.2005.05.007>

Altunkaynak, B., Gamgam, H., 2018, Bootstrap confidence intervals for
the coefficient of quartile variation, Simulation and Computation, 1-9,
DOI: <https://doi.org/10.1080/03610918.2018.1435800>

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ----echo=FALSE, warning=FALSE, message=FALSE, include = FALSE----------------
library(dplyr)
library(boot)
x <- c(
  0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
  4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
)
cv_versatile <- function(
  x, # Currently there are methods for numeric vectors
  na.rm = FALSE, # indicating whether NA values should be stripped
  digits = 1, # digits of output after rounding. default is 4
  method = NULL, # method for the computation of confidence interval (CI)
  correction = FALSE, # indicating whether to compute the unbiased statistics
  alpha = 0.05, # The allowed type I error probability
  R = NULL, # integer indicating the number of bootstrap replicates
  ...
) {
  # library(MBESS)
  # require(dplyr)
  # require(SciViews)
  # require(boot)
  if (missing(x) || is.null(x)) {
    stop("object 'x' not found")
  } else if (!missing(x)) {
    x <- x
  }
  if (!is.numeric(x)) {
    stop("argument is not a numeric vector: returning NA")
    return(NA_real_)
  }
  na.rm <- na.rm # removes NAs if TRUE
  if (na.rm == TRUE) {
    x <- x[!is.na(x)]
  } else if (anyNA(x)) {
    stop(
      "missing values and NaN's not allowed if 'na.rm' is FALSE"
    )
  }

  if (is.null(digits)) {
    digits <- 1
  }
  if (is.null(R)) {
    R <- 1000
  }
  digits <- digits # digits required for rounding
  shortest_length <- data.frame( # "a" and "b" values for shortest-length CI
    v = c( # degrees of freedom
      2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
      21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 40, 50, 60, 70, 80, 90,
      100, 150, 200, 250, 300, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13,
      14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
      30, 40, 50, 60, 70, 80, 90, 100, 150, 200, 250, 300, 2, 3, 4,
      5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21,
      22, 23, 24, 25, 26, 27, 28, 29, 30, 40, 50, 60, 70, 80, 90,
      100, 150, 200, 250, 300
    ),
    al = c( # al is the allowed type I error probability
      0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,
      0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,
      0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,
      0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,
      0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
      0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
      0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
      0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
      0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
      0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01,
      0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01,
      0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01,
      0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01,
      0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01
    ),
    a = c( # a is an attribute for the length of CI
      0.2065, 0.5654, 1.02, 1.5352, 2.093, 2.6828, 3.2981, 3.9343,
      4.5883, 5.2573, 5.9397, 6.6337, 7.3382, 8.0521, 8.7745,
      9.5047, 10.2421, 10.9861, 11.7362, 12.4919, 13.253,
      14.0191, 14.7899, 15.565, 16.3443, 17.1275, 17.9144,
      18.7049, 19.4987, 27.5919, 35.9012, 44.3661, 52.9501,
      61.629, 70.386, 79.2086, 124.0372, 169.6646, 215.8057,
      262.3132, 0.1015, 0.3449, 0.6918, 1.1092, 1.5776,
      2.0851, 2.6235, 3.1874, 3.7729, 4.3768, 4.9967,
      5.6308, 6.2776, 6.9357, 7.6042, 8.282, 8.9685,
      9.6629, 10.3647, 11.0733, 11.7882, 12.5092,
      13.2357, 13.9675, 14.7043, 15.4458, 16.1917,
      16.9419, 17.6961, 25.4233, 33.4085, 41.5794, 49.8923,
      58.3183, 66.8374, 75.4347, 119.2737, 164.0642,
      209.4667, 255.3057, 0.02, 0.114, 0.2937, 0.5461,
      0.8567, 1.2143, 1.6107, 2.0394, 2.4958, 2.976, 3.4771,
      3.9968, 4.5329, 5.084, 5.6487, 6.2256, 6.8139, 7.4126,
      8.0209, 8.6383, 9.264, 9.8976, 10.5385, 11.1864, 11.8408,
      12.5014, 13.1678, 13.8397, 14.517, 21.5331, 28.8879,
      36.4863, 44.2711, 52.2044, 60.2597, 68.4177, 110.3262,
      153.4834, 197.444, 241.9776
    ),
    b = c( # b is an attribute for the length of CI
      12.5208, 13.1532, 14.18, 15.3498, 16.5807, 17.8391,
      19.1099, 20.3848, 21.6598, 22.9325, 24.2016,
      25.4666, 26.7269, 27.9825, 29.2334, 30.4796,
      31.7212, 32.9585, 34.1915, 35.4205, 36.6455,
      37.8668, 39.0844, 40.2986, 41.5095, 42.7171,
      43.9217, 45.1234, 46.3222, 58.1755, 69.8342,
      81.3479, 92.7487, 104.0584, 115.2925, 126.4628,
      181.6128, 235.9748, 289.8273, 343.3155, 15.1194,
      15.5897, 16.5735, 17.7432, 18.9954, 20.2863,
      21.5953, 22.9118, 24.2303, 25.5476, 26.8618,
      28.1717, 29.4769, 30.777, 32.072, 33.3619,
      34.6467, 35.9266, 37.2016, 38.472, 39.7379,
      40.9995, 42.257, 43.5105, 44.7601, 46.006,
      47.2483, 48.4872, 49.7229, 61.9217, 73.892,
      85.6914, 97.3573, 108.9153, 120.3839, 131.7767,
      187.9079, 243.1025, 297.691, 351.8461, 20.8264,
      20.9856, 21.8371, 22.9867, 24.2618, 25.6017,
      26.9749, 28.3643, 29.7602, 31.158, 32.5543,
      33.9474, 35.3358, 36.7192, 38.0968, 39.4688,
      40.8347, 42.1952, 43.5498, 44.8989, 46.2426,
      47.581, 48.9144, 50.2428, 51.5665, 52.8856,
      54.2002, 55.5107, 56.8169, 69.6808, 82.2534,
      94.6063, 106.7867, 118.8272, 130.7514, 142.5771,
      200.6194, 257.4375, 313.462, 368.9185
    )
  )
  if ("kelley" %in% method) {
    if (!require(MBESS)) {
      warning(
        "package 'MBESS' required to calculate Kelley's confidence interval"
      )
    }
  }
  cv <- (
    sd(x, na.rm = na.rm) / mean(x, na.rm = na.rm) # coefficient of variation
  )
  cv_corr <- cv * (
    (1 - (1 / (4 * (length(x) - 1))) + # corrected coefficient of variation
      (1 / length(x)) * cv^2) +
      (1 / (2 * (length(x) - 1)^2))
  )
  if (is.null(method) == TRUE & correction == FALSE) {
    return(
      list(
        method = "cv = sd/mean (may be biased)",
        statistics = data.frame(
          est = round(cv * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if (is.null(method) == TRUE & correction == TRUE) {
    return(
      list(
        method = "Corrected (i.e., unbiased) cv",
        statistics = data.frame(
          est = round(cv_corr * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if (!is.null(method)) {
    method <- tolower(method) # convert user's input to lower-case
    method <- match.arg( # match the user's input with available methods
      arg = method,
      choices = c(
        "kelley", "mckay", "miller", "vangel", "mahmoudvand_hassani",
        "equal_tailed", "shortest_length", "normal_approximation",
        "norm", "basic", "perc", "bca", "all"
      ),
      several.ok = TRUE
    )
  }

  # calculating cv's bootstrap CI
  boot.cv <- boot::boot(
    x,
    function(x, i) { # coefficient of variation
      sd(x[i], na.rm = na.rm) / mean(x[i], na.rm = na.rm)
    },
    R = R
  )
  boot.cv_corr <- boot::boot(
    x,
    function(x, i) { # corrected coefficient of variation
      sd(x[i], na.rm = na.rm) / mean(x[i], na.rm = na.rm) * (
        (1 - (1 / (4 * (length(x[i]) - 1))) +
          (1 / length(x)) * (
            sd(x[i], na.rm = na.rm) / mean(x[i], na.rm = na.rm)
          )^2) +
          (1 / (2 * (length(x) - 1)^2))
      )
    },
    R = R
  )
  # calculating cv and its CI attributes based on selected methods
  if ("kelley" %in% method && correction == FALSE) {
    ci <- MBESS::conf.limits.nct(
      ncp = sqrt(length(x)) / cv_corr,
      df = length(x) - 1,
      conf.level = (1 - alpha)
    )
    est.kelley <- cv # cv is an estimate of CV
    lower.tile.kelley <- unname(sqrt(length(x)) / ci$Upper.Limit)
    upper.tile.kelley <- unname(sqrt(length(x)) / ci$Lower.Limit)
  } else if ("kelley" %in% method && correction == TRUE) {
    ci <- MBESS::conf.limits.nct(
      ncp = sqrt(length(x)) / cv_corr,
      df = length(x) - 1,
      conf.level = (1 - alpha)
    )
    est.kelley <- cv_corr # corrected cv is an estimate of CV
    lower.tile.kelley <- unname(sqrt(length(x)) / ci$Upper.Limit)
    upper.tile.kelley <- unname(sqrt(length(x)) / ci$Lower.Limit)
  } else if ("mckay" %in% method && correction == FALSE) {
    if (cv > 0.33) {
      warning("Confidence interval may be very approximate")
    }
    v <- length(x) - 1
    t1 <- qchisq(1 - alpha / 2, v) / v
    t2 <- qchisq(alpha / 2, v) / v
    u1 <- v * t1
    u2 <- v * t2
    est.mckay <- cv # cv is an estimate of CV
    lower.tile.mckay <- cv / sqrt((u1 / (v + 1) - 1) * (cv^2) + u1 / v)
    upper.tile.mckay <- cv / sqrt((u2 / (v + 1) - 1) * (cv^2) + u2 / v)
  } else if ("mckay" %in% method && correction == TRUE) {
    if (cv_corr > 0.33) {
      warning("Confidence interval may be very approximate")
    }
    v <- length(x) - 1
    t1 <- qchisq(1 - alpha / 2, v) / v
    t2 <- qchisq(alpha / 2, v) / v
    u1 <- v * t1
    u2 <- v * t2
    est.mckay <- cv_corr # corrected cv is an estimate of CV
    lower.tile.mckay <- cv_corr / sqrt((u1 / (v + 1) - 1) * (cv_corr^2) + u1 / v)
    upper.tile.mckay <- cv_corr / sqrt((u2 / (v + 1) - 1) * (cv_corr^2) + u2 / v)
  } else if ("miller" %in% method && correction == FALSE) {
    v <- length(x) - 1
    z_alpha_over2 <- qnorm(1 - (alpha / 2))
    u <- sqrt(
      (cv^2 / v) * (0.5 + cv^2)
    )
    zu <- z_alpha_over2 * u
    est.miller <- cv # cv is an estimate of CV
    lower.tile.miller <- cv - zu
    upper.tile.miller <- cv + zu
  } else if ("miller" %in% method && correction == TRUE) {
    v <- length(x) - 1
    z_alpha_over2 <- qnorm(1 - (alpha / 2))
    u <- sqrt(
      (cv_corr^2 / v) * (0.5 + cv_corr^2)
    )
    zu <- z_alpha_over2 * u
    est.miller <- cv_corr # corrected cv is an estimate of CV
    lower.tile.miller <- cv_corr - zu
    upper.tile.miller <- cv_corr + zu
  } else if ("vangel" %in% method && correction == FALSE) {
    if (cv > 0.33) {
      warning("Confidence interval may be very approximate")
    }
    v <- length(x) - 1
    t1 <- qchisq(1 - alpha / 2, v) / v
    t2 <- qchisq(alpha / 2, v) / v
    u1 <- v * t1
    u2 <- v * t2
    est.vangel <- cv # cv is an estimate of CV
    lower.tile.vangel <- cv / sqrt(((u1 + 1) / (v + 1) - 1) * (cv^2) + u1 / v)
    upper.tile.vangel <- cv / sqrt(((u2 + 1) / (v + 1) - 1) * (cv^2) + u2 / v)
  } else if ("vangel" %in% method && correction == TRUE) {
    if (cv_corr > 0.33) {
      warning("Confidence interval may be very approximate")
    }
    v <- length(x) - 1
    t1 <- qchisq(1 - alpha / 2, v) / v
    t2 <- qchisq(alpha / 2, v) / v
    u1 <- v * t1
    u2 <- v * t2
    est.vangel <- cv_corr # corrected cv is an estimate of CV
    lower.tile.vangel <- cv_corr / sqrt(
      ((u1 + 1) / (v + 1) - 1) * (cv_corr^2) + u1 / v
    )
    upper.tile.vangel <- cv_corr / sqrt(
      ((u2 + 1) / (v + 1) - 1) * (cv_corr^2) + u2 / v
    )
  } else if ("mahmoudvand_hassani" %in% method && correction == FALSE) {
    if (length(x) <= 340) {
      cn <- sqrt(2 / (length(x) - 1)) * (
        (gamma(length(x) / 2)) / (gamma((length(x) - 1) / 2))
      )
    } else {
      cn <- sqrt(2 / (length(x) - 1)) * (
        (lgamma(length(x) / 2)) / (lgamma((length(x) - 1) / 2))
      )
    }
    ul <- 2 - (cn + (qnorm((alpha / 2)) * sqrt(1 - cn^2)))
    uu <- 2 - (cn - (qnorm((alpha / 2)) * sqrt(1 - cn^2)))
    est.mahmoud <- cv # cv is an estimate of CV
    lower.tile.mahmoud <- cv / ul
    upper.tile.mahmoud <- cv / uu
  } else if ("mahmoudvand_hassani" %in% method && correction == TRUE) {
    if (length(x) <= 340) {
      cn <- sqrt(2 / (length(x) - 1)) * (
        (gamma(length(x) / 2)) / (gamma((length(x) - 1) / 2))
      )
    } else {
      cn <- sqrt(2 / (length(x) - 1)) * (
        (lgamma(length(x) / 2)) / (lgamma((length(x) - 1) / 2))
      )
    }
    ul <- 2 - (cn + (qnorm((alpha / 2)) * sqrt(1 - cn^2)))
    uu <- 2 - (cn - (qnorm((alpha / 2)) * sqrt(1 - cn^2)))
    est.mahmoud <- cv_corr # corrected cv is an estimate of CV
    lower.tile.mahmoud <- cv_corr / ul
    upper.tile.mahmoud <- cv_corr / uu
  } else if ("normal_approximation" %in% method && correction == FALSE) {
    cn <- sqrt(1 - (1 / (2 * length(x))))
    ul <- cn + (qnorm(1 - (alpha / 2)) * sqrt(1 - cn^2))
    uu <- cn - (qnorm(1 - (alpha / 2)) * sqrt(1 - cn^2))
    est.normaapprox <- cv # cv is an estimate of CV
    lower.tile.normaapprox <- cv / ul
    upper.tile.normaapprox <- cv / uu
  } else if ("normal_approximation" %in% method && correction == TRUE) {
    cn <- sqrt(1 - (1 / (2 * length(x))))
    ul <- cn + (qnorm(1 - (alpha / 2)) * sqrt(1 - cn^2))
    uu <- cn - (qnorm(1 - (alpha / 2)) * sqrt(1 - cn^2))
    est.normaapprox <- cv_corr # corrected cv is an estimate of CV
    lower.tile.normaapprox <- cv_corr / ul
    upper.tile.normaapprox <- cv_corr / uu
  } else if ("shortest_length" %in% method && correction == FALSE) {
    if (length(x) <= 300) {
      a_value <- shortest_length %>%
        subset(al == alpha & v == length(x) - 1) %>%
        dplyr::select(a)
      b_value <- shortest_length %>%
        subset(al == alpha & v == length(x) - 1) %>%
        dplyr::select(b)
    } else if (length(x) > 300) {
      a_value <- shortest_length %>%
        subset(al == alpha & v == 300) %>%
        dplyr::select(a)
      b_value <- shortest_length %>%
        subset(al == alpha & v == 300) %>%
        dplyr::select(b)
    }
    est.shortest <- cv # cv is an estimate of CV
    lower.tile.shortest <- (cv * sqrt(length(x) - 1)) / sqrt(b_value$b)
    upper.tile.shortest <- (cv * sqrt(length(x) - 1)) / sqrt(a_value$a)
  } else if ("shortest_length" %in% method && correction == TRUE) {
    if (length(x) <= 300) {
      a_value <- shortest_length %>%
        subset(al == alpha & v == length(x) - 1) %>%
        dplyr::select(a)
      b_value <- shortest_length %>%
        subset(al == alpha & v == length(x) - 1) %>%
        dplyr::select(b)
    } else if (length(x) > 300) {
      a_value <- shortest_length %>%
        subset(al == alpha & v == 300) %>%
        dplyr::select(a)
      b_value <- shortest_length %>%
        subset(al == alpha & v == 300) %>%
        dplyr::select(b)
    }
    est.shortest <- cv_corr # corrected cv is an estimate of CV
    lower.tile.shortest <- (cv_corr * sqrt(length(x) - 1)) / sqrt(b_value$b)
    upper.tile.shortest <- (cv_corr * sqrt(length(x) - 1)) / sqrt(a_value$a)
  } else if ("equal_tailed" %in% method && correction == FALSE) {
    v <- length(x) - 1
    tt1 <- qchisq(1 - alpha / 2, v)
    tt2 <- qchisq(alpha / 2, v)
    est.equal <- cv # cv is an estimate of CV
    lower.tile.equal <- (cv * sqrt(v)) / (sqrt(tt1))
    upper.tile.equal <- (cv * sqrt(v)) / (sqrt(tt2))
  } else if ("equal_tailed" %in% method && correction == TRUE) {
    v <- length(x) - 1
    tt1 <- qchisq(1 - alpha / 2, v)
    tt2 <- qchisq(alpha / 2, v)
    est.equal <- cv_corr # corrected cv is an estimate of CV
    lower.tile.equal <- (cv_corr * sqrt(v)) / (sqrt(tt1))
    upper.tile.equal <- (cv_corr * sqrt(v)) / (sqrt(tt2))
  } else if ("norm" %in% method && correction == FALSE) {
    boot.normcv.ci <- boot::boot.ci(
      boot.cv,
      conf = (1 - alpha), type = "norm"
    )
    est.norm <- cv # cv is an estimate of CV
    lower.tile.norm <- boot.normcv.ci$normal[2]
    upper.tile.norm <- boot.normcv.ci$normal[3]
  } else if ("norm" %in% method && correction == TRUE) {
    boot.normcv_corr.ci <- boot::boot.ci(
      boot.cv_corr,
      conf = (1 - alpha), type = "norm"
    )
    est.norm <- cv_corr # corrected cv is an estimate of CV
    lower.tile.norm <- boot.normcv_corr.ci$normal[2]
    upper.tile.norm <- boot.normcv_corr.ci$normal[3]
  } else if ("basic" %in% method && correction == FALSE) {
    boot.basiccv.ci <- boot::boot.ci(
      boot.cv,
      conf = (1 - alpha), type = "basic"
    )
    est.basic <- cv # cv is an estimate of CV
    lower.tile.basic <- boot.basiccv.ci$basic[4]
    upper.tile.basic <- boot.basiccv.ci$basic[5]
  } else if ("basic" %in% method && correction == TRUE) {
    boot.basiccv_corr.ci <- boot::boot.ci(
      boot.cv_corr,
      conf = (1 - alpha), type = "basic"
    )
    est.basic <- cv_corr # corrected cv is an estimate of CV
    lower.tile.basic <- boot.basiccv_corr.ci$basic[4]
    upper.tile.basic <- boot.basiccv_corr.ci$basic[5]
  } else if ("perc" %in% method && correction == FALSE) {
    boot.percentcv.ci <- boot::boot.ci(
      boot.cv,
      conf = (1 - alpha), type = "basic"
    )
    est.percent <- cv # cv is an estimate of CV
    lower.tile.percent <- boot.percentcv.ci$percent[4]
    upper.tile.percent <- boot.percentcv.ci$percent[5]
  } else if ("perc" %in% method && correction == TRUE) {
    boot.percentcv_corr.ci <- boot::boot.ci(
      boot.cv_corr,
      conf = (1 - alpha), type = "basic"
    )
    est.percent <- cv_corr # corrected cv is an estimate of CV
    lower.tile.percent <- boot.percentcv_corr.ci$percent[4]
    upper.tile.percent <- boot.percentcv_corr.ci$percent[5]
  } else if ("bca" %in% method && correction == FALSE) {
    boot.bcacv.ci <- boot::boot.ci(
      boot.cv,
      conf = (1 - alpha), type = "basic"
    )
    est.bca <- cv # cv is an estimate of CV
    lower.tile.bca <- boot.bcacv.ci$bca[4]
    upper.tile.bca <- boot.bcacv.ci$bca[5]
  } else if ("bca" %in% method && correction == TRUE) {
    boot.bcacv_corr.ci <- boot::boot.ci(
      boot.cv_corr,
      conf = (1 - alpha), type = "basic"
    )
    est.bca <- cv_corr # corrected cv is an estimate of CV
    lower.tile.bca <- boot.bcacv_corr.ci$percent[4]
    upper.tile.bca <- boot.bcacv_corr.ci$percent[5]
  } else if (method == "all" && correction == FALSE) {
    est.all <- cv # cv is an estimate of CV
    ci <- MBESS::conf.limits.nct(
      ncp = sqrt(length(x)) / cv_corr,
      df = length(x) - 1,
      conf.level = (1 - alpha)
    )
    lower.tile.kelley <- unname(sqrt(length(x)) / ci$Upper.Limit)
    upper.tile.kelley <- unname(sqrt(length(x)) / ci$Lower.Limit)
    if (cv > 0.33) {
      warning("Confidence interval may be very approximate")
    }
    v <- length(x) - 1
    t1 <- qchisq(1 - alpha / 2, v) / v
    t2 <- qchisq(alpha / 2, v) / v
    u1 <- v * t1
    u2 <- v * t2
    est.mckay <- cv # cv is an estimate of CV
    lower.tile.mckay <- cv / sqrt((u1 / (v + 1) - 1) * (cv^2) + u1 / v)
    upper.tile.mckay <- cv / sqrt((u2 / (v + 1) - 1) * (cv^2) + u2 / v)
    z_alpha_over2 <- qnorm(1 - (alpha / 2))
    u <- sqrt(
      (cv^2 / v) * (0.5 + cv^2)
    )
    zu <- z_alpha_over2 * u
    est.miller <- cv # cv is an estimate of CV
    lower.tile.miller <- cv - zu
    upper.tile.miller <- cv + zu
    tt1 <- qchisq(1 - alpha / 2, v) / v
    tt2 <- qchisq(alpha / 2, v) / v
    uu1 <- v * tt1
    uu2 <- v * tt2
    est.vangel <- cv # cv is an estimate of CV
    lower.tile.vangel <- cv / sqrt(((uu1 + 1) / (v + 1) - 1) * (cv^2) + uu1 / v)
    upper.tile.vangel <- cv / sqrt(((uu2 + 1) / (v + 1) - 1) * (cv^2) + uu2 / v)
    if (length(x) <= 340) {
      cn <- sqrt(2 / (length(x) - 1)) * (
        (gamma(length(x) / 2)) / (gamma((length(x) - 1) / 2))
      )
    } else {
      cn <- sqrt(2 / (length(x) - 1)) * (
        (lgamma(length(x) / 2)) / (lgamma((length(x) - 1) / 2))
      )
    }
    ul <- 2 - (cn + (qnorm((alpha / 2)) * sqrt(1 - cn^2)))
    uu <- 2 - (cn - (qnorm((alpha / 2)) * sqrt(1 - cn^2)))
    est.mahmoud <- cv # cv is an estimate of CV
    lower.tile.mahmoud <- cv / ul
    upper.tile.mahmoud <- cv / uu
    cnt <- sqrt(1 - (1 / (2 * length(x))))
    ult <- cnt + (qnorm(1 - (alpha / 2)) * sqrt(1 - cnt^2))
    uut <- cnt - (qnorm(1 - (alpha / 2)) * sqrt(1 - cnt^2))
    est.normaapprox <- cv # cv is an estimate of CV
    lower.tile.normaapprox <- cv / ult
    upper.tile.normaapprox <- cv / uut
    if (length(x) <= 300) {
      a_value <- shortest_length %>%
        subset(al == alpha & v == length(x) - 1) %>%
        dplyr::select(a)
      b_value <- shortest_length %>%
        subset(al == alpha & v == length(x) - 1) %>%
        dplyr::select(b)
    } else if (length(x) > 300) {
      a_value <- shortest_length %>%
        subset(al == alpha & v == 300) %>%
        dplyr::select(a)
      b_value <- shortest_length %>%
        subset(al == alpha & v == 300) %>%
        dplyr::select(b)
    }
    est.shortest <- cv # cv is an estimate of CV
    lower.tile.shortest <- (cv * sqrt(length(x) - 1)) / sqrt(b_value$b)
    upper.tile.shortest <- (cv * sqrt(length(x) - 1)) / sqrt(a_value$a)
    ttt1 <- qchisq(1 - alpha / 2, v)
    ttt2 <- qchisq(alpha / 2, v)
    est.equal <- cv # cv is an estimate of CV
    lower.tile.equal <- (cv * sqrt(v)) / (sqrt(ttt1))
    upper.tile.equal <- (cv * sqrt(v)) / (sqrt(ttt2))
    boot.normcv.ci <- boot::boot.ci(
      boot.cv,
      conf = (1 - alpha), type = "norm"
    )
    est.norm <- cv # cv is an estimate of CV
    lower.tile.norm <- boot.normcv.ci$normal[2]
    upper.tile.norm <- boot.normcv.ci$normal[3]
    boot.basiccv.ci <- boot::boot.ci(
      boot.cv,
      conf = (1 - alpha), type = "basic"
    )
    est.basic <- cv # cv is an estimate of CV
    lower.tile.basic <- boot.basiccv.ci$basic[4]
    upper.tile.basic <- boot.basiccv.ci$basic[5]
    boot.percentcv.ci <- boot::boot.ci(
      boot.cv,
      conf = (1 - alpha), type = "basic"
    )
    est.percent <- cv # cv is an estimate of CV
    lower.tile.percent <- boot.percentcv.ci$percent[4]
    upper.tile.percent <- boot.percentcv.ci$percent[5]
    boot.bcacv.ci <- boot::boot.ci(
      boot.cv,
      conf = (1 - alpha), type = "basic"
    )
    est.bca <- cv # cv is an estimate of CV
    lower.tile.bca <- boot.bcacv.ci$bca[4]
    upper.tile.bca <- boot.bcacv.ci$bca[5]
  } else if (method == "all" && correction == TRUE) {
    est.all <- cv_corr # cv_corr is an estimate of CV
    ci <- MBESS::conf.limits.nct(
      ncp = sqrt(length(x)) / cv_corr,
      df = length(x) - 1,
      conf.level = (1 - alpha)
    )
    lower.tile.kelley <- unname(sqrt(length(x)) / ci$Upper.Limit)
    upper.tile.kelley <- unname(sqrt(length(x)) / ci$Lower.Limit)
    if (cv_corr > 0.33) {
      warning("Confidence interval may be very approximate")
    }
    v <- length(x) - 1
    t1 <- qchisq(1 - alpha / 2, v) / v
    t2 <- qchisq(alpha / 2, v) / v
    u1 <- v * t1
    u2 <- v * t2
    est.mckay <- cv_corr # cv_corr is an estimate of CV
    lower.tile.mckay <- cv_corr / sqrt((u1 / (v + 1) - 1) * (cv_corr^2) + u1 / v)
    upper.tile.mckay <- cv_corr / sqrt((u2 / (v + 1) - 1) * (cv_corr^2) + u2 / v)
    z_alpha_over2 <- qnorm(1 - (alpha / 2))
    u <- sqrt(
      (cv_corr^2 / v) * (0.5 + cv_corr^2)
    )
    zu <- z_alpha_over2 * u
    est.miller <- cv_corr # cv_corr is an estimate of CV
    lower.tile.miller <- cv_corr - zu
    upper.tile.miller <- cv_corr + zu
    tt1 <- qchisq(1 - alpha / 2, v) / v
    tt2 <- qchisq(alpha / 2, v) / v
    uu1 <- v * tt1
    uu2 <- v * tt2
    est.vangel <- cv_corr # cv_corr is an estimate of CV
    lower.tile.vangel <- cv_corr / sqrt(((uu1 + 1) / (v + 1) - 1) * (cv_corr^2) + uu1 / v)
    upper.tile.vangel <- cv_corr / sqrt(((uu2 + 1) / (v + 1) - 1) * (cv_corr^2) + uu2 / v)
    if (length(x) <= 340) {
      cn <- sqrt(2 / (length(x) - 1)) * (
        (gamma(length(x) / 2)) / (gamma((length(x) - 1) / 2))
      )
    } else {
      cn <- sqrt(2 / (length(x) - 1)) * (
        (lgamma(length(x) / 2)) / (lgamma((length(x) - 1) / 2))
      )
    }
    ul <- 2 - (cn + (qnorm((alpha / 2)) * sqrt(1 - cn^2)))
    uu <- 2 - (cn - (qnorm((alpha / 2)) * sqrt(1 - cn^2)))
    est.mahmoud <- cv_corr # cv_corr is an estimate of CV
    lower.tile.mahmoud <- cv_corr / ul
    upper.tile.mahmoud <- cv_corr / uu
    cnt <- sqrt(1 - (1 / (2 * length(x))))
    ult <- cnt + (qnorm(1 - (alpha / 2)) * sqrt(1 - cnt^2))
    uut <- cnt - (qnorm(1 - (alpha / 2)) * sqrt(1 - cnt^2))
    est.normaapprox <- cv_corr # cv_corr is an estimate of CV
    lower.tile.normaapprox <- cv_corr / ult
    upper.tile.normaapprox <- cv_corr / uut
    if (length(x) <= 300) {
      a_value <- shortest_length %>%
        subset(al == alpha & v == length(x) - 1) %>%
        dplyr::select(a)
      b_value <- shortest_length %>%
        subset(al == alpha & v == length(x) - 1) %>%
        dplyr::select(b)
    } else if (length(x) > 300) {
      a_value <- shortest_length %>%
        subset(al == alpha & v == 300) %>%
        dplyr::select(a)
      b_value <- shortest_length %>%
        subset(al == alpha & v == 300) %>%
        dplyr::select(b)
    }
    est.shortest <- cv_corr # cv_corr is an estimate of CV
    lower.tile.shortest <- (cv_corr * sqrt(length(x) - 1)) / sqrt(b_value$b)
    upper.tile.shortest <- (cv_corr * sqrt(length(x) - 1)) / sqrt(a_value$a)
    ttt1 <- qchisq(1 - alpha / 2, v)
    ttt2 <- qchisq(alpha / 2, v)
    est.equal <- cv_corr # cv_corr is an estimate of CV
    lower.tile.equal <- (cv_corr * sqrt(v)) / (sqrt(ttt1))
    upper.tile.equal <- (cv_corr * sqrt(v)) / (sqrt(ttt2))
    boot.normcv_corr.ci <- boot::boot.ci(
      boot.cv_corr,
      conf = (1 - alpha), type = "norm"
    )
    est.norm <- cv_corr # cv_corr is an estimate of CV
    lower.tile.norm <- boot.normcv_corr.ci$normal[2]
    upper.tile.norm <- boot.normcv_corr.ci$normal[3]
    boot.basiccv_corr.ci <- boot::boot.ci(
      boot.cv_corr,
      conf = (1 - alpha), type = "basic"
    )
    est.basic <- cv_corr # cv_corr is an estimate of CV
    lower.tile.basic <- boot.basiccv_corr.ci$basic[4]
    upper.tile.basic <- boot.basiccv_corr.ci$basic[5]
    boot.percentcv_corr.ci <- boot::boot.ci(
      boot.cv_corr,
      conf = (1 - alpha), type = "basic"
    )
    est.percent <- cv_corr # cv_corr is an estimate of CV
    lower.tile.percent <- boot.percentcv_corr.ci$percent[4]
    upper.tile.percent <- boot.percentcv_corr.ci$percent[5]
    boot.bcacv_corr.ci <- boot::boot.ci(
      boot.cv_corr,
      conf = (1 - alpha), type = "basic"
    )
    est.bca <- cv_corr # cv_corr is an estimate of CV
    lower.tile.bca <- boot.bcacv_corr.ci$bca[4]
    upper.tile.bca <- boot.bcacv_corr.ci$bca[5]
  }
  # preparing the output based on the selected methods
  if ("kelley" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Kelley 95% CI"
        ),
        statistics = data.frame(
          est = round(est.kelley * 100, digits = digits),
          lower = round(lower.tile.kelley * 100, digits = digits),
          upper = round(upper.tile.kelley * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("kelley" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Kelley 95% CI"
        ),
        statistics = data.frame(
          est = round(est.kelley * 100, digits = digits),
          lower = round(lower.tile.kelley * 100, digits = digits),
          upper = round(upper.tile.kelley * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("mckay" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with McKay 95% CI"
        ),
        statistics = data.frame(
          est = round(est.mckay * 100, digits = digits),
          lower = round(lower.tile.mckay * 100, digits = digits),
          upper = round(upper.tile.mckay * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("mckay" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with McKay 95% CI"
        ),
        statistics = data.frame(
          est = round(est.mckay * 100, digits = digits),
          lower = round(lower.tile.mckay * 100, digits = digits),
          upper = round(upper.tile.mckay * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("miller" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Miller 95% CI"
        ),
        statistics = data.frame(
          est = round(est.miller * 100, digits = digits),
          lower = round(lower.tile.miller * 100, digits = digits),
          upper = round(upper.tile.miller * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("miller" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Miller 95% CI"
        ),
        statistics = data.frame(
          est = round(est.miller * 100, digits = digits),
          lower = round(lower.tile.miller * 100, digits = digits),
          upper = round(upper.tile.miller * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("vangel" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Vangel 95% CI"
        ),
        statistics = data.frame(
          est = round(est.vangel * 100, digits = digits),
          lower = round(lower.tile.vangel * 100, digits = digits),
          upper = round(upper.tile.vangel * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("vangel" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Vangel 95% CI"
        ),
        statistics = data.frame(
          est = round(est.vangel * 100, digits = digits),
          lower = round(lower.tile.vangel * 100, digits = digits),
          upper = round(upper.tile.vangel * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("mahmoudvand_hassani" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Mahmoudvand-Hassani 95% CI"
        ),
        statistics = data.frame(
          est = round(est.mahmoud * 100, digits = digits),
          lower = round(lower.tile.mahmoud * 100, digits = digits),
          upper = round(upper.tile.mahmoud * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("mahmoudvand_hassani" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Mahmoudvand-Hassani 95% CI"
        ),
        statistics = data.frame(
          est = round(est.mahmoud * 100, digits = digits),
          lower = round(lower.tile.mahmoud * 100, digits = digits),
          upper = round(upper.tile.mahmoud * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("normal_approximation" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Normal Approximation 95% CI"
        ),
        statistics = data.frame(
          est = round(est.normaapprox * 100, digits = digits),
          lower = round(
            lower.tile.normaapprox * 100,
            digits = digits
          ),
          upper = round(
            upper.tile.normaapprox * 100,
            digits = digits
          ),
          row.names = c(" ")
        )
      )
    )
  } else if ("normal_approximation" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Normal Approximation 95% CI"
        ),
        statistics = data.frame(
          est = round(est.normaapprox * 100, digits = digits),
          lower = round(
            lower.tile.normaapprox * 100,
            digits = digits
          ),
          upper = round(
            upper.tile.normaapprox * 100,
            digits = digits
          ),
          row.names = c(" ")
        )
      )
    )
  } else if ("shortest_length" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Shortest-Length 95% CI"
        ),
        statistics = data.frame(
          est = round(est.shortest * 100, digits = digits),
          lower = round(lower.tile.shortest * 100, digits = digits),
          upper = round(upper.tile.shortest * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("shortest_length" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Shortest-Length 95% CI"
        ),
        statistics = data.frame(
          est = round(est.shortest * 100, digits = digits),
          lower = round(lower.tile.shortest * 100, digits = digits),
          upper = round(upper.tile.shortest * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("equal_tailed" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Equal-Tailed 95% CI"
        ),
        statistics = data.frame(
          est = round(est.equal * 100, digits = digits),
          lower = round(lower.tile.equal * 100, digits = digits),
          upper = round(upper.tile.equal * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("equal_tailed" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Equal-Tailed 95% CI"
        ),
        statistics = data.frame(
          est = round(est.equal * 100, digits = digits),
          lower = round(lower.tile.equal * 100, digits = digits),
          upper = round(upper.tile.equal * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("norm" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Normal Approximation Bootstrap 95% CI"
        ),
        statistics = data.frame(
          est = round(est.norm * 100, digits = digits),
          lower = round(lower.tile.norm * 100, digits = digits),
          upper = round(upper.tile.norm * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("norm" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Normal Approximation Bootstrap 95% CI"
        ),
        statistics = data.frame(
          est = round(est.norm * 100, digits = digits),
          lower = round(lower.tile.norm * 100, digits = digits),
          upper = round(upper.tile.norm * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("basic" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Basic Bootstrap 95% CI"
        ),
        statistics = data.frame(
          est = round(est.basic * 100, digits = digits),
          lower = round(lower.tile.basic * 100, digits = digits),
          upper = round(upper.tile.basic * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("basic" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Basic Bootstrap 95% CI"
        ),
        statistics = data.frame(
          est = round(est.basic * 100, digits = digits),
          lower = round(lower.tile.basic * 100, digits = digits),
          upper = round(upper.tile.basic * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("percent" %in% method && correction == FALSE) {
    stop("percent method is not developed yet")
    # return(
    #     list(
    #         method = (
    #             "cv with Bootstrap Percentile 95% CI"
    #         ),
    #         statistics = data.frame(
    #             est = round(est.percent * 100, digits = digits),
    #             lower = round(lower.tile.percent * 100, digits = digits),
    #             upper = round(upper.tile.percent * 100, digits = digits),
    #             row.names = c(" ")
    #         )
    #     )
    #     )
  } else if ("percent" %in% method && correction == TRUE) {
    stop("percent method is not developed yet")
    # return(
    #     list(
    #         method = (
    #             "Corrected cv with Bootstrap Percentile 95% CI"
    #         ),
    #         statistics = data.frame(
    #             est = round(est.percent * 100, digits = digits),
    #             lower = round(lower.tile.percent * 100, digits = digits),
    #             upper = round(upper.tile.percent * 100, digits = digits),
    #             row.names = c(" ")
    #         )
    #     )
    # )
  } else if ("bca" %in% method && correction == FALSE) {
    stop("BCa method is not developed yet")
    # return(
    #     list(
    #         method = (
    #             "cv with Adjusted Bootstrap Percentile (BCa) 95% CI"
    #         ),
    #         statistics = data.frame(
    #             est = round(est.bca * 100, digits = digits),
    #             lower = round(lower.tile.bca * 100, digits = digits),
    #             upper = round(upper.tile.bca * 100, digits = digits),
    #             row.names = c(" ")
    #         )
    #     )
    #     )
  } else if ("bca" %in% method && correction == TRUE) {
    stop("BCa method is not developed yet")
    # return(
    #     list(
    #         method = (
    #         "Corrected cv with Adjusted Bootstrap Percentile (BCa) 95% CI"
    #         ),
    #         statistics = data.frame(
    #             est = round(est.bca * 100, digits = digits),
    #             lower = round(lower.tile.bca * 100, digits = digits),
    #             upper = round(upper.tile.bca * 100, digits = digits),
    #             row.names = c(" ")
    #         )
    #     )
    # )
  } else if (method == "all" && correction == FALSE) {
    return(
      list(
        method = "All methods",
        statistics = data.frame(
          row.names = c(
            "kelley", # 1
            "mckay", # 2
            "miller", # 3
            "vangel", # 4
            "mahmoudvand_hassani", # 5
            "equal_tailed", # 6
            "shortest_length", # 7
            "normal_approximation", # 8
            "norm", # 9
            "basic" # 10
            # "perc"  # 11
            # "bca"  # 12
          ),
          est = c(
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            # round(cv * 100, digits = digits),
            # round(cv * 100, digits = digits),
            round(cv * 100, digits = digits)
          ),
          lower = c(
            round(lower.tile.kelley * 100, digits = digits),
            round(lower.tile.mckay * 100, digits = digits),
            round(lower.tile.miller * 100, digits = digits),
            round(lower.tile.vangel * 100, digits = digits),
            round(lower.tile.mahmoud * 100, digits = digits),
            round(lower.tile.equal * 100, digits = digits),
            round(lower.tile.shortest * 100, digits = digits),
            round(lower.tile.normaapprox * 100, digits = digits),
            round(lower.tile.norm * 100, digits = digits),
            round(lower.tile.basic * 100, digits = digits)
            # ,
            # round(lower.tile.percent * 100, digits = digits)
            # ,
            # round(lower.tile.bca * 100, digits = digits)
          ),
          upper = c(
            round(upper.tile.kelley * 100, digits = digits),
            round(upper.tile.mckay * 100, digits = digits),
            round(upper.tile.miller * 100, digits = digits),
            round(upper.tile.vangel * 100, digits = digits),
            round(upper.tile.mahmoud * 100, digits = digits),
            round(upper.tile.equal * 100, digits = digits),
            round(upper.tile.shortest * 100, digits = digits),
            round(upper.tile.normaapprox * 100, digits = digits),
            round(upper.tile.norm * 100, digits = digits),
            round(upper.tile.basic * 100, digits = digits)
            # ,
            # round(upper.tile.percent * 100, digits = digits)
            # ,
            # round(upper.tile.bca * 100, digits = digits)
          ),
          description = c(
            "cv with Kelley 95% CI",
            "cv with McKay 95% CI",
            "cv with Miller 95% CI",
            "cv with Vangel 95% CI",
            "cv with Mahmoudvand-Hassani 95% CI",
            "cv with Equal-Tailed 95% CI",
            "cv with Shortest-Length 95% CI",
            "cv with Normal Approximation 95% CI",
            "cv with Normal Approximation Bootstrap 95% CI",
            "cv with Basic Bootstrap 95% CI"
            # ,
            # "cv with Bootstrap Percentile 95% CI"
            # ,
            # "cv with Adjusted Bootstrap Percentile (BCa) 95% CI"
          )
        )
      )
    )
  } else if (method == "all" && correction == TRUE) {
    return(
      list(
        method = "All methods",
        statistics = data.frame(
          row.names = c(
            "kelley", # 1
            "mckay", # 2
            "miller", # 3
            "vangel", # 4
            "mahmoudvand_hassani", # 5
            "equal_tailed", # 6
            "shortest_length", # 7
            "normal_approximation", # 8
            "norm", # 9
            "basic" # 10
            # "perc"  # 11
            # "bca"  # 12
          ),
          est = c(
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            # round(cv_corr * 100, digits = digits),
            # round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits)
          ),
          lower = c(
            round(lower.tile.kelley * 100, digits = digits),
            round(lower.tile.mckay * 100, digits = digits),
            round(lower.tile.miller * 100, digits = digits),
            round(lower.tile.vangel * 100, digits = digits),
            round(lower.tile.mahmoud * 100, digits = digits),
            round(lower.tile.equal * 100, digits = digits),
            round(lower.tile.shortest * 100, digits = digits),
            round(lower.tile.normaapprox * 100, digits = digits),
            round(lower.tile.norm * 100, digits = digits),
            round(lower.tile.basic * 100, digits = digits)
            # ,
            # round(lower.tile.percent * 100, digits = digits)
            # ,
            # round(lower.tile.bca * 100, digits = digits)
          ),
          upper = c(
            round(upper.tile.kelley * 100, digits = digits),
            round(upper.tile.mckay * 100, digits = digits),
            round(upper.tile.miller * 100, digits = digits),
            round(upper.tile.vangel * 100, digits = digits),
            round(upper.tile.mahmoud * 100, digits = digits),
            round(upper.tile.equal * 100, digits = digits),
            round(upper.tile.shortest * 100, digits = digits),
            round(upper.tile.normaapprox * 100, digits = digits),
            round(upper.tile.norm * 100, digits = digits),
            round(upper.tile.basic * 100, digits = digits)
            # ,
            # round(upper.tile.percent * 100, digits = digits)
            # ,
            # round(upper.tile.bca * 100, digits = digits)
          ),
          description = c(
            "Corrected cv with Kelley 95% CI",
            "Corrected cv with McKay 95% CI",
            "Corrected cv with Miller 95% CI",
            "Corrected cv with Vangel 95% CI",
            "Corrected cv with Mahmoudvand-Hassani 95% CI",
            "Corrected cv with Equal-Tailed 95% CI",
            "Corrected cv with Shortest-Length 95% CI",
            "Corrected cv with Normal Approximation 95% CI",
            "Corrected cv with Normal Approximation Bootstrap 95% CI",
            "Corrected cv with Basic Bootstrap 95% CI"
            # ,
            # "Corrected cv with Bootstrap Percentile 95% CI"
            # ,
            # "Corrected cv with Adjusted Bootstrap Percentile (BCa) 95% CI"
          )
        )
      )
    )
  }
}

## ----eval = TRUE, warning=FALSE, message=FALSE--------------------------------
x <- c(
  0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
  4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
)
cv_versatile(
  x,
  na.rm = TRUE,
  digits = 3,
  method = "kelley",
  correction = TRUE,
  alpha = 0.05
)

## ----eval = TRUE, warning=FALSE, message=FALSE--------------------------------
cv_versatile(
  x,
  na.rm = TRUE,
  digits = 3,
  method = "mckay",
  correction = TRUE,
  alpha = 0.05
)

## ----eval = TRUE, warning=FALSE, message=FALSE--------------------------------
cv_versatile(
  x,
  na.rm = TRUE,
  digits = 3,
  method = "miller",
  correction = TRUE,
  alpha = 0.05
)

## ----eval = TRUE, warning=FALSE, message=FALSE--------------------------------
cv_versatile(
  x,
  na.rm = TRUE,
  digits = 3,
  method = "vangel",
  correction = TRUE,
  alpha = 0.05
)

## ----eval = TRUE, warning=FALSE, message=FALSE--------------------------------
cv_versatile(
  x,
  na.rm = TRUE,
  digits = 3,
  method = "mahmoudvand_hassani",
  correction = TRUE,
  alpha = 0.05
)

## ----eval = TRUE, warning=FALSE, message=FALSE--------------------------------
cv_versatile(
  x,
  na.rm = TRUE,
  digits = 3,
  method = "normal_approximation",
  correction = TRUE,
  alpha = 0.05
)

## ----eval = TRUE, warning=FALSE, message=FALSE--------------------------------
cv_versatile(
  x,
  na.rm = TRUE,
  digits = 3,
  method = "shortest_length",
  correction = TRUE,
  alpha = 0.05
)

## ----eval = TRUE, warning=FALSE, message=FALSE--------------------------------
cv_versatile(
  x,
  na.rm = TRUE,
  digits = 3,
  method = "equal_tailed",
  correction = TRUE,
  alpha = 0.05
)

## ----eval = TRUE, warning=FALSE, message=FALSE--------------------------------
cv_versatile(
  x,
  na.rm = TRUE,
  digits = 3,
  method = "basic",
  correction = TRUE,
  alpha = 0.05
)

## ----eval = TRUE, warning=FALSE, message=FALSE--------------------------------
cv_versatile(
  x,
  na.rm = TRUE,
  digits = 3,
  method = "all",
  correction = FALSE,
  alpha = 0.05
)

context("state_x")
test_that(
  desc = "error message is thrown when x is NULL",
  {
    x <- NULL
    expect_error(
      CoefVar$new(x, na.rm = FALSE)$est(),
      "object 'x' not found"
    )
    expect_error(
      CoefQuartVar$new(x, na.rm = FALSE)$est(),
      "object 'x' not found"
    )
    expect_error(
      CoefVarCI$new(x, na.rm = FALSE)$kelley_ci(),
      "object 'x' not found"
    )
    expect_error(
      CoefQuartVarCI$new(x, na.rm = FALSE)$bonett_ci(),
      "object 'x' not found"
    )
    expect_error(
      BootCoefVar$new(x, na.rm = FALSE)$boot_cv(),
      "object 'x' not found"
    )
    expect_error(
      BootCoefQuartVar$new(x, na.rm = FALSE)$boot_cqv(),
      "object 'x' not found"
    )
    expect_error(
      SampleQuantiles$new(x, na.rm = FALSE)$qx(),
      "object 'x' not found"
    )
    expect_error(
      cv_versatile(x, na.rm = FALSE),
      "object 'x' not found"
    )
    expect_error(
      cqv_versatile(x, na.rm = FALSE),
      "object 'x' not found"
    )
  }
)
test_that(
  desc = "error message is thrown when x is missing",
  {
    expect_error(
      CoefVar$new(na.rm = FALSE)$est(),
      "object 'x' not found"
    )
    expect_error(
      CoefQuartVar$new(na.rm = FALSE)$est(),
      "object 'x' not found"
    )
    expect_error(
      CoefVarCI$new(na.rm = FALSE)$kelley_ci(),
      "object 'x' not found"
    )
    expect_error(
      CoefQuartVarCI$new(na.rm = FALSE)$bonett_ci(),
      "object 'x' not found"
    )
    expect_error(
      BootCoefVar$new(na.rm = FALSE)$boot_cv(),
      "object 'x' not found"
    )
    expect_error(
      BootCoefQuartVar$new(na.rm = FALSE)$boot_cqv(),
      "object 'x' not found"
    )
    expect_error(
      SampleQuantiles$new(na.rm = FALSE)$qx(),
      "object 'x' not found"
    )
    expect_error(
      cv_versatile(na.rm = FALSE),
      "object 'x' not found"
    )
    expect_error(
      cqv_versatile(na.rm = FALSE),
      "object 'x' not found"
    )
  }
)
test_that(
  desc = "error message is thrown when x is NA",
  {
    x <- NA
    expect_error(
      CoefVar$new(x, na.rm = FALSE)$est(),
      "missing values and NaN's not allowed if 'na.rm' is FALSE"
    )
    expect_error(
      CoefQuartVar$new(x, na.rm = FALSE)$est(),
      "missing values and NaN's not allowed if 'na.rm' is FALSE"
    )
    expect_error(
      CoefVarCI$new(x, na.rm = FALSE)$kelley_ci(),
      "missing values and NaN's not allowed if 'na.rm' is FALSE"
    )
    expect_error(
      CoefQuartVarCI$new(x, na.rm = FALSE)$bonett_ci(),
      "missing values and NaN's not allowed if 'na.rm' is FALSE"
    )
    expect_error(
      BootCoefVar$new(x, na.rm = FALSE)$boot_cv(),
      "missing values and NaN's not allowed if 'na.rm' is FALSE"
    )
    expect_error(
      BootCoefQuartVar$new(x, na.rm = FALSE)$boot_cqv(),
      "missing values and NaN's not allowed if 'na.rm' is FALSE"
    )
    expect_error(
      SampleQuantiles$new(x, na.rm = FALSE)$qx(),
      "missing values and NaN's not allowed if 'na.rm' is FALSE"
    )
    expect_error(
      cv_versatile(x, na.rm = FALSE),
      "argument is not a numeric vector"
    )
    expect_error(
      cqv_versatile(x, na.rm = FALSE),
      "argument is not a numeric vector"
    )
  }
)
test_that(
  desc = "error message is thrown when x has NA",
  {
    y <- c(NA, 6)
    expect_error(
      CoefVar$new(x = y, na.rm = FALSE)$est(),
      "missing values and NaN's not allowed if 'na.rm' is FALSE"
    )
    expect_error(
      CoefQuartVar$new(x = y, na.rm = FALSE)$est(),
      "missing values and NaN's not allowed if 'na.rm' is FALSE"
    )
    expect_error(
      CoefVarCI$new(x = y, na.rm = FALSE)$kelley_ci(),
      "missing values and NaN's not allowed if 'na.rm' is FALSE"
    )
    expect_error(
      CoefQuartVarCI$new(x = y, na.rm = FALSE)$bonett_ci(),
      "missing values and NaN's not allowed if 'na.rm' is FALSE"
    )
    expect_error(
      BootCoefVar$new(x = y, na.rm = FALSE)$boot_cv(),
      "missing values and NaN's not allowed if 'na.rm' is FALSE"
    )
    expect_error(
      BootCoefQuartVar$new(x = y, na.rm = FALSE)$boot_cqv(),
      "missing values and NaN's not allowed if 'na.rm' is FALSE"
    )
    expect_error(
      SampleQuantiles$new(x = y, na.rm = FALSE)$qx(),
      "missing values and NaN's not allowed if 'na.rm' is FALSE"
    )
    expect_error(
      cv_versatile(x = y, na.rm = FALSE),
      "missing values and NaN's not allowed if 'na.rm' is FALSE"
    )
    expect_error(
      cqv_versatile(x = y, na.rm = FALSE),
      "missing values and NaN's not allowed if 'na.rm' is FALSE"
    )
  }
)
test_that(
  desc = "error message is thrown when x is not numeric",
  {
    y <- c("a", "b")
    expect_error(
      CoefVar$new(x = y)$est(),
      "argument is not a numeric vector"
    )
    expect_error(
      CoefQuartVar$new(x = y)$est(),
      "argument is not a numeric vector"
    )
    expect_error(
      CoefVarCI$new(x = y)$kelley_ci(),
      "argument is not a numeric vector"
    )
    expect_error(
      CoefQuartVarCI$new(x = y)$bonett_ci(),
      "argument is not a numeric vector"
    )
    expect_error(
      BootCoefVar$new(x = y)$boot_cv(),
      "argument is not a numeric vector"
    )
    expect_error(
      BootCoefQuartVar$new(x = y)$boot_cqv(),
      "argument is not a numeric vector"
    )
    expect_error(
      SampleQuantiles$new(x = y)$qx(),
      "argument is not a numeric vector"
    )
    expect_error(
      cv_versatile(x = y),
      "argument is not a numeric vector"
    )
    expect_error(
      cqv_versatile(x = y),
      "argument is not a numeric vector"
    )
  }
)

context("state_aak_adj")

# Infants weights (in grams) data, Abu-Shawiesh, Akyuz & Kibria (2019),
# Section 6.1 (n = 61). Used in their Tables 4 & 5.
infants_data <- c(
  4960, 5130, 4260, 5160, 4050, 5240, 4350, 4360, 3930, 4410,
  4610, 4102, 3530, 4550, 4460, 2940, 4160, 4110, 4410, 4800,
  5130, 3670, 4550, 4290, 5210, 4950, 5210, 3210, 4030, 3580,
  4360, 4360, 3920, 4050, 4630, 3756, 4382, 4586, 5336, 2828,
  4172, 4256, 4594, 4866, 4784, 4520, 5238, 4320, 5070, 5330,
  3836, 5916, 5010, 4344, 3496, 4148, 4044, 5192, 4368, 4180,
  5044
)

# Postmortem interval (PMI) data, Abu-Shawiesh, Akyuz & Kibria (2019),
# Section 6.2 (n = 22). Used in their Tables 4 & 6.
pmi_data <- c(
  5.5, 14.5, 6.0, 5.5, 5.3, 5.8, 11.0, 6.1, 7.0, 14.5,
  10.4, 4.6, 4.3, 7.2, 10.5, 6.5, 3.3, 7.0, 4.1, 6.2,
  10.4, 4.9
)

small_data <- c(
  0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
  4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
)

test_that(
  desc = "AA&K-ADJ matches paper Table 5 on infants data",
  {
    # Paper reports (0.1163, 0.1706). Allow 0.5 percentage points
    # of slack.
    result <- CoefVarCI$new(
      x = infants_data, alpha = 0.05, digits = 6
    )$aak_adj_ci()
    expect_equal(
      result$statistics$lower / 100, 0.1163,
      tolerance = 0.005
    )
    expect_equal(
      result$statistics$upper / 100, 0.1706,
      tolerance = 0.005
    )
  }
)

test_that(
  desc = "AA&K-ADJ matches paper Table 6 on PMI data",
  {
    # Paper reports (0.3275, 0.6532).
    result <- CoefVarCI$new(
      x = pmi_data, alpha = 0.05, digits = 6
    )$aak_adj_ci()
    expect_equal(
      result$statistics$lower / 100, 0.3275,
      tolerance = 0.005
    )
    expect_equal(
      result$statistics$upper / 100, 0.6532,
      tolerance = 0.005
    )
  }
)

test_that(
  desc = "AA&K-ADJ bounds bracket the point estimate",
  {
    result <- CoefVarCI$new(
      x = small_data, digits = 6
    )$aak_adj_ci()
    expect_lt(result$statistics$lower, result$statistics$est)
    expect_gt(result$statistics$upper, result$statistics$est)
  }
)

test_that(
  desc = "AA&K-ADJ method label changes with correction flag",
  {
    no_corr <- CoefVarCI$new(
      x = small_data, correction = FALSE
    )$aak_adj_ci()
    with_corr <- CoefVarCI$new(
      x = small_data, correction = TRUE
    )$aak_adj_ci()
    expect_true(grepl("Abu-Shawiesh-Akyuz-Kibria", no_corr$method))
    expect_true(grepl("Abu-Shawiesh-Akyuz-Kibria", with_corr$method))
    expect_true(grepl("corrected", with_corr$method))
  }
)

test_that(
  desc = "AA&K-ADJ requires at least 4 observations",
  {
    expect_error(
      CoefVarCI$new(x = c(1.0, 2.0, 3.0))$aak_adj_ci(),
      regexp = "(at least 4|G_2|gamma_hat)"
    )
  }
)

test_that(
  desc = "cv_versatile aak_adj matches the R6 method",
  {
    r6 <- CoefVarCI$new(
      x = small_data, alpha = 0.05, digits = 6
    )$aak_adj_ci()
    proc <- cv_versatile(
      x = small_data, method = "aak_adj", alpha = 0.05, digits = 6
    )
    expect_equal(
      r6$statistics$lower, proc$statistics$lower,
      tolerance = 0.0001
    )
    expect_equal(
      r6$statistics$upper, proc$statistics$upper,
      tolerance = 0.0001
    )
  }
)

context("state_cv_versatile")
test_that(
    desc = "finds the correct findings", {
        x <- c(
            0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
            4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
        )
        expect_equal(
            cv_versatile(
                x,
                method = "kelley",
                correction = FALSE
                )$method,
            "cv with Kelley 95% CI"
        )
        expect_equal(
            cv_versatile(
                x,
                method = "kelley",
                correction = FALSE
            )$statistics$est,
            57.8, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "kelley",
                correction = FALSE
            )$statistics$lower,
            41.5, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "kelley",
                correction = FALSE
            )$statistics$upper,
            98.5, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "kelley",
                correction = TRUE
            )$method,
            "Corrected cv with Kelley 95% CI"
        )
        expect_equal(
            cv_versatile(
                x,
                method = "kelley",
                correction = TRUE
            )$statistics$est,
            58.1, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "kelley",
                correction = TRUE
            )$statistics$lower,
            41.5, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "kelley",
                correction = TRUE
            )$statistics$upper,
            98.5, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "mckay",
                correction = FALSE
            )$method,
            "cv with McKay 95% CI"
        )
        expect_equal(
            cv_versatile(
                x,
                method = "mckay",
                correction = FALSE
            )$statistics$est,
            57.8, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "mckay",
                correction = FALSE
            )$statistics$lower,
            41.4, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "mckay",
                correction = FALSE
            )$statistics$upper,
            108.5, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "mckay",
                correction = TRUE
            )$method,
            "Corrected cv with McKay 95% CI"
        )
        expect_equal(
            cv_versatile(
                x,
                method = "mckay",
                correction = TRUE
            )$statistics$est,
            58.1, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "mckay",
                correction = TRUE
            )$statistics$lower,
            41.6, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "mckay",
                correction = TRUE
            )$statistics$upper,
            109.4, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "miller",
                correction = FALSE
            )$method,
            "cv with Miller 95% CI"
        )
        expect_equal(
            cv_versatile(
                x,
                method = "miller",
                correction = FALSE
            )$statistics$est,
            57.8, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "miller",
                correction = FALSE
            )$statistics$lower,
            34.1, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "miller",
                correction = FALSE
            )$statistics$upper,
            81.5, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "miller",
                correction = TRUE
            )$method,
            "Corrected cv with Miller 95% CI"
        )
        expect_equal(
            cv_versatile(
                x,
                method = "miller",
                correction = TRUE
            )$statistics$est,
            58.1, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "miller",
                correction = TRUE
            )$statistics$lower,
            34.2, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "miller",
                correction = TRUE
            )$statistics$upper,
            81.9, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "vangel",
                correction = FALSE
            )$method,
            "cv with Vangel 95% CI"
        )
        expect_equal(
            cv_versatile(
                x,
                method = "vangel",
                correction = FALSE
            )$statistics$est,
            57.8, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "vangel",
                correction = FALSE,
                digits = 2,
            )$statistics$lower,
            40.95, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "vangel",
                correction = FALSE,
                digits = 2,
            )$statistics$upper,
            103.93, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "vangel",
                correction = TRUE,
                digits = 2
            )$method,
            "Corrected cv with Vangel 95% CI"
        )
        expect_equal(
            cv_versatile(
                x,
                method = "vangel",
                correction = TRUE,
                digits = 2
            )$statistics$est,
            58.06, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "vangel",
                correction = TRUE,
                digits = 2
            )$statistics$lower,
            41.13, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "vangel",
                correction = TRUE,
                digits = 2
            )$statistics$upper,
            104.71, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "mahmoudvand_hassani",
                correction = FALSE,
                digits = 2
            )$method,
            "cv with Mahmoudvand-Hassani 95% CI"
        )
        expect_equal(
            cv_versatile(
                x,
                method = "mahmoudvand_hassani",
                correction = FALSE
            )$statistics$est,
            57.8, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "mahmoudvand_hassani",
                correction = FALSE
            )$statistics$lower,
            43.5, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "mahmoudvand_hassani",
                correction = FALSE
            )$statistics$upper,
            82.9, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "mahmoudvand_hassani",
                correction = TRUE
            )$method,
            "Corrected cv with Mahmoudvand-Hassani 95% CI"
        )
        expect_equal(
            cv_versatile(
                x,
                method = "mahmoudvand_hassani",
                correction = TRUE
            )$statistics$est,
            58.1, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "mahmoudvand_hassani",
                correction = TRUE
            )$statistics$lower,
            43.7, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "mahmoudvand_hassani",
                correction = TRUE
            )$statistics$upper,
            83.3, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "normal_approximation",
                correction = FALSE
            )$method,
            "cv with Normal Approximation 95% CI"
        )
        expect_equal(
            cv_versatile(
                x,
                method = "normal_approximation",
                correction = FALSE
            )$statistics$est,
            57.8, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "normal_approximation",
                correction = FALSE
            )$statistics$lower,
            44.5, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "normal_approximation",
                correction = FALSE
            )$statistics$upper,
            85.3, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "normal_approximation",
                correction = TRUE
            )$method,
            "Corrected cv with Normal Approximation 95% CI"
        )
        expect_equal(
            cv_versatile(
                x,
                method = "normal_approximation",
                correction = TRUE
            )$statistics$est,
            58.1, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "normal_approximation",
                correction = TRUE
            )$statistics$lower,
            44.8, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "normal_approximation",
                correction = TRUE
            )$statistics$upper,
            85.7, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "shortest_length",
                correction = FALSE
            )$method,
            "cv with Shortest-Length 95% CI"
        )
        expect_equal(
            cv_versatile(
                x,
                method = "shortest_length",
                correction = FALSE
            )$statistics$est,
            57.8, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "shortest_length",
                correction = FALSE
            )$statistics$lower,
            42, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "shortest_length",
                correction = FALSE
            )$statistics$upper,
            81, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "shortest_length",
                correction = TRUE
            )$method,
            "Corrected cv with Shortest-Length 95% CI"
        )
        expect_equal(
            cv_versatile(
                x,
                method = "shortest_length",
                correction = TRUE
            )$statistics$est,
            58.1, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "shortest_length",
                correction = TRUE
            )$statistics$lower,
            42.2, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "shortest_length",
                correction = TRUE
            )$statistics$upper,
            81.4, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "equal_tailed",
                correction = FALSE
            )$method,
            "cv with Equal-Tailed 95% CI"
        )
        expect_equal(
            cv_versatile(
                x,
                method = "equal_tailed",
                correction = FALSE
            )$statistics$est,
            57.8, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "equal_tailed",
                correction = FALSE
            )$statistics$lower,
            43.9, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "equal_tailed",
                correction = FALSE
            )$statistics$upper,
            84.4, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "equal_tailed",
                correction = TRUE
            )$method,
            "Corrected cv with Equal-Tailed 95% CI"
        )
        expect_equal(
            cv_versatile(
                x,
                method = "equal_tailed",
                correction = TRUE
            )$statistics$est,
            58.1, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "equal_tailed",
                correction = TRUE
            )$statistics$lower,
            44.2, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "equal_tailed",
                correction = TRUE
            )$statistics$upper,
            84.8, tolerance = 0.0001
        )
        expect_equal(
            cv_versatile(
                x,
                method = "norm",
                correction = FALSE
            )$method,
            "cv with Normal Approximation Bootstrap 95% CI"
        )
        expect_equal(
            cv_versatile(
                x,
                method = "norm",
                correction = FALSE
            )$statistics$est,
            57.8, tolerance = 0.0001
        )
        expect_true(
            abs(
                cv_versatile(
                x,
                method = "norm",
                correction = FALSE
            )$statistics$lower -
            38.6
            )/100 < 0.05
        )
        expect_true(
            abs(
                cv_versatile(
                    x,
                    method = "norm",
                    correction = FALSE
                )$statistics$upper -
                    77.6
            )/100 < 0.05
        )
        expect_equal(
            cv_versatile(
                x,
                method = "norm",
                correction = TRUE
            )$method,
            "Corrected cv with Normal Approximation Bootstrap 95% CI"
        )
        expect_equal(
            cv_versatile(
                x,
                method = "norm",
                correction = TRUE
            )$statistics$est,
            58.1, tolerance = 0.0001
        )
        expect_true(
            abs(
                cv_versatile(
                    x,
                    method = "norm",
                    correction = TRUE
                )$statistics$lower -
                    38.6
            )/100 < 0.05
        )
        expect_true(
            abs(
                cv_versatile(
                    x,
                    method = "norm",
                    correction = TRUE
                )$statistics$upper -
                    77.6
            )/100 < 0.05
        )
        expect_equal(
            cv_versatile(
                x,
                method = "basic",
                correction = FALSE
            )$method,
            "cv with Basic Bootstrap 95% CI"
        )
        expect_equal(
            cv_versatile(
                x,
                method = "basic",
                correction = FALSE
            )$statistics$est,
            57.8, tolerance = 0.0001
        )
        expect_true(
            abs(
                cv_versatile(
                    x,
                    method = "basic",
                    correction = FALSE
                )$statistics$lower -
                    37.1
            )/100 < 0.05
        )
        expect_true(
            abs(
                cv_versatile(
                    x,
                    method = "basic",
                    correction = FALSE
                )$statistics$upper -
                    77
            )/100 < 0.05
        )
        expect_equal(
            cv_versatile(
                x,
                method = "basic",
                correction = TRUE
            )$method,
            "Corrected cv with Basic Bootstrap 95% CI"
        )
        expect_equal(
            cv_versatile(
                x,
                method = "basic",
                correction = TRUE
            )$statistics$est,
            58.1, tolerance = 0.0001
        )
        expect_true(
            abs(
                cv_versatile(
                    x,
                    method = "basic",
                    correction = TRUE
                )$statistics$lower -
                    38
            )/100 < 0.05
        )
        expect_true(
            abs(
                cv_versatile(
                    x,
                    method = "basic",
                    correction = TRUE
                )$statistics$upper -
                    77
            )/100 < 0.05
        )
        expect_error(
            cv_versatile(
                x,
                method = "perc",
                correction = FALSE
            ),
            "percent method is not developed yet"
        )

        expect_error(
            cv_versatile(
                x,
                method = "perc",
                correction = TRUE
            ),
            "percent method is not developed yet"
        )
        expect_error(
            cv_versatile(
                x,
                method = "bca",
                correction = FALSE
            ),
            "BCa method is not developed yet"
        )

        expect_error(
            cv_versatile(
                x,
                method = "bca",
                correction = TRUE
            ),
            "BCa method is not developed yet"
        )
        expect_equal(
            cv_versatile(
                x,
                method = "all",
                correction = TRUE
            )$method,
            "All methods"
        )
        expect_equal(
            nrow(cv_versatile(
                x,
                method = "all",
                correction = TRUE
            )$statistics),
            10
        )
        expect_equal(
            ncol(cv_versatile(
                x,
                method = "all",
                correction = TRUE
            )$statistics),
            4
        )
        expect_equal(
            cv_versatile(
                x,
                correction = FALSE
            )$method,
            "cv = sd/mean (may be biased)"
        )
        expect_equal(
            cv_versatile(
                x,
                correction = TRUE
            )$method,
            "Corrected (i.e., unbiased) cv"
        )
        expect_equal(
            cv_versatile(
                x,
                correction = FALSE
            )$statistics$est,
            57.8
        )
        expect_equal(
            cv_versatile(
                x,
                correction = TRUE
            )$statistics$est,
            58.1
        )
        expect_equal(
            cv_versatile(
                x,
                method = "all",
                correction = FALSE
            )$method,
            "All methods"
        )
        expect_equal(
            nrow(cv_versatile(
                x,
                method = "all",
                correction = FALSE
            )$statistics),
            10
        )
        expect_equal(
            ncol(cv_versatile(
                x,
                method = "all",
                correction = FALSE
            )$statistics),
            4
        )
    }
)
test_that(
    desc = "for sample sizes higher than 340", {
        x = c(
            56.28, 52.29, 58.66, 56.05, 37.32, 18.18, 78.08, 38.39, 93.93,
            41.96, 49.09, 78.49, 49.76, 51.65, 55.9, 65.85, 37.08, 46.69,
            44.49, 36.83, 69.96, 21.2, 77.01, 26.32, 62.31, 21.54, 74.73,
            61.32, 43.84, 49.76, 13.89, 73.07, 26.38, 55.03, 11.61, 17.78,
            32.12, 72.92, 41.44, 60.3, 45, 100.65, 71.87, 52.2, 54.73, 29.99,
            38.21, 13.18, 41.32, 48.59, 46.76, 33.38, 28.29, 65.12, 53.73,
            62.54, 49.66, 83.33, 66.21, 27.8, 42.6, 55.05, 30.01, 87.68,
            32.93, 76.31, 13.26, 20.2, 46.94, 27.05, 36.75, 44.95, 62.23,
            58.24, 51, 30.79, 57.8, 52.22, 49.84, 61.98, 39.96, 46.91,
            13.99, 54.04, 53.13, 93.67, 56.72, 57.17, 20.4, 23.73, 21.97,
            54.37, 66.22, 39.5, 50.42, 20.67, 68.66, 65.03, 33.55, 100.59,
            45.43, 40.03, 53.42, 78.23, 63.51, 61.04, 30.8, 88.69, 69.16,
            39.33, 14.18, 60.43, 35.21, 98.71, 35.01, 39.88, 27.6, 32.33,
            69.26, 21.67, 62.35, 46.8, 59.15, 64.16, 72.91, 24.68, 36.44,
            45.34, 41.84, 36.28, 54.17, 45.36, 82.28, 45.25, 55.22, 30.04,
            51.01, 34.79, 46.13, 49.59, 54.52, 23.64, 57.98, 42.13, 43.14,
            67.11, 30.43, 47.95, 63.44, 54.26, 37.75, 49.57, 17.56, 67.66,
            32.16, 71.98, 36.13, 68.23, 38.82, 47.03, 34.53, 41.77, 43.25,
            47.08, 88.6, 31.61, 58.32, 58.27, 47.02, 38.34, 73.21, 45.28, 38,
            58.01, 79, 51.11, 61.2, 13.3, 50.21, 57.53, 71.4, 105.32, 39.89,
            51.98, 32.98, 20.59, 28.2, 33.96, 85.3, 63.19, 100.91, 44.62, 35.41,
            56.94, 39.75, 71.22, 12.32, 48.4, 80.24, 33.41, 73.47, 34.65, 71.1,
            40.3, 46.4, 52.37, 20.21, 5.5, 81.66, 78.47, 53.05, 47.3, 71.99,
            48.34, 56.94, 36.78, 57.21, 46.05, 54.79, 32.73, 50.19, 61.07,
            54.22, 37.26, 64.59, 50.72, 78.73, 65.26, 66.1, 52.95, 67.44,
            34.29, 60.13, 39.77, 54.8, 52.46, 60.72, 72.22, 25.38, 61.72,
            50.99, 43.96, 2.36, 71.13, 43.24, 82.37, 65.28, 88.13, 40.22,
            50.06, 62.81, 46.98, 42.26, 63.3, 46.53, 71.14, 49.58, 37.13,
            22.93, 90.91, 33.28, 52.73, 62.67, 43.31, 58.22, 43.04, 76.42,
            61.37, 32.68, 24.24, 55, 71.75, 55.95, 53.42, 16.42, 49.25,
            55.18, 77.89, 55.21, 24.17, 105.72, 21.06, 60.25, 32.05, 25.76,
            17.94, 48.27, 71.78, 58.91, 34.12, 12.98, 76.39, 13.59, 69.43,
            42.54, 81.56, 67.75, 45.33, 49.66, 53.43, 50.95, 30.04, 68.6,
            55.81, 18.16, 30.59, 72.27, 68.67, 28.25, 61.37, 40, 43.59,
            54.38, 34.63, 90.22, 69.85, 47.67, 38.38, 54.81, 47.04, 49.96,
            52.27, 76.31, 49.16, 45.46, 49.27, 59.86, 35, 50.06, 41.74, 64.21,
            68.83, 42.18, 57.58, 61.69, 44.13, 75.31, 68.29, 51.46, 57.52, 39.74
        )
        expect_equal(
            cv_versatile(
                x, digits = 7, method = "mahmoudvand_hassani"
                )$statistics$est,
            37.97088,
            tolerance = 0.01
        )
        expect_equal(
            cv_versatile(
                x, digits = 7, method = "mahmoudvand_hassani", correction = TRUE
                )$statistics$est,
            37.95918,
            tolerance = 0.01
        )
        expect_equal(
            cv_versatile(
                x, digits = 7, method = "shortest_length"
            )$statistics$lower,
            37.32619,
            tolerance = 0.01
        )
        expect_equal(
            cv_versatile(
                x, digits = 7, method = "shortest_length", correction = TRUE
            )$statistics$lower,
            37.31469,
            tolerance = 0.01
        )
        expect_equal(
            cv_versatile(
                x,
                method = "all",
                correction = FALSE
            )$method,
            "All methods"
        )
        expect_equal(
            nrow(cv_versatile(
                x,
                method = "all",
                correction = FALSE
            )$statistics),
            10
        )
        expect_equal(
            ncol(cv_versatile(
                x,
                method = "all",
                correction = FALSE
            )$statistics),
            4
        )
        expect_equal(
            cv_versatile(
                x,
                method = "all",
                correction = TRUE
            )$method,
            "All methods"
        )
        expect_equal(
            nrow(cv_versatile(
                x,
                method = "all",
                correction = TRUE
            )$statistics),
            10
        )
        expect_equal(
            ncol(cv_versatile(
                x,
                method = "all",
                correction = TRUE
            )$statistics),
            4
        )
    }
)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ----echo=FALSE, warning=FALSE, message=FALSE, include = FALSE----------------
library(dplyr)
library(boot)
library(SciViews)
x <- c(
    0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
    4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
)
cqv_versatile <- function(
    x,
    na.rm = FALSE,
    digits = 1,
    method = NULL,
    R = NULL,
    ...
) {
    # require(dplyr)
    # require(SciViews)
    # require(boot)
    if (missing(x) || is.null(x)) {
        stop("object 'x' not found")
    } else if (!missing(x)) {
        x <- x
    }
    if (!is.numeric(x)) {
        stop("argument is not a numeric vector: returning NA")
        return(NA_real_)
    }
    na.rm = na.rm  # removes NAs if TRUE
    if (na.rm == TRUE) {
        x <- x[!is.na(x)]
    } else if (anyNA(x)) {
        stop(
            "missing values and NaN's not allowed if 'na.rm' is FALSE"
        )
    }
    if (is.null(digits)) {
        digits = 1
    }
    digits = digits  # digits required for rounding
    method = method  # returns 95% confidence interval
    if (is.null(R)) {
        R = 1000
    }
    q3 <- unname(
        quantile(
            x,
            probs = 0.75,  # third quartile (0.75 percentile)
            na.rm = na.rm
        )
    )
    q1 <- unname(
        quantile(
            x,
            probs = 0.25,  # first quartile (0.25 percentile)
            na.rm = na.rm
        )
    )
    if (q3 == 0) {  # to avoid NaNs when q3 and q1 are zero
        warning(
            "cqv is NaN because q3 and q1 are 0, max was used instead of q3"
        )
        q3 <- max(x, na.rm = na.rm)
    }
    a <- ceiling(
        (length(x)/4) - (1.96 * (((3 * length(x))/16)^(0.5)))
    )
    b <- round(
        (length(x)/4) + (1.96 * (((3 * length(x))/16)^(0.5))),
        digits = 0
    )
    c <- length(x) + 1 - b
    d <- length(x) + 1 - a
    Ya <- dplyr::nth(x, a, order_by = x)
    Yb <- dplyr::nth(x, b, order_by = x)
    Yc <- dplyr::nth(x, c, order_by = x)
    Yd <- dplyr::nth(x, d, order_by = x)
    star <- 0
    for (i in a:(b - 1)) {
        star[i] <- (
            (choose(length(x), i)) * (0.25^(i)) * (0.75^(length(x) - i))
        )
        alphastar <- 1 - sum(star[i], na.rm = na.rm)
    }
    zzz <- qnorm((1 - ((1 - alphastar)/2)))
    f1square <- (3 * (zzz)^2)/(4 * length(x) * ((Yb - Ya)^2))
    f3square <- (3 * (zzz)^2)/(4 * length(x) * ((Yd - Yc)^2))
    D <- q3 - q1
    S <- q3 + q1
    v <- (
        (1/(16 * length(x))) * (
            (((3/f1square) + (3/f3square) - (2/sqrt(f1square * f3square))) / D^2) +
                (((3/f1square) + (3/f3square) + (2/sqrt(f1square * f3square))) / S^2) -
                ((2 * ((3/f3square) - (3/f1square)))/(D*S))
        )
    )
    ccc <- length(x)/(length(x) - 1)
    upper.tile <- exp(((SciViews::ln((D/S)) * ccc)) + (zzz * (v^(0.5))))
    lower.tile <- exp(((SciViews::ln((D/S)) * ccc)) - (zzz * (v^(0.5))))
    if (
        unname(quantile(x, probs = 0.75, na.rm = na.rm)) != 0
    ) {
        boot.cqv <- boot::boot(
            x,
            function(x, i) {
                round(((
                    unname(quantile(x[i], probs = 0.75, na.rm = na.rm)) -
                        unname(quantile(x[i], probs = 0.25, na.rm = na.rm))
                ) / (
                    unname(quantile(x[i], probs = 0.75, na.rm = na.rm)) +
                        unname(quantile(x[i], probs = 0.25, na.rm = na.rm))
                )) * 100, digits = digits)
            },
            R = R
        )
    } else if (
        unname(quantile(x, probs = 0.75, na.rm = na.rm)) == 0
    ) {
        boot.cqv <- boot::boot(
            x,
            function(x, i) {
                round(((
                    max(x[i], na.rm = na.rm) -
                        unname(quantile(x[i], probs = 0.25, na.rm = na.rm))
                ) / (
                    max(x[i], na.rm = na.rm) +
                        unname(quantile(x[i], probs = 0.25, na.rm = na.rm))
                )) * 100, digits = digits)
            },
            R = R
        )
    }

    if (is.null(method)) {
        boot.cqv.ci <- NA
    } else if (method == "bonett") {
        boot.cqv.ci <- NA
    } else if (method == "norm") {
        boot.norm.ci <- boot::boot.ci(boot.cqv, conf = 0.95, type = "norm")
    } else if (method == "basic") {
        boot.basic.ci <- boot::boot.ci(boot.cqv, conf = 0.95, type = "basic")
    } else if (method == "perc") {
        boot.perc.ci <- boot::boot.ci(boot.cqv, conf = 0.95, type = "perc")
    } else if (method == "bca") {
        boot.bca.ci <- boot::boot.ci(boot.cqv, conf = 0.95, type = "bca")
    } else if (method == "all") {
        boot.norm.ci <- boot::boot.ci(boot.cqv, conf = 0.95, type = "norm")
        boot.basic.ci <- boot::boot.ci(boot.cqv, conf = 0.95, type = "basic")
        boot.perc.ci <- boot::boot.ci(boot.cqv, conf = 0.95, type = "perc")
        boot.bca.ci <- boot::boot.ci(boot.cqv, conf = 0.95, type = "bca")
    }

    if (is.null(method)) {
        cqv <- round(
            100 * ((q3 - q1)/(q3 + q1)), digits = digits
        )
    } else if (method == "bonett") {
        cqv <- round(
            100 * ((q3 - q1)/(q3 + q1)), digits = digits
        )
    } else if (method == "norm") {
        cqv <- round(
            100 * ((q3 - q1)/(q3 + q1)), digits = digits
        )
    } else if (method == "basic") {
        cqv <- round(
            100 * ((q3 - q1)/(q3 + q1)), digits = digits
        )
    } else if (method == "perc") {
        cqv <- round(
            100 * ((q3 - q1)/(q3 + q1)), digits = digits
        )
    } else if (method == "bca") {
        cqv <- round(
            100 * ((q3 - q1)/(q3 + q1)), digits = digits
        )
    } else if (method == "all") {
        cqv <- round(
            100 * ((q3 - q1)/(q3 + q1)), digits = digits
        )
    }

    if (is.null(method)) {
        lower <- NA
        upper <- NA
    } else if (method == "bonett") {
        lower <- round(lower.tile * 100, digits = digits)
        upper <- round(upper.tile * 100, digits = digits)
    } else if (method == "norm") {
        lower <- round(boot.norm.ci$normal[2], digits = digits)
        upper <- round(boot.norm.ci$normal[3], digits = digits)
    } else if (method == "basic") {
        lower <- round(boot.basic.ci$basic[4], digits = digits)
        upper <- round(boot.basic.ci$basic[5], digits = digits)
    } else if (method == "perc") {
        lower <- round(boot.perc.ci$percent[4], digits = digits)
        upper <- round(boot.perc.ci$percent[5], digits = digits)
    } else if (method == "bca") {
        lower <- round(boot.bca.ci$bca[4], digits = digits)
        upper <- round(boot.bca.ci$bca[5], digits = digits)
    }

    if (is.null(method)) {
        return(
            list(
                method = "cqv = (q3-q1)/(q3+q1)",
                statistics = data.frame(
                    est = cqv,
                    row.names = c(" ")
                )
            )
        )
    } else if (method == "bonett" && cqv != 100) {
        return(
            list(
                method = "cqv with Bonett 95% CI",
                statistics = data.frame(
                    est = cqv,
                    lower = lower,
                    upper = upper,
                    row.names = c(" ")
                )
            )
        )
    } else if (method == "norm" && cqv != 100) {
        return(
            list(
                method = "cqv with normal approximation 95% CI",
                statistics = data.frame(
                    est = cqv,
                    lower = lower,
                    upper = upper,
                    row.names = c(" ")
                )
            )
        )
    } else if (method == "basic" && cqv != 100) {
        return(
            list(
                method = "cqv with basic bootstrap 95% CI",
                statistics = data.frame(
                    est = cqv,
                    lower = lower,
                    upper = upper,
                    row.names = c(" ")
                )
            )
        )
    } else if (method == "perc" && cqv != 100) {
        return(
            list(
                method = "cqv with bootstrap percentile 95% CI",
                statistics = data.frame(
                    est = cqv,
                    lower = lower,
                    upper = upper,
                    row.names = c(" ")
                )
            )
        )
    } else if (method == "bca" && cqv != 100) {
        return(
            list(
                method = "cqv with adjusted bootstrap percentile (BCa) 95% CI",
                statistics = data.frame(
                    est = cqv,
                    lower = lower,
                    upper = upper,
                    row.names = c(" ")
                )
            )
        )
    } else if (
        (
            method == "norm" | method == "bonett" | method == "basic" | method == "perc" |
            method == "bca" | method == "all"
        ) && cqv == 100
    ) {
        warning(
            "All values of t are equal to  100 \n Cannot calculate confidence intervals \n"
        )
        return(
            list(
                method = "cqv with Bonett 95% CI",
                statistics = data.frame(
                    est = cqv,
                    lower = round(lower.tile * 100, digits = digits),
                    upper = round(upper.tile * 100, digits = digits),
                    row.names = c(" ")
                )
            )
        )
    } else if (method == "all" && cqv != 100) {
        return(
            list(
                method = "All methods",
                statistics = data.frame(
                    row.names = c(
                        "bonett",
                        "norm",
                        "basic",
                        "percent",
                        "bca"
                    ),
                    est = c(cqv, cqv, cqv, cqv, cqv),
                    lower = c(
                        round(lower.tile * 100, digits = digits),
                        round(boot.norm.ci$normal[2], digits = digits),
                        round(boot.basic.ci$basic[4], digits = digits),
                        round(boot.perc.ci$percent[4], digits = digits),
                        round(boot.bca.ci$bca[4], digits = digits)
                    ),
                    upper = c(
                        round(upper.tile * 100, digits = digits),
                        round(boot.norm.ci$normal[3], digits = digits),
                        round(boot.basic.ci$basic[5], digits = digits),
                        round(boot.perc.ci$percent[5], digits = digits),
                        round(boot.bca.ci$bca[5], digits = digits)
                    ),
                    description = c(
                        "cqv with Bonett 95% CI",
                        "cqv with normal approximation 95% CI",
                        "cqv with basic bootstrap 95% CI",
                        "cqv with bootstrap percentile 95% CI",
                        "cqv with adjusted bootstrap percentile (BCa) 95% CI"
                    )
                )
            )
        )
    } else {
        stop("method for confidence interval is not available")
        return(NA_real_)
    }
}

## ----eval = TRUE, warning=FALSE, message=FALSE--------------------------------
x <- c(
    0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
    4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
)
cqv_versatile(
    x, 
    na.rm = TRUE, 
    digits = 3, 
    method = "bonett"
)

## ----eval = TRUE, warning=FALSE, message=FALSE--------------------------------
cqv_versatile(
    x, 
    na.rm = TRUE, 
    digits = 3, 
    method = "bca"
)

## ----eval = TRUE, warning=FALSE, message=FALSE--------------------------------
cqv_versatile(
    x, 
    na.rm = TRUE, 
    digits = 3, 
    method = "all"
)

context("state_cqv_versatile")
test_that(
  desc = "check if 0.75 percentile is zero to avoid NANs",
  {
    x <- c(
      0L, 0L, 0L, 0L, 0L, 0L, 0L, 1771L, 0L, 0L, 106L, 0L, 0L, 0L,
      0L, 0L, 0L, 0L, 84L, 168L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 124L,
      0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 127L, 0L, 0L, 0L, 100L, 0L,
      0L, 0L, 0L, 0L, 104L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
      0L, 554L, 1174L, 0L, 0L, 0L, 0L, 8L, 0L, 0L, 0L, 0L, 263L, 0L,
      0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
      0L, 0L, 0L, 0L, 2L, 130L, 0L, 0L, 0L, 272L, 0L, 0L, 0L, 0L, 42L,
      0L, 0L, 130L, 0L, 0L, 0L, 0L, 0L, 9L, 0L, 0L, 2L, 0L, 0L, 0L,
      0L, 0L, 826L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
      0L, 0L, 0L, 231L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
      0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
      0L, 0L, 0L, 0L, 177L, 0L, 550L, 0L, 0L, 0L, 0L, 9L, 531L, 0L,
      152L, 0L, 347L, 0L, 0L, 0L, 0L, 827L, 290L, 0L, 0L, 0L, 525L,
      0L, 0L, 122L, 0L, 0L, 0L, 546L, 0L, 10L, 0L, 0L, 0L, 81L, 5L,
      0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
      0L, 0L, 53L, 0L, 0L, 0L, 0L, 0L, 241L, 0L, 0L, 261L, 0L, 0L,
      0L, 0L, 211L, 263L, 588L, 0L, 0L, 197L, 0L, 63L, 0L, 0L, 0L,
      0L, 0L, 297L, 0L, 177L, 0L, 0L, 0L, 0L, 5L, 203L, 139L, 73L,
      0L, 47L, 19L, 0L, 0L, 0L, 0L, 0L, 243L, 0L, 0L, 0L, 0L, 0L, 110L,
      0L, 0L, 0L, 0L, 0L, 0L, 0L, 12L, 110L, 1217L, 386L, 0L, 600L,
      0L, 574L, 0L, 131L, 103L, 0L, 0L, 0L, 0L, 0L, 50L, 565L, 0L,
      0L, 0L, 1961L, 15L, 304L, 0L, 389L, 133L, 585L, 0L, 0L, 0L, 0L,
      0L, 240L, 12L, 0L, 0L, 119L, 21L, 0L, 717L, 0L, 729L, 0L, 1L,
      0L, 0L, 0L, 0L, 0L, 41L, 0L, 0L, 0L, 176L, 0L, 0L, 0L, 0L, 26L,
      322L, 0L, 0L, 37L, 0L, 528L, 271L, 0L, 0L, 0L, 0L, 127L, 0L,
      0L, 0L, 43L, 290L, 0L, 0L, 22L, 52L, 0L, 57L, 112L, 0L, 29L,
      0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 1235L, 0L, 0L, 0L, 3131L,
      0L, 0L, 0L, 0L, 0L, 0L, 284L, 0L, 610L, 760L, 937L, 0L, 0L, 1176L,
      0L, 160L, 1463L, 0L, 607L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
      44L, 0L, 0L, 0L, 0L, 0L, 0L, 1423L, 0L, 618L, 0L, 927L, 0L, 0L,
      0L, 0L, 483L, 111L, 805L, 0L, 0L, 0L, 356L, 0L, 0L, 0L, 0L, 0L,
      0L, 0L, 0L, 954L, 0L, 845L, 0L, 182L, 475L, 328L, 0L, 0L, 0L,
      256L, 0L, 1126L, 0L, 422L, 0L, 0L, 0L, 0L, 0L, 501L, 0L, 0L,
      212L, 4299L, 4096L, 2148L, 12059L, 3945L, 4254L, 978L, 1406L,
      3500L, 1573L, 4207L, 1168L, 3153L, 3028L, 3155L, 1260L, 2469L,
      522L, 2313L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
      0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 836L, 0L, 0L, 0L,
      0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
      0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
      0L, 0L, 0L, 0L, 0L, 49L, 1176L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
      0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L
    )
    randstring <- function(n = 5000) {
      a <- do.call(paste0, replicate(5, sample(LETTERS, n, TRUE), FALSE))
      paste0(
        a, sprintf("%04d", sample(9999, n, TRUE)),
        sample(LETTERS, n, TRUE)
      )
    }
    expect_true(
      quantile(x, probs = 0.75) == 0
    )
    expect_warning(
      cqv_versatile(x),
      "cqv is NaN because q3 and q1 are 0, max was used instead of q3"
    )
    expect_equal(
      cqv_versatile(x)$statistic$est,
      100
    )
    expect_equal(
      cqv_versatile(x)$method,
      "cqv = (q3-q1)/(q3+q1)"
    )
    expect_equal(
      cqv_versatile(x, method = "bonett")$method,
      "cqv with Bonett 95% CI"
    )
    expect_equal(
      cqv_versatile(x, method = "bonett")$statistic$est,
      100
    )
    expect_equal(
      cqv_versatile(x, method = "bonett")$statistic$lower,
      100
    )
    expect_equal(
      cqv_versatile(x, method = "bonett")$statistic$upper,
      100
    )
    expect_warning(
      cqv_versatile(x, method = "bonett"),
      "All values of t are equal to  100 \n Cannot calculate confidence intervals \n"
    )
    expect_equal(
      cqv_versatile(x, method = "norm", R = 10)$method,
      "cqv with Bonett 95% CI"
    )
    expect_equal(
      cqv_versatile(x, method = "norm", R = 10)$statistic$est,
      100
    )
    expect_equal(
      cqv_versatile(x, method = "norm", R = 10)$statistic$lower,
      100
    )
    expect_equal(
      cqv_versatile(x, method = "norm", R = 10)$statistic$upper,
      100
    )
    expect_warning(
      cqv_versatile(x, method = "norm", R = 10),
      "All values of t are equal to  100 \n Cannot calculate confidence intervals \n"
    )
    expect_equal(
      cqv_versatile(x, method = "basic", R = 10)$method,
      "cqv with Bonett 95% CI"
    )
    expect_equal(
      cqv_versatile(x, method = "basic", R = 10)$statistic$est,
      100
    )
    expect_equal(
      cqv_versatile(x, method = "basic", R = 10)$statistic$lower,
      100
    )
    expect_equal(
      cqv_versatile(x, method = "basic", R = 10)$statistic$upper,
      100
    )
    expect_warning(
      cqv_versatile(x, method = "basic", R = 10),
      "All values of t are equal to  100 \n Cannot calculate confidence intervals \n"
    )
    expect_equal(
      cqv_versatile(x, method = "perc", R = 10)$method,
      "cqv with Bonett 95% CI"
    )
    expect_equal(
      cqv_versatile(x, method = "perc", R = 10)$statistic$est,
      100
    )
    expect_equal(
      cqv_versatile(x, method = "perc", R = 10)$statistic$lower,
      100
    )
    expect_equal(
      cqv_versatile(x, method = "perc", R = 10)$statistic$upper,
      100
    )
    expect_warning(
      cqv_versatile(x, method = "perc", R = 10),
      "All values of t are equal to  100 \n Cannot calculate confidence intervals \n"
    )
    expect_equal(
      cqv_versatile(x, method = "bca", R = 10)$method,
      "cqv with Bonett 95% CI"
    )
    expect_equal(
      cqv_versatile(x, method = "bca", R = 10)$statistic$est,
      100
    )
    expect_equal(
      cqv_versatile(x, method = "bca", R = 10)$statistic$lower,
      100
    )
    expect_equal(
      cqv_versatile(x, method = "bca", R = 10)$statistic$upper,
      100
    )
    expect_warning(
      cqv_versatile(x, method = "bca", R = 10),
      "All values of t are equal to  100 \n Cannot calculate confidence intervals \n"
    )
    expect_equal(
      cqv_versatile(x, method = "all", R = 10)$method,
      "cqv with Bonett 95% CI"
    )
    expect_equal(
      cqv_versatile(x, method = "all", R = 10)$statistic$est,
      100
    )
    expect_equal(
      cqv_versatile(x, method = "all", R = 10)$statistic$lower,
      100
    )
    expect_equal(
      cqv_versatile(x, method = "all", R = 10)$statistic$upper,
      100
    )
    expect_warning(
      cqv_versatile(x, method = "all"),
      "All values of t are equal to  100 \n Cannot calculate confidence intervals \n"
    )
    expect_error(
      cqv_versatile(x, method = randstring(1)),
      "method for confidence interval is not available"
    )
  }
)
context("state_cqv_versatile")
test_that(
  desc = "check if 0.75 percentile is non-zero to avoid NANs",
  {
    x <- c(
      0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
      4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
    )
    expect_true(
      quantile(x, probs = 0.75) != 0
    )
    expect_equal(
      cqv_versatile(x)$statistic$est,
      45.6
    )
    expect_equal(
      cqv_versatile(x)$method,
      "cqv = (q3-q1)/(q3+q1)"
    )
    expect_equal(
      cqv_versatile(x, method = "bonett")$statistic$lower,
      24.8,
      tolerance = 0.0001
    )
    expect_equal(
      cqv_versatile(x, method = "bonett")$statistic$upper,
      77.3,
      tolerance = 0.0001
    )
    expect_equal(
      cqv_versatile(x, method = "norm", R = 10)$statistic$est,
      45.6
    )
    expect_equal(
      cqv_versatile(x, method = "norm", R = 10)$method,
      "cqv with normal approximation 95% CI"
    )
    expect_true(
      abs(
        cqv_versatile(x, method = "norm")$statistic$lower - 19.5
      ) / 100 < 0.05
    )
    expect_true(
      abs(
        cqv_versatile(x, method = "norm")$statistic$upper - 71
      ) / 100 < 0.05
    )
    expect_equal(
      cqv_versatile(x, method = "basic")$statistic$est,
      45.6
    )
    expect_equal(
      cqv_versatile(x, method = "basic")$method,
      "cqv with basic bootstrap 95% CI"
    )
    expect_true(
      abs(
        cqv_versatile(x, method = "basic")$statistic$lower - 19.7
      ) / 100 < 0.05
    )
    expect_true(
      abs(
        cqv_versatile(x, method = "basic")$statistic$upper - 73.5
      ) / 100 < 0.05
    )
    expect_equal(
      cqv_versatile(x, method = "perc")$statistic$est,
      45.6
    )
    expect_equal(
      cqv_versatile(x, method = "perc")$method,
      "cqv with bootstrap percentile 95% CI"
    )
    expect_true(
      abs(
        cqv_versatile(x, method = "perc")$statistic$lower - 17.3
      ) / 100 < 0.05
    )
    expect_true(
      abs(
        cqv_versatile(x, method = "perc")$statistic$upper - 67.9
      ) / 100 < 0.05
    )
    expect_equal(
      cqv_versatile(x, method = "bca")$statistic$est,
      45.6
    )
    expect_equal(
      cqv_versatile(x, method = "bca")$method,
      "cqv with adjusted bootstrap percentile (BCa) 95% CI"
    )
    expect_true(
      abs(
        cqv_versatile(x, method = "bca")$statistic$lower - 20.7
      ) / 100 < 0.3
    )
    expect_true(
      abs(
        cqv_versatile(x, method = "bca")$statistic$upper - 76.5
      ) / 100 < 0.3
    )
    expect_equal(
      cqv_versatile(
        x,
        method = "all"
      )$method,
      "All methods"
    )
    expect_equal(
      nrow(cqv_versatile(
        x,
        method = "all"
      )$statistics),
      5
    )
    expect_equal(
      ncol(cqv_versatile(
        x,
        method = "all"
      )$statistics),
      4
    )
  }
)

#' @title Coefficient of Variation (cv)
#' @name cv_versatile
#' @description Versatile function for the coefficient of variation (cv)
#' @param x An \code{R} object. Currently there are methods for numeric vectors
#' @param na.rm a logical value indicating whether \code{NA} values should be
#'   stripped before the computation proceeds.
#' @param digits integer indicating the number of decimal places to be used.
#' @param method a scalar representing the type of confidence intervals
#'   required. The value should be any of the values "kelley", "mckay",
#'   "miller", "vangel", "mahmoudvand_hassani", "equal_tailed",
#'   "shortest_length", "normal_approximation", "norm","basic", "aak_adj",
#'   "aak_ls", "aak_als", or "all".
#' @param correction returns the unbiased estimate of the coefficient of
#'   variation
#' @param alpha The allowed type I error probability
#' @param R integer indicating the number of bootstrap replicates.
#' @details \describe{ \item{\strong{Coefficient of Variation}}{ The \emph{cv}
#'   is a measure of relative dispersion representing the degree of variability
#'   relative to the mean \code{[1]}. Since \eqn{cv} is unitless, it is useful
#'   for comparison of variables with different units. It is also a measure of
#'   homogeneity \code{[1]}. } }
#' @return An object of type "list" which contains the estimate, the intervals,
#'   and the computation method. It has two main components:
#' @return \describe{ \item{$method}{ A description of statistical method used
#'   for the computations. } \item{$statistics}{ A data frame representing three
#'   vectors: est, lower and upper limits of confidence interval \code{(CI)};
#'   additional description vector is provided when "all" is selected: \cr \cr
#'   \strong{est:} \code{cv*100}  \cr \cr \strong{Kelley Confidence
#'   Interval:} Thanks to package \pkg{MBESS} \code{[2]} for the
#'   computation of confidence limits for the noncentrality parameter from a
#'   \emph{t} distribution \link[MBESS]{conf.limits.nct} \code{[3]}.  \cr \cr
#'   \strong{McKay Confidence Interval:} The intervals calculated by the method
#'   introduced by McKay \code{[4]}, using chi-square distribution.  \cr \cr
#'   \strong{Miller Confidence Interval:} The intervals calculated by the
#'   method introduced by Miller \code{[5]}, using the standard normal
#'   distribution.  \cr \cr \strong{Vangel Confidence Interval:} Vangel
#'   \code{[6]} proposed a method for the calculation of CI for \emph{cv}; which
#'   is a modification on McKay’s CI.  \cr \cr \strong{Mahmoudvand-Hassani
#'   Confidence Interval:} Mahmoudvand and Hassani \code{[7]} proposed a new CI
#'   for \emph{cv}; which is obtained using ranked set sampling \emph{(RSS)}
#'   \cr \cr \strong{Normal Approximation Confidence Interval:} Wararit
#'   Panichkitkosolkul \code{[8]} proposed another CI for \emph{cv}; which is a
#'   normal approximation.  \cr \cr \strong{Shortest-Length Confidence
#'   Interval:} Wararit Panichkitkosolkul \code{[8]} proposed another CI for
#'   \emph{cv}; which is obtained through minimizing the length of CI.  \cr \cr
#'   \strong{Equal-Tailed Confidence Interval:} Wararit Panichkitkosolkul
#'   \code{[8]} proposed another CI for \emph{cv}; which is obtained using
#'   chi-square distribution.  \cr \cr \strong{Bootstrap Confidence
#'   Intervals:} Thanks to package \pkg{boot} by Canty & Ripley \code{[9]} we
#'   can obtain bootstrap CI around \emph{cv} using \link[boot]{boot.ci}.  \cr
#'   \cr \strong{Abu-Shawiesh-Akyuz-Kibria Confidence Intervals:}
#'   Abu-Shawiesh, Akyuz, & Kibria \code{[10]} proposed three CIs for the
#'   population \emph{cv} that adjust for non-normality through the sample
#'   kurtosis: the adjusted-degrees-of-freedom CI (\code{aak_adj}), the
#'   large-sample CI (\code{aak_ls}), and the augmented-large-sample CI
#'   (\code{aak_als}).  \cr \cr } }
#' @examples
#' x <- c(
#'     0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
#'     4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
#' )
#' cv_versatile(x)
#' cv_versatile(x, correction = TRUE)
#' cv_versatile(x, na.rm = TRUE, digits = 3, method = "kelley", correction = TRUE)
#' cv_versatile(x, na.rm = TRUE, method = "mahmoudvand_hassani", correction = TRUE)
#' @references \code{[1]} Albatineh, AN., Kibria, BM., Wilcox, ML., & Zogheib,
#'   B, 2014, Confidence interval estimation for the population coefficient of
#'   variation using ranked set sampling: A simulation study, Journal of Applied
#'   Statistics, 41(4), 733–751, DOI:
#'   \doi{10.1080/02664763.2013.847405}
#'
#' @references \code{[2]} Kelley, K., 2018, MBESS: The MBESS R Package. R
#'   package version 4.4. 3.
#'
#' @references \code{[3]} Kelley, K., 2007, Sample size planning for the
#'   coefficient of variation from the accuracy in parameter estimation
#'   approach, Behavior Research Methods, 39(4), 755–766, DOI:
#'   \doi{10.3758/BF03192966}
#' @references \code{[4]} McKay, AT., 1932, Distribution of the Coefficient of
#'   Variation and the Extended“ t” Distribution, Journal of the Royal
#'   Statistical Society, 95(4), 695–698
#' @references \code{[5]} Miller, E., 1991, Asymptotic test statistics for
#'   coefficients of variation, Communications in Statistics-Theory and Methods,
#'   20(10), 3351–3363
#' @references \code{[6]} Vangel, MG., 1996, Confidence intervals for a normal
#'   coefficient of variation, The American Statistician, 50(1), 21–26
#' @references \code{[7]} Mahmoudvand, R., & Hassani, H., 2009, Two new
#'   confidence intervals for the coefficient of variation in a normal
#'   distribution, Journal of Applied Statistics, 36(4), 429–442
#' @references \code{[8]} Panichkitkosolkul, W., 2013, Confidence Intervals for
#'   the Coefficient of Variation in a Normal Distribution with a Known
#'   Population Mean, Journal of Probability and Statistics, 2013, 1–11,
#'   \doi{10.1155/2013/324940}
#'
#' @references \code{[9]} Canty, A., & Ripley, B., 2017, boot: Bootstrap R
#'   (S-Plus) Functions, R package version 1.3-20
#'
#' @references \code{[10]} Abu-Shawiesh, MOA., Akyuz, HE., & Kibria, BG.,
#'   2019, Performance of Some Confidence Intervals for Estimating the
#'   Population Coefficient of Variation under both Symmetric and Skewed
#'   Distributions, Statistics, Optimization & Information Computing, 7(2),
#'   277-290, DOI: \doi{10.19139/soic.v7i2.630}
#' @export
#' @import dplyr SciViews boot R6 utils
NULL
#' @importFrom stats quantile sd qchisq qnorm
#' @importFrom MBESS conf.limits.nct
NULL
globalVariables(c("al", "a", "b"))
cv_versatile <- function(
  x, # Currently there are methods for numeric vectors
  na.rm = FALSE, # indicating whether NA values should be stripped
  digits = 1, # digits of output after rounding. default is 4
  method = NULL, # method for the computation of confidence interval (CI)
  correction = FALSE, # indicating whether to compute the unbiased statistics
  alpha = 0.05, # The allowed type I error probability
  R = NULL, # integer indicating the number of bootstrap replicates
  ...
) {
  # library(MBESS)
  # require(dplyr)
  # require(SciViews)
  # require(boot)
  if (missing(x) || is.null(x)) {
    stop("object 'x' not found")
  } else if (!missing(x)) {
    x <- x
  }
  if (!is.numeric(x)) {
    stop("argument is not a numeric vector")
  }
  na.rm <- na.rm # removes NAs if TRUE
  if (na.rm == TRUE) {
    x <- x[!is.na(x)]
  } else if (anyNA(x)) {
    stop(
      "missing values and NaN's not allowed if 'na.rm' is FALSE"
    )
  }

  # if (is.null(digits)) {
  #     digits = 1
  # }
  if (is.null(R)) {
    R <- 1000
  }
  digits <- digits # digits required for rounding
  shortest_length <- data.frame( # "a" and "b" values for shortest-length CI
    v = c( # degrees of freedom
      2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
      21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 40, 50, 60, 70, 80, 90,
      100, 150, 200, 250, 300, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13,
      14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
      30, 40, 50, 60, 70, 80, 90, 100, 150, 200, 250, 300, 2, 3, 4,
      5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21,
      22, 23, 24, 25, 26, 27, 28, 29, 30, 40, 50, 60, 70, 80, 90,
      100, 150, 200, 250, 300
    ),
    al = c( # al is the allowed type I error probability
      0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,
      0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,
      0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,
      0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,
      0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
      0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
      0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
      0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
      0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
      0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01,
      0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01,
      0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01,
      0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01,
      0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01
    ),
    a = c( # a is an attribute for the length of CI
      0.2065, 0.5654, 1.02, 1.5352, 2.093, 2.6828, 3.2981, 3.9343,
      4.5883, 5.2573, 5.9397, 6.6337, 7.3382, 8.0521, 8.7745,
      9.5047, 10.2421, 10.9861, 11.7362, 12.4919, 13.253,
      14.0191, 14.7899, 15.565, 16.3443, 17.1275, 17.9144,
      18.7049, 19.4987, 27.5919, 35.9012, 44.3661, 52.9501,
      61.629, 70.386, 79.2086, 124.0372, 169.6646, 215.8057,
      262.3132, 0.1015, 0.3449, 0.6918, 1.1092, 1.5776,
      2.0851, 2.6235, 3.1874, 3.7729, 4.3768, 4.9967,
      5.6308, 6.2776, 6.9357, 7.6042, 8.282, 8.9685,
      9.6629, 10.3647, 11.0733, 11.7882, 12.5092,
      13.2357, 13.9675, 14.7043, 15.4458, 16.1917,
      16.9419, 17.6961, 25.4233, 33.4085, 41.5794, 49.8923,
      58.3183, 66.8374, 75.4347, 119.2737, 164.0642,
      209.4667, 255.3057, 0.02, 0.114, 0.2937, 0.5461,
      0.8567, 1.2143, 1.6107, 2.0394, 2.4958, 2.976, 3.4771,
      3.9968, 4.5329, 5.084, 5.6487, 6.2256, 6.8139, 7.4126,
      8.0209, 8.6383, 9.264, 9.8976, 10.5385, 11.1864, 11.8408,
      12.5014, 13.1678, 13.8397, 14.517, 21.5331, 28.8879,
      36.4863, 44.2711, 52.2044, 60.2597, 68.4177, 110.3262,
      153.4834, 197.444, 241.9776
    ),
    b = c( # b is an attribute for the length of CI
      12.5208, 13.1532, 14.18, 15.3498, 16.5807, 17.8391,
      19.1099, 20.3848, 21.6598, 22.9325, 24.2016,
      25.4666, 26.7269, 27.9825, 29.2334, 30.4796,
      31.7212, 32.9585, 34.1915, 35.4205, 36.6455,
      37.8668, 39.0844, 40.2986, 41.5095, 42.7171,
      43.9217, 45.1234, 46.3222, 58.1755, 69.8342,
      81.3479, 92.7487, 104.0584, 115.2925, 126.4628,
      181.6128, 235.9748, 289.8273, 343.3155, 15.1194,
      15.5897, 16.5735, 17.7432, 18.9954, 20.2863,
      21.5953, 22.9118, 24.2303, 25.5476, 26.8618,
      28.1717, 29.4769, 30.777, 32.072, 33.3619,
      34.6467, 35.9266, 37.2016, 38.472, 39.7379,
      40.9995, 42.257, 43.5105, 44.7601, 46.006,
      47.2483, 48.4872, 49.7229, 61.9217, 73.892,
      85.6914, 97.3573, 108.9153, 120.3839, 131.7767,
      187.9079, 243.1025, 297.691, 351.8461, 20.8264,
      20.9856, 21.8371, 22.9867, 24.2618, 25.6017,
      26.9749, 28.3643, 29.7602, 31.158, 32.5543,
      33.9474, 35.3358, 36.7192, 38.0968, 39.4688,
      40.8347, 42.1952, 43.5498, 44.8989, 46.2426,
      47.581, 48.9144, 50.2428, 51.5665, 52.8856,
      54.2002, 55.5107, 56.8169, 69.6808, 82.2534,
      94.6063, 106.7867, 118.8272, 130.7514, 142.5771,
      200.6194, 257.4375, 313.462, 368.9185
    )
  )
  # if ("kelley" %in% method) {
  #     if (!requireNamespace("MBESS")) {
  #         warning(
  #     "package 'MBESS' required to calculate Kelley's confidence interval"
  #         )
  #     }
  # }
  cv <- (
    stats::sd(x, na.rm = na.rm) / mean(x, na.rm = na.rm) # coefficient of variation
  )
  cv_corr <- cv * (
    (1 - (1 / (4 * (length(x) - 1))) + # corrected coefficient of variation
      (1 / length(x)) * cv^2) +
      (1 / (2 * (length(x) - 1)^2))
  )
  if (is.null(method) == TRUE && correction == FALSE) {
    return(
      list(
        method = "cv = sd/mean (may be biased)",
        statistics = data.frame(
          est = round(cv * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if (is.null(method) == TRUE && correction == TRUE) {
    return(
      list(
        method = "Corrected (i.e., unbiased) cv",
        statistics = data.frame(
          est = round(cv_corr * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if (!is.null(method)) {
    method <- tolower(method) # convert user's input to lower-case
    method <- match.arg( # match the user's input with available methods
      arg = method,
      choices = c(
        "kelley", "mckay", "miller", "vangel", "mahmoudvand_hassani",
        "equal_tailed", "shortest_length", "normal_approximation",
        "norm", "basic", "perc", "bca",
        "aak_adj", "aak_ls", "aak_als", "all"
      ),
      several.ok = TRUE
    )
  }

  # calculating cv's bootstrap CI
  boot.cv <- boot::boot(
    x,
    function(x, i) { # coefficient of variation
      stats::sd(x[i], na.rm = na.rm) / mean(x[i], na.rm = na.rm)
    },
    R = R
  )
  boot.cv_corr <- boot::boot(
    x,
    function(x, i) { # corrected coefficient of variation
      stats::sd(x[i], na.rm = na.rm) / mean(x[i], na.rm = na.rm) * (
        (1 - (1 / (4 * (length(x[i]) - 1))) +
          (1 / length(x)) * (
            stats::sd(x[i], na.rm = na.rm) / mean(x[i], na.rm = na.rm)
          )^2) +
          (1 / (2 * (length(x) - 1)^2))
      )
    },
    R = R
  )
  # calculating cv and its CI attributes based on selected methods
  if ("kelley" %in% method && correction == FALSE) {
    ci <- MBESS::conf.limits.nct(
      ncp = sqrt(length(x)) / cv_corr,
      df = length(x) - 1,
      conf.level = (1 - alpha)
    )
    est.kelley <- cv # cv is an estimate of CV
    lower.tile.kelley <- unname(sqrt(length(x)) / ci$Upper.Limit)
    upper.tile.kelley <- unname(sqrt(length(x)) / ci$Lower.Limit)
  } else if ("kelley" %in% method && correction == TRUE) {
    ci <- MBESS::conf.limits.nct(
      ncp = sqrt(length(x)) / cv_corr,
      df = length(x) - 1,
      conf.level = (1 - alpha)
    )
    est.kelley <- cv_corr # corrected cv is an estimate of CV
    lower.tile.kelley <- unname(sqrt(length(x)) / ci$Upper.Limit)
    upper.tile.kelley <- unname(sqrt(length(x)) / ci$Lower.Limit)
  } else if ("mckay" %in% method && correction == FALSE) {
    # if (cv > 0.33) {
    #     warning("Confidence interval may be very approximate")
    # }
    v <- length(x) - 1
    t1 <- stats::qchisq(1 - alpha / 2, v) / v
    t2 <- stats::qchisq(alpha / 2, v) / v
    u1 <- v * t1
    u2 <- v * t2
    est.mckay <- cv # cv is an estimate of CV
    lower.tile.mckay <- cv / sqrt((u1 / (v + 1) - 1) * (cv^2) + u1 / v)
    upper.tile.mckay <- cv / sqrt((u2 / (v + 1) - 1) * (cv^2) + u2 / v)
  } else if ("mckay" %in% method && correction == TRUE) {
    # if (cv_corr > 0.33) {
    #     warning("Confidence interval may be very approximate")
    # }
    v <- length(x) - 1
    t1 <- stats::qchisq(1 - alpha / 2, v) / v
    t2 <- stats::qchisq(alpha / 2, v) / v
    u1 <- v * t1
    u2 <- v * t2
    est.mckay <- cv_corr # corrected cv is an estimate of CV
    lower.tile.mckay <- cv_corr / sqrt((u1 / (v + 1) - 1) * (cv_corr^2) + u1 / v)
    upper.tile.mckay <- cv_corr / sqrt((u2 / (v + 1) - 1) * (cv_corr^2) + u2 / v)
  } else if ("miller" %in% method && correction == FALSE) {
    v <- length(x) - 1
    z_alpha_over2 <- stats::qnorm(1 - (alpha / 2))
    u <- sqrt(
      (cv^2 / v) * (0.5 + cv^2)
    )
    zu <- z_alpha_over2 * u
    est.miller <- cv # cv is an estimate of CV
    lower.tile.miller <- cv - zu
    upper.tile.miller <- cv + zu
  } else if ("miller" %in% method && correction == TRUE) {
    v <- length(x) - 1
    z_alpha_over2 <- stats::qnorm(1 - (alpha / 2))
    u <- sqrt(
      (cv_corr^2 / v) * (0.5 + cv_corr^2)
    )
    zu <- z_alpha_over2 * u
    est.miller <- cv_corr # corrected cv is an estimate of CV
    lower.tile.miller <- cv_corr - zu
    upper.tile.miller <- cv_corr + zu
  } else if ("vangel" %in% method && correction == FALSE) {
    # if (cv > 0.33) {
    #     warning("Confidence interval may be very approximate")
    # }
    v <- length(x) - 1
    t1 <- stats::qchisq(1 - alpha / 2, v) / v
    t2 <- stats::qchisq(alpha / 2, v) / v
    u1 <- v * t1
    u2 <- v * t2
    est.vangel <- cv # cv is an estimate of CV
    lower.tile.vangel <- cv / sqrt((((u1 + 1) / (v)) - 1) * (cv^2) + u1 / v)
    upper.tile.vangel <- cv / sqrt((((u2 + 1) / (v)) - 1) * (cv^2) + u2 / v)
  } else if ("vangel" %in% method && correction == TRUE) {
    # if (cv_corr > 0.33) {
    #     warning("Confidence interval may be very approximate")
    # }
    v <- length(x) - 1
    t1 <- stats::qchisq(1 - alpha / 2, v) / v
    t2 <- stats::qchisq(alpha / 2, v) / v
    u1 <- v * t1
    u2 <- v * t2
    est.vangel <- cv_corr # corrected cv is an estimate of CV
    lower.tile.vangel <- cv_corr / sqrt(
      (((((u1 + 1) / (v)) - 1)) * (cv_corr^2)) + u1 / v
    )
    upper.tile.vangel <- cv_corr / sqrt(
      (((((u2 + 1) / (v)) - 1)) * (cv_corr^2)) + u2 / v
    )
  } else if ("mahmoudvand_hassani" %in% method && correction == FALSE) {
    if (length(x) <= 340) {
      cn <- sqrt(2 / (length(x) - 1)) * (
        (gamma(length(x) / 2)) / (gamma((length(x) - 1) / 2))
      )
    } else {
      cn <- sqrt(2 / (length(x) - 1)) * (
        (lgamma(length(x) / 2)) / (lgamma((length(x) - 1) / 2))
      )
    }
    ul <- 2 - (cn + (stats::qnorm((alpha / 2)) * sqrt(1 - cn^2)))
    uu <- 2 - (cn - (stats::qnorm((alpha / 2)) * sqrt(1 - cn^2)))
    est.mahmoud <- cv # cv is an estimate of CV
    lower.tile.mahmoud <- cv / ul
    upper.tile.mahmoud <- cv / uu
  } else if ("mahmoudvand_hassani" %in% method && correction == TRUE) {
    if (length(x) <= 340) {
      cn <- sqrt(2 / (length(x) - 1)) * (
        (gamma(length(x) / 2)) / (gamma((length(x) - 1) / 2))
      )
    } else {
      cn <- sqrt(2 / (length(x) - 1)) * (
        (lgamma(length(x) / 2)) / (lgamma((length(x) - 1) / 2))
      )
    }
    ul <- 2 - (cn + (stats::qnorm((alpha / 2)) * sqrt(1 - cn^2)))
    uu <- 2 - (cn - (stats::qnorm((alpha / 2)) * sqrt(1 - cn^2)))
    est.mahmoud <- cv_corr # corrected cv is an estimate of CV
    lower.tile.mahmoud <- cv_corr / ul
    upper.tile.mahmoud <- cv_corr / uu
  } else if ("normal_approximation" %in% method && correction == FALSE) {
    cn <- sqrt(1 - (1 / (2 * length(x))))
    ul <- cn + (stats::qnorm(1 - (alpha / 2)) * sqrt(1 - cn^2))
    uu <- cn - (stats::qnorm(1 - (alpha / 2)) * sqrt(1 - cn^2))
    est.normaapprox <- cv # cv is an estimate of CV
    lower.tile.normaapprox <- cv / ul
    upper.tile.normaapprox <- cv / uu
  } else if ("normal_approximation" %in% method && correction == TRUE) {
    cn <- sqrt(1 - (1 / (2 * length(x))))
    ul <- cn + (stats::qnorm(1 - (alpha / 2)) * sqrt(1 - cn^2))
    uu <- cn - (stats::qnorm(1 - (alpha / 2)) * sqrt(1 - cn^2))
    est.normaapprox <- cv_corr # corrected cv is an estimate of CV
    lower.tile.normaapprox <- cv_corr / ul
    upper.tile.normaapprox <- cv_corr / uu
  } else if ("shortest_length" %in% method && correction == FALSE) {
    if (length(x) <= 300) {
      a_value <- shortest_length[
        shortest_length$al == alpha & shortest_length$v == length(x) - 1,
        "a",
        drop = FALSE
      ]
      b_value <- shortest_length[
        shortest_length$al == alpha & shortest_length$v == length(x) - 1,
        "b",
        drop = FALSE
      ]
    } else if (length(x) > 300) {
      a_value <- shortest_length[shortest_length$al == alpha & shortest_length$v == 300, "a", drop = FALSE]
      b_value <- shortest_length[shortest_length$al == alpha & shortest_length$v == 300, "b", drop = FALSE]
    }
    est.shortest <- cv # cv is an estimate of CV
    lower.tile.shortest <- (cv * sqrt(length(x) - 1)) / sqrt(b_value$b)
    upper.tile.shortest <- (cv * sqrt(length(x) - 1)) / sqrt(a_value$a)
  } else if ("shortest_length" %in% method && correction == TRUE) {
    if (length(x) <= 300) {
      a_value <- shortest_length[
        shortest_length$al == alpha & shortest_length$v == length(x) - 1,
        "a",
        drop = FALSE
      ]
      b_value <- shortest_length[
        shortest_length$al == alpha & shortest_length$v == length(x) - 1,
        "b",
        drop = FALSE
      ]
    } else if (length(x) > 300) {
      a_value <- shortest_length[shortest_length$al == alpha & shortest_length$v == 300, "a", drop = FALSE]
      b_value <- shortest_length[shortest_length$al == alpha & shortest_length$v == 300, "b", drop = FALSE]
    }
    est.shortest <- cv_corr # corrected cv is an estimate of CV
    lower.tile.shortest <- (cv_corr * sqrt(length(x) - 1)) / sqrt(b_value$b)
    upper.tile.shortest <- (cv_corr * sqrt(length(x) - 1)) / sqrt(a_value$a)
  } else if ("equal_tailed" %in% method && correction == FALSE) {
    v <- length(x) - 1
    tt1 <- stats::qchisq(1 - alpha / 2, v)
    tt2 <- stats::qchisq(alpha / 2, v)
    est.equal <- cv # cv is an estimate of CV
    lower.tile.equal <- (cv * sqrt(v)) / (sqrt(tt1))
    upper.tile.equal <- (cv * sqrt(v)) / (sqrt(tt2))
  } else if ("equal_tailed" %in% method && correction == TRUE) {
    v <- length(x) - 1
    tt1 <- stats::qchisq(1 - alpha / 2, v)
    tt2 <- stats::qchisq(alpha / 2, v)
    est.equal <- cv_corr # corrected cv is an estimate of CV
    lower.tile.equal <- (cv_corr * sqrt(v)) / (sqrt(tt1))
    upper.tile.equal <- (cv_corr * sqrt(v)) / (sqrt(tt2))
  } else if ("aak_adj" %in% method && correction == FALSE) {
    if (length(x) < 4) {
      stop("AA&K-ADJ CI requires at least 4 observations.")
    }
    n <- length(x)
    gh <- aak_gamma_hat_hummel(x, ddof = 1)
    r_hat <- (2 * n) / (gh + 2 * n / (n - 1))
    if (r_hat <= 0) {
      stop("AA&K-ADJ: non-positive adjusted degrees of freedom")
    }
    u_high <- stats::qchisq(1 - alpha / 2, r_hat)
    u_low <- stats::qchisq(alpha / 2, r_hat)
    est.aak_adj <- cv # cv is an estimate of CV
    lower.tile.aak_adj <- cv * sqrt(r_hat / u_high)
    upper.tile.aak_adj <- cv * sqrt(r_hat / u_low)
  } else if ("aak_adj" %in% method && correction == TRUE) {
    if (length(x) < 4) {
      stop("AA&K-ADJ CI requires at least 4 observations.")
    }
    n <- length(x)
    gh <- aak_gamma_hat_hummel(x, ddof = 1)
    r_hat <- (2 * n) / (gh + 2 * n / (n - 1))
    if (r_hat <= 0) {
      stop("AA&K-ADJ: non-positive adjusted degrees of freedom")
    }
    u_high <- stats::qchisq(1 - alpha / 2, r_hat)
    u_low <- stats::qchisq(alpha / 2, r_hat)
    est.aak_adj <- cv_corr # corrected cv is an estimate of CV
    lower.tile.aak_adj <- cv_corr * sqrt(r_hat / u_high)
    upper.tile.aak_adj <- cv_corr * sqrt(r_hat / u_low)
  } else if ("aak_ls" %in% method && correction == FALSE) {
    if (length(x) < 4) {
      stop("AA&K-LS CI requires at least 4 observations.")
    }
    n <- length(x)
    big_g2 <- aak_g2_bias_corrected(x)
    a_value <- (big_g2 + 2 * n / (n - 1)) / n
    z <- stats::qnorm(1 - alpha / 2)
    radical <- z * sqrt(a_value)
    est.aak_ls <- cv
    lower.tile.aak_ls <- cv * sqrt(exp(-radical))
    upper.tile.aak_ls <- cv * sqrt(exp(+radical))
  } else if ("aak_ls" %in% method && correction == TRUE) {
    if (length(x) < 4) {
      stop("AA&K-LS CI requires at least 4 observations.")
    }
    n <- length(x)
    big_g2 <- aak_g2_bias_corrected(x)
    a_value <- (big_g2 + 2 * n / (n - 1)) / n
    z <- stats::qnorm(1 - alpha / 2)
    radical <- z * sqrt(a_value)
    est.aak_ls <- cv_corr
    lower.tile.aak_ls <- cv_corr * sqrt(exp(-radical))
    upper.tile.aak_ls <- cv_corr * sqrt(exp(+radical))
  } else if ("aak_als" %in% method && correction == FALSE) {
    if (length(x) < 4) {
      stop("AA&K-ALS CI requires at least 4 observations.")
    }
    n <- length(x)
    kappa <- aak_kappa_e5(x)
    k_value <- kappa + 2 * n / (n - 1)
    b_value <- (k_value / n) * (1 + k_value / (2 * n))
    c_value <- k_value / (2 * n)
    z <- stats::qnorm(1 - alpha / 2)
    radical <- z * sqrt(b_value)
    est.aak_als <- cv
    lower.tile.aak_als <- cv * sqrt(exp(-radical + c_value))
    upper.tile.aak_als <- cv * sqrt(exp(+radical + c_value))
  } else if ("aak_als" %in% method && correction == TRUE) {
    if (length(x) < 4) {
      stop("AA&K-ALS CI requires at least 4 observations.")
    }
    n <- length(x)
    kappa <- aak_kappa_e5(x)
    k_value <- kappa + 2 * n / (n - 1)
    b_value <- (k_value / n) * (1 + k_value / (2 * n))
    c_value <- k_value / (2 * n)
    z <- stats::qnorm(1 - alpha / 2)
    radical <- z * sqrt(b_value)
    est.aak_als <- cv_corr
    lower.tile.aak_als <- cv_corr * sqrt(exp(-radical + c_value))
    upper.tile.aak_als <- cv_corr * sqrt(exp(+radical + c_value))
  } else if ("norm" %in% method && correction == FALSE) {
    boot.normcv.ci <- boot::boot.ci(
      boot.cv,
      conf = (1 - alpha), type = "norm"
    )
    est.norm <- cv # cv is an estimate of CV
    lower.tile.norm <- boot.normcv.ci$normal[2]
    upper.tile.norm <- boot.normcv.ci$normal[3]
  } else if ("norm" %in% method && correction == TRUE) {
    boot.normcv_corr.ci <- boot::boot.ci(
      boot.cv_corr,
      conf = (1 - alpha), type = "norm"
    )
    est.norm <- cv_corr # corrected cv is an estimate of CV
    lower.tile.norm <- boot.normcv_corr.ci$normal[2]
    upper.tile.norm <- boot.normcv_corr.ci$normal[3]
  } else if ("basic" %in% method && correction == FALSE) {
    boot.basiccv.ci <- boot::boot.ci(
      boot.cv,
      conf = (1 - alpha), type = "basic"
    )
    est.basic <- cv # cv is an estimate of CV
    lower.tile.basic <- boot.basiccv.ci$basic[4]
    upper.tile.basic <- boot.basiccv.ci$basic[5]
  } else if ("basic" %in% method && correction == TRUE) {
    boot.basiccv_corr.ci <- boot::boot.ci(
      boot.cv_corr,
      conf = (1 - alpha), type = "basic"
    )
    est.basic <- cv_corr # corrected cv is an estimate of CV
    lower.tile.basic <- boot.basiccv_corr.ci$basic[4]
    upper.tile.basic <- boot.basiccv_corr.ci$basic[5]
  } else if ("perc" %in% method && correction == FALSE) {
    boot.percentcv.ci <- boot::boot.ci(
      boot.cv,
      conf = (1 - alpha), type = "basic"
    )
    lower.tile.percent <- boot.percentcv.ci$percent[4] # nolint: object_usage_linter.
    upper.tile.percent <- boot.percentcv.ci$percent[5] # nolint: object_usage_linter.
  } else if ("perc" %in% method && correction == TRUE) {
    boot.percentcv_corr.ci <- boot::boot.ci(
      boot.cv_corr,
      conf = (1 - alpha), type = "basic"
    )
    lower.tile.percent <- boot.percentcv_corr.ci$percent[4] # nolint: object_usage_linter.
    upper.tile.percent <- boot.percentcv_corr.ci$percent[5] # nolint: object_usage_linter.
  } else if ("bca" %in% method && correction == FALSE) {
    boot.bcacv.ci <- boot::boot.ci(
      boot.cv,
      conf = (1 - alpha), type = "basic"
    )
    est.bca <- cv # nolint: object_usage_linter. retained for symmetry with siblings
    lower.tile.bca <- boot.bcacv.ci$bca[4] # nolint: object_usage_linter.
    upper.tile.bca <- boot.bcacv.ci$bca[5] # nolint: object_usage_linter.
  } else if ("bca" %in% method && correction == TRUE) {
    boot.bcacv_corr.ci <- boot::boot.ci(
      boot.cv_corr,
      conf = (1 - alpha), type = "basic"
    )
    est.bca <- cv_corr # nolint: object_usage_linter.
    lower.tile.bca <- boot.bcacv_corr.ci$percent[4] # nolint: object_usage_linter.
    upper.tile.bca <- boot.bcacv_corr.ci$percent[5] # nolint: object_usage_linter.
  } else if (method == "all" && correction == FALSE) {
    est.all <- cv # nolint: object_usage_linter.
    ci <- MBESS::conf.limits.nct(
      ncp = sqrt(length(x)) / cv_corr,
      df = length(x) - 1,
      conf.level = (1 - alpha)
    )
    lower.tile.kelley <- unname(sqrt(length(x)) / ci$Upper.Limit)
    upper.tile.kelley <- unname(sqrt(length(x)) / ci$Lower.Limit)
    # if (cv > 0.33) {
    #     warning("Confidence interval may be very approximate")
    # }
    v <- length(x) - 1
    t1 <- stats::qchisq(1 - alpha / 2, v) / v
    t2 <- stats::qchisq(alpha / 2, v) / v
    u1 <- v * t1
    u2 <- v * t2
    est.mckay <- cv # cv is an estimate of CV
    lower.tile.mckay <- cv / sqrt((u1 / (v + 1) - 1) * (cv^2) + u1 / v)
    upper.tile.mckay <- cv / sqrt((u2 / (v + 1) - 1) * (cv^2) + u2 / v)
    z_alpha_over2 <- stats::qnorm(1 - (alpha / 2))
    u <- sqrt(
      (cv^2 / v) * (0.5 + cv^2)
    )
    zu <- z_alpha_over2 * u
    est.miller <- cv # cv is an estimate of CV
    lower.tile.miller <- cv - zu
    upper.tile.miller <- cv + zu
    tt1 <- stats::qchisq(1 - alpha / 2, v) / v
    tt2 <- stats::qchisq(alpha / 2, v) / v
    uu1 <- v * tt1
    uu2 <- v * tt2
    est.vangel <- cv # cv is an estimate of CV
    lower.tile.vangel <- cv / sqrt((((uu1 + 1) / (v)) - 1) * (cv^2) + uu1 / v)
    upper.tile.vangel <- cv / sqrt((((uu2 + 1) / (v)) - 1) * (cv^2) + uu2 / v)
    if (length(x) <= 340) {
      cn <- sqrt(2 / (length(x) - 1)) * (
        (gamma(length(x) / 2)) / (gamma((length(x) - 1) / 2))
      )
    } else if (length(x) > 340) {
      cn <- sqrt(2 / (length(x) - 1)) * (
        (lgamma(length(x) / 2)) / (lgamma((length(x) - 1) / 2))
      )
    }
    ul <- 2 - (cn + (stats::qnorm((alpha / 2)) * sqrt(1 - cn^2)))
    uu <- 2 - (cn - (stats::qnorm((alpha / 2)) * sqrt(1 - cn^2)))
    est.mahmoud <- cv # cv is an estimate of CV
    lower.tile.mahmoud <- cv / ul
    upper.tile.mahmoud <- cv / uu
    cnt <- sqrt(1 - (1 / (2 * length(x))))
    ult <- cnt + (stats::qnorm(1 - (alpha / 2)) * sqrt(1 - cnt^2))
    uut <- cnt - (stats::qnorm(1 - (alpha / 2)) * sqrt(1 - cnt^2))
    est.normaapprox <- cv # cv is an estimate of CV
    lower.tile.normaapprox <- cv / ult
    upper.tile.normaapprox <- cv / uut
    if (length(x) <= 300) {
      a_value <- shortest_length[
        shortest_length$al == alpha & shortest_length$v == length(x) - 1,
        "a",
        drop = FALSE
      ]
      b_value <- shortest_length[
        shortest_length$al == alpha & shortest_length$v == length(x) - 1,
        "b",
        drop = FALSE
      ]
    } else if (length(x) > 300) {
      a_value <- shortest_length[shortest_length$al == alpha & shortest_length$v == 300, "a", drop = FALSE]
      b_value <- shortest_length[shortest_length$al == alpha & shortest_length$v == 300, "b", drop = FALSE]
    }
    est.shortest <- cv # cv is an estimate of CV
    lower.tile.shortest <- (cv * sqrt(length(x) - 1)) / sqrt(b_value$b)
    upper.tile.shortest <- (cv * sqrt(length(x) - 1)) / sqrt(a_value$a)
    ttt1 <- stats::qchisq(1 - alpha / 2, v)
    ttt2 <- stats::qchisq(alpha / 2, v)
    est.equal <- cv # cv is an estimate of CV
    lower.tile.equal <- (cv * sqrt(v)) / (sqrt(ttt1))
    upper.tile.equal <- (cv * sqrt(v)) / (sqrt(ttt2))
    # ------------------- AA&K family (population CIs) --------------------
    if (length(x) >= 4) {
      n_all <- length(x)
      gh_all <- aak_gamma_hat_hummel(x, ddof = 1)
      r_hat_all <- (2 * n_all) / (gh_all + 2 * n_all / (n_all - 1))
      if (r_hat_all <= 0) {
        stop("AA&K-ADJ: non-positive adjusted degrees of freedom")
      }
      u_high_all <- stats::qchisq(1 - alpha / 2, r_hat_all)
      u_low_all <- stats::qchisq(alpha / 2, r_hat_all)
      est.aak_adj <- cv
      lower.tile.aak_adj <- cv * sqrt(r_hat_all / u_high_all)
      upper.tile.aak_adj <- cv * sqrt(r_hat_all / u_low_all)
      big_g2_all <- aak_g2_bias_corrected(x)
      a_val_all <- (big_g2_all + 2 * n_all / (n_all - 1)) / n_all
      z_all <- stats::qnorm(1 - alpha / 2)
      rad_ls_all <- z_all * sqrt(a_val_all)
      est.aak_ls <- cv
      lower.tile.aak_ls <- cv * sqrt(exp(-rad_ls_all))
      upper.tile.aak_ls <- cv * sqrt(exp(+rad_ls_all))
      kappa_all <- aak_kappa_e5(x)
      k_val_all <- kappa_all + 2 * n_all / (n_all - 1)
      b_val_all <- (k_val_all / n_all) * (1 + k_val_all / (2 * n_all))
      c_val_all <- k_val_all / (2 * n_all)
      rad_als_all <- z_all * sqrt(b_val_all)
      est.aak_als <- cv
      lower.tile.aak_als <- cv * sqrt(exp(-rad_als_all + c_val_all))
      upper.tile.aak_als <- cv * sqrt(exp(+rad_als_all + c_val_all))
    } else {
      stop("AA&K CIs require at least 4 observations.")
    }
    boot.normcv.ci <- boot::boot.ci(
      boot.cv,
      conf = (1 - alpha), type = "norm"
    )
    est.norm <- cv # cv is an estimate of CV
    lower.tile.norm <- boot.normcv.ci$normal[2]
    upper.tile.norm <- boot.normcv.ci$normal[3]
    boot.basiccv.ci <- boot::boot.ci(
      boot.cv,
      conf = (1 - alpha), type = "basic"
    )
    est.basic <- cv # cv is an estimate of CV
    lower.tile.basic <- boot.basiccv.ci$basic[4]
    upper.tile.basic <- boot.basiccv.ci$basic[5]
    boot.percentcv.ci <- boot::boot.ci(
      boot.cv,
      conf = (1 - alpha), type = "basic"
    )
    lower.tile.percent <- boot.percentcv.ci$percent[4]
    upper.tile.percent <- boot.percentcv.ci$percent[5]
    boot.bcacv.ci <- boot::boot.ci(
      boot.cv,
      conf = (1 - alpha), type = "basic"
    )
    est.bca <- cv # cv is an estimate of CV
    lower.tile.bca <- boot.bcacv.ci$bca[4]
    upper.tile.bca <- boot.bcacv.ci$bca[5]
  } else if (method == "all" && correction == TRUE) {
    est.all <- cv_corr # cv_corr is an estimate of CV
    ci <- MBESS::conf.limits.nct(
      ncp = sqrt(length(x)) / cv_corr,
      df = length(x) - 1,
      conf.level = (1 - alpha)
    )
    lower.tile.kelley <- unname(sqrt(length(x)) / ci$Upper.Limit)
    upper.tile.kelley <- unname(sqrt(length(x)) / ci$Lower.Limit)
    # if (cv_corr > 0.33) {
    #     warning("Confidence interval may be very approximate")
    # }
    v <- length(x) - 1
    t1 <- stats::qchisq(1 - alpha / 2, v) / v
    t2 <- stats::qchisq(alpha / 2, v) / v
    u1 <- v * t1
    u2 <- v * t2
    est.mckay <- cv_corr # cv_corr is an estimate of CV
    lower.tile.mckay <- cv_corr / sqrt((u1 / (v + 1) - 1) * (cv_corr^2) + u1 / v)
    upper.tile.mckay <- cv_corr / sqrt((u2 / (v + 1) - 1) * (cv_corr^2) + u2 / v)
    z_alpha_over2 <- stats::qnorm(1 - (alpha / 2))
    u <- sqrt(
      (cv_corr^2 / v) * (0.5 + cv_corr^2)
    )
    zu <- z_alpha_over2 * u
    est.miller <- cv_corr # cv_corr is an estimate of CV
    lower.tile.miller <- cv_corr - zu
    upper.tile.miller <- cv_corr + zu
    tt1 <- stats::qchisq(1 - alpha / 2, v) / v
    tt2 <- stats::qchisq(alpha / 2, v) / v
    uu1 <- v * tt1
    uu2 <- v * tt2
    est.vangel <- cv_corr # cv_corr is an estimate of CV
    lower.tile.vangel <- cv_corr / sqrt((((uu1 + 1) / (v)) - 1) * (cv_corr^2) + uu1 / v)

    upper.tile.vangel <- cv_corr / sqrt((((uu2 + 1) / (v)) - 1) * (cv_corr^2) + uu2 / v)
    if (length(x) <= 340) {
      cn <- sqrt(2 / (length(x) - 1)) * (
        (gamma(length(x) / 2)) / (gamma((length(x) - 1) / 2))
      )
    } else {
      cn <- sqrt(2 / (length(x) - 1)) * (
        (lgamma(length(x) / 2)) / (lgamma((length(x) - 1) / 2))
      )
    }
    ul <- 2 - (cn + (stats::qnorm((alpha / 2)) * sqrt(1 - cn^2)))
    uu <- 2 - (cn - (stats::qnorm((alpha / 2)) * sqrt(1 - cn^2)))
    est.mahmoud <- cv_corr # cv_corr is an estimate of CV
    lower.tile.mahmoud <- cv_corr / ul
    upper.tile.mahmoud <- cv_corr / uu
    cnt <- sqrt(1 - (1 / (2 * length(x))))
    ult <- cnt + (stats::qnorm(1 - (alpha / 2)) * sqrt(1 - cnt^2))
    uut <- cnt - (stats::qnorm(1 - (alpha / 2)) * sqrt(1 - cnt^2))
    est.normaapprox <- cv_corr # cv_corr is an estimate of CV
    lower.tile.normaapprox <- cv_corr / ult
    upper.tile.normaapprox <- cv_corr / uut
    if (length(x) <= 300) {
      a_value <- shortest_length[
        shortest_length$al == alpha & shortest_length$v == length(x) - 1,
        "a",
        drop = FALSE
      ]
      b_value <- shortest_length[
        shortest_length$al == alpha & shortest_length$v == length(x) - 1,
        "b",
        drop = FALSE
      ]
    } else if (length(x) > 300) {
      a_value <- shortest_length[shortest_length$al == alpha & shortest_length$v == 300, "a", drop = FALSE]
      b_value <- shortest_length[shortest_length$al == alpha & shortest_length$v == 300, "b", drop = FALSE]
    }
    est.shortest <- cv_corr # cv_corr is an estimate of CV
    lower.tile.shortest <- (cv_corr * sqrt(length(x) - 1)) / sqrt(b_value$b)
    upper.tile.shortest <- (cv_corr * sqrt(length(x) - 1)) / sqrt(a_value$a)
    ttt1 <- stats::qchisq(1 - alpha / 2, v)
    ttt2 <- stats::qchisq(alpha / 2, v)
    est.equal <- cv_corr # cv_corr is an estimate of CV
    lower.tile.equal <- (cv_corr * sqrt(v)) / (sqrt(ttt1))
    upper.tile.equal <- (cv_corr * sqrt(v)) / (sqrt(ttt2))
    # ------------------- AA&K family (population CIs) --------------------
    if (length(x) >= 4) {
      n_all <- length(x)
      gh_all <- aak_gamma_hat_hummel(x, ddof = 1)
      r_hat_all <- (2 * n_all) / (gh_all + 2 * n_all / (n_all - 1))
      if (r_hat_all <= 0) {
        stop("AA&K-ADJ: non-positive adjusted degrees of freedom")
      }
      u_high_all <- stats::qchisq(1 - alpha / 2, r_hat_all)
      u_low_all <- stats::qchisq(alpha / 2, r_hat_all)
      est.aak_adj <- cv_corr
      lower.tile.aak_adj <- cv_corr * sqrt(r_hat_all / u_high_all)
      upper.tile.aak_adj <- cv_corr * sqrt(r_hat_all / u_low_all)
      big_g2_all <- aak_g2_bias_corrected(x)
      a_val_all <- (big_g2_all + 2 * n_all / (n_all - 1)) / n_all
      z_all <- stats::qnorm(1 - alpha / 2)
      rad_ls_all <- z_all * sqrt(a_val_all)
      est.aak_ls <- cv_corr
      lower.tile.aak_ls <- cv_corr * sqrt(exp(-rad_ls_all))
      upper.tile.aak_ls <- cv_corr * sqrt(exp(+rad_ls_all))
      kappa_all <- aak_kappa_e5(x)
      k_val_all <- kappa_all + 2 * n_all / (n_all - 1)
      b_val_all <- (k_val_all / n_all) * (1 + k_val_all / (2 * n_all))
      c_val_all <- k_val_all / (2 * n_all)
      rad_als_all <- z_all * sqrt(b_val_all)
      est.aak_als <- cv_corr
      lower.tile.aak_als <-
        cv_corr * sqrt(exp(-rad_als_all + c_val_all))
      upper.tile.aak_als <-
        cv_corr * sqrt(exp(+rad_als_all + c_val_all))
    } else {
      stop("AA&K CIs require at least 4 observations.")
    }
    boot.normcv_corr.ci <- boot::boot.ci(
      boot.cv_corr,
      conf = (1 - alpha), type = "norm"
    )
    est.norm <- cv_corr # cv_corr is an estimate of CV
    lower.tile.norm <- boot.normcv_corr.ci$normal[2]
    upper.tile.norm <- boot.normcv_corr.ci$normal[3]
    boot.basiccv_corr.ci <- boot::boot.ci(
      boot.cv_corr,
      conf = (1 - alpha), type = "basic"
    )
    est.basic <- cv_corr # cv_corr is an estimate of CV
    lower.tile.basic <- boot.basiccv_corr.ci$basic[4]
    upper.tile.basic <- boot.basiccv_corr.ci$basic[5]
    boot.percentcv_corr.ci <- boot::boot.ci(
      boot.cv_corr,
      conf = (1 - alpha), type = "basic"
    )
    lower.tile.percent <- boot.percentcv_corr.ci$percent[4]
    upper.tile.percent <- boot.percentcv_corr.ci$percent[5]
    boot.bcacv_corr.ci <- boot::boot.ci(
      boot.cv_corr,
      conf = (1 - alpha), type = "basic"
    )
    est.bca <- cv_corr # cv_corr is an estimate of CV
    lower.tile.bca <- boot.bcacv_corr.ci$bca[4]
    upper.tile.bca <- boot.bcacv_corr.ci$bca[5]
  }
  # preparing the output based on the selected methods
  if ("kelley" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Kelley 95% CI"
        ),
        statistics = data.frame(
          est = round(est.kelley * 100, digits = digits),
          lower = round(lower.tile.kelley * 100, digits = digits),
          upper = round(upper.tile.kelley * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("kelley" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Kelley 95% CI"
        ),
        statistics = data.frame(
          est = round(est.kelley * 100, digits = digits),
          lower = round(lower.tile.kelley * 100, digits = digits),
          upper = round(upper.tile.kelley * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("mckay" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with McKay 95% CI"
        ),
        statistics = data.frame(
          est = round(est.mckay * 100, digits = digits),
          lower = round(lower.tile.mckay * 100, digits = digits),
          upper = round(upper.tile.mckay * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("mckay" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with McKay 95% CI"
        ),
        statistics = data.frame(
          est = round(est.mckay * 100, digits = digits),
          lower = round(lower.tile.mckay * 100, digits = digits),
          upper = round(upper.tile.mckay * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("miller" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Miller 95% CI"
        ),
        statistics = data.frame(
          est = round(est.miller * 100, digits = digits),
          lower = round(lower.tile.miller * 100, digits = digits),
          upper = round(upper.tile.miller * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("miller" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Miller 95% CI"
        ),
        statistics = data.frame(
          est = round(est.miller * 100, digits = digits),
          lower = round(lower.tile.miller * 100, digits = digits),
          upper = round(upper.tile.miller * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("vangel" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Vangel 95% CI"
        ),
        statistics = data.frame(
          est = round(est.vangel * 100, digits = digits),
          lower = round(lower.tile.vangel * 100, digits = digits),
          upper = round(upper.tile.vangel * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("vangel" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Vangel 95% CI"
        ),
        statistics = data.frame(
          est = round(est.vangel * 100, digits = digits),
          lower = round(lower.tile.vangel * 100, digits = digits),
          upper = round(upper.tile.vangel * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("mahmoudvand_hassani" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Mahmoudvand-Hassani 95% CI"
        ),
        statistics = data.frame(
          est = round(est.mahmoud * 100, digits = digits),
          lower = round(lower.tile.mahmoud * 100, digits = digits),
          upper = round(upper.tile.mahmoud * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("mahmoudvand_hassani" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Mahmoudvand-Hassani 95% CI"
        ),
        statistics = data.frame(
          est = round(est.mahmoud * 100, digits = digits),
          lower = round(lower.tile.mahmoud * 100, digits = digits),
          upper = round(upper.tile.mahmoud * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("normal_approximation" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Normal Approximation 95% CI"
        ),
        statistics = data.frame(
          est = round(est.normaapprox * 100, digits = digits),
          lower = round(
            lower.tile.normaapprox * 100,
            digits = digits
          ),
          upper = round(
            upper.tile.normaapprox * 100,
            digits = digits
          ),
          row.names = c(" ")
        )
      )
    )
  } else if ("normal_approximation" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Normal Approximation 95% CI"
        ),
        statistics = data.frame(
          est = round(est.normaapprox * 100, digits = digits),
          lower = round(
            lower.tile.normaapprox * 100,
            digits = digits
          ),
          upper = round(
            upper.tile.normaapprox * 100,
            digits = digits
          ),
          row.names = c(" ")
        )
      )
    )
  } else if ("shortest_length" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Shortest-Length 95% CI"
        ),
        statistics = data.frame(
          est = round(est.shortest * 100, digits = digits),
          lower = round(lower.tile.shortest * 100, digits = digits),
          upper = round(upper.tile.shortest * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("shortest_length" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Shortest-Length 95% CI"
        ),
        statistics = data.frame(
          est = round(est.shortest * 100, digits = digits),
          lower = round(lower.tile.shortest * 100, digits = digits),
          upper = round(upper.tile.shortest * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("equal_tailed" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Equal-Tailed 95% CI"
        ),
        statistics = data.frame(
          est = round(est.equal * 100, digits = digits),
          lower = round(lower.tile.equal * 100, digits = digits),
          upper = round(upper.tile.equal * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("equal_tailed" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Equal-Tailed 95% CI"
        ),
        statistics = data.frame(
          est = round(est.equal * 100, digits = digits),
          lower = round(lower.tile.equal * 100, digits = digits),
          upper = round(upper.tile.equal * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("aak_adj" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Abu-Shawiesh-Akyuz-Kibria ADJ 95% CI"
        ),
        statistics = data.frame(
          est = round(est.aak_adj * 100, digits = digits),
          lower = round(lower.tile.aak_adj * 100, digits = digits),
          upper = round(upper.tile.aak_adj * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("aak_adj" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Abu-Shawiesh-Akyuz-Kibria ADJ 95% CI"
        ),
        statistics = data.frame(
          est = round(est.aak_adj * 100, digits = digits),
          lower = round(lower.tile.aak_adj * 100, digits = digits),
          upper = round(upper.tile.aak_adj * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("aak_ls" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Abu-Shawiesh-Akyuz-Kibria LS 95% CI"
        ),
        statistics = data.frame(
          est = round(est.aak_ls * 100, digits = digits),
          lower = round(lower.tile.aak_ls * 100, digits = digits),
          upper = round(upper.tile.aak_ls * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("aak_ls" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Abu-Shawiesh-Akyuz-Kibria LS 95% CI"
        ),
        statistics = data.frame(
          est = round(est.aak_ls * 100, digits = digits),
          lower = round(lower.tile.aak_ls * 100, digits = digits),
          upper = round(upper.tile.aak_ls * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("aak_als" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Abu-Shawiesh-Akyuz-Kibria ALS 95% CI"
        ),
        statistics = data.frame(
          est = round(est.aak_als * 100, digits = digits),
          lower = round(lower.tile.aak_als * 100, digits = digits),
          upper = round(upper.tile.aak_als * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("aak_als" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Abu-Shawiesh-Akyuz-Kibria ALS 95% CI"
        ),
        statistics = data.frame(
          est = round(est.aak_als * 100, digits = digits),
          lower = round(lower.tile.aak_als * 100, digits = digits),
          upper = round(upper.tile.aak_als * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("norm" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Normal Approximation Bootstrap 95% CI"
        ),
        statistics = data.frame(
          est = round(est.norm * 100, digits = digits),
          lower = round(lower.tile.norm * 100, digits = digits),
          upper = round(upper.tile.norm * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("norm" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Normal Approximation Bootstrap 95% CI"
        ),
        statistics = data.frame(
          est = round(est.norm * 100, digits = digits),
          lower = round(lower.tile.norm * 100, digits = digits),
          upper = round(upper.tile.norm * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("basic" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Basic Bootstrap 95% CI"
        ),
        statistics = data.frame(
          est = round(est.basic * 100, digits = digits),
          lower = round(lower.tile.basic * 100, digits = digits),
          upper = round(upper.tile.basic * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("basic" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Basic Bootstrap 95% CI"
        ),
        statistics = data.frame(
          est = round(est.basic * 100, digits = digits),
          lower = round(lower.tile.basic * 100, digits = digits),
          upper = round(upper.tile.basic * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("perc" %in% method && correction == FALSE) {
    stop("percent method is not developed yet")
    # return(
    #     list(
    #         method = (
    #             "cv with Bootstrap Percentile 95% CI"
    #         ),
    #         statistics = data.frame(
    #             est = round(est.percent * 100, digits = digits),
    #             lower = round(lower.tile.percent * 100, digits = digits),
    #             upper = round(upper.tile.percent * 100, digits = digits),
    #             row.names = c(" ")
    #         )
    #     )
    #     )
  } else if ("perc" %in% method && correction == TRUE) {
    stop("percent method is not developed yet")
    # return(
    #     list(
    #         method = (
    #             "Corrected cv with Bootstrap Percentile 95% CI"
    #         ),
    #         statistics = data.frame(
    #             est = round(est.percent * 100, digits = digits),
    #             lower = round(lower.tile.percent * 100, digits = digits),
    #             upper = round(upper.tile.percent * 100, digits = digits),
    #             row.names = c(" ")
    #         )
    #     )
    # )
  } else if ("bca" %in% method && correction == FALSE) {
    stop("BCa method is not developed yet")
    # return(
    #     list(
    #         method = (
    #             "cv with Adjusted Bootstrap Percentile (BCa) 95% CI"
    #         ),
    #         statistics = data.frame(
    #             est = round(est.bca * 100, digits = digits),
    #             lower = round(lower.tile.bca * 100, digits = digits),
    #             upper = round(upper.tile.bca * 100, digits = digits),
    #             row.names = c(" ")
    #         )
    #     )
    #     )
  } else if ("bca" %in% method && correction == TRUE) {
    stop("BCa method is not developed yet")
    # return(
    #     list(
    #         method = (
    #         "Corrected cv with Adjusted Bootstrap Percentile (BCa) 95% CI"
    #         ),
    #         statistics = data.frame(
    #             est = round(est.bca * 100, digits = digits),
    #             lower = round(lower.tile.bca * 100, digits = digits),
    #             upper = round(upper.tile.bca * 100, digits = digits),
    #             row.names = c(" ")
    #         )
    #     )
    # )
  } else if (method == "all" && correction == FALSE) {
    return(
      list(
        method = "All methods",
        statistics = data.frame(
          row.names = c(
            "kelley", # 1
            "mckay", # 2
            "miller", # 3
            "vangel", # 4
            "mahmoudvand_hassani", # 5
            "equal_tailed", # 6
            "shortest_length", # 7
            "normal_approximation", # 8
            "norm", # 9
            "basic", # 10
            # "perc"  # 11
            # "bca"  # 12
            "aak_adj", # 13
            "aak_ls", # 14
            "aak_als" # 15
          ),
          est = c(
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            # round(cv * 100, digits = digits),
            # round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits)
          ),
          lower = c(
            round(lower.tile.kelley * 100, digits = digits),
            round(lower.tile.mckay * 100, digits = digits),
            round(lower.tile.miller * 100, digits = digits),
            round(lower.tile.vangel * 100, digits = digits),
            round(lower.tile.mahmoud * 100, digits = digits),
            round(lower.tile.equal * 100, digits = digits),
            round(lower.tile.shortest * 100, digits = digits),
            round(lower.tile.normaapprox * 100, digits = digits),
            round(lower.tile.norm * 100, digits = digits),
            round(lower.tile.basic * 100, digits = digits),
            # ,
            # round(lower.tile.percent * 100, digits = digits)
            # ,
            # round(lower.tile.bca * 100, digits = digits)
            round(lower.tile.aak_adj * 100, digits = digits),
            round(lower.tile.aak_ls * 100, digits = digits),
            round(lower.tile.aak_als * 100, digits = digits)
          ),
          upper = c(
            round(upper.tile.kelley * 100, digits = digits),
            round(upper.tile.mckay * 100, digits = digits),
            round(upper.tile.miller * 100, digits = digits),
            round(upper.tile.vangel * 100, digits = digits),
            round(upper.tile.mahmoud * 100, digits = digits),
            round(upper.tile.equal * 100, digits = digits),
            round(upper.tile.shortest * 100, digits = digits),
            round(upper.tile.normaapprox * 100, digits = digits),
            round(upper.tile.norm * 100, digits = digits),
            round(upper.tile.basic * 100, digits = digits),
            # ,
            # round(upper.tile.percent * 100, digits = digits)
            # ,
            # round(upper.tile.bca * 100, digits = digits)
            round(upper.tile.aak_adj * 100, digits = digits),
            round(upper.tile.aak_ls * 100, digits = digits),
            round(upper.tile.aak_als * 100, digits = digits)
          ),
          description = c(
            "cv with Kelley 95% CI",
            "cv with McKay 95% CI",
            "cv with Miller 95% CI",
            "cv with Vangel 95% CI",
            "cv with Mahmoudvand-Hassani 95% CI",
            "cv with Equal-Tailed 95% CI",
            "cv with Shortest-Length 95% CI",
            "cv with Normal Approximation 95% CI",
            "cv with Normal Approximation Bootstrap 95% CI",
            "cv with Basic Bootstrap 95% CI",
            # ,
            # "cv with Bootstrap Percentile 95% CI"
            # ,
            # "cv with Adjusted Bootstrap Percentile (BCa) 95% CI"
            "cv with Abu-Shawiesh-Akyuz-Kibria ADJ 95% CI",
            "cv with Abu-Shawiesh-Akyuz-Kibria LS 95% CI",
            "cv with Abu-Shawiesh-Akyuz-Kibria ALS 95% CI"
          )
        )
      )
    )
  } else if (method == "all" && correction == TRUE) {
    return(
      list(
        method = "All methods",
        statistics = data.frame(
          row.names = c(
            "kelley", # 1
            "mckay", # 2
            "miller", # 3
            "vangel", # 4
            "mahmoudvand_hassani", # 5
            "equal_tailed", # 6
            "shortest_length", # 7
            "normal_approximation", # 8
            "norm", # 9
            "basic", # 10
            # "perc"  # 11
            # "bca"  # 12
            "aak_adj", # 13
            "aak_ls", # 14
            "aak_als" # 15
          ),
          est = c(
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            # round(cv_corr * 100, digits = digits),
            # round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits)
          ),
          lower = c(
            round(lower.tile.kelley * 100, digits = digits),
            round(lower.tile.mckay * 100, digits = digits),
            round(lower.tile.miller * 100, digits = digits),
            round(lower.tile.vangel * 100, digits = digits),
            round(lower.tile.mahmoud * 100, digits = digits),
            round(lower.tile.equal * 100, digits = digits),
            round(lower.tile.shortest * 100, digits = digits),
            round(lower.tile.normaapprox * 100, digits = digits),
            round(lower.tile.norm * 100, digits = digits),
            round(lower.tile.basic * 100, digits = digits),
            # ,
            # round(lower.tile.percent * 100, digits = digits)
            # ,
            # round(lower.tile.bca * 100, digits = digits)
            round(lower.tile.aak_adj * 100, digits = digits),
            round(lower.tile.aak_ls * 100, digits = digits),
            round(lower.tile.aak_als * 100, digits = digits)
          ),
          upper = c(
            round(upper.tile.kelley * 100, digits = digits),
            round(upper.tile.mckay * 100, digits = digits),
            round(upper.tile.miller * 100, digits = digits),
            round(upper.tile.vangel * 100, digits = digits),
            round(upper.tile.mahmoud * 100, digits = digits),
            round(upper.tile.equal * 100, digits = digits),
            round(upper.tile.shortest * 100, digits = digits),
            round(upper.tile.normaapprox * 100, digits = digits),
            round(upper.tile.norm * 100, digits = digits),
            round(upper.tile.basic * 100, digits = digits),
            # ,
            # round(upper.tile.percent * 100, digits = digits)
            # ,
            # round(upper.tile.bca * 100, digits = digits)
            round(upper.tile.aak_adj * 100, digits = digits),
            round(upper.tile.aak_ls * 100, digits = digits),
            round(upper.tile.aak_als * 100, digits = digits)
          ),
          description = c(
            "Corrected cv with Kelley 95% CI",
            "Corrected cv with McKay 95% CI",
            "Corrected cv with Miller 95% CI",
            "Corrected cv with Vangel 95% CI",
            "Corrected cv with Mahmoudvand-Hassani 95% CI",
            "Corrected cv with Equal-Tailed 95% CI",
            "Corrected cv with Shortest-Length 95% CI",
            "Corrected cv with Normal Approximation 95% CI",
            "Corrected cv with Normal Approximation Bootstrap 95% CI",
            "Corrected cv with Basic Bootstrap 95% CI",
            # ,
            # "Corrected cv with Bootstrap Percentile 95% CI"
            # ,
            # "Corrected cv with Adjusted Bootstrap Percentile (BCa) 95% CI"
            "Corrected cv with Abu-Shawiesh-Akyuz-Kibria ADJ 95% CI",
            "Corrected cv with Abu-Shawiesh-Akyuz-Kibria LS 95% CI",
            "Corrected cv with Abu-Shawiesh-Akyuz-Kibria ALS 95% CI"
          )
        )
      )
    )
  }
}

# --------------------------------------------------------------------------- #
#  Kurtosis helpers used by the AA&K (Abu-Shawiesh, Akyuz & Kibria) family
#  of confidence intervals for the coefficient of variation.
#
#  References:
#    Abu-Shawiesh, M. O. A., Akyuz, H. E., & Kibria, B. G. (2019).
#    Performance of Some Confidence Intervals for Estimating the Population
#    Coefficient of Variation under both Symmetric and Skewed Distributions.
#    Statistics, Optimization & Information Computing, 7(2), 277-290.
#    DOI: 10.19139/soic.v7i2.630
#
#  All helpers are package-internal (not exported) and assume that NA values
#  have already been stripped by the caller. They carry `@noRd` so roxygen
#  does NOT generate man pages for them, while still being visible to
#  lintr's static-analysis pass as legitimate package-scope functions.
# --------------------------------------------------------------------------- #

#' Central moment of order k
#'
#' Computes \eqn{m_k = (1/n) \sum_i (x_i - \bar{x})^k}. Matches \eqn{m_2}
#' and \eqn{m_4} in Abu-Shawiesh, Akyuz & Kibria (2019), footer of Eq. 10.
#' No Bessel correction.
#'
#' @keywords internal
#' @noRd
aak_central_moment <- function(x, order) {
  mean((x - mean(x))^order)
}

#' Sample excess kurtosis
#'
#' \eqn{g_2 = m_4 / m_2^2 - 3}; matches the expression for \eqn{g_2} under
#' Abu-Shawiesh, Akyuz & Kibria (2019), footer of Eq. 10.
#'
#' @keywords internal
#' @noRd
aak_g2_excess_kurtosis <- function(x) {
  m_2 <- aak_central_moment(x, 2)
  m_4 <- aak_central_moment(x, 4)
  m_4 / (m_2^2) - 3
}

#' Bias-adjusted kurtosis estimator G_2
#'
#' Abu-Shawiesh, Akyuz & Kibria (2019), Eq. 10:
#' \eqn{G_2 = (n - 1) / ((n - 2)(n - 3)) [(n - 1) g_2 + 6]}.
#'
#' @keywords internal
#' @noRd
aak_g2_bias_corrected <- function(x) {
  n <- length(x)
  if (n < 4) {
    stop("G_2 requires at least 4 observations.")
  }
  g_2 <- aak_g2_excess_kurtosis(x)
  (n - 1) / ((n - 2) * (n - 3)) * ((n - 1) * g_2 + 6)
}

#' Modified kurtosis estimator kappa_e5
#'
#' Abu-Shawiesh, Akyuz & Kibria (2019), Eq. 13:
#' \eqn{kappa_{e5} = ((n + 1) / (n - 1)) G_2 (1 + 5 G_2 / n)}.
#'
#' @keywords internal
#' @noRd
aak_kappa_e5 <- function(x) {
  n <- length(x)
  big_g2 <- aak_g2_bias_corrected(x)
  ((n + 1) / (n - 1)) * big_g2 * (1 + 5 * big_g2 / n)
}

#' Hummel-style adjusted-kurtosis estimator
#'
#' Abu-Shawiesh, Akyuz & Kibria (2019), Eq. 5:
#' \eqn{gamma_hat = [n(n + 1) / ((n - 1)(n - 2)(n - 3))]
#'   \sum (x - \bar{x})^4 / S^4 - 3(n - 1)^2 / ((n - 2)(n - 3))},
#' where \eqn{S} is the Bessel-corrected sample standard deviation
#' (i.e. \code{stats::var(x)} with \code{ddof = 1}).
#'
#' @keywords internal
#' @noRd
aak_gamma_hat_hummel <- function(x, ddof = 1) {
  n <- length(x)
  if (n < 4) {
    stop("gamma_hat requires at least 4 observations.")
  }
  mu <- mean(x)
  sum_fourth <- sum((x - mu)^4)
  sample_var <- stats::var(x) # R's var uses ddof = 1 by default
  if (ddof != 1) {
    sample_var <- sum((x - mu)^2) / (n - ddof)
  }
  first_term <- (n * (n + 1) / ((n - 1) * (n - 2) * (n - 3))) *
    (sum_fourth / (sample_var^2))
  second_term <- 3 * (n - 1)^2 / ((n - 2) * (n - 3))
  first_term - second_term
}

# Declare bare-name columns that flow through dplyr::select() inside the
# shortest-length lookup table (R/sysdata.rda). Without this, R CMD check
# and lintr's object_usage_linter flag `al`, `a`, and `b` as undefined
# globals — they are columns of the internal `shortest_length` data
# frame, accessed via dplyr's non-standard evaluation.
utils::globalVariables(c("al", "a", "b"))

#' @title R6 Confidence Intervals for the Coefficient of Variation (cv)
#' @name CoefVarCI
#' @description The R6 class \code{CoefVarCI} for the confidence intervals of
#'   coefficient of variation (cv)
#' @param x An \code{R} object. Currently there are methods for numeric vectors
#' @param na.rm a logical value indicating whether \code{NA} values should be
#'   stripped before the computation proceeds.
#' @param digits integer indicating the number of decimal places to be used.
#' @param method a scalar representing the type of confidence intervals
#'   required. The value should be any of the values "kelley_ci", "mckay_ci",
#'   "miller_ci", "vangel_ci", "mahmoudvand_hassani_ci", "equal_tailed_ci",
#'   "shortest_length_ci", "normal_approximation_ci", "norm_ci","basic_ci",
#'   "aak_adj_ci", "aak_ls_ci", "aak_als_ci", or "all_ci".
#' @param alpha The allowed type I error probability
#' @param R integer indicating the number of bootstrap replicates.
#' @param correction returns the unbiased estimate of the coefficient of
#'   variation if TRUE is determined.
#' @details \describe{ \item{\strong{Coefficient of Variation}}{ The \emph{cv}
#'   is a measure of relative dispersion representing the degree of variability
#'   relative to the mean \code{[1]}. Since \eqn{cv} is unitless, it is useful
#'   for comparison of variables with different units. It is also a measure of
#'   homogeneity \code{[1]}. } }
#' @return An object of type "list" which contains the estimate, the intervals,
#'   and the computation method. It has two main components:
#' @return \describe{ \item{$method}{ A description of statistical method used
#'   for the computations. } \item{$statistics}{ A data frame representing three
#'   vectors: est/, lower and upper limits of confidence interval \code{(CI)};
#'   additional description vector is provided when "all" is selected: \cr \cr
#'   \strong{est:} \code{cv*100}  \cr \cr \strong{Kelley Confidence
#'   Interval:} Thanks to package \pkg{MBESS} \code{[2]} for the
#'   computation of confidence limits for the noncentrality parameter from a
#'   \emph{t} distribution \link[MBESS]{conf.limits.nct} \code{[3]}.  \cr \cr
#'   \strong{McKay Confidence Interval:} The intervals calculated by the method
#'   introduced by McKay \code{[4]}, using chi-square distribution.  \cr \cr
#'   \strong{Miller Confidence Interval:} The intervals calculated by the
#'   method introduced by Miller \code{[5]}, using the standard normal
#'   distribution.  \cr \cr \strong{Vangel Confidence Interval:} Vangel
#'   \code{[6]} proposed a method for the calculation of CI for \emph{cv}; which
#'   is a modification on McKay’s CI.  \cr \cr \strong{Mahmoudvand-Hassani
#'   Confidence Interval:} Mahmoudvand and Hassani \code{[7]} proposed a new CI
#'   for \emph{cv}; which is obtained using ranked set sampling \emph{(RSS)}
#'   \cr \cr \strong{Normal Approximation Confidence Interval:} Wararit
#'   Panichkitkosolkul \code{[8]} proposed another CI for \emph{cv}; which is a
#'   normal approximation.  \cr \cr \strong{Shortest-Length Confidence
#'   Interval:} Wararit Panichkitkosolkul \code{[8]} proposed another CI for
#'   \emph{cv}; which is obtained through minimizing the length of CI.  \cr \cr
#'   \strong{Equal-Tailed Confidence Interval:} Wararit Panichkitkosolkul
#'   \code{[8]} proposed another CI for \emph{cv}; which is obtained using
#'   chi-square distribution.  \cr \cr \strong{Bootstrap Confidence
#'   Intervals:} Thanks to package \pkg{boot} by Canty & Ripley \code{[9]} we
#'   can obtain bootstrap CI around \emph{cv} using \link[boot]{boot.ci}.  \cr
#'   \cr \strong{Abu-Shawiesh-Akyuz-Kibria Confidence Intervals:}
#'   Abu-Shawiesh, Akyuz, & Kibria \code{[10]} proposed three CIs for the
#'   population \emph{cv} that adjust for non-normality through the sample
#'   kurtosis: the adjusted-degrees-of-freedom CI (\code{aak_adj_ci}) derived
#'   from Hummel et al.'s CI for the variance, the large-sample CI
#'   (\code{aak_ls_ci}) derived from the log-transformed CI for the variance,
#'   and the augmented-large-sample CI (\code{aak_als_ci}) derived from
#'   Burch's CI for the variance.  \cr \cr } }
#' @examples
#' y <- c(
#'     0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
#'     4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
#' )
#' CoefVarCI$new(x = y)$kelley_ci()
#' cv_y <- CoefVarCI$new(
#'    x = y,
#'    alpha = 0.05,
#'    R = 1000,
#'    digits = 2,
#'    correction = TRUE
#' )
#' cv_y$kelley_ci()
#' cv_y$mckay_ci()
#' R6::is.R6(cv_y)
#' @references \code{[1]} Albatineh, AN., Kibria, BM., Wilcox, ML., & Zogheib,
#'   B, 2014, Confidence interval estimation for the population coefficient of
#'   variation using ranked set sampling: A simulation study, Journal of Applied
#'   Statistics, 41(4), 733–751, DOI:
#'   \doi{10.1080/02664763.2013.847405}
#'
#' @references \code{[2]} Kelley, K., 2018, MBESS: The MBESS R Package. R
#'   package version 4.4. 3.
#'
#' @references \code{[3]} Kelley, K., 2007, Sample size planning for the
#'   coefficient of variation from the accuracy in parameter estimation
#'   approach, Behavior Research Methods, 39(4), 755–766, DOI:
#'   \doi{10.3758/BF03192966}
#' @references \code{[4]} McKay, AT., 1932, Distribution of the Coefficient of
#'   Variation and the Extended“ t” Distribution, Journal of the Royal
#'   Statistical Society, 95(4), 695–698
#' @references \code{[5]} Miller, E., 1991, Asymptotic test statistics for
#'   coefficients of variation, Communications in Statistics-Theory and Methods,
#'   20(10), 3351–3363
#' @references \code{[6]} Vangel, MG., 1996, Confidence intervals for a normal
#'   coefficient of variation, The American Statistician, 50(1), 21–26
#' @references \code{[7]} Mahmoudvand, R., & Hassani, H., 2009, Two new
#'   confidence intervals for the coefficient of variation in a normal
#'   distribution, Journal of Applied Statistics, 36(4), 429–442
#' @references \code{[8]} Panichkitkosolkul, W., 2013, Confidence Intervals for
#'   the Coefficient of Variation in a Normal Distribution with a Known
#'   Population Mean, Journal of Probability and Statistics, 2013, 1–11,
#'   \doi{10.1155/2013/324940}
#'
#' @references \code{[9]} Canty, A., & Ripley, B., 2017, boot: Bootstrap R
#'   (S-Plus) Functions, R package version 1.3-20
#'
#' @references \code{[10]} Abu-Shawiesh, MOA., Akyuz, HE., & Kibria, BG.,
#'   2019, Performance of Some Confidence Intervals for Estimating the
#'   Population Coefficient of Variation under both Symmetric and Skewed
#'   Distributions, Statistics, Optimization & Information Computing, 7(2),
#'   277-290, DOI: \doi{10.19139/soic.v7i2.630}
#' @export
#' @import dplyr SciViews boot R6 utils
NULL
#' @importFrom stats quantile sd qchisq qnorm
#' @importFrom MBESS conf.limits.nct
NULL
CoefVarCI <- R6::R6Class(
  classname = "CoefVarCI",
  inherit = CoefVar,
  public = list(
    # ---------------- determining defaults for arguments -----------------
    x = NA,
    na.rm = FALSE,
    digits = 1,
    R = 1000,
    alpha = 0.05,
    correction = FALSE,
    # --------------------- adding some internal fields -------------------
    ncp_cv = NA,
    ncp_cv_corr = NA,
    v = NA,
    est = NA,
    kelley_nctci_cv = NA,
    kelley_nctci_cv_corr = NA,
    t1_mckay = NA,
    t2_mckay = NA,
    u1_mckay = NA,
    u2_mckay = NA,
    lower_tile_kelley_cv = NA,
    upper_tile_kelley_cv = NA,
    lower_tile_kelley_cv_corr = NA,
    upper_tile_kelley_cv_corr = NA,
    lower_tile_mckay_cv = NA,
    upper_tile_mckay_cv = NA,
    lower_tile_mckay_cv_corr = NA,
    upper_tile_mckay_cv_corr = NA,
    z_miller = NA,
    u_miller_cv = NA,
    u_miller_cv_corr = NA,
    zu_miller_cv = NA,
    zu_miller_cv_corr = NA,
    lower_tile_miller_cv = NA,
    upper_tile_miller_cv = NA,
    lower_tile_miller_cv_corr = NA,
    upper_tile_miller_cv_corr = NA,
    t1_vangel = NA,
    t2_vangel = NA,
    u1_vangel = NA,
    u2_vangel = NA,
    lower_tile_vangel_cv = NA,
    upper_tile_vangel_cv = NA,
    lower_tile_vangel_cv_corr = NA,
    upper_tile_vangel_cv_corr = NA,
    cn_mh = NA,
    ul_mh = NA,
    uu_mh = NA,
    lower_tile_mh_cv = NA,
    upper_tile_mh_cv = NA,
    lower_tile_mh_cv_corr = NA,
    upper_tile_mh_cv_corr = NA,
    cn_normaapprox = NA,
    ul_normaapprox = NA,
    uu_normaapprox = NA,
    lower_tile_normaapprox_cv = NA,
    upper_tile_normaapprox_cv = NA,
    lower_tile_normaapprox_cv_corr = NA,
    upper_tile_normaapprox_cv_corr = NA,
    a_value = NA,
    b_value = NA,
    lower_tile_shortest_cv = NA,
    upper_tile_shortest_cv = NA,
    lower_tile_shortest_cv_corr = NA,
    upper_tile_shortest_cv_corr = NA,
    t1_equal = NA,
    t2_equal = NA,
    lower_tile_equal_cv = NA,
    upper_tile_equal_cv = NA,
    lower_tile_equal_cv_corr = NA,
    upper_tile_equal_cv_corr = NA,
    # ---- AA&K (Abu-Shawiesh, Akyuz & Kibria, 2019) population CIs -------
    gamma_hat_aak = NA,
    r_hat_aak = NA,
    aak_adj_qchi_low = NA,
    aak_adj_qchi_high = NA,
    lower_tile_aak_adj_cv = NA,
    upper_tile_aak_adj_cv = NA,
    lower_tile_aak_adj_cv_corr = NA,
    upper_tile_aak_adj_cv_corr = NA,
    g2_aak = NA,
    a_value_aak_ls = NA,
    lower_tile_aak_ls_cv = NA,
    upper_tile_aak_ls_cv = NA,
    lower_tile_aak_ls_cv_corr = NA,
    upper_tile_aak_ls_cv_corr = NA,
    kappa_e5_aak = NA,
    k_value_aak_als = NA,
    b_value_aak_als = NA,
    c_value_aak_als = NA,
    lower_tile_aak_als_cv = NA,
    upper_tile_aak_als_cv = NA,
    lower_tile_aak_als_cv_corr = NA,
    upper_tile_aak_als_cv_corr = NA,
    shortest_length = data.frame(
      # ----------- "a" and "b" values for shortest-length CI -----------
      v = c( # degrees of freedom
        2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
        20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 40, 50, 60, 70, 80,
        90, 100, 150, 200, 250, 300, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12,
        13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28,
        29, 30, 40, 50, 60, 70, 80, 90, 100, 150, 200, 250, 300, 2, 3,
        4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
        21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 40, 50, 60, 70, 80, 90,
        100, 150, 200, 250, 300
      ),
      al = c( # al is the allowed type I error probability
        0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,
        0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,
        0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,
        0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,
        0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
        0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
        0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
        0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
        0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
        0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01,
        0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01,
        0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01,
        0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01,
        0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01
      ),
      a = c( # a is an attribute for the length of CI
        0.2065, 0.5654, 1.02, 1.5352, 2.093, 2.6828, 3.2981, 3.9343,
        4.5883, 5.2573, 5.9397, 6.6337, 7.3382, 8.0521, 8.7745,
        9.5047, 10.2421, 10.9861, 11.7362, 12.4919, 13.253,
        14.0191, 14.7899, 15.565, 16.3443, 17.1275, 17.9144,
        18.7049, 19.4987, 27.5919, 35.9012, 44.3661, 52.9501,
        61.629, 70.386, 79.2086, 124.0372, 169.6646, 215.8057,
        262.3132, 0.1015, 0.3449, 0.6918, 1.1092, 1.5776,
        2.0851, 2.6235, 3.1874, 3.7729, 4.3768, 4.9967,
        5.6308, 6.2776, 6.9357, 7.6042, 8.282, 8.9685,
        9.6629, 10.3647, 11.0733, 11.7882, 12.5092,
        13.2357, 13.9675, 14.7043, 15.4458, 16.1917,
        16.9419, 17.6961, 25.4233, 33.4085, 41.5794, 49.8923,
        58.3183, 66.8374, 75.4347, 119.2737, 164.0642,
        209.4667, 255.3057, 0.02, 0.114, 0.2937, 0.5461,
        0.8567, 1.2143, 1.6107, 2.0394, 2.4958, 2.976, 3.4771,
        3.9968, 4.5329, 5.084, 5.6487, 6.2256, 6.8139, 7.4126,
        8.0209, 8.6383, 9.264, 9.8976, 10.5385, 11.1864, 11.8408,
        12.5014, 13.1678, 13.8397, 14.517, 21.5331, 28.8879,
        36.4863, 44.2711, 52.2044, 60.2597, 68.4177, 110.3262,
        153.4834, 197.444, 241.9776
      ),
      b = c( # b is an attribute for the length of CI
        12.5208, 13.1532, 14.18, 15.3498, 16.5807, 17.8391,
        19.1099, 20.3848, 21.6598, 22.9325, 24.2016,
        25.4666, 26.7269, 27.9825, 29.2334, 30.4796,
        31.7212, 32.9585, 34.1915, 35.4205, 36.6455,
        37.8668, 39.0844, 40.2986, 41.5095, 42.7171,
        43.9217, 45.1234, 46.3222, 58.1755, 69.8342,
        81.3479, 92.7487, 104.0584, 115.2925, 126.4628,
        181.6128, 235.9748, 289.8273, 343.3155, 15.1194,
        15.5897, 16.5735, 17.7432, 18.9954, 20.2863,
        21.5953, 22.9118, 24.2303, 25.5476, 26.8618,
        28.1717, 29.4769, 30.777, 32.072, 33.3619,
        34.6467, 35.9266, 37.2016, 38.472, 39.7379,
        40.9995, 42.257, 43.5105, 44.7601, 46.006,
        47.2483, 48.4872, 49.7229, 61.9217, 73.892,
        85.6914, 97.3573, 108.9153, 120.3839, 131.7767,
        187.9079, 243.1025, 297.691, 351.8461, 20.8264,
        20.9856, 21.8371, 22.9867, 24.2618, 25.6017,
        26.9749, 28.3643, 29.7602, 31.158, 32.5543,
        33.9474, 35.3358, 36.7192, 38.0968, 39.4688,
        40.8347, 42.1952, 43.5498, 44.8989, 46.2426,
        47.581, 48.9144, 50.2428, 51.5665, 52.8856,
        54.2002, 55.5107, 56.8169, 69.6808, 82.2534,
        94.6063, 106.7867, 118.8272, 130.7514, 142.5771,
        200.6194, 257.4375, 313.462, 368.9185
      )
    ),
    bootcv = NA,
    # --------- determining constructor defaults for arguments ------------
    initialize = function(x = NA,
                          na.rm = FALSE,
                          digits = 1,
                          R = 1000,
                          alpha = 0.05,
                          correction = FALSE,
                          ...) {
      # ---------------------- check NA or NAN -------------------------
      if (missing(x) || is.null(x)) {
        stop("object 'x' not found")
      } else if (!missing(x)) {
        self$x <- x
      }
      if (!missing(na.rm)) {
        self$na.rm <- na.rm
      }
      if (self$na.rm == TRUE) {
        self$x <- x[!is.na(x)]
      } else if (anyNA(x) & self$na.rm == FALSE) {
        stop(
          "missing values and NaN's not allowed if 'na.rm' is FALSE"
        )
      }
      # ------------- stop if input x vector is not numeric -------------
      if (!is.numeric(x)) {
        stop("argument is not a numeric vector")
      }
      # ------------------- set digits with user input ------------------
      if (!missing(digits)) {
        self$digits <- digits
      }
      # ---- set the number of bootstrap replicates with user input -----
      if (!missing(R)) {
        self$R <- R
      }
      # ------ set the probability of type I error with user input ------
      if (!missing(alpha)) {
        self$alpha <- alpha
      }
      # ----------------- set correction with user input ----------------
      if (!missing(correction)) {
        self$correction <- correction
      }
      # ------ initialize some internal fields for public methods -------
      # ------ initialize est function from parent class CoefVar --------
      self$est <- function(...) {
        super$initialize(
          self$x,
          na.rm = self$na.rm,
          digits = self$digits
        )
      }
      self$est()
      # --- initialize the internal functions for the public methods ----
      # ------ initialize internal functions of confidence limits -------
      # ------------------------ Kelley method --------------------------
      self$ncp_cv <- function(...) {
        # non-centrality parameter of cv
        return(
          sqrt(length(self$x)) / (self$est() / 100)
        )
      }
      self$ncp_cv_corr <- function(...) {
        # non-centrality parameter of corrected cv
        return(
          sqrt(length(self$x)) / (self$est_corr() / 100)
        )
      }
      self$v <- function(...) {
        # v is the n-1 degrees of freedom
        return(
          length(self$x) - 1
        )
      }
      self$kelley_nctci_cv <- function(...) {
        # for Kelley method of CI
        # noncentrality parameter from a t distribution for cv
        return(
          MBESS::conf.limits.nct(
            ncp = self$ncp_cv(),
            df = self$v(),
            conf.level = (1 - self$alpha)
          )
        )
      }
      self$kelley_nctci_cv_corr <- function(...) {
        # for Kelley method of CI
        # noncentrality parameter from a t distribution for corrected cv
        return(
          MBESS::conf.limits.nct(
            ncp = self$ncp_cv_corr(),
            df = self$v(),
            conf.level = (1 - self$alpha)
          )
        )
      }
      self$lower_tile_kelley_cv <- function(...) {
        # lower confidence limit for Kelley method of cv
        return(
          unname(
            sqrt(
              length(
                self$x
              )
            ) / self$kelley_nctci_cv()$Upper.Limit
          )
        )
      }
      self$upper_tile_kelley_cv <- function(...) {
        # upper confidence limit for Kelley method of cv
        return(
          unname(
            sqrt(
              length(
                self$x
              )
            ) / self$kelley_nctci_cv()$Lower.Limit
          )
        )
      }
      self$lower_tile_kelley_cv_corr <- function(...) {
        # lower confidence limit for Kelley method of corrected cv
        return(
          unname(
            sqrt(
              length(
                self$x
              )
            ) / self$kelley_nctci_cv_corr()$Upper.Limit
          )
        )
      }
      self$upper_tile_kelley_cv_corr <- function(...) {
        # upper confidence limit for Kelley method of corrected cv
        return(
          unname(
            sqrt(
              length(
                self$x
              )
            ) / self$kelley_nctci_cv_corr()$Lower.Limit
          )
        )
      }
      # ------------------------ McKay method ---------------------------
      self$t1_mckay <- function(...) {
        # chi-square (1-alpha/2)% quantile
        return(
          qchisq(1 - self$alpha / 2, self$v()) / self$v()
        )
      }
      self$t2_mckay <- function(...) {
        # chi-square (alpha/2)% quantile
        return(
          qchisq(self$alpha / 2, self$v()) / self$v()
        )
      }
      self$u1_mckay <- function(...) {
        # u1 parameter based on chi-square (1-alpha/2)% quantile
        return(
          self$v() * self$t1_mckay()
        )
      }
      self$u2_mckay <- function(...) {
        # u2 parameter based on chi-square (alpha/2)% quantile
        return(
          self$v() * self$t2_mckay()
        )
      }
      self$lower_tile_mckay_cv <- function(...) {
        # lower confidence limit for McKay method of cv
        return(
          (self$est() / 100) / sqrt(
            (self$u1_mckay() / (self$v() + 1) - 1
            ) * (
              (self$est() / 100)^2) + self$u1_mckay() / self$v()
          )
        )
      }
      self$upper_tile_mckay_cv <- function(...) {
        # upper confidence limit for McKay method of cv
        return(
          (self$est() / 100) / sqrt(
            (self$u2_mckay() / (self$v() + 1) - 1
            ) * (
              (self$est() / 100)^2) + self$u2_mckay() / self$v()
          )
        )
      }
      self$lower_tile_mckay_cv_corr <- function(...) {
        # lower confidence limit for McKay method of corrected cv
        return(
          (self$est_corr() / 100) / sqrt(
            (self$u1_mckay() / (self$v() + 1) - 1
            ) * (
              (self$est_corr() / 100)^2) + self$u1_mckay() / self$v()
          )
        )
      }
      self$upper_tile_mckay_cv_corr <- function(...) {
        # upper confidence limit for McKay method of corrected cv
        return(
          (self$est_corr() / 100) / sqrt(
            (self$u2_mckay() / (self$v() + 1) - 1
            ) * (
              (self$est_corr() / 100)^2) + self$u2_mckay() / self$v()
          )
        )
      }
      # ------------------------ Miller method --------------------------
      self$z_miller <- function(...) {
        # z(1-alpha/2)% percentile of Miller Method
        return(
          qnorm(1 - (self$alpha / 2))
        )
      }
      self$u_miller_cv <- function(...) {
        # u parameter based on z(1-alpha/2)% percentile for cv
        return(
          sqrt(
            (
              (self$est() / 100)^2 / self$v()) *
              (0.5 + (self$est() / 100)^2)
          )
        )
      }
      self$u_miller_cv_corr <- function(...) {
        # u parameter based on z(1-alpha/2)% percentile for corrected cv
        return(
          sqrt(
            ((self$est_corr() / 100)^2 / self$v()) *
              (0.5 + (self$est_corr() / 100)^2)
          )
        )
      }
      self$zu_miller_cv <- function(...) {
        # Miller z * Miller u for cv
        return(
          self$z_miller() * self$u_miller_cv()
        )
      }
      self$zu_miller_cv_corr <- function(...) {
        # Miller z * Miller u for corrected cv
        return(
          self$z_miller() * self$u_miller_cv_corr()
        )
      }
      self$lower_tile_miller_cv <- function(...) {
        # lower confidence limit for Miller method of cv
        return(
          (self$est() / 100) - self$zu_miller_cv()
        )
      }
      self$upper_tile_miller_cv <- function(...) {
        # upper confidence limit for Miller method of cv
        return(
          (self$est() / 100) + self$zu_miller_cv()
        )
      }
      self$lower_tile_miller_cv_corr <- function(...) {
        # lower confidence limit for Miller method of corrected cv
        return(
          (self$est_corr() / 100) - self$zu_miller_cv_corr()
        )
      }
      self$upper_tile_miller_cv_corr <- function(...) {
        # upper confidence limit for Miller method of corrected cv
        return(
          (self$est_corr() / 100) + self$zu_miller_cv_corr()
        )
      }
      # ------------------------ Vangel method --------------------------
      self$t1_vangel <- function(...) {
        # chi-square (1-alpha/2)% quantile
        return(
          qchisq(1 - self$alpha / 2, self$v()) / self$v()
        )
      }
      self$t2_vangel <- function(...) {
        # chi-square (alpha/2)% quantile
        return(
          qchisq(self$alpha / 2, self$v()) / self$v()
        )
      }
      self$u1_vangel <- function(...) {
        # u1 parameter based on chi-square (1-alpha/2)% quantile
        return(
          self$v() * self$t1_vangel()
        )
      }
      self$u2_vangel <- function(...) {
        # u2 parameter based on chi-square (alpha/2)% quantile
        return(
          self$v() * self$t2_vangel()
        )
      }
      self$lower_tile_vangel_cv <- function(...) {
        # lower confidence limit for Vangel method of cv
        return(
          (self$est() / 100) / sqrt(
            ((((self$u1_vangel() + 1) / (self$v())) - 1) *
              ((self$est() / 100)^2)) + self$u1_vangel() / self$v()
          )
        )
      }
      self$upper_tile_vangel_cv <- function(...) {
        # upper confidence limit for Vangel method of cv
        return(
          (self$est() / 100) / sqrt(
            ((((self$u2_vangel() + 1) / (self$v())) - 1) *
              ((self$est() / 100)^2)) + self$u2_vangel() / self$v()
          )
        )
      }
      self$lower_tile_vangel_cv_corr <- function(...) {
        # lower confidence limit for Vangel method of corrected cv
        return(
          (self$est_corr() / 100) / sqrt(
            (
              (((self$u1_vangel() + 1) / (self$v())) - 1) *
                ((self$est_corr() / 100)^2)) +
              self$u1_vangel() / self$v()
          )
        )
      }
      self$upper_tile_vangel_cv_corr <- function(...) {
        # upper confidence limit for Vangel method of corrected cv
        return(
          (self$est_corr() / 100) / sqrt(
            (
              (((self$u2_vangel() + 1) / (self$v())) - 1) *
                ((self$est_corr() / 100)^2)) +
              self$u2_vangel() / self$v()
          )
        )
      }
      # ------------------ Mahmoudvand-Hassani method -------------------
      self$cn_mh <- function(...) {
        # ranked set sampling (RSS) parameter C_n
        if (length(self$x) <= 340) {
          # gamma() works well with sample sizes less than 340
          return(
            sqrt(2 / (length(self$x) - 1)) * (
              (gamma(length(self$x) / 2)) / (
                gamma((length(self$x) - 1) / 2))
            )
          )
        } else {
          return(
            # gamma() leads to infinity for sample sizes higher than 340
            # lgamma() i.e., large gamma is used instead.
            sqrt(2 / (length(self$x) - 1)) * (
              (lgamma(length(self$x) / 2)) / (
                lgamma((length(self$x) - 1) / 2))
            )
          )
        }
      }
      self$ul_mh <- function(...) {
        # RSS u parameter for the lower bound
        return(
          2 - (
            self$cn_mh() + (
              qnorm((self$alpha / 2)) * sqrt(1 - self$cn_mh()^2)
            )
          )
        )
      }
      self$uu_mh <- function(...) {
        # RSS u parameter for the upper bound
        return(
          2 - (
            self$cn_mh() - (
              qnorm((self$alpha / 2)) * sqrt(1 - self$cn_mh()^2)
            )
          )
        )
      }
      self$lower_tile_mh_cv <- function(...) {
        # lower confidence limit for MH method of cv
        return(
          (self$est() / 100) / self$ul_mh()
        )
      }
      self$upper_tile_mh_cv <- function(...) {
        # upper confidence limit for MH method of cv
        return(
          (self$est() / 100) / self$uu_mh()
        )
      }
      self$lower_tile_mh_cv_corr <- function(...) {
        # lower confidence limit for MH method of corrected cv
        return(
          (self$est_corr() / 100) / self$ul_mh()
        )
      }
      self$upper_tile_mh_cv_corr <- function(...) {
        # upper confidence limit for MH method of corrected cv
        return(
          (self$est_corr() / 100) / self$uu_mh()
        )
      }
      # ------------------ Normal Approximation method ------------------
      self$cn_normaapprox <- function(...) {
        # Normal Approximation parameter C_n
        return(
          sqrt(1 - (1 / (2 * length(self$x))))
        )
      }
      self$ul_normaapprox <- function(...) {
        # Normal Approximation u parameter for the lower bound
        return(
          self$cn_normaapprox() + (
            qnorm(
              1 - (self$alpha / 2)
            ) * sqrt(1 - self$cn_normaapprox()^2)
          )
        )
      }
      self$uu_normaapprox <- function(...) {
        # Normal Approximation u parameter for the upper bound
        return(
          self$cn_normaapprox() - (
            qnorm(
              1 - (self$alpha / 2)
            ) * sqrt(1 - self$cn_normaapprox()^2)
          )
        )
      }
      self$lower_tile_normaapprox_cv <- function(...) {
        # lower confidence limit for Normal Approximation method of cv
        return(
          (self$est() / 100) / self$ul_normaapprox()
        )
      }
      self$upper_tile_normaapprox_cv <- function(...) {
        # upper confidence limit for Normal Approximation method of cv
        return(
          (self$est() / 100) / self$uu_normaapprox()
        )
      }
      self$lower_tile_normaapprox_cv_corr <- function(...) {
        # lower limit for Normal Approximation method of corrected cv
        return(
          (self$est_corr() / 100) / self$ul_normaapprox()
        )
      }
      self$upper_tile_normaapprox_cv_corr <- function(...) {
        # upper limit for Normal Approximation method of corrected cv
        return(
          (self$est_corr() / 100) / self$uu_normaapprox()
        )
      }
      # -------------------- Shortest-Length method ---------------------
      self$a_value <- function(...) {
        # a_value of the Shortest-Length interval method
        if ((length(self$x) - 1) <= 300) {
          return(
            self$shortest_length[
              self$shortest_length$al == self$alpha &
                self$shortest_length$v == length(self$x) - 1,
              "a",
              drop = FALSE
            ]
          )
        } else if ((length(self$x) - 1) > 300) {
          return(
            shortest_length[shortest_length$al == self$alpha & shortest_length$v == 300, "a", drop = FALSE]
          )
        }
      }
      self$b_value <- function(...) {
        # b_value of the Shortest-Length interval method
        if ((length(self$x) - 1) <= 300) {
          return(
            self$shortest_length[
              self$shortest_length$al == self$alpha &
                self$shortest_length$v == length(self$x) - 1,
              "b",
              drop = FALSE
            ]
          )
        } else if ((length(self$x) - 1) > 300) {
          return(
            shortest_length[shortest_length$al == self$alpha & shortest_length$v == 300, "b", drop = FALSE]
          )
        }
      }
      self$lower_tile_shortest_cv <- function(...) {
        # lower confidence limit for Shortest-Length method of cv
        return(
          ((self$est() / 100) * sqrt(length(self$x) - 1)
          ) / sqrt(self$b_value()$b)
        )
      }
      self$upper_tile_shortest_cv <- function(...) {
        # upper confidence limit for Shortest-Length method of cv
        return(
          ((self$est() / 100) * sqrt(length(self$x) - 1)
          ) / sqrt(self$a_value()$a)
        )
      }
      self$lower_tile_shortest_cv_corr <- function(...) {
        # lower limit for Shortest-Length method of corrected cv
        return(
          ((self$est_corr() / 100) * sqrt(length(self$x) - 1)
          ) / sqrt(self$b_value()$b)
        )
      }
      self$upper_tile_shortest_cv_corr <- function(...) {
        # upper limit for Shortest-Length method of corrected cv
        return(
          ((self$est_corr() / 100) * sqrt(length(self$x) - 1)
          ) / sqrt(self$a_value()$a)
        )
      }
      # ---------------------- Equal-Tailed method ----------------------
      self$t1_equal <- function(...) {
        # t1 which is (1-alhpa/2)% percentile of chi-scquare
        return(
          qchisq(1 - self$alpha / 2, self$v())
        )
      }
      self$t2_equal <- function(...) {
        # t2 which is (alhpa/2)% percentile of chi-scquare
        return(
          qchisq(alpha / 2, self$v())
        )
      }
      self$lower_tile_equal_cv <- function(...) {
        # lower confidence limit for Equal-Tailed method of cv
        return(
          ((self$est() / 100) * sqrt(self$v())) / (sqrt(self$t1_equal()))
        )
      }
      self$upper_tile_equal_cv <- function(...) {
        # upper confidence limit for Equal-Tailed method of cv
        return(
          ((self$est() / 100) * sqrt(self$v())) / (sqrt(self$t2_equal()))
        )
      }
      self$lower_tile_equal_cv_corr <- function(...) {
        # lower limit for Equal-Tailed method of corrected cv
        return(
          ((self$est_corr() / 100) * sqrt(self$v())) / (
            sqrt(self$t1_equal())
          )
        )
      }
      self$upper_tile_equal_cv_corr <- function(...) {
        # upper limit for Equal-Tailed method of corrected cv
        return(
          ((self$est_corr() / 100) * sqrt(self$v())) / (sqrt(
            self$t2_equal()
          )
          )
        )
      }
      # ------------- AA&K-ADJ (Abu-Shawiesh et al., 2019) --------------
      # adjusted-degrees-of-freedom CI derived from Hummel et al.'s
      # CI for the population variance (Eq. 21).
      self$gamma_hat_aak <- function(...) {
        # Hummel-style adjusted-kurtosis estimator (Eq. 5)
        return(aak_gamma_hat_hummel(self$x, ddof = 1))
      }
      self$r_hat_aak <- function(...) {
        # adjusted degrees of freedom r_hat = 2n / (gamma_hat + 2n/(n-1))
        n <- length(self$x)
        gh <- self$gamma_hat_aak()
        r_hat <- (2 * n) / (gh + 2 * n / (n - 1))
        if (r_hat <= 0) {
          stop(
            "AA&K-ADJ: non-positive adjusted degrees of freedom"
          )
        }
        return(r_hat)
      }
      self$aak_adj_qchi_high <- function(...) {
        # (1 - alpha/2) quantile of chi-square with df = r_hat
        return(stats::qchisq(1 - self$alpha / 2, self$r_hat_aak()))
      }
      self$aak_adj_qchi_low <- function(...) {
        # (alpha/2) quantile of chi-square with df = r_hat
        return(stats::qchisq(self$alpha / 2, self$r_hat_aak()))
      }
      self$lower_tile_aak_adj_cv <- function(...) {
        # lower confidence limit for AA&K-ADJ method of cv
        return(
          (self$est() / 100) *
            sqrt(self$r_hat_aak() / self$aak_adj_qchi_high())
        )
      }
      self$upper_tile_aak_adj_cv <- function(...) {
        # upper confidence limit for AA&K-ADJ method of cv
        return(
          (self$est() / 100) *
            sqrt(self$r_hat_aak() / self$aak_adj_qchi_low())
        )
      }
      self$lower_tile_aak_adj_cv_corr <- function(...) {
        # lower confidence limit for AA&K-ADJ method of corrected cv
        return(
          (self$est_corr() / 100) *
            sqrt(self$r_hat_aak() / self$aak_adj_qchi_high())
        )
      }
      self$upper_tile_aak_adj_cv_corr <- function(...) {
        # upper confidence limit for AA&K-ADJ method of corrected cv
        return(
          (self$est_corr() / 100) *
            sqrt(self$r_hat_aak() / self$aak_adj_qchi_low())
        )
      }
      # -------------- AA&K-LS (Abu-Shawiesh et al., 2019) --------------
      # large-sample CI for the population CV derived from the
      # log-transformed CI for the variance (Eq. 26).
      self$g2_aak <- function(...) {
        # bias-adjusted kurtosis estimator G_2 (Eq. 10)
        return(aak_g2_bias_corrected(self$x))
      }
      self$a_value_aak_ls <- function(...) {
        # A = (G_2 + 2n/(n-1)) / n
        n <- length(self$x)
        return((self$g2_aak() + 2 * n / (n - 1)) / n)
      }
      self$lower_tile_aak_ls_cv <- function(...) {
        # lower confidence limit for AA&K-LS method of cv
        z <- stats::qnorm(1 - self$alpha / 2)
        radical <- z * sqrt(self$a_value_aak_ls())
        return((self$est() / 100) * sqrt(exp(-radical)))
      }
      self$upper_tile_aak_ls_cv <- function(...) {
        # upper confidence limit for AA&K-LS method of cv
        z <- stats::qnorm(1 - self$alpha / 2)
        radical <- z * sqrt(self$a_value_aak_ls())
        return((self$est() / 100) * sqrt(exp(+radical)))
      }
      self$lower_tile_aak_ls_cv_corr <- function(...) {
        # lower confidence limit for AA&K-LS method of corrected cv
        z <- stats::qnorm(1 - self$alpha / 2)
        radical <- z * sqrt(self$a_value_aak_ls())
        return((self$est_corr() / 100) * sqrt(exp(-radical)))
      }
      self$upper_tile_aak_ls_cv_corr <- function(...) {
        # upper confidence limit for AA&K-LS method of corrected cv
        z <- stats::qnorm(1 - self$alpha / 2)
        radical <- z * sqrt(self$a_value_aak_ls())
        return((self$est_corr() / 100) * sqrt(exp(+radical)))
      }
      # ------------- AA&K-ALS (Abu-Shawiesh et al., 2019) --------------
      # augmented-large-sample CI for the population CV derived from
      # Burch's CI for the variance using a three-term Taylor expansion
      # of log(S^2) (Eq. 32).
      self$kappa_e5_aak <- function(...) {
        # modified kurtosis estimator kappa_e5 (Eq. 13)
        return(aak_kappa_e5(self$x))
      }
      self$k_value_aak_als <- function(...) {
        # K = kappa_e5 + 2n / (n - 1)
        n <- length(self$x)
        return(self$kappa_e5_aak() + 2 * n / (n - 1))
      }
      self$b_value_aak_als <- function(...) {
        # B = (K/n) * (1 + K/(2n))  -- variance of log(S^2)
        n <- length(self$x)
        k <- self$k_value_aak_als()
        return((k / n) * (1 + k / (2 * n)))
      }
      self$c_value_aak_als <- function(...) {
        # C = K / (2n)  -- bias of log(S^2)
        n <- length(self$x)
        return(self$k_value_aak_als() / (2 * n))
      }
      self$lower_tile_aak_als_cv <- function(...) {
        # lower confidence limit for AA&K-ALS method of cv
        z <- stats::qnorm(1 - self$alpha / 2)
        radical <- z * sqrt(self$b_value_aak_als())
        c_v <- self$c_value_aak_als()
        return((self$est() / 100) * sqrt(exp(-radical + c_v)))
      }
      self$upper_tile_aak_als_cv <- function(...) {
        # upper confidence limit for AA&K-ALS method of cv
        z <- stats::qnorm(1 - self$alpha / 2)
        radical <- z * sqrt(self$b_value_aak_als())
        c_v <- self$c_value_aak_als()
        return((self$est() / 100) * sqrt(exp(+radical + c_v)))
      }
      self$lower_tile_aak_als_cv_corr <- function(...) {
        # lower confidence limit for AA&K-ALS method of corrected cv
        z <- stats::qnorm(1 - self$alpha / 2)
        radical <- z * sqrt(self$b_value_aak_als())
        c_v <- self$c_value_aak_als()
        return((self$est_corr() / 100) * sqrt(exp(-radical + c_v)))
      }
      self$upper_tile_aak_als_cv_corr <- function(...) {
        # upper confidence limit for AA&K-ALS method of corrected cv
        z <- stats::qnorm(1 - self$alpha / 2)
        radical <- z * sqrt(self$b_value_aak_als())
        c_v <- self$c_value_aak_als()
        return((self$est_corr() / 100) * sqrt(exp(+radical + c_v)))
      }
      # ----------------------- Bootstrap methods -----------------------
      self$bootcv <- function(...) {
        # call grand-parent method of bootstrap for cv
        return(
          super$super_$initialize(
            x = self$x,
            na.rm = self$na.rm,
            R = self$R,
            alpha = self$alpha
          )
        )
        # invisible(self)
      }
      self$bootcv()
      invisible(self)
    },
    # -------------- public method kelley_ci() i.e., Kelley CI ------------
    kelley_ci = function(...) {
      if (self$correction == FALSE) {
        return(
          list(
            method = (
              "cv with Kelley CI"
            ),
            statistics = data.frame(
              est = round(self$est(), digits = self$digits),
              lower = round(
                (self$lower_tile_kelley_cv() * 100),
                digits = self$digits
              ),
              upper = round(
                (self$upper_tile_kelley_cv() * 100),
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      } else {
        return(
          list(
            method = (
              "corrected cv with Kelley CI"
            ),
            statistics = data.frame(
              est = round(self$est_corr(), digits = self$digits),
              lower = round(
                (self$lower_tile_kelley_cv_corr() * 100),
                digits = self$digits
              ),
              upper = round(
                (self$upper_tile_kelley_cv_corr() * 100),
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      }
    },
    # --------------- public method mckay_ci() i.e., McKay CI -------------
    mckay_ci = function(...) {
      if (self$correction == FALSE) {
        return(
          list(
            method = (
              "cv with McKay CI"
            ),
            statistics = data.frame(
              est = round(self$est(), digits = self$digits),
              lower = round(
                self$lower_tile_mckay_cv() * 100,
                digits = self$digits
              ),
              upper = round(
                self$upper_tile_mckay_cv() * 100,
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      } else {
        return(
          list(
            method = (
              "corrected cv with McKay CI"
            ),
            statistics = data.frame(
              est = round(self$est_corr(), digits = self$digits),
              lower = round(
                self$lower_tile_mckay_cv_corr() * 100,
                digits = self$digits
              ),
              upper = round(
                self$upper_tile_mckay_cv_corr() * 100,
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      }
    },
    # -------------- public method miller_ci() i.e., Miller CI ------------
    miller_ci = function(...) {
      if (self$correction == FALSE) {
        return(
          list(
            method = (
              "cv with Miller CI"
            ),
            statistics = data.frame(
              est = round(self$est(), digits = self$digits),
              lower = round(
                self$lower_tile_miller_cv() * 100,
                digits = self$digits
              ),
              upper = round(
                self$upper_tile_miller_cv() * 100,
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      } else {
        return(
          list(
            method = (
              "corrected cv with Miller CI"
            ),
            statistics = data.frame(
              est = round(self$est_corr(), digits = self$digits),
              lower = round(
                self$lower_tile_miller_cv_corr() * 100,
                digits = self$digits
              ),
              upper = round(
                self$upper_tile_miller_cv_corr() * 100,
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      }
    },
    # -------------- public method vangel_ci() i.e., Vangel CI ------------
    vangel_ci = function(...) {
      if (self$correction == FALSE) {
        return(
          list(
            method = (
              "cv with Vangel CI"
            ),
            statistics = data.frame(
              est = round(self$est(), digits = self$digits),
              lower = round(
                self$lower_tile_vangel_cv() * 100,
                digits = self$digits
              ),
              upper = round(
                self$upper_tile_vangel_cv() * 100,
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      } else {
        return(
          list(
            method = (
              "corrected cv with Vangel CI"
            ),
            statistics = data.frame(
              est = round(self$est_corr(), digits = self$digits),
              lower = round(
                self$lower_tile_vangel_cv_corr() * 100,
                digits = self$digits
              ),
              upper = round(
                self$upper_tile_vangel_cv_corr() * 100,
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      }
    },
    # -------- public method mh_ci() i.e., Mahmoudvand-Hassani CI ---------
    mh_ci = function(...) {
      if (self$correction == FALSE) {
        return(
          list(
            method = (
              "cv with Mahmoudvand-Hassani CI"
            ),
            statistics = data.frame(
              est = round(self$est(), digits = self$digits),
              lower = round(
                self$lower_tile_mh_cv() * 100,
                digits = self$digits
              ),
              upper = round(
                self$upper_tile_mh_cv() * 100,
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      } else {
        return(
          list(
            method = (
              "corrected cv with Mahmoudvand-Hassani CI"
            ),
            statistics = data.frame(
              est = round(self$est_corr(), digits = self$digits),
              lower = round(
                self$lower_tile_mh_cv_corr() * 100,
                digits = self$digits
              ),
              upper = round(
                self$upper_tile_mh_cv_corr() * 100,
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      }
    },
    # --- public method normaapprox_ci() i.e., Normal Approximation CI ----
    normaapprox_ci = function(...) {
      if (self$correction == FALSE) {
        return(
          list(
            method = (
              "cv with Normal Approximation CI"
            ),
            statistics = data.frame(
              est = round(self$est(), digits = self$digits),
              lower = round(
                self$lower_tile_normaapprox_cv() * 100,
                digits = self$digits
              ),
              upper = round(
                self$upper_tile_normaapprox_cv() * 100,
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      } else {
        return(
          list(
            method = (
              "corrected cv with Normal Approximation CI"
            ),
            statistics = data.frame(
              est = round(self$est_corr(), digits = self$digits),
              lower = round(
                self$lower_tile_normaapprox_cv_corr() * 100,
                digits = self$digits
              ),
              upper = round(
                self$upper_tile_normaapprox_cv_corr() * 100,
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      }
    },
    # ------- public method shortest_ci() i.e., Shortest-Length CI --------
    shortest_ci = function(...) {
      if (self$correction == FALSE) {
        return(
          list(
            method = (
              "cv with Shortest-Length CI"
            ),
            statistics = data.frame(
              est = round(self$est(), digits = self$digits),
              lower = round(
                self$lower_tile_shortest_cv() * 100,
                digits = self$digits
              ),
              upper = round(
                self$upper_tile_shortest_cv() * 100,
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      } else {
        return(
          list(
            method = (
              "corrected cv with Shortest-Length CI"
            ),
            statistics = data.frame(
              est = round(self$est_corr(), digits = self$digits),
              lower = round(
                self$lower_tile_shortest_cv_corr() * 100,
                digits = self$digits
              ),
              upper = round(
                self$upper_tile_shortest_cv_corr() * 100,
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      }
    },
    # ----------- public method equal_ci() i.e., Equal-Tailed CI ----------
    equal_ci = function(...) {
      if (self$correction == FALSE) {
        return(
          list(
            method = (
              "cv with Equal-Tailed CI"
            ),
            statistics = data.frame(
              est = round(self$est(), digits = self$digits),
              lower = round(
                self$lower_tile_equal_cv() * 100,
                digits = self$digits
              ),
              upper = round(
                self$upper_tile_equal_cv() * 100,
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      } else {
        return(
          list(
            method = (
              "corrected cv with Equal-Tailed CI"
            ),
            statistics = data.frame(
              est = round(self$est_corr(), digits = self$digits),
              lower = round(
                self$lower_tile_equal_cv_corr() * 100,
                digits = self$digits
              ),
              upper = round(
                self$upper_tile_equal_cv_corr() * 100,
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      }
    },
    # --------- public method aak_adj_ci() i.e., AA&K-ADJ CI --------------
    aak_adj_ci = function(...) {
      if (self$correction == FALSE) {
        return(
          list(
            method = (
              "cv with Abu-Shawiesh-Akyuz-Kibria ADJ CI"
            ),
            statistics = data.frame(
              est = round(self$est(), digits = self$digits),
              lower = round(
                self$lower_tile_aak_adj_cv() * 100,
                digits = self$digits
              ),
              upper = round(
                self$upper_tile_aak_adj_cv() * 100,
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      } else {
        return(
          list(
            method = (
              "corrected cv with Abu-Shawiesh-Akyuz-Kibria ADJ CI"
            ),
            statistics = data.frame(
              est = round(self$est_corr(), digits = self$digits),
              lower = round(
                self$lower_tile_aak_adj_cv_corr() * 100,
                digits = self$digits
              ),
              upper = round(
                self$upper_tile_aak_adj_cv_corr() * 100,
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      }
    },
    # ---------- public method aak_ls_ci() i.e., AA&K-LS CI ---------------
    aak_ls_ci = function(...) {
      if (self$correction == FALSE) {
        return(
          list(
            method = (
              "cv with Abu-Shawiesh-Akyuz-Kibria LS CI"
            ),
            statistics = data.frame(
              est = round(self$est(), digits = self$digits),
              lower = round(
                self$lower_tile_aak_ls_cv() * 100,
                digits = self$digits
              ),
              upper = round(
                self$upper_tile_aak_ls_cv() * 100,
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      } else {
        return(
          list(
            method = (
              "corrected cv with Abu-Shawiesh-Akyuz-Kibria LS CI"
            ),
            statistics = data.frame(
              est = round(self$est_corr(), digits = self$digits),
              lower = round(
                self$lower_tile_aak_ls_cv_corr() * 100,
                digits = self$digits
              ),
              upper = round(
                self$upper_tile_aak_ls_cv_corr() * 100,
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      }
    },
    # --------- public method aak_als_ci() i.e., AA&K-ALS CI --------------
    aak_als_ci = function(...) {
      if (self$correction == FALSE) {
        return(
          list(
            method = (
              "cv with Abu-Shawiesh-Akyuz-Kibria ALS CI"
            ),
            statistics = data.frame(
              est = round(self$est(), digits = self$digits),
              lower = round(
                self$lower_tile_aak_als_cv() * 100,
                digits = self$digits
              ),
              upper = round(
                self$upper_tile_aak_als_cv() * 100,
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      } else {
        return(
          list(
            method = (
              "corrected cv with Abu-Shawiesh-Akyuz-Kibria ALS CI"
            ),
            statistics = data.frame(
              est = round(self$est_corr(), digits = self$digits),
              lower = round(
                self$lower_tile_aak_als_cv_corr() * 100,
                digits = self$digits
              ),
              upper = round(
                self$upper_tile_aak_als_cv_corr() * 100,
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      }
    },
    # -- public method norm_ci() i.e., bootstrap normal approximation CI --
    norm_ci = function(...) {
      if (self$correction == FALSE) {
        return(
          list(
            method = (
              "cv with normal approximation bootstrap CI"
            ),
            statistics = data.frame(
              est = round(self$est(), digits = self$digits),
              lower = round(
                self$boot_norm_ci_cv()$normal[2],
                digits = self$digits
              ),
              upper = round(
                self$boot_norm_ci_cv()$normal[3],
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      } else {
        return(
          list(
            method = (
              "corrected cv with normal approximation bootstrap CI"
            ),
            statistics = data.frame(
              est = round(self$est_corr(), digits = self$digits),
              lower = round(
                self$boot_norm_ci_cv_corr()$normal[2],
                digits = self$digits
              ),
              upper = round(
                self$boot_norm_ci_cv_corr()$normal[3],
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      }
    },
    # --------- public method basic_ci() i.e., Basic Bootstrap CI ---------
    basic_ci = function(...) {
      if (self$correction == FALSE) {
        return(
          list(
            method = (
              "cv with basic bootstrap CI"
            ),
            statistics = data.frame(
              est = round(self$est(), digits = self$digits),
              lower = round(
                self$boot_basic_ci_cv()$basic[4],
                digits = self$digits
              ),
              upper = round(
                self$boot_basic_ci_cv()$basic[5],
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      } else {
        return(
          list(
            method = (
              "corrected cv with basic bootstrap CI"
            ),
            statistics = data.frame(
              est = round(self$est_corr(), digits = self$digits),
              lower = round(
                self$boot_basic_ci_cv_corr()$basic[4],
                digits = self$digits
              ),
              upper = round(
                self$boot_basic_ci_cv_corr()$basic[5],
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      }
    },
    # ------- public method perc_ci() i.e., Bootstrap Percentile CI -------
    perc_ci = function(...) {
      if (self$correction == FALSE) {
        return(
          list(
            method = (
              "cv with bootstrap percentile CI"
            ),
            statistics = data.frame(
              est = round(self$est(), digits = self$digits),
              lower = round(
                self$boot_perc_ci_cv()$percent[4],
                digits = self$digits
              ),
              upper = round(
                self$boot_perc_ci_cv()$percent[5],
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      } else {
        return(
          list(
            method = (
              "corrected cv with bootstrap percentile CI"
            ),
            statistics = data.frame(
              est = round(self$est_corr(), digits = self$digits),
              lower = round(
                self$boot_perc_ci_cv_corr()$percent[4],
                digits = self$digits
              ),
              upper = round(
                self$boot_perc_ci_cv_corr()$percent[5],
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      }
    },
    # - public method bca_ci() i.e., Adjusted Bootstrap Percentile (BCa) CI
    bca_ci = function(...) {
      if (self$correction == FALSE) {
        return(
          list(
            method = (
              "cv with adjusted bootstrap percentile (BCa) CI"
            ),
            statistics = data.frame(
              est = round(self$est(), digits = self$digits),
              lower = round(
                self$boot_bca_ci_cv()$bca[4],
                digits = self$digits
              ),
              upper = round(
                self$boot_bca_ci_cv()$bca[5],
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      } else {
        return(
          list(
            method = (
              "corrected cv with adjusted bootstrap percentile (BCa) CI"
            ),
            statistics = data.frame(
              est = round(self$est_corr(), digits = self$digits),
              lower = round(
                self$boot_bca_ci_cv_corr()$bca[4],
                digits = self$digits
              ),
              upper = round(
                self$boot_bca_ci_cv_corr()$bca[5],
                digits = self$digits
              ),
              row.names = c(" ")
            )
          )
        )
      }
    },
    # --------- public method all_ci() i.e., All Bootstrap CIs ------------
    all_ci = function(...) {
      if (self$correction == FALSE) {
        return(
          list(
            method = "All methods",
            statistics = data.frame(
              row.names = c(
                "kelley", # 1
                "mckay", # 2
                "miller", # 3
                "vangel", # 4
                "mahmoudvand_hassani", # 5
                "equal_tailed", # 6
                "shortest_length", # 7
                "normal_approximation", # 8
                "norm", # 9
                "basic", # 10
                "perc", # 11
                "bca", # 12
                "aak_adj", # 13
                "aak_ls", # 14
                "aak_als" # 15
              ),
              est = c(
                round(self$est(), digits = self$digits),
                round(self$est(), digits = self$digits),
                round(self$est(), digits = self$digits),
                round(self$est(), digits = self$digits),
                round(self$est(), digits = self$digits),
                round(self$est(), digits = self$digits),
                round(self$est(), digits = self$digits),
                round(self$est(), digits = self$digits),
                round(self$est(), digits = self$digits),
                round(self$est(), digits = self$digits),
                round(self$est(), digits = self$digits),
                round(self$est(), digits = self$digits),
                round(self$est(), digits = self$digits),
                round(self$est(), digits = self$digits),
                round(self$est(), digits = self$digits)
              ),
              lower = c(
                round(
                  (self$lower_tile_kelley_cv() * 100),
                  digits = self$digits
                ),
                round(
                  self$lower_tile_mckay_cv() * 100,
                  digits = self$digits
                ),
                round(
                  self$lower_tile_miller_cv() * 100,
                  digits = self$digits
                ),
                round(
                  self$lower_tile_vangel_cv() * 100,
                  digits = self$digits
                ),
                round(
                  self$lower_tile_mh_cv() * 100,
                  digits = self$digits
                ),
                round(
                  self$lower_tile_equal_cv() * 100,
                  digits = self$digits
                ),
                round(
                  self$lower_tile_shortest_cv() * 100,
                  digits = self$digits
                ),
                round(
                  self$lower_tile_normaapprox_cv() * 100,
                  digits = self$digits
                ),
                round(
                  self$boot_norm_ci_cv()$normal[2],
                  digits = self$digits
                ),
                round(
                  self$boot_basic_ci_cv()$basic[4],
                  digits = self$digits
                ),
                round(
                  self$boot_perc_ci_cv()$percent[4],
                  digits = self$digits
                ),
                round(
                  self$boot_bca_ci_cv()$bca[4],
                  digits = self$digits
                ),
                round(
                  self$lower_tile_aak_adj_cv() * 100,
                  digits = self$digits
                ),
                round(
                  self$lower_tile_aak_ls_cv() * 100,
                  digits = self$digits
                ),
                round(
                  self$lower_tile_aak_als_cv() * 100,
                  digits = self$digits
                )
              ),
              upper = c(
                round(
                  (self$upper_tile_kelley_cv() * 100),
                  digits = self$digits
                ),
                round(
                  self$upper_tile_mckay_cv() * 100,
                  digits = self$digits
                ),
                round(
                  self$upper_tile_miller_cv() * 100,
                  digits = self$digits
                ),
                round(
                  self$upper_tile_vangel_cv() * 100,
                  digits = self$digits
                ),
                round(
                  self$upper_tile_mh_cv() * 100,
                  digits = self$digits
                ),
                round(
                  self$upper_tile_equal_cv() * 100,
                  digits = self$digits
                ),
                round(
                  self$upper_tile_shortest_cv() * 100,
                  digits = self$digits
                ),
                round(
                  self$upper_tile_normaapprox_cv() * 100,
                  digits = self$digits
                ),
                round(
                  self$boot_norm_ci_cv()$normal[3],
                  digits = self$digits
                ),
                round(
                  self$boot_basic_ci_cv()$basic[5],
                  digits = self$digits
                ),
                round(
                  self$boot_perc_ci_cv()$percent[5],
                  digits = self$digits
                ),
                round(
                  self$boot_bca_ci_cv()$bca[5],
                  digits = self$digits
                ),
                round(
                  self$upper_tile_aak_adj_cv() * 100,
                  digits = self$digits
                ),
                round(
                  self$upper_tile_aak_ls_cv() * 100,
                  digits = self$digits
                ),
                round(
                  self$upper_tile_aak_als_cv() * 100,
                  digits = self$digits
                )
              ),
              description = c(
                "cv with Kelley 95% CI",
                "cv with McKay 95% CI",
                "cv with Miller 95% CI",
                "cv with Vangel 95% CI",
                "cv with Mahmoudvand-Hassani 95% CI",
                "cv with Equal-Tailed 95% CI",
                "cv with Shortest-Length 95% CI",
                "cv with Normal Approximation 95% CI",
                "cv with Normal Approximation Bootstrap 95% CI",
                "cv with Basic Bootstrap 95% CI",
                "cv with Bootstrap Percentile 95% CI",
                "cv with Adjusted Bootstrap Percentile (BCa) 95% CI",
                "cv with Abu-Shawiesh-Akyuz-Kibria ADJ 95% CI",
                "cv with Abu-Shawiesh-Akyuz-Kibria LS 95% CI",
                "cv with Abu-Shawiesh-Akyuz-Kibria ALS 95% CI"
              )
            )
          )
        )
      } else {
        return(
          list(
            method = "All methods",
            statistics = data.frame(
              row.names = c(
                "kelley", # 1
                "mckay", # 2
                "miller", # 3
                "vangel", # 4
                "mahmoudvand_hassani", # 5
                "equal_tailed", # 6
                "shortest_length", # 7
                "normal_approximation", # 8
                "norm", # 9
                "basic", # 10
                "perc", # 11
                "bca", # 12
                "aak_adj", # 13
                "aak_ls", # 14
                "aak_als" # 15
              ),
              est = c(
                round(self$est_corr(), digits = self$digits),
                round(self$est_corr(), digits = self$digits),
                round(self$est_corr(), digits = self$digits),
                round(self$est_corr(), digits = self$digits),
                round(self$est_corr(), digits = self$digits),
                round(self$est_corr(), digits = self$digits),
                round(self$est_corr(), digits = self$digits),
                round(self$est_corr(), digits = self$digits),
                round(self$est_corr(), digits = self$digits),
                round(self$est_corr(), digits = self$digits),
                round(self$est_corr(), digits = self$digits),
                round(self$est_corr(), digits = self$digits),
                round(self$est_corr(), digits = self$digits),
                round(self$est_corr(), digits = self$digits),
                round(self$est_corr(), digits = self$digits)
              ),
              lower = c(
                round(
                  (self$lower_tile_kelley_cv_corr() * 100),
                  digits = self$digits
                ),
                round(
                  self$lower_tile_mckay_cv_corr() * 100,
                  digits = self$digits
                ),
                round(
                  self$lower_tile_miller_cv_corr() * 100,
                  digits = self$digits
                ),
                round(
                  self$lower_tile_vangel_cv_corr() * 100,
                  digits = self$digits
                ),
                round(
                  self$lower_tile_mh_cv_corr() * 100,
                  digits = self$digits
                ),
                round(
                  self$lower_tile_equal_cv_corr() * 100,
                  digits = self$digits
                ),
                round(
                  self$lower_tile_shortest_cv_corr() * 100,
                  digits = self$digits
                ),
                round(
                  self$lower_tile_normaapprox_cv_corr() * 100,
                  digits = self$digits
                ),
                round(
                  self$boot_norm_ci_cv_corr()$normal[2],
                  digits = self$digits
                ),
                round(
                  self$boot_basic_ci_cv_corr()$basic[4],
                  digits = self$digits
                ),
                round(
                  self$boot_perc_ci_cv_corr()$percent[4],
                  digits = self$digits
                ),
                round(
                  self$boot_bca_ci_cv_corr()$bca[4],
                  digits = self$digits
                ),
                round(
                  self$lower_tile_aak_adj_cv_corr() * 100,
                  digits = self$digits
                ),
                round(
                  self$lower_tile_aak_ls_cv_corr() * 100,
                  digits = self$digits
                ),
                round(
                  self$lower_tile_aak_als_cv_corr() * 100,
                  digits = self$digits
                )
              ),
              upper = c(
                round(
                  (self$upper_tile_kelley_cv_corr() * 100),
                  digits = self$digits
                ),
                round(
                  self$upper_tile_mckay_cv_corr() * 100,
                  digits = self$digits
                ),
                round(
                  self$upper_tile_miller_cv_corr() * 100,
                  digits = self$digits
                ),
                round(
                  self$upper_tile_vangel_cv_corr() * 100,
                  digits = self$digits
                ),
                round(
                  self$upper_tile_mh_cv_corr() * 100,
                  digits = self$digits
                ),
                round(
                  self$upper_tile_equal_cv_corr() * 100,
                  digits = self$digits
                ),
                round(
                  self$upper_tile_shortest_cv_corr() * 100,
                  digits = self$digits
                ),
                round(
                  self$upper_tile_normaapprox_cv_corr() * 100,
                  digits = self$digits
                ),
                round(
                  self$boot_norm_ci_cv_corr()$normal[3],
                  digits = self$digits
                ),
                round(
                  self$boot_basic_ci_cv_corr()$basic[5],
                  digits = self$digits
                ),
                round(
                  self$boot_perc_ci_cv_corr()$percent[5],
                  digits = self$digits
                ),
                round(
                  self$boot_bca_ci_cv_corr()$bca[5],
                  digits = self$digits
                ),
                round(
                  self$upper_tile_aak_adj_cv_corr() * 100,
                  digits = self$digits
                ),
                round(
                  self$upper_tile_aak_ls_cv_corr() * 100,
                  digits = self$digits
                ),
                round(
                  self$upper_tile_aak_als_cv_corr() * 100,
                  digits = self$digits
                )
              ),
              description = c(
                "Corrected cv with Kelley 95% CI",
                "Corrected cv with McKay 95% CI",
                "Corrected cv with Miller 95% CI",
                "Corrected cv with Vangel 95% CI",
                "Corrected cv with Mahmoudvand-Hassani 95% CI",
                "Corrected cv with Equal-Tailed 95% CI",
                "Corrected cv with Shortest-Length 95% CI",
                "Corrected cv with Normal Approximation 95% CI",
                "Corrected cv with Normal Approximation Bootstrap 95% CI",
                "Corrected cv with Basic Bootstrap 95% CI",
                "Corrected cv with Bootstrap Percentile 95% CI",
                "Corrected cv with Adjusted Bootstrap Percentile (BCa) 95% CI",
                "Corrected cv with Abu-Shawiesh-Akyuz-Kibria ADJ 95% CI",
                "Corrected cv with Abu-Shawiesh-Akyuz-Kibria LS 95% CI",
                "Corrected cv with Abu-Shawiesh-Akyuz-Kibria ALS 95% CI"
              )
            )
          )
        )
      }
    }
  )
)

context("state_CoefVarCI")
test_that(
  desc = "understands the default value of correction = FALSE",
  {
    x <- c(
      0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
      4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
    )
    expect_false(
      CoefVarCI$new(x)$correction
    )
  }
)
test_that(
  desc = "understands the user input value of correction = TRUE",
  {
    x <- c(
      0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
      4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
    )
    expect_true(
      CoefVarCI$new(x, correction = TRUE)$correction
    )
  }
)
test_that(
  desc = "understands the user input value of correction = FALSE",
  {
    x <- c(
      0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
      4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
    )
    expect_false(
      CoefVarCI$new(x, correction = FALSE)$correction
    )
  }
)
test_that(
  desc = "finds the correct findings",
  {
    x <- c(
      0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
      4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
    )
    expect_equal(
      CoefVarCI$new(x)$ncp_cv(),
      7.737259,
      tolerance = 0.000001
    )
    expect_equal(
      CoefVarCI$new(x)$ncp_cv_corr(),
      7.697308,
      tolerance = 0.000001
    )
    expect_equal(
      CoefVarCI$new(x)$v(),
      19
    )
    expect_equal(
      CoefVarCI$new(x)$kelley_nctci_cv()$Lower.Limit,
      4.565712,
      tolerance = 0.000001
    )
    expect_equal(
      CoefVarCI$new(x)$kelley_nctci_cv()$Upper.Limit,
      10.82754,
      tolerance = 0.00001
    )
    expect_equal(
      CoefVarCI$new(x)$kelley_nctci_cv_corr()$Lower.Limit,
      4.535534,
      tolerance = 0.000001
    )
    expect_equal(
      CoefVarCI$new(x)$kelley_nctci_cv_corr()$Upper.Limit,
      10.77774,
      tolerance = 0.00001
    )
    expect_equal(
      CoefVarCI$new(x)$lower_tile_kelley_cv(),
      0.4130336,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$upper_tile_kelley_cv(),
      0.9795047,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$lower_tile_kelley_cv_corr(),
      0.4149419,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$upper_tile_kelley_cv_corr(),
      0.9860219,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$t1_mckay(),
      1.72907,
      tolerance = 0.00001
    )
    expect_equal(
      CoefVarCI$new(x)$t2_mckay(),
      0.468764,
      tolerance = 0.000001
    )
    expect_equal(
      CoefVarCI$new(x)$u1_mckay(),
      32.85233,
      tolerance = 0.00001
    )
    expect_equal(
      CoefVarCI$new(x)$u2_mckay(),
      8.906516,
      tolerance = 0.000001
    )
    expect_equal(
      CoefVarCI$new(x)$lower_tile_mckay_cv(),
      0.4145785,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$upper_tile_mckay_cv(),
      1.085637,
      tolerance = 0.000001
    )
    expect_equal(
      CoefVarCI$new(x)$lower_tile_mckay_cv_corr(),
      0.416491,
      tolerance = 0.000001
    )
    expect_equal(
      CoefVarCI$new(x)$upper_tile_mckay_cv_corr(),
      1.095004,
      tolerance = 0.000001
    )
    expect_equal(
      CoefVarCI$new(x)$z_miller(),
      1.959964,
      tolerance = 0.000001
    )
    expect_equal(
      CoefVarCI$new(x)$u_miller_cv(),
      0.1211033,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$u_miller_cv_corr(),
      0.1219853,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$zu_miller_cv(),
      0.2373581,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$zu_miller_cv_corr(),
      0.2390868,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$lower_tile_miller_cv(),
      0.3406419,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$upper_tile_miller_cv(),
      0.8153581,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$lower_tile_miller_cv_corr(),
      0.3419132,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$upper_tile_miller_cv_corr(),
      0.8200868,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$t1_vangel(),
      1.72907,
      tolerance = 0.00001
    )
    expect_equal(
      CoefVarCI$new(x)$t2_vangel(),
      0.468764,
      tolerance = 0.000001
    )
    expect_equal(
      CoefVarCI$new(x)$u1_vangel(),
      32.85233,
      tolerance = 0.00001
    )
    expect_equal(
      CoefVarCI$new(x)$u2_vangel(),
      8.906516,
      tolerance = 0.000001
    )
    expect_equal(
      CoefVarCI$new(x)$lower_tile_vangel_cv(),
      0.4097103,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$upper_tile_vangel_cv(),
      1.040016,
      tolerance = 0.000001
    )
    expect_equal(
      CoefVarCI$new(x)$lower_tile_vangel_cv_corr(),
      0.4115559,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$upper_tile_vangel_cv_corr(),
      1.048242,
      tolerance = 0.000001
    )
    expect_equal(
      CoefVarCI$new(x)$cn_mh(),
      0.9869343,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$ul_mh(),
      1.328862,
      tolerance = 0.000001
    )
    expect_equal(
      CoefVarCI$new(x)$uu_mh(),
      0.6972697,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$lower_tile_mh_cv(),
      0.4349587,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$upper_tile_mh_cv(),
      0.8289476,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$lower_tile_mh_cv_corr(),
      0.4372163,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$upper_tile_mh_cv_corr(),
      0.8332501,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$cn_normaapprox(),
      0.9874209,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$ul_normaapprox(),
      1.297318,
      tolerance = 0.000001
    )
    expect_equal(
      CoefVarCI$new(x)$uu_normaapprox(),
      0.6775234,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$lower_tile_normaapprox_cv(),
      0.4455344,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$upper_tile_normaapprox_cv(),
      0.8531071,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$lower_tile_normaapprox_cv_corr(),
      0.4478469,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$upper_tile_normaapprox_cv_corr(),
      0.857535,
      tolerance = 0.000001
    )
    expect_equal(
      unname(CoefVarCI$new(x)$a_value()$a),
      9.6629,
      tolerance = 0.0001
    )
    expect_equal(
      unname(CoefVarCI$new(x)$b_value()$b),
      35.9266,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x)$lower_tile_shortest_cv(),
      0.420336,
      tolerance = 0.000001
    )
    expect_equal(
      CoefVarCI$new(x)$upper_tile_shortest_cv(),
      0.810496,
      tolerance = 0.000001
    )
    expect_equal(
      CoefVarCI$new(x)$lower_tile_shortest_cv_corr(),
      0.4225177,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$upper_tile_shortest_cv_corr(),
      0.8147028,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$t1_equal(),
      32.85233,
      tolerance = 0.00001
    )
    expect_equal(
      CoefVarCI$new(x)$t2_equal(),
      8.906516,
      tolerance = 0.000001
    )
    expect_equal(
      CoefVarCI$new(x)$lower_tile_equal_cv(),
      0.4395634,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$upper_tile_equal_cv(),
      0.8442104,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$lower_tile_equal_cv_corr(),
      0.4418449,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x)$upper_tile_equal_cv_corr(),
      0.8485921,
      tolerance = 0.0000001
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$kelley_ci()$method,
      "cv with Kelley CI"
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$kelley_ci()$statistics$est,
      57.8,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$kelley_ci()$statistics$lower,
      41.3,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$kelley_ci()$statistics$upper,
      98,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$kelley_ci()$method,
      "corrected cv with Kelley CI"
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$kelley_ci()$statistics$est,
      58.1,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$kelley_ci()$statistics$lower,
      41.5,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$kelley_ci()$statistics$upper,
      98.6,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$mckay_ci()$method,
      "cv with McKay CI"
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$mckay_ci()$statistics$est,
      57.8,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$mckay_ci()$statistics$lower,
      41.5,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$mckay_ci()$statistics$upper,
      108.6,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$mckay_ci()$method,
      "corrected cv with McKay CI"
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$mckay_ci()$statistics$est,
      58.1,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$mckay_ci()$statistics$lower,
      41.6,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$mckay_ci()$statistics$upper,
      109.5,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$miller_ci()$method,
      "cv with Miller CI"
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$miller_ci()$statistics$est,
      57.8,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$miller_ci()$statistics$lower,
      34.1,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$miller_ci()$statistics$upper,
      81.5,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$miller_ci()$method,
      "corrected cv with Miller CI"
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$miller_ci()$statistics$est,
      58.1,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$miller_ci()$statistics$lower,
      34.2,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$miller_ci()$statistics$upper,
      82,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$vangel_ci()$method,
      "cv with Vangel CI"
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$vangel_ci()$statistics$est,
      57.8,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = FALSE, digits = 2
      )$vangel_ci()$statistics$lower,
      40.95,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = FALSE, digits = 2
      )$vangel_ci()$statistics$upper,
      103.92,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$vangel_ci()$method,
      "corrected cv with Vangel CI"
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$vangel_ci()$statistics$est,
      58.1,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$vangel_ci()$statistics$lower,
      41.2,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$vangel_ci()$statistics$upper,
      104.8,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$mh_ci()$method,
      "cv with Mahmoudvand-Hassani CI"
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$mh_ci()$statistics$est,
      57.8,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$mh_ci()$statistics$lower,
      43.5,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$mh_ci()$statistics$upper,
      82.9,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$mh_ci()$method,
      "corrected cv with Mahmoudvand-Hassani CI"
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$mh_ci()$statistics$est,
      58.1,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$mh_ci()$statistics$lower,
      43.7,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$mh_ci()$statistics$upper,
      83.3,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$normaapprox_ci()$method,
      "cv with Normal Approximation CI"
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = FALSE
      )$normaapprox_ci()$statistics$est,
      57.8,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = FALSE
      )$normaapprox_ci()$statistics$lower,
      44.6,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = FALSE
      )$normaapprox_ci()$statistics$upper,
      85.3,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = TRUE
      )$normaapprox_ci()$method,
      "corrected cv with Normal Approximation CI"
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = TRUE
      )$normaapprox_ci()$statistics$est,
      58.1,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = TRUE
      )$normaapprox_ci()$statistics$lower,
      44.8,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = TRUE
      )$normaapprox_ci()$statistics$upper,
      85.8,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$shortest_ci()$method,
      "cv with Shortest-Length CI"
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$shortest_ci()$statistics$est,
      57.8,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$shortest_ci()$statistics$lower,
      42,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$shortest_ci()$statistics$upper,
      81,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$shortest_ci()$method,
      "corrected cv with Shortest-Length CI"
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$shortest_ci()$statistics$est,
      58.1,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$shortest_ci()$statistics$lower,
      42.3,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$shortest_ci()$statistics$upper,
      81.5,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$equal_ci()$method,
      "cv with Equal-Tailed CI"
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = FALSE
      )$equal_ci()$statistics$est,
      57.8,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = FALSE
      )$equal_ci()$statistics$lower,
      44,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = FALSE
      )$equal_ci()$statistics$upper,
      84.4,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = TRUE
      )$equal_ci()$method,
      "corrected cv with Equal-Tailed CI"
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = TRUE
      )$equal_ci()$statistics$est,
      58.1,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = TRUE
      )$equal_ci()$statistics$lower,
      44.2,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = TRUE
      )$equal_ci()$statistics$upper,
      84.9,
      tolerance = 0.0001
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$norm_ci()$method,
      "cv with normal approximation bootstrap CI"
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = FALSE
      )$norm_ci()$statistics$est,
      57.8,
      tolerance = 0.0001
    )
    expect_true(
      abs(
        CoefVarCI$new(
          x,
          correction = FALSE
        )$norm_ci()$statistics$lower -
          CoefVarCI$new(
            x,
            correction = FALSE
          )$boot_norm_ci_cv()$normal[2]
      ) / 100 < 0.05
    )
    expect_true(
      abs(
        CoefVarCI$new(
          x,
          correction = FALSE
        )$norm_ci()$statistics$upper -
          CoefVarCI$new(
            x,
            correction = FALSE
          )$boot_norm_ci_cv()$normal[3]
      ) / 100 < 0.05
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = TRUE
      )$norm_ci()$method,
      "corrected cv with normal approximation bootstrap CI"
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = TRUE
      )$norm_ci()$statistics$est,
      58.1,
      tolerance = 0.0001
    )
    expect_true(
      abs(
        CoefVarCI$new(
          x,
          correction = TRUE
        )$norm_ci()$statistics$lower -
          CoefVarCI$new(
            x,
            correction = TRUE
          )$boot_norm_ci_cv_corr()$normal[2]
      ) / 100 < 0.05
    )
    expect_true(
      abs(
        CoefVarCI$new(
          x,
          correction = TRUE
        )$norm_ci()$statistics$upper -
          CoefVarCI$new(
            x,
            correction = TRUE
          )$boot_norm_ci_cv_corr()$normal[3]
      ) / 100 < 0.05
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$basic_ci()$method,
      "cv with basic bootstrap CI"
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = FALSE
      )$basic_ci()$statistics$est,
      57.8,
      tolerance = 0.0001
    )
    expect_true(
      abs(
        CoefVarCI$new(
          x,
          correction = FALSE
        )$basic_ci()$statistics$lower -
          CoefVarCI$new(
            x,
            correction = FALSE
          )$boot_basic_ci_cv()$basic[4]
      ) / 100 < 0.05
    )
    expect_true(
      abs(
        CoefVarCI$new(
          x,
          correction = FALSE
        )$basic_ci()$statistics$upper -
          CoefVarCI$new(
            x,
            correction = FALSE
          )$boot_basic_ci_cv()$basic[5]
      ) / 100 < 0.05
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = TRUE
      )$basic_ci()$method,
      "corrected cv with basic bootstrap CI"
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = TRUE
      )$basic_ci()$statistics$est,
      58.1,
      tolerance = 0.0001
    )
    expect_true(
      abs(
        CoefVarCI$new(
          x,
          correction = TRUE
        )$basic_ci()$statistics$lower -
          CoefVarCI$new(
            x,
            correction = TRUE
          )$boot_basic_ci_cv_corr()$basic[4]
      ) / 100 < 0.05
    )
    expect_true(
      abs(
        CoefVarCI$new(
          x,
          correction = TRUE
        )$basic_ci()$statistics$upper -
          CoefVarCI$new(
            x,
            correction = TRUE
          )$boot_basic_ci_cv_corr()$basic[5]
      ) / 100 < 0.05
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$perc_ci()$method,
      "cv with bootstrap percentile CI"
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = FALSE
      )$perc_ci()$statistics$est,
      57.8,
      tolerance = 0.0001
    )
    expect_true(
      abs(
        CoefVarCI$new(
          x,
          correction = FALSE
        )$perc_ci()$statistics$lower -
          CoefVarCI$new(
            x,
            correction = FALSE
          )$boot_perc_ci_cv()$percent[4]
      ) / 100 < 0.05
    )
    expect_true(
      abs(
        CoefVarCI$new(
          x,
          correction = FALSE
        )$perc_ci()$statistics$upper -
          CoefVarCI$new(
            x,
            correction = FALSE
          )$boot_perc_ci_cv()$percent[5]
      ) / 100 < 0.05
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = TRUE
      )$perc_ci()$method,
      "corrected cv with bootstrap percentile CI"
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = TRUE
      )$perc_ci()$statistics$est,
      58.1,
      tolerance = 0.0001
    )
    expect_true(
      abs(
        CoefVarCI$new(
          x,
          correction = TRUE
        )$perc_ci()$statistics$lower -
          CoefVarCI$new(
            x,
            correction = TRUE
          )$boot_perc_ci_cv_corr()$percent[4]
      ) / 100 < 0.05
    )
    expect_true(
      abs(
        CoefVarCI$new(
          x,
          correction = TRUE
        )$perc_ci()$statistics$upper -
          CoefVarCI$new(
            x,
            correction = TRUE
          )$boot_perc_ci_cv_corr()$percent[5]
      ) / 100 < 0.05
    )
    expect_equal(
      CoefVarCI$new(x, correction = FALSE)$bca_ci()$method,
      "cv with adjusted bootstrap percentile (BCa) CI"
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = FALSE
      )$bca_ci()$statistics$est,
      57.8,
      tolerance = 0.0001
    )
    expect_true(
      abs(
        CoefVarCI$new(
          x,
          correction = FALSE
        )$bca_ci()$statistics$lower -
          CoefVarCI$new(
            x,
            correction = FALSE
          )$boot_bca_ci_cv()$bca[4]
      ) / 100 < 0.3
    )
    expect_true(
      abs(
        CoefVarCI$new(
          x,
          correction = FALSE
        )$bca_ci()$statistics$upper -
          CoefVarCI$new(
            x,
            correction = FALSE
          )$boot_bca_ci_cv()$bca[5]
      ) / 100 < 0.3
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = TRUE
      )$bca_ci()$method,
      "corrected cv with adjusted bootstrap percentile (BCa) CI"
    )
    expect_equal(
      CoefVarCI$new(
        x,
        correction = TRUE
      )$bca_ci()$statistics$est,
      58.1,
      tolerance = 0.0001
    )
    expect_true(
      abs(
        CoefVarCI$new(
          x,
          correction = TRUE
        )$bca_ci()$statistics$lower -
          CoefVarCI$new(
            x,
            correction = TRUE
          )$boot_bca_ci_cv_corr()$bca[4]
      ) / 100 < 0.3
    )
    expect_true(
      abs(
        CoefVarCI$new(
          x,
          correction = TRUE
        )$bca_ci()$statistics$upper -
          CoefVarCI$new(
            x,
            correction = TRUE
          )$boot_bca_ci_cv_corr()$bca[5]
      ) / 100 < 0.3
    )
    expect_equal(
      CoefVarCI$new(x)$all_ci()$method,
      "All methods"
    )
    expect_equal(
      nrow(CoefVarCI$new(x)$all_ci()$statistics),
      15
    )
    expect_equal(
      ncol(CoefVarCI$new(x)$all_ci()$statistics),
      4
    )
    expect_equal(
      CoefVarCI$new(x, correction = TRUE)$all_ci()$method,
      "All methods"
    )
    expect_equal(
      nrow(CoefVarCI$new(x, correction = TRUE)$all_ci()$statistics),
      15
    )
    expect_equal(
      ncol(CoefVarCI$new(x, correction = TRUE)$all_ci()$statistics),
      4
    )
  }
)
test_that(
  desc = "detect R6 class",
  {
    y <- c(
      0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
      4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
    )
    cv_y <- CoefVarCI$new(
      x = y,
      alpha = 0.05,
      R = 1000,
      digits = 2,
      correction = TRUE
    )
    expect_true(
      R6::is.R6(cv_y)
    )
  }
)

test_that(
  desc = "cn_mh for sample sizes higher than 340",
  {
    x <- c(
      56.28, 52.29, 58.66, 56.05, 37.32, 18.18, 78.08, 38.39, 93.93,
      41.96, 49.09, 78.49, 49.76, 51.65, 55.9, 65.85, 37.08, 46.69,
      44.49, 36.83, 69.96, 21.2, 77.01, 26.32, 62.31, 21.54, 74.73,
      61.32, 43.84, 49.76, 13.89, 73.07, 26.38, 55.03, 11.61, 17.78,
      32.12, 72.92, 41.44, 60.3, 45, 100.65, 71.87, 52.2, 54.73, 29.99,
      38.21, 13.18, 41.32, 48.59, 46.76, 33.38, 28.29, 65.12, 53.73,
      62.54, 49.66, 83.33, 66.21, 27.8, 42.6, 55.05, 30.01, 87.68,
      32.93, 76.31, 13.26, 20.2, 46.94, 27.05, 36.75, 44.95, 62.23,
      58.24, 51, 30.79, 57.8, 52.22, 49.84, 61.98, 39.96, 46.91,
      13.99, 54.04, 53.13, 93.67, 56.72, 57.17, 20.4, 23.73, 21.97,
      54.37, 66.22, 39.5, 50.42, 20.67, 68.66, 65.03, 33.55, 100.59,
      45.43, 40.03, 53.42, 78.23, 63.51, 61.04, 30.8, 88.69, 69.16,
      39.33, 14.18, 60.43, 35.21, 98.71, 35.01, 39.88, 27.6, 32.33,
      69.26, 21.67, 62.35, 46.8, 59.15, 64.16, 72.91, 24.68, 36.44,
      45.34, 41.84, 36.28, 54.17, 45.36, 82.28, 45.25, 55.22, 30.04,
      51.01, 34.79, 46.13, 49.59, 54.52, 23.64, 57.98, 42.13, 43.14,
      67.11, 30.43, 47.95, 63.44, 54.26, 37.75, 49.57, 17.56, 67.66,
      32.16, 71.98, 36.13, 68.23, 38.82, 47.03, 34.53, 41.77, 43.25,
      47.08, 88.6, 31.61, 58.32, 58.27, 47.02, 38.34, 73.21, 45.28, 38,
      58.01, 79, 51.11, 61.2, 13.3, 50.21, 57.53, 71.4, 105.32, 39.89,
      51.98, 32.98, 20.59, 28.2, 33.96, 85.3, 63.19, 100.91, 44.62, 35.41,
      56.94, 39.75, 71.22, 12.32, 48.4, 80.24, 33.41, 73.47, 34.65, 71.1,
      40.3, 46.4, 52.37, 20.21, 5.5, 81.66, 78.47, 53.05, 47.3, 71.99,
      48.34, 56.94, 36.78, 57.21, 46.05, 54.79, 32.73, 50.19, 61.07,
      54.22, 37.26, 64.59, 50.72, 78.73, 65.26, 66.1, 52.95, 67.44,
      34.29, 60.13, 39.77, 54.8, 52.46, 60.72, 72.22, 25.38, 61.72,
      50.99, 43.96, 2.36, 71.13, 43.24, 82.37, 65.28, 88.13, 40.22,
      50.06, 62.81, 46.98, 42.26, 63.3, 46.53, 71.14, 49.58, 37.13,
      22.93, 90.91, 33.28, 52.73, 62.67, 43.31, 58.22, 43.04, 76.42,
      61.37, 32.68, 24.24, 55, 71.75, 55.95, 53.42, 16.42, 49.25,
      55.18, 77.89, 55.21, 24.17, 105.72, 21.06, 60.25, 32.05, 25.76,
      17.94, 48.27, 71.78, 58.91, 34.12, 12.98, 76.39, 13.59, 69.43,
      42.54, 81.56, 67.75, 45.33, 49.66, 53.43, 50.95, 30.04, 68.6,
      55.81, 18.16, 30.59, 72.27, 68.67, 28.25, 61.37, 40, 43.59,
      54.38, 34.63, 90.22, 69.85, 47.67, 38.38, 54.81, 47.04, 49.96,
      52.27, 76.31, 49.16, 45.46, 49.27, 59.86, 35, 50.06, 41.74, 64.21,
      68.83, 42.18, 57.58, 61.69, 44.13, 75.31, 68.29, 51.46, 57.52, 39.74
    )
    expect_equal(
      CoefVarCI$new(x)$cn_mh(), 0.0769772,
      tolerance = 0.0000001
    )
  }
)
test_that(
  desc = "a_value and b_value for sample sizes higher than 300",
  {
    x <- rnorm(n = 302, mean = 50, sd = 20)
    expect_equal(
      unname(CoefVarCI$new(x)$a_value()$a),
      255.3057,
      tolerance = 0.0001
    )
    expect_equal(
      unname(CoefVarCI$new(x)$b_value()$b),
      351.8461,
      tolerance = 0.0001
    )
  }
)

context("state_na_rm")
test_that(
  desc = "understands the default value of na.rm = FALSE",
  {
    x <- 1:10
    expect_false(
      CoefVar$new(x)$na.rm
    )
    expect_false(
      CoefQuartVar$new(x)$na.rm
    )
    expect_false(
      CoefVarCI$new(x)$na.rm
    )
    expect_false(
      CoefQuartVarCI$new(x)$na.rm
    )
    expect_false(
      BootCoefVar$new(x)$na.rm
    )
    expect_false(
      BootCoefQuartVar$new(x)$na.rm
    )
    expect_false(
      SampleQuantiles$new(x)$na.rm
    )
    expect_false(
      formals(cv_versatile)$na.rm
    )
    expect_false(
      formals(cqv_versatile)$na.rm
    )
  }
)
test_that(
  desc = "understands the user input of na.rm = TRUE",
  {
    y <- c(1, 2.6, 10, NA, 6.8, NA)
    expect_true(
      CoefVar$new(x = y, na.rm = TRUE)$na.rm
    )
    expect_equal(
      CoefVar$new(x = y, na.rm = TRUE)$est(), 80
    )
    expect_true(
      CoefQuartVar$new(x = y, na.rm = TRUE)$na.rm
    )
    expect_equal(
      CoefQuartVar$new(x = y, na.rm = TRUE)$est(), 55.1
    )
    expect_true(
      CoefVarCI$new(x = y, na.rm = TRUE)$na.rm
    )
    expect_equal(
      CoefVarCI$new(x = y, na.rm = TRUE)$kelley_ci()$statistics$est, 80
    )
    expect_true(
      CoefQuartVarCI$new(x = y, na.rm = TRUE)$na.rm
    )
    expect_equal(
      CoefQuartVarCI$new(x = y, na.rm = TRUE)$norm_ci()$statistics$est,
      55.1
    )
    expect_true(
      BootCoefVar$new(x = y, na.rm = TRUE)$na.rm
    )
    expect_equal(
      mean(BootCoefVar$new(x = y, na.rm = TRUE)$boot_cv()$data), 5.1
    )
    expect_true(
      BootCoefQuartVar$new(x = y, na.rm = TRUE)$na.rm
    )
    expect_equal(
      BootCoefQuartVar$new(x = y, na.rm = TRUE)$boot_cqv()$t0, 55.10204
    )
    expect_true(
      SampleQuantiles$new(x = y, na.rm = TRUE)$na.rm
    )
    expect_equal(
      SampleQuantiles$new(y, na.rm = TRUE, names = FALSE)$qx(), 4.7
    )
    expect_equal(
      cv_versatile(y, na.rm = TRUE, method = "kelley")$statistics$est, 80
    )
    expect_equal(
      cqv_versatile(y, na.rm = TRUE, method = "bonett")$statistics$est,
      55.1
    )
  }
)
# test_that(
#     desc = "correct missing handling when x is data.frame", {
#         df <- data.frame(
#             id = c(1:3, NA),
#             gender = c("m", "f", "m", NA),
#             fbs = c(104, 98, 129, NA)
#         )
#     }
# )

#' @title R6 Coefficient of Variation (cv)
#' @name CoefVar
#' @description The R6 class \code{CoefVar} for the coefficient of variation
#'   (cv)
#' @param x An \code{R} object. Currently there are methods for numeric vectors
#' @param na.rm a logical value indicating whether \code{NA} values should be
#'   stripped before the computation proceeds.
#' @param digits integer indicating the number of decimal places to be used.
#' @details \describe{ \item{\strong{Coefficient of Variation}}{ The \emph{cv}
#'   is a measure of relative dispersion representing the degree of variability
#'   relative to the mean \code{[1]}. Since \eqn{cv} is unitless, it is useful
#'   for comparison of variables with different units. It is also a measure of
#'   homogeneity \code{[1]}. } }
#' @examples
#' x <- c(
#'     0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
#'     4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
#' )
#' CoefVar$new(x)$est()
#' cv_x <- CoefVar$new(x, digits = 2)
#' cv_x$est()
#' cv_x$est_corr()
#' R6::is.R6(cv_x)
#' @references \code{[1]} Albatineh, AN., Kibria, BM., Wilcox, ML., & Zogheib,
#'   B, 2014, Confidence interval estimation for the population coefficient of
#'   variation using ranked set sampling: A simulation study, Journal of Applied
#'   Statistics, 41(4), 733–751, DOI:
#'   \doi{10.1080/02664763.2013.847405}
#'
#' @export
#' @import dplyr SciViews boot R6 utils
NULL
#' @importFrom stats quantile sd qchisq qnorm
#' @importFrom MBESS conf.limits.nct
NULL
CoefVar <- R6::R6Class(
  classname = "CoefVar",
  inherit = BootCoefVar,
  public = list(
    # ---------------- determining defaults for arguments -----------------
    x = NA,
    na.rm = FALSE,
    digits = 1,
    est = NA,
    # --------- determining constructor defaults for arguments ------------
    initialize = function(x = NA,
                          na.rm = FALSE,
                          digits = 1,
                          ...) {
      # ---------------------- check NA or NAN -------------------------
      if (missing(x) || is.null(x)) {
        stop("object 'x' not found")
      } else if (!missing(x)) {
        self$x <- x
      }
      if (!missing(na.rm)) {
        self$na.rm <- na.rm
      }
      if (self$na.rm == TRUE) {
        self$x <- x[!is.na(x)]
      } else if (anyNA(x) & self$na.rm == FALSE) {
        stop(
          "missing values and NaN's not allowed if 'na.rm' is FALSE"
        )
      }
      # ------------- stop if input x vector is not numeric -------------
      if (!is.numeric(x)) {
        stop("argument is not a numeric vector")
      }
      # ------------------- set digits with user input ------------------
      if (!missing(digits)) {
        self$digits <- digits
      }
      # ----------- initialize cv estimate i.e., est() function ---------
      self$est <- function(...) {
        return(
          round(
            100 * (
              (sd(self$x, na.rm = self$na.rm)) /
                (mean(self$x, na.rm = self$na.rm))
            ),
            digits = self$digits
          )
        )
      }
    },
    # -- public method of corrected cv estimate i.e., est_corr() function -
    est_corr = function(...) {
      return(
        round(
          100 * ((self$est() / 100) * (
            (1 - (1 / (4 * (length(self$x) - 1))) +
              (1 / length(self$x)) * (self$est() / 100)^2) +
              (1 / (2 * (length(self$x) - 1)^2))
          )),
          digits = self$digits
        )
      )
    }
  ),
  # ---- define super_ function to enable multiple levels of inheritance ----
  active = list(
    super_ = function() super
  )
)

#' @title R6 Confidence Intervals for the Coefficient of Quartile Variation
#'   (cqv)
#' @name CoefQuartVarCI
#' @description The R6 class \code{CoefQuartVarCI} for the confidence intervals
#'   of coefficient of quartile variation (cqv)
#' @param x An \code{R} object. Currently there are methods for numeric vectors
#' @param na.rm a logical value indicating whether \code{NA} values should be
#'   stripped before the computation proceeds.
#' @param digits integer indicating the number of decimal places to be used.
#' @param methods the available computation methods of confidence intervals are:
#'   "bonett_ci", "norm_ci", "basic_ci", "perc_ci", "bca_ci" or "all_ci".
#' @param R integer indicating the number of bootstrap replicates.
#' @details \describe{ \item{\strong{Coefficient of Quartile Variation}}{ The
#'   \emph{cqv} is a measure of relative dispersion that is based on
#'   interquartile range \emph{(iqr)}. Since \eqn{cqv} is unitless, it is useful
#'   for comparison of variables with different units. It is also a measure of
#'   homogeneity \code{[1, 2]}. } }
#' @return An object of type "list" which contains the estimate, the intervals,
#'   and the computation method. It has two components:
#' @return \describe{ \item{$method}{ A description of statistical method used
#'   for the computations. } \item{$statistics}{ A data frame representing three
#'   vectors: est, lower and upper limits of 95\% confidence interval
#'   \code{(CI)}: \cr \cr \strong{est:} \code{cqv*100}  \cr \cr \strong{Bonett
#'   95\% CI:} It uses a centering adjustment which helps to equalize the tail
#'   error probabilities \code{[1, 2]}.  \cr \cr \strong{Normal approximation
#'   95\% CI:} The intervals calculated by the normal approximation \code{[3,
#'   4]}, using \link[boot]{boot.ci}.  \cr \cr \strong{Basic bootstrap 95\%
#'   CI:} The intervals calculated by the basic bootstrap method \code{[3, 4]},
#'   using \link[boot]{boot.ci}.  \cr \cr \strong{Bootstrap percentile 95\%
#'   CI:} The intervals calculated by the bootstrap percentile method \code{[3,
#'   4]}, using \link[boot]{boot.ci}.  \cr \cr \strong{Adjusted bootstrap
#'   percentile (BCa) 95\% CI:} The intervals calculated by the adjusted
#'   bootstrap percentile (BCa) method \code{[3, 4]}, using
#'   \link[boot]{boot.ci}.  } }
#' @examples
#' y <- c(
#' 0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
#' 4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
#' )
#' CoefQuartVarCI$new(x = y)$bonett_ci()
#' cqv_y <- CoefQuartVarCI$new(
#'     x = y,
#'     alpha = 0.05,
#'     R = 1000,
#'     digits = 2
#' )
#' cqv_y$bonett_ci()
#' R6::is.R6(cqv_y)
#' @references \code{[1]} Bonett, DG., 2006, Confidence interval for a
#'   coefficient of quartile variation, Computational Statistics & Data
#'   Analysis, 50(11), 2953-7, DOI:
#'   \doi{10.1016/j.csda.2005.05.007}
#'
#' @references \code{[2]} Altunkaynak, B., Gamgam, H., 2018, Bootstrap
#'   confidence intervals for the coefficient of quartile variation, Simulation
#'   and Computation, 1-9, DOI:
#'   \doi{10.1080/03610918.2018.1435800}
#'
#' @references \code{[3]} Canty, A., & Ripley, B, 2017, boot: Bootstrap R
#'   (S-Plus) Functions. R package version 1.3-20.
#' @references \code{[4]} Davison, AC., & Hinkley, DV., 1997, Bootstrap Methods
#'   and Their Applications. Cambridge University Press, Cambridge. ISBN
#'   0-521-57391-2
#' @export
#' @import dplyr SciViews boot R6 utils
NULL
#' @importFrom stats quantile sd qchisq qnorm
#' @importFrom MBESS conf.limits.nct
NULL
CoefQuartVarCI <- R6::R6Class(
  classname = "CoefQuartVarCI",
  inherit = CoefQuartVar,
  public = list(
    # ---------------- determining defaults for arguments -----------------
    x = NA,
    na.rm = FALSE,
    digits = 1,
    R = 1000,
    alpha = 0.05,
    # --------------------- adding some internal fields -------------------
    zzz = NA,
    f1square = NA,
    f3square = NA,
    D = NA,
    S = NA,
    v = NA,
    ccc = NA,
    bootcqv = NA,
    # --------- determining constructor defaults for arguments ------------
    initialize = function(x = NA,
                          na.rm = FALSE,
                          digits = 1,
                          R = 1000,
                          alpha = 0.05,
                          ...) {
      # ---------------------- check NA or NAN -------------------------
      if (missing(x) || is.null(x)) {
        stop("object 'x' not found")
      } else if (!missing(x)) {
        self$x <- x
      }
      if (!missing(na.rm)) {
        self$na.rm <- na.rm
      }
      if (self$na.rm == TRUE) {
        self$x <- x[!is.na(x)]
      } else if (anyNA(x) & self$na.rm == FALSE) {
        stop(
          "missing values and NaN's not allowed if 'na.rm' is FALSE"
        )
      }
      # ------------- stop if input x vector is not numeric -------------
      if (!is.numeric(x)) {
        stop("argument is not a numeric vector")
      }
      # ------------------- set digits with user input ------------------
      if (!missing(digits)) {
        self$digits <- digits
      }
      # ---- set the number of bootstrap replicates with user input -----
      if (!missing(R)) {
        self$R <- R
      }
      # ------ set the probability of type I error with user input ------
      if (!missing(alpha)) {
        self$alpha <- alpha
      }
      # ------------- initialize zzz() i.e., z(1 - alpha/2) -------------
      self$zzz <- function(...) {
        qnorm((1 - ((1 - super$alphastar()) / 2)))
      }
      # --------------- initialize internal function f1^2 ---------------
      self$f1square <- function(...) {
        (3 * (self$zzz())^2) / (
          4 * length(self$x) * ((super$Yb() - super$Ya())^2))
      }
      # --------------- initialize internal function f3^2 ---------------
      self$f3square <- function(...) {
        (3 * (self$zzz())^2) / (
          4 * length(self$x) * ((super$Yd() - super$Yc())^2))
      }
      # ----------- initialize internal function D = q3 - q1 ------------
      self$D <- function(...) {
        super$super_$super_$initialize(
          x = self$x,
          na.rm = self$na.rm,
          probs = 0.75,
          names = FALSE
        ) - super$super_$super_$initialize(
          x = self$x,
          na.rm = self$na.rm,
          probs = 0.25,
          names = FALSE
        )
      }
      # ----------- initialize internal function S = q3 + q1 ------------
      self$S <- function(...) {
        super$super_$super_$initialize(
          x = self$x,
          na.rm = self$na.rm,
          probs = 0.75,
          names = FALSE
        ) + super$super_$super_$initialize(
          x = self$x,
          na.rm = self$na.rm,
          probs = 0.25,
          names = FALSE
        )
      }
      # -------- initialize internal function v = var{ln(D/S)} ----------
      self$v <- function(...) {
        (
          (1 / (16 * length(self$x))) * (
            (((3 / self$f1square()) + (3 / self$f3square()) - (
              2 / sqrt(self$f1square() * self$f3square()))) /
              self$D()^2) +
              (((3 / self$f1square()) + (3 / self$f3square()) +
                (2 / sqrt(self$f1square() * self$f3square()))) /
                self$S()^2) -
              ((2 * ((3 / self$f3square()) - (3 / self$f1square()))) /
                (self$D() * self$S()))
          )
        )
      }
      # ----------- initialize internal function c = n/(n-1) ------------
      # ---------------- which is a centering adjustment ----------------
      self$ccc <- function(...) {
        length(self$x) / (length(self$x) - 1)
      }
      # ---- initialize the internal functions for public methods -------
      self$bootcqv <- function(...) {
        return(
          super$super_$initialize(
            x = self$x,
            na.rm = self$na.rm,
            R = self$R,
            alpha = self$alpha
          )
        )
        # invisible(self)
      }
      self$bootcqv()
      # invisible(self)
    },
    # -------------- public method bonett_ci() i.e., Bonett CI ------------
    bonett_ci = function(...) {
      return(
        list(
          method = "cqv with Bonett CI",
          statistics = data.frame(
            est = round(super$est(), digits = self$digits),
            lower = (
              round(
                (100 * exp(
                  ((SciViews::ln((self$D() / self$S())) *
                    self$ccc())) -
                    (self$zzz() * (self$v()^(0.5)))
                )),
                digits = self$digits
              )
            ),
            upper = (
              round(
                (100 * exp(
                  ((SciViews::ln((self$D() / self$S())) *
                    self$ccc())) +
                    (self$zzz() * (self$v()^(0.5)))
                )),
                digits = self$digits
              )
            ),
            row.names = c(" ")
          )
        )
      )
    },
    # -- public method norm_ci() i.e., Normal Approximation Bootstrap CI --
    norm_ci = function(...) {
      return(
        list(
          method = "cqv with normal approximation bootstrap CI",
          statistics = data.frame(
            est = round(super$est(), digits = self$digits),
            lower = round(
              self$boot_norm_ci()$normal[2],
              digits = self$digits
            ),
            upper = round(
              self$boot_norm_ci()$normal[3],
              digits = self$digits
            ),
            row.names = c(" ")
          )
        )
      )
    },
    # ---------- public method basic_ci() i.e., Basic Bootstrap CI --------
    basic_ci = function(...) {
      return(
        list(
          method = "cqv with basic bootstrap CI",
          statistics = data.frame(
            est = round(super$est(), digits = self$digits),
            lower = round(
              self$boot_basic_ci()$basic[4],
              digits = self$digits
            ),
            upper = round(
              self$boot_basic_ci()$basic[5],
              digits = self$digits
            ),
            row.names = c(" ")
          )
        )
      )
    },
    # ----- public method perc_ci() i.e., Bootstrap Percentile CI ---------
    perc_ci = function(...) {
      return(
        list(
          method = "cqv with bootstrap percentile CI",
          statistics = data.frame(
            est = round(super$est(), digits = self$digits),
            lower = round(
              self$boot_perc_ci()$percent[4],
              digits = self$digits
            ),
            upper = round(
              self$boot_perc_ci()$percent[5],
              digits = self$digits
            ),
            row.names = c(" ")
          )
        )
      )
    },
    # - public method bca_ci() i.e., Adjusted Bootstrap Percentile (BCa) CI
    bca_ci = function(...) {
      return(
        list(
          method = "cqv with adjusted bootstrap percentile (BCa) CI",
          statistics = data.frame(
            est = round(super$est(), digits = self$digits),
            lower = round(
              self$boot_bca_ci()$bca[4],
              digits = self$digits
            ),
            upper = round(
              self$boot_bca_ci()$bca[5],
              digits = self$digits
            ),
            row.names = c(" ")
          )
        )
      )
    },
    # --------- public method all_ci() i.e., All Bootstrap CIs ------------
    all_ci = function(...) {
      return(
        list(
          method = "All methods",
          statistics = data.frame(
            row.names = c(
              "bonett",
              "norm",
              "basic",
              "percent",
              "bca"
            ),
            est = c(
              round(super$est(), digits = self$digits),
              round(super$est(), digits = self$digits),
              round(super$est(), digits = self$digits),
              round(super$est(), digits = self$digits),
              round(super$est(), digits = self$digits)
            ),
            lower = c(
              round(
                (100 * exp(
                  ((SciViews::ln((self$D() / self$S())) *
                    self$ccc())) -
                    (self$zzz() * (self$v()^(0.5)))
                )),
                digits = self$digits
              ),
              round(
                self$boot_norm_ci()$normal[2],
                digits = self$digits
              ),
              round(
                self$boot_basic_ci()$basic[4],
                digits = self$digits
              ),
              round(
                self$boot_perc_ci()$percent[4],
                digits = self$digits
              ),
              round(
                self$boot_bca_ci()$bca[4],
                digits = self$digits
              )
            ),
            upper = c(
              round(
                (100 * exp(
                  ((SciViews::ln((self$D() / self$S())) *
                    self$ccc())) +
                    (self$zzz() * (self$v()^(0.5)))
                )),
                digits = self$digits
              ),
              round(
                self$boot_norm_ci()$normal[3],
                digits = self$digits
              ),
              round(
                self$boot_basic_ci()$basic[5],
                digits = self$digits
              ),
              round(
                self$boot_perc_ci()$percent[5],
                digits = self$digits
              ),
              round(
                self$boot_bca_ci()$bca[5],
                digits = self$digits
              )
            ),
            description = c(
              "cqv with Bonett CI",
              "cqv with normal approximation CI",
              "cqv with basic bootstrap CI",
              "cqv with bootstrap percentile CI",
              "cqv with adjusted bootstrap percentile (BCa) CI"
            )
          )
        )
      )
    }
  )
)

context("state_CoefQuartVarCI")
test_that(
  desc = "finds the correct findings",
  {
    x <- c(
      0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
      4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
    )
    expect_equal(
      CoefQuartVarCI$new(x)$zzz(),
      1.832024,
      tolerance = 0.000001
    )
    expect_equal(
      CoefQuartVarCI$new(x)$f1square(),
      0.01398464,
      tolerance = 0.00000001
    )
    expect_equal(
      CoefQuartVarCI$new(x)$f3square(),
      0.04355077,
      tolerance = 0.00000001
    )
    expect_equal(
      CoefQuartVarCI$new(x)$D(),
      3.6,
      tolerance = 0.0001
    )
    expect_equal(
      CoefQuartVarCI$new(x)$S(),
      8,
      tolerance = 0.0001
    )
    expect_equal(
      CoefQuartVarCI$new(x)$v(),
      0.09819588,
      tolerance = 0.00000001
    )
    expect_equal(
      CoefQuartVarCI$new(x)$ccc(),
      1.052632,
      tolerance = 0.000001
    )
    expect_equal(
      CoefQuartVarCI$new(x)$bonett_ci()$method,
      "cqv with Bonett CI"
    )
    expect_equal(
      CoefQuartVarCI$new(x)$bonett_ci()$statistics$est,
      45,
      tolerance = 0.0001
    )
    expect_equal(
      CoefQuartVarCI$new(x)$bonett_ci()$statistics$lower,
      24.3,
      tolerance = 0.0001
    )
    expect_equal(
      CoefQuartVarCI$new(x)$bonett_ci()$statistics$upper,
      76.6,
      tolerance = 0.0001
    )
    expect_equal(
      CoefQuartVarCI$new(x)$norm_ci()$method,
      "cqv with normal approximation bootstrap CI"
    )
    expect_equal(
      CoefQuartVarCI$new(x)$norm_ci()$statistics$est,
      45,
      tolerance = 0.0001
    )
    expect_true(
      abs(
        CoefQuartVarCI$new(x)$norm_ci()$statistics$lower -
          CoefQuartVarCI$new(x)$boot_norm_ci()$normal[2]
      ) / 100 < 0.05
    )
    expect_true(
      abs(
        CoefQuartVarCI$new(x)$norm_ci()$statistics$upper -
          CoefQuartVarCI$new(x)$boot_norm_ci()$normal[3]
      ) / 100 < 0.05
    )
    expect_equal(
      CoefQuartVarCI$new(x)$basic_ci()$method,
      "cqv with basic bootstrap CI"
    )
    expect_equal(
      CoefQuartVarCI$new(x)$basic_ci()$statistics$est,
      45,
      tolerance = 0.0001
    )
    expect_true(
      abs(
        CoefQuartVarCI$new(x)$basic_ci()$statistics$lower -
          CoefQuartVarCI$new(x)$boot_basic_ci()$basic[4]
      ) / 100 < 0.3
    )
    expect_true(
      abs(
        CoefQuartVarCI$new(x)$basic_ci()$statistics$upper -
          CoefQuartVarCI$new(x)$boot_basic_ci()$basic[5]
      ) / 100 < 0.3
    )
    expect_equal(
      CoefQuartVarCI$new(x)$perc_ci()$method,
      "cqv with bootstrap percentile CI"
    )
    expect_equal(
      CoefQuartVarCI$new(x)$perc_ci()$statistics$est,
      45,
      tolerance = 0.0001
    )
    expect_true(
      abs(
        CoefQuartVarCI$new(x)$perc_ci()$statistics$lower -
          CoefQuartVarCI$new(x)$boot_perc_ci()$percent[4]
      ) / 100 < 0.05
    )
    expect_true(
      abs(
        CoefQuartVarCI$new(x)$perc_ci()$statistics$upper -
          CoefQuartVarCI$new(x)$boot_perc_ci()$percent[5]
      ) / 100 < 0.05
    )
    expect_equal(
      CoefQuartVarCI$new(x)$bca_ci()$method,
      "cqv with adjusted bootstrap percentile (BCa) CI"
    )
    expect_equal(
      CoefQuartVarCI$new(x)$bca_ci()$statistics$est,
      45,
      tolerance = 0.0001
    )
    expect_true(
      abs(
        CoefQuartVarCI$new(x)$bca_ci()$statistics$lower -
          CoefQuartVarCI$new(x)$boot_bca_ci()$bca[4]
      ) / 100 < 0.3
    )
    expect_true(
      abs(
        CoefQuartVarCI$new(x)$bca_ci()$statistics$upper -
          CoefQuartVarCI$new(x)$boot_bca_ci()$bca[5]
      ) / 100 < 0.3
    )
    expect_equal(
      CoefQuartVarCI$new(x)$all_ci()$method,
      "All methods"
    )
    expect_equal(
      nrow(CoefQuartVarCI$new(x)$all_ci()$statistics),
      5
    )
    expect_equal(
      ncol(CoefQuartVarCI$new(x)$all_ci()$statistics),
      4
    )
  }
)
test_that(
  desc = "detect R6 class",
  {
    y <- c(
      0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
      4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
    )
    cqv_y <- CoefQuartVarCI$new(
      x = y,
      alpha = 0.05,
      R = 1000,
      digits = 2
    )
    expect_true(
      R6::is.R6(cqv_y)
    )
  }
)

#' @title Coefficient of Quartile Variation (cqv)
#' @name cqv_versatile
#' @description Versatile function for the coefficient of quartile variation
#'   (cqv)
#' @param x An \code{R} object. Currently there are methods for numeric vectors
#' @param na.rm a logical value indicating whether \code{NA} values should be
#'   stripped before the computation proceeds.
#' @param digits integer indicating the number of decimal places to be used.
#' @param method a scalar representing the type of confidence intervals
#'   required. The value should be any of the values "bonett", "norm", "basic",
#'   "perc", "bca" or "all".
#' @param R integer indicating the number of bootstrap replicates.
#' @details \describe{ \item{\strong{Coefficient of Quartile Variation}}{ The
#'   \emph{cqv} is a measure of relative dispersion that is based on
#'   interquartile range \emph{(iqr)}. Since \eqn{cqv} is unitless, it is useful
#'   for comparison of variables with different units. It is also a measure of
#'   homogeneity \code{[1, 2]}. } }
#' @return An object of type "list" which contains the estimate, the intervals,
#'   and the computation method. It has two components:
#' @return \describe{ \item{$method}{ A description of statistical method used
#'   for the computations. } \item{$statistics}{ A data frame representing three
#'   vectors: est, lower and upper limits of 95\% confidence interval
#'   \code{(CI)}: \cr \cr \strong{est:} \code{cqv*100}  \cr \cr \strong{Bonett
#'   95\% CI:} It uses a centering adjustment which helps to equalize the tail
#'   error probabilities \code{[1, 2]}.  \cr \cr \strong{Normal approximation
#'   95\% CI:} The intervals calculated by the normal approximation \code{[3,
#'   4]}, using \link[boot]{boot.ci}.  \cr \cr \strong{Basic bootstrap 95\%
#'   CI:} The intervals calculated by the basic bootstrap method \code{[3, 4]},
#'   using \link[boot]{boot.ci}.  \cr \cr \strong{Bootstrap percentile 95\%
#'   CI:} The intervals calculated by the bootstrap percentile method \code{[3,
#'   4]}, using \link[boot]{boot.ci}.  \cr \cr \strong{Adjusted bootstrap
#'   percentile (BCa) 95\% CI:} The intervals calculated by the adjusted
#'   bootstrap percentile (BCa) method \code{[3, 4]}, using
#'   \link[boot]{boot.ci}.  } }
#' @examples
#' x <- c(
#'     0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
#'     4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
#' )
#' cqv_versatile(x)
#' cqv_versatile(x, na.rm = TRUE, digits = 2)
#' cqv_versatile(x, na.rm = TRUE, digits = 2, method = "bonett")
#' @references \code{[1]} Bonett, DG., 2006, Confidence interval for a
#'   coefficient of quartile variation, Computational Statistics & Data
#'   Analysis, 50(11), 2953-7, DOI:
#'   \doi{10.1016/j.csda.2005.05.007}
#'
#' @references \code{[2]} Altunkaynak, B., Gamgam, H., 2018, Bootstrap
#'   confidence intervals for the coefficient of quartile variation, Simulation
#'   and Computation, 1-9, DOI:
#'   \doi{10.1080/03610918.2018.1435800}
#'
#' @references \code{[3]} Canty, A., & Ripley, B, 2017, boot: Bootstrap R
#'   (S-Plus) Functions. R package version 1.3-20.
#' @references \code{[4]} Davison, AC., & Hinkley, DV., 1997, Bootstrap Methods
#'   and Their Applications. Cambridge University Press, Cambridge. ISBN
#'   0-521-57391-2
#' @export
#' @import dplyr SciViews boot R6 utils
NULL
#' @importFrom stats quantile sd qchisq qnorm
#' @importFrom MBESS conf.limits.nct
NULL
cqv_versatile <- function(
  x,
  na.rm = FALSE,
  digits = 1,
  method = NULL,
  R = NULL,
  ...
) {
  # require(dplyr)
  # require(SciViews)
  # require(boot)
  if (missing(x) || is.null(x)) {
    stop("object 'x' not found")
  } else if (!missing(x)) {
    x <- x
  }
  if (!is.numeric(x)) {
    stop("argument is not a numeric vector")
  }
  na.rm <- na.rm # removes NAs if TRUE
  if (na.rm == TRUE) {
    x <- x[!is.na(x)]
  } else if (anyNA(x)) {
    stop(
      "missing values and NaN's not allowed if 'na.rm' is FALSE"
    )
  }
  # if (is.null(digits)) {
  #     digits = 1
  # }
  digits <- digits # digits required for rounding
  method <- method # returns 95% confidence interval
  if (is.null(R)) {
    R <- 1000
  }
  q3 <- unname(
    stats::quantile(
      x,
      probs = 0.75, # third quartile (0.75 percentile)
      na.rm = na.rm
    )
  )
  q1 <- unname(
    stats::quantile(
      x,
      probs = 0.25, # first quartile (0.25 percentile)
      na.rm = na.rm
    )
  )
  if (q3 == 0) { # to avoid NaNs when q3 and q1 are zero
    warning(
      "cqv is NaN because q3 and q1 are 0, max was used instead of q3"
    )
    q3 <- max(x, na.rm = na.rm)
  }
  a <- ceiling(
    (length(x) / 4) - (1.96 * (((3 * length(x)) / 16)^(0.5)))
  )
  b <- round(
    (length(x) / 4) + (1.96 * (((3 * length(x)) / 16)^(0.5))),
    digits = 0
  )
  c <- length(x) + 1 - b
  d <- length(x) + 1 - a
  Ya <- dplyr::nth(x, a, order_by = x)
  Yb <- dplyr::nth(x, b, order_by = x)
  Yc <- dplyr::nth(x, c, order_by = x)
  Yd <- dplyr::nth(x, d, order_by = x)
  star <- 0
  for (i in a:(b - 1)) {
    star[i] <- (
      (choose(length(x), i)) * (0.25^(i)) * (0.75^(length(x) - i))
    )
    alphastar <- 1 - sum(star[i], na.rm = na.rm)
  }
  zzz <- stats::qnorm((1 - ((1 - alphastar) / 2)))
  f1square <- (3 * (zzz)^2) / (4 * length(x) * ((Yb - Ya)^2))
  f3square <- (3 * (zzz)^2) / (4 * length(x) * ((Yd - Yc)^2))
  D <- q3 - q1
  S <- q3 + q1
  v <- (
    (1 / (16 * length(x))) * (
      (((3 / f1square) + (3 / f3square) - (2 / sqrt(f1square * f3square))) / D^2) +
        (((3 / f1square) + (3 / f3square) + (2 / sqrt(f1square * f3square))) / S^2) -
        ((2 * ((3 / f3square) - (3 / f1square))) / (D * S))
    )
  )
  ccc <- length(x) / (length(x) - 1)
  upper.tile <- exp(((SciViews::ln((D / S)) * ccc)) + (zzz * (v^(0.5))))
  lower.tile <- exp(((SciViews::ln((D / S)) * ccc)) - (zzz * (v^(0.5))))
  if (
    unname(stats::quantile(x, probs = 0.75, na.rm = na.rm)) != 0
  ) {
    boot.cqv <- boot::boot(
      x,
      function(x, i) {
        round(((
          unname(stats::quantile(x[i], probs = 0.75, na.rm = na.rm)) -
            unname(stats::quantile(x[i], probs = 0.25, na.rm = na.rm))
        ) / (
          unname(stats::quantile(x[i], probs = 0.75, na.rm = na.rm)) +
            unname(stats::quantile(x[i], probs = 0.25, na.rm = na.rm))
        )) * 100, digits = digits)
      },
      R = R
    )
  } else if (
    unname(stats::quantile(x, probs = 0.75, na.rm = na.rm)) == 0
  ) {
    boot.cqv <- boot::boot(
      x,
      function(x, i) {
        round(((
          max(x[i], na.rm = na.rm) -
            unname(stats::quantile(x[i], probs = 0.25, na.rm = na.rm))
        ) / (
          max(x[i], na.rm = na.rm) +
            unname(stats::quantile(x[i], probs = 0.25, na.rm = na.rm))
        )) * 100, digits = digits)
      },
      R = R
    )
  }

  if (is.null(method) || method == "bonett") {
    # no bootstrap CI needed for the analytic Bonett method
  } else if (method == "norm") {
    boot.norm.ci <- boot::boot.ci(boot.cqv, conf = 0.95, type = "norm")
  } else if (method == "basic") {
    boot.basic.ci <- boot::boot.ci(boot.cqv, conf = 0.95, type = "basic")
  } else if (method == "perc") {
    boot.perc.ci <- boot::boot.ci(boot.cqv, conf = 0.95, type = "perc")
  } else if (method == "bca") {
    boot.bca.ci <- boot::boot.ci(boot.cqv, conf = 0.95, type = "bca")
  } else if (method == "all") {
    boot.norm.ci <- boot::boot.ci(boot.cqv, conf = 0.95, type = "norm")
    boot.basic.ci <- boot::boot.ci(boot.cqv, conf = 0.95, type = "basic")
    boot.perc.ci <- boot::boot.ci(boot.cqv, conf = 0.95, type = "perc")
    boot.bca.ci <- boot::boot.ci(boot.cqv, conf = 0.95, type = "bca")
  }

  if (is.null(method)) {
    cqv <- round(
      100 * ((q3 - q1) / (q3 + q1)),
      digits = digits
    )
  } else if (method == "bonett") {
    cqv <- round(
      100 * ((q3 - q1) / (q3 + q1)),
      digits = digits
    )
  } else if (method == "norm") {
    cqv <- round(
      100 * ((q3 - q1) / (q3 + q1)),
      digits = digits
    )
  } else if (method == "basic") {
    cqv <- round(
      100 * ((q3 - q1) / (q3 + q1)),
      digits = digits
    )
  } else if (method == "perc") {
    cqv <- round(
      100 * ((q3 - q1) / (q3 + q1)),
      digits = digits
    )
  } else if (method == "bca") {
    cqv <- round(
      100 * ((q3 - q1) / (q3 + q1)),
      digits = digits
    )
  } else if (method == "all") {
    cqv <- round(
      100 * ((q3 - q1) / (q3 + q1)),
      digits = digits
    )
  }

  if (is.null(method)) {
    lower <- NA
    upper <- NA
  } else if (method == "bonett" && cqv != 100) {
    lower <- round(lower.tile * 100, digits = digits)
    upper <- round(upper.tile * 100, digits = digits)
  } else if (method == "norm" && cqv != 100) {
    lower <- round(boot.norm.ci$normal[2], digits = digits)
    upper <- round(boot.norm.ci$normal[3], digits = digits)
  } else if (method == "basic" && cqv != 100) {
    lower <- round(boot.basic.ci$basic[4], digits = digits)
    upper <- round(boot.basic.ci$basic[5], digits = digits)
  } else if (method == "perc" && cqv != 100) {
    lower <- round(boot.perc.ci$percent[4], digits = digits)
    upper <- round(boot.perc.ci$percent[5], digits = digits)
  } else if (method == "bca" && cqv != 100) {
    lower <- round(boot.bca.ci$bca[4], digits = digits)
    upper <- round(boot.bca.ci$bca[5], digits = digits)
  } else if (method == "bonett" && cqv == 100) {
    lower <- round(lower.tile * 100, digits = digits)
    upper <- round(upper.tile * 100, digits = digits)
  } else if (method == "norm" && cqv == 100) {
    lower <- round(lower.tile * 100, digits = digits)
    upper <- round(upper.tile * 100, digits = digits)
  } else if (method == "basic" && cqv == 100) {
    lower <- round(lower.tile * 100, digits = digits)
    upper <- round(upper.tile * 100, digits = digits)
  } else if (method == "perc" && cqv == 100) {
    lower <- round(lower.tile * 100, digits = digits)
    upper <- round(upper.tile * 100, digits = digits)
  } else if (method == "bca" && cqv == 100) {
    lower <- round(lower.tile * 100, digits = digits)
    upper <- round(upper.tile * 100, digits = digits)
  }
  if (is.null(method)) {
    return(
      list(
        method = "cqv = (q3-q1)/(q3+q1)",
        statistics = data.frame(
          est = cqv,
          row.names = c(" ")
        )
      )
    )
  } else if (method == "bonett" && cqv != 100) {
    return(
      list(
        method = "cqv with Bonett 95% CI",
        statistics = data.frame(
          est = cqv,
          lower = lower,
          upper = upper,
          row.names = c(" ")
        )
      )
    )
  } else if (method == "norm" && cqv != 100) {
    return(
      list(
        method = "cqv with normal approximation 95% CI",
        statistics = data.frame(
          est = cqv,
          lower = lower,
          upper = upper,
          row.names = c(" ")
        )
      )
    )
  } else if (method == "basic" && cqv != 100) {
    return(
      list(
        method = "cqv with basic bootstrap 95% CI",
        statistics = data.frame(
          est = cqv,
          lower = lower,
          upper = upper,
          row.names = c(" ")
        )
      )
    )
  } else if (method == "perc" && cqv != 100) {
    return(
      list(
        method = "cqv with bootstrap percentile 95% CI",
        statistics = data.frame(
          est = cqv,
          lower = lower,
          upper = upper,
          row.names = c(" ")
        )
      )
    )
  } else if (method == "bca" && cqv != 100) {
    return(
      list(
        method = "cqv with adjusted bootstrap percentile (BCa) 95% CI",
        statistics = data.frame(
          est = cqv,
          lower = lower,
          upper = upper,
          row.names = c(" ")
        )
      )
    )
  } else if (
    method %in% c("norm", "bonett", "basic", "perc", "bca", "all") && cqv == 100
  ) {
    warning(
      "All values of t are equal to  100 \n Cannot calculate confidence intervals \n"
    )
    return(
      list(
        method = "cqv with Bonett 95% CI",
        statistics = data.frame(
          est = cqv,
          lower = round(lower.tile * 100, digits = digits),
          upper = round(upper.tile * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if (method == "all" && cqv != 100) {
    return(
      list(
        method = "All methods",
        statistics = data.frame(
          row.names = c(
            "bonett",
            "norm",
            "basic",
            "percent",
            "bca"
          ),
          est = c(cqv, cqv, cqv, cqv, cqv),
          lower = c(
            round(lower.tile * 100, digits = digits),
            round(boot.norm.ci$normal[2], digits = digits),
            round(boot.basic.ci$basic[4], digits = digits),
            round(boot.perc.ci$percent[4], digits = digits),
            round(boot.bca.ci$bca[4], digits = digits)
          ),
          upper = c(
            round(upper.tile * 100, digits = digits),
            round(boot.norm.ci$normal[3], digits = digits),
            round(boot.basic.ci$basic[5], digits = digits),
            round(boot.perc.ci$percent[5], digits = digits),
            round(boot.bca.ci$bca[5], digits = digits)
          ),
          description = c(
            "cqv with Bonett 95% CI",
            "cqv with normal approximation 95% CI",
            "cqv with basic bootstrap 95% CI",
            "cqv with bootstrap percentile 95% CI",
            "cqv with adjusted bootstrap percentile (BCa) 95% CI"
          )
        )
      )
    )
  } else {
    stop("method for confidence interval is not available")
  }
}

#' @title R6 Coefficient of Quartile Variation (cqv)
#' @name CoefQuartVar
#' @description The R6 class \code{CoefQuartVar} for the coefficient of quartile
#'   variation (cqv)
#' @param x An \code{R} object. Currently there are methods for numeric vectors
#' @param na.rm a logical value indicating whether \code{NA} values should be
#'   stripped before the computation proceeds.
#' @param digits integer indicating the number of decimal places to be used.
#' @details \describe{ \item{\strong{Coefficient of Quartile Variation}}{
#'   \emph{cqv} is a measure of relative dispersion that is based on
#'   interquartile range \emph{(iqr)}. Since \eqn{cqv} is unitless, it is useful
#'   for comparison of variables with different units. It is also a measure of
#'   homogeneity \code{[1]}. } }
#' @examples
#' x <- c(
#'    0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
#'    4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
#' )
#' CoefQuartVar$new(x)$est()
#' cqv_x <- CoefQuartVar$new(x, digits = 2)
#' cqv_x$est()
#' R6::is.R6(cqv_x)
#' @references \code{[1]} Bonett, DG., 2006, Confidence interval for a
#'   coefficient of quartile variation, Computational Statistics & Data
#'   Analysis, 50(11), 2953-7, DOI:
#'   \doi{10.1016/j.csda.2005.05.007}
#'
#' @export
#' @import dplyr SciViews boot R6 utils
NULL
#' @importFrom stats quantile sd qchisq qnorm
#' @importFrom MBESS conf.limits.nct
NULL
CoefQuartVar <- R6::R6Class(
  classname = "CoefQuartVar",
  inherit = BootCoefQuartVar,
  public = list(
    # ---------------- determining defaults for arguments -----------------
    x = NA,
    na.rm = FALSE,
    digits = 1,
    star = 0,
    # --------- determining constructor defaults for arguments ------------
    initialize = function(x = NA,
                          na.rm = FALSE,
                          digits = 1,
                          ...) {
      # ---------------------- check NA or NAN -------------------------
      if (missing(x) || is.null(x)) {
        stop("object 'x' not found")
      } else if (!missing(x)) {
        self$x <- x
      }
      if (!missing(na.rm)) {
        self$na.rm <- na.rm
      }
      if (self$na.rm == TRUE) {
        self$x <- x[!is.na(x)]
      } else if (anyNA(x) & self$na.rm == FALSE) {
        stop(
          "missing values and NaN's not allowed if 'na.rm' is FALSE"
        )
      }
      # ------------- stop if input x vector is not numeric -------------
      if (!is.numeric(x)) {
        stop("argument is not a numeric vector")
      }
      # ------------------- set digits with user input ------------------
      if (!missing(digits)) {
        self$digits <- digits
      }
      # ---------- initialize cqv estimate i.e., est() function ---------
      self$est()
    },
    # --------- public method of cqv estimate i.e., est() function --------
    est = function(...) {
      if ( # check if 0.75 percentile is non-zero to avoid NANs
        super$super_$initialize(
          x = self$x, na.rm = self$na.rm, probs = 0.75
        ) != 0
      ) {
        return(
          round(
            (((super$super_$initialize(
              x = self$x,
              na.rm = self$na.rm,
              probs = 0.75,
              names = FALSE,
              digits = self$digits
            )) - (super$super_$initialize(
              x = self$x,
              na.rm = self$na.rm,
              probs = 0.25,
              names = FALSE,
              digits = self$digits
            ))) / ((super$super_$initialize(
              x = self$x,
              na.rm = self$na.rm,
              probs = 0.75,
              names = FALSE,
              digits = self$digits
            )) + (super$super_$initialize(
              x = self$x,
              na.rm = self$na.rm,
              probs = 0.25,
              names = FALSE,
              digits = self$digits
            )))) * 100,
            digits = self$digits
          )
        )
      } else if (
        super$super_$initialize(
          x = self$x, na.rm = self$na.rm, probs = 0.75
        ) == 0
      ) {
        warning(
          "cqv is NaN because both q3 and q1 are 0, max was used instead of q3"
        )
        return(
          round(
            ((max(x = self$x) - (super$super_$initialize(
              x = self$x,
              na.rm = self$na.rm,
              probs = 0.25,
              names = FALSE
            ))) / (max(x = self$x) + (super$super_$initialize(
              x = self$x,
              na.rm = self$na.rm,
              probs = 0.25,
              names = FALSE
            )))) * 100,
            digits = self$digits
          )
        )
      }
    },
    # ------------------- adding some internal methods --------------------
    # --------------------- adding some internal fields -------------------
    a = function(...) {
      ceiling(
        (length(self$x) / 4) - (1.96 * (((3 * length(self$x)) / 16)^(0.5)))
      )
    },
    b = function(...) {
      round(
        (length(self$x) / 4) + (1.96 * (((3 * length(self$x)) / 16)^(0.5))),
        digits = 0
      )
    },
    c = function(...) {
      length(self$x) + 1 - self$b()
    },
    d = function(...) {
      length(self$x) + 1 - self$a()
    },
    Ya = function(...) {
      dplyr::nth(self$x, self$a(), order_by = self$x)
    },
    Yb = function(...) {
      dplyr::nth(self$x, self$b(), order_by = self$x)
    },
    Yc = function(...) {
      dplyr::nth(self$x, self$c(), order_by = self$x)
    },
    Yd = function(...) {
      dplyr::nth(self$x, self$d(), order_by = self$x)
    },
    alphastar = function(...) {
      for (i in self$a():(self$b() - 1)) {
        self$star[i] <- (
          (choose(length(self$x), i)) *
            (0.25^(i)) * (0.75^(length(self$x) - i))
        )
        return(alphastar = 1 - sum(self$star[i], na.rm = self$na.rm))
      }
    }
  ),
  # ---- define super_ function to enable multiple levels of inheritance ----
  active = list(
    super_ = function() super
  )
)

#' @title R6 Bootstrap Resampling for Coefficient of Variation
#' @name BootCoefVar
#' @description The R6 class \code{BootCoefVar} produces the bootstrap
#'   resampling for the coefficient of variation (cv) of the given numeric
#'   vectors. It uses \link[boot]{boot} and \link[boot]{boot.ci} from the
#'   package \pkg{boot}.
#' @param x An \code{R} object. Currently there are methods for numeric vectors
#' @param na.rm a logical value indicating whether \code{NA} values should be
#'   stripped before the computation proceeds.
#' @param alpha The allowed type I error probability
#' @param R integer indicating the number of bootstrap replicates.
#' @examples
#' x <- c(
#'     0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
#'     4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
#' )
#' cv_x <- BootCoefVar$new(x)
#' cv_x$boot_cv()
#' cv_x$boot_cv_corr()
#' cv_x$boot_basic_ci_cv()
#' cv_x$boot_norm_ci_cv()
#' cv_x$boot_perc_ci_cv()
#' cv_x$boot_bca_ci_cv()
#' cv_x$boot_basic_ci_cv_corr()
#' cv_x$boot_norm_ci_cv_corr()
#' cv_x$boot_perc_ci_cv_corr()
#' cv_x$boot_bca_ci_cv_corr()
#' R6::is.R6(cv_x)
#' @export
#' @references Canty, A., & Ripley, B, 2017, boot: Bootstrap R (S-Plus)
#'   Functions. R package version 1.3-20.
#' @references Davison, AC., & Hinkley, DV., 1997, Bootstrap Methods and Their
#'   Applications. Cambridge University Press, Cambridge. ISBN 0-521-57391-2
#' @import dplyr SciViews boot R6 utils
NULL
#' @importFrom stats quantile sd qchisq qnorm
#' @importFrom MBESS conf.limits.nct
NULL
BootCoefVar <- R6::R6Class(
  classname = "BootCoefVar",
  public = list(
    # ---------------- determining defaults for arguments -----------------
    x = NA,
    na.rm = FALSE,
    R = 1000,
    alpha = 0.05,
    boot_cv = NA,
    boot_cv_corr = NA,
    # --------- determining constructor defaults for arguments ------------
    initialize = function(x = NA,
                          na.rm = FALSE,
                          R = 1000,
                          alpha = 0.05,
                          ...) {
      # ---------------------- check NA or NAN -------------------------
      if (missing(x) || is.null(x)) {
        stop("object 'x' not found")
      } else if (!missing(x)) {
        self$x <- x
      }
      if (!missing(na.rm)) {
        self$na.rm <- na.rm
      }
      if (na.rm == TRUE) {
        self$x <- x[!is.na(x)]
      } else if (anyNA(x) & self$na.rm == FALSE) {
        stop(
          "missing values and NaN's not allowed if 'na.rm' is FALSE"
        )
      }
      # ------------- stop if input x vector is not numeric -------------
      if (!is.numeric(x)) {
        stop("argument is not a numeric vector")
      }
      # ---- set the number of bootstrap replicates with user input -----
      if (!missing(R)) {
        self$R <- R
      }
      # ------ set the probability of type I error with user input ------
      if (!missing(alpha)) {
        self$alpha <- alpha
      }
      # ------ initialize boot_cv() i.e., bootstrap of cv function ------
      self$boot_cv <- function(...) {
        return(
          boot::boot(self$x, function(x, i) {
            100 * (
              (sd(self$x[i], na.rm = self$na.rm)) /
                (mean(self$x[i], na.rm = self$na.rm))
            )
          },
          R = self$R
          )
        )
      }
      # - initialize boot_cv_corr() i.e., bootstrap of cv_corr function -
      self$boot_cv_corr <- function(...) {
        return(
          boot::boot(self$x, function(x, i) {
            100 * (sd(
              self$x[i],
              na.rm = self$na.rm
            ) / mean(
              self$x[i],
              na.rm = self$na.rm
            ) * ((1 - (1 / (4 * (length(self$x[i]) - 1))) +
              (1 / length(self$x)) * (
                sd(
                  self$x[i],
                  na.rm = self$na.rm
                ) / mean(
                  self$x[i],
                  na.rm = self$na.rm
                ))^2) + (1 / (2 * (
              length(self$x) - 1)^2))))
          },
          R = R
          )
        )
      }
    },
    # ------- public methods of cv bootstrap confidence intervals ---------
    boot_norm_ci_cv = function(...) {
      return(
        boot::boot.ci(
          self$boot_cv(),
          conf = (1 - self$alpha),
          type = "norm"
        )
      )
    },
    boot_basic_ci_cv = function(...) {
      return(
        boot::boot.ci(
          self$boot_cv(),
          conf = (1 - self$alpha),
          type = "basic"
        )
      )
    },
    boot_perc_ci_cv = function(...) {
      return(
        boot::boot.ci(
          self$boot_cv(),
          conf = (1 - self$alpha),
          type = "perc"
        )
      )
    },
    boot_bca_ci_cv = function(...) {
      return(
        boot::boot.ci(
          self$boot_cv(),
          conf = (1 - self$alpha),
          type = "bca"
        )
      )
    },
    # ---- public methods of cv_corr bootstrap confidence intervals -------
    boot_norm_ci_cv_corr = function(...) {
      return(
        boot::boot.ci(
          self$boot_cv_corr(),
          conf = (1 - self$alpha),
          type = "norm"
        )
      )
    },
    boot_basic_ci_cv_corr = function(...) {
      return(
        boot::boot.ci(
          self$boot_cv_corr(),
          conf = (1 - self$alpha),
          type = "basic"
        )
      )
    },
    boot_perc_ci_cv_corr = function(...) {
      return(
        boot::boot.ci(
          self$boot_cv_corr(),
          conf = (1 - self$alpha),
          type = "perc"
        )
      )
    },
    boot_bca_ci_cv_corr = function(...) {
      return(
        boot::boot.ci(
          self$boot_cv_corr(),
          conf = (1 - self$alpha),
          type = "bca"
        )
      )
    }
  ),
  # ---- define super_ function to enable multiple levels of inheritance ----
  active = list(
    super_ = function() {
      super
    }
  )
)

context("state_alpha")
test_that(
  desc = "understands the default value of alpha = 0.05",
  {
    x <- c(
      0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
      4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
    )
    expect_equal(
      (1 - BootCoefQuartVar$new(x)$boot_basic_ci()$basic[1]), 0.05
    )
    expect_equal(
      (1 - BootCoefVar$new(x)$boot_basic_ci_cv()$basic[1]), 0.05
    )
    expect_equal(
      (1 - CoefQuartVarCI$new(x)$boot_basic_ci()$basic[1]), 0.05
    )
    expect_equal(
      (1 - CoefVarCI$new(x)$boot_basic_ci_cv()$basic[1]), 0.05
    )
    expect_equal(BootCoefQuartVar$new(x)$alpha, 0.05)
    expect_equal(BootCoefVar$new(x)$alpha, 0.05)
    expect_equal(CoefQuartVarCI$new(x)$alpha, 0.05)
    expect_equal(CoefVarCI$new(x)$alpha, 0.05)
    expect_equal(formals(cv_versatile)$alpha, 0.05)
  }
)
test_that(
  desc = "understands the user input value of alpha",
  {
    x <- c(
      0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
      4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
    )
    expect_equal(
      (1 - BootCoefQuartVar$new(
        x,
        alpha = 0.025
      )$boot_basic_ci()$basic[1]),
      0.025
    )
    expect_equal(
      (1 - BootCoefVar$new(
        x,
        alpha = 0.025
      )$boot_basic_ci_cv()$basic[1]),
      0.025
    )
    expect_equal(
      (1 - CoefQuartVarCI$new(
        x,
        alpha = 0.025
      )$boot_basic_ci()$basic[1]),
      0.025
    )
    expect_equal(
      (1 - CoefVarCI$new(
        x,
        alpha = 0.025
      )$boot_basic_ci_cv()$basic[1]),
      0.025
    )
    expect_equal(BootCoefQuartVar$new(x, alpha = 0.025)$alpha, 0.025)
    expect_equal(BootCoefVar$new(x, alpha = 0.025)$alpha, 0.025)
    expect_equal(CoefQuartVarCI$new(x, alpha = 0.025)$alpha, 0.025)
    expect_equal(CoefVarCI$new(x, alpha = 0.025)$alpha, 0.025)
  }
)

context("state_BootCoefVar")
test_that(
  desc = "finds the correct bootstraps",
  {
    x <- c(
      0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
      4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
    )
    expect_equal(
      BootCoefVar$new(x)$boot_cv()$statistic(), 57.77352,
      tolerance = 0.00001
    )
    expect_equal(
      BootCoefVar$new(x)$boot_cv_corr()$statistic(), 58.05753,
      tolerance = 0.00001
    )
    expect_true(
      abs(
        BootCoefVar$new(x)$boot_norm_ci_cv()$normal[2] -
          39.24367
      ) / 100 < 0.05
    )
    expect_true(
      abs(
        BootCoefVar$new(x)$boot_norm_ci_cv()$normal[3] -
          78.14946
      ) / 100 < 0.05
    )
    expect_true(
      abs(
        BootCoefVar$new(x)$boot_basic_ci_cv()$basic[4] -
          38.07423
      ) / 100 < 0.05
    )
    expect_true(
      abs(
        BootCoefVar$new(x)$boot_basic_ci_cv()$basic[5] -
          76.64171
      ) / 100 < 0.05
    )
    expect_true(
      abs(
        BootCoefVar$new(x)$boot_perc_ci_cv()$percent[4] -
          37.20539
      ) / 100 < 0.05
    )
    expect_true(
      abs(
        BootCoefVar$new(x)$boot_perc_ci_cv()$percent[5] -
          76.55577
      ) / 100 < 0.05
    )
    expect_true(
      abs(
        BootCoefVar$new(x)$boot_bca_ci_cv()$bca[4] -
          41.72823
      ) / 100 < 0.1
    )
    expect_true(
      abs(
        BootCoefVar$new(x)$boot_bca_ci_cv()$bca[5] -
          82.96899
      ) / 100 < 0.1
    )
    expect_true(
      abs(
        BootCoefVar$new(x)$boot_norm_ci_cv_corr()$normal[2] -
          38.67237
      ) / 100 < 0.05
    )
    expect_true(
      abs(
        BootCoefVar$new(x)$boot_norm_ci_cv_corr()$normal[3] -
          80.07603
      ) / 100 < 0.3
    )
    expect_true(
      abs(
        BootCoefVar$new(x)$boot_basic_ci_cv_corr()$basic[4] -
          34.87221
      ) / 100 < 0.3
    )
    expect_true(
      abs(
        BootCoefVar$new(x)$boot_basic_ci_cv_corr()$basic[5] -
          76.89616
      ) / 100 < 0.05
    )
    expect_true(
      abs(
        BootCoefVar$new(x)$boot_perc_ci_cv_corr()$percent[4] -
          38.0117
      ) / 100 < 0.05
    )
    expect_true(
      abs(
        BootCoefVar$new(x)$boot_perc_ci_cv_corr()$percent[5] -
          79.59805
      ) / 100 < 0.05
    )
    expect_true(
      abs(
        BootCoefVar$new(x)$boot_bca_ci_cv_corr()$bca[4] -
          40.62717
      ) / 100 < 0.3
    )
    expect_true(
      abs(
        BootCoefVar$new(x)$boot_bca_ci_cv_corr()$bca[5] -
          84.75401
      ) / 100 < 0.3
    )
  }
)
test_that(
  desc = "detect R6 class",
  {
    x <- c(
      0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
      4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
    )
    cv_x <- BootCoefVar$new(x)
    expect_true(
      R6::is.R6(cv_x)
    )
  }
)

#' @title R6 Bootstrap Resampling for Coefficient of Quartile Variation
#' @name BootCoefQuartVar
#' @description The R6 class \code{BootCoefQuartVar} produces the bootstrap
#'   resampling for the coefficient of quartile variation (cqv) of the given
#'   numeric vectors. It uses \link[boot]{boot} from the package \pkg{boot}.
#'   Also, it produces the bootstrap confidence intervals for the cqv based on
#'   the \link[boot]{boot.ci} from the package \pkg{boot}.
#' @param x An \code{R} object. Currently there are methods for numeric vectors
#' @param na.rm a logical value indicating whether \code{NA} values should be
#'   stripped before the computation proceeds.
#' @param alpha The allowed type I error probability
#' @param R integer indicating the number of bootstrap replicates.
#' @examples
#' x <- c(
#'     0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
#'     4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
#' )
#' cqv_x <- BootCoefQuartVar$new(x)
#' cqv_x$boot_cqv()
#' cqv_x$boot_basic_ci()
#' cqv_x$boot_norm_ci()
#' cqv_x$boot_perc_ci()
#' cqv_x$boot_bca_ci()
#' R6::is.R6(cqv_x)
#' @export
#' @references Canty, A., & Ripley, B, 2017, boot: Bootstrap R (S-Plus)
#'   Functions. R package version 1.3-20.
#' @references Davison, AC., & Hinkley, DV., 1997, Bootstrap Methods and Their
#'   Applications. Cambridge University Press, Cambridge. ISBN 0-521-57391-2
#' @references Altunkaynak, B., Gamgam, H., 2018, Bootstrap confidence intervals
#'   for the coefficient of quartile variation, Simulation and Computation, 1-9,
#'   DOI:
#'   \doi{10.1080/03610918.2018.1435800}
#'
#' @import dplyr SciViews boot R6 utils
NULL
#' @importFrom stats quantile sd qchisq qnorm
#' @importFrom MBESS conf.limits.nct
NULL
BootCoefQuartVar <- R6::R6Class(
  classname = "BootCoefQuartVar",
  inherit = SampleQuantiles,
  public = list(
    # ---------------- determining defaults for arguments -----------------
    x = NA,
    na.rm = FALSE,
    R = 1000,
    alpha = 0.05,
    boot_cqv = NA,
    # --------- determining constructor defaults for arguments ------------
    initialize = function(x = NA,
                          na.rm = FALSE,
                          R = 1000,
                          alpha = 0.05,
                          ...) {
      # ---------------------- check NA or NAN -------------------------
      if (missing(x) || is.null(x)) {
        stop("object 'x' not found")
      } else if (!missing(x)) {
        self$x <- x
      }
      if (!missing(na.rm)) {
        self$na.rm <- na.rm
      }
      if (na.rm == TRUE) {
        self$x <- x[!is.na(x)]
      } else if (anyNA(x) & self$na.rm == FALSE) {
        stop(
          "missing values and NaN's not allowed if 'na.rm' is FALSE"
        )
      }
      # ------------- stop if input x vector is not numeric -------------
      if (!is.numeric(x)) {
        stop("argument is not a numeric vector")
      }
      # ---- set the number of bootstrap replicates with user input -----
      if (!missing(R)) {
        self$R <- R
      }
      # ------ set the probability of type I error with user input ------
      if (!missing(alpha)) {
        self$alpha <- alpha
      }
      # ----- initialize boot_cqv() i.e., bootstrap of cqv function -----
      self$boot_cqv <- function(...) {
        if ( # check if 0.75 percentile is non-zero to avoid NANs
          super$initialize(x = self$x, na.rm = TRUE, probs = 0.75) != 0
        ) {
          return(
            boot::boot(self$x, function(x, i) {
              (((
                unname(
                  quantile(
                    self$x[i],
                    probs = 0.75,
                    na.rm = self$na.rm
                  )
                ) - unname(
                  quantile(
                    self$x[i],
                    probs = 0.25,
                    na.rm = self$na.rm
                  )
                )
              ) / (unname(
                quantile(
                  self$x[i],
                  probs = 0.75,
                  na.rm = self$na.rm
                )
              ) + unname(
                quantile(
                  self$x[i],
                  probs = 0.25,
                  na.rm = self$na.rm
                )
              )
              )) * 100)
            },
            R = self$R
            )
          )
        } else if ( # check if 0.75 percentile is zero to avoid NANs
          super$initialize(x = self$x, na.rm = TRUE, probs = 0.75) == 0
        ) {
          warning(
            "cqv is NaN because both q3 and q1 are 0, max was used instead of q3"
          )
          return(
            boot::boot(self$x, function(x, i) {
              (((
                unname(
                  max(
                    self$x[i],
                    na.rm = self$na.rm
                  )
                ) - unname(
                  quantile(
                    self$x[i],
                    probs = 0.25,
                    na.rm = self$na.rm
                  )
                )
              ) / (unname(
                max(
                  self$x[i],
                  na.rm = self$na.rm
                )
              ) + unname(
                quantile(
                  self$x[i],
                  probs = 0.25,
                  na.rm = self$na.rm
                )
              )
              )) * 100)
            },
            R = self$R
            )
          )
        }
      }
    },
    # --------- public methods of bootstrap confidence intervals ----------
    boot_norm_ci = function(...) {
      return(
        boot::boot.ci(
          self$boot_cqv(),
          conf = (1 - self$alpha),
          type = "norm"
        )
      )
    },
    boot_basic_ci = function(...) {
      return(
        boot::boot.ci(
          self$boot_cqv(),
          conf = (1 - self$alpha),
          type = "basic"
        )
      )
    },
    boot_perc_ci = function(...) {
      return(
        boot::boot.ci(
          self$boot_cqv(),
          conf = (1 - self$alpha),
          type = "perc"
        )
      )
    },
    boot_bca_ci = function(...) {
      return(
        boot::boot.ci(
          self$boot_cqv(),
          conf = (1 - self$alpha),
          type = "bca"
        )
      )
    }
  ),
  # ---- define super_ function to enable multiple levels of inheritance ----
  active = list(
    super_ = function() {
      super
    }
  )
)

#' @title R6 Sample Quantiles
#' @name SampleQuantiles
#' @description The R6 class \code{SampleQuantiles} produces the sample
#'   quantiles corresponding to the given probabilities. It uses
#'   \link[stats]{quantile} from the package \pkg{stats}.
#' @param x An \code{R} object. Currently there are methods for numeric vectors
#' @param na.rm a logical value indicating whether \code{NA} values should be
#'   stripped before the computation proceeds.
#' @param digits integer indicating the number of decimal places to be used.
#' @param probs numeric vector of probabilities with values in \code{[0,1]}.
#' @param names logical; if TRUE, the result has a \link[base]{names} attribute
#'   regarding the percentiles.
#' @param type an integer between 1 and 9 selecting one of the nine quantile
#'   algorithms explained in \link[stats]{quantile} to be used.
#' @examples
#' x <- c(
#'    0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
#'    4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
#' )
#' SampleQuantiles$new(x)$qx()
#' percentile_95 <-  SampleQuantiles$new(x, na.rm = TRUE, digits = 2, probs = 0.95)
#' percentile_95$qx()
#' percentile_75 <-  SampleQuantiles$new(x, na.rm = TRUE, digits = 3, probs = 0.75)
#' percentile_75$qx()
#' R6::is.R6(percentile_95)
#' @export
#' @import dplyr SciViews boot R6 utils
NULL
#' @importFrom stats quantile sd qchisq qnorm
#' @importFrom MBESS conf.limits.nct
NULL
SampleQuantiles <- R6::R6Class(
  classname = "SampleQuantiles",
  public = list(
    # ---------------- determining defaults for arguments -----------------
    x = NA,
    na.rm = FALSE,
    digits = 1,
    probs = 0.5,
    names = TRUE,
    type = 7,
    eps = NA,
    epsp1 = NA,
    epsm1 = NA,
    # --------- determining constructor defaults for arguments ------------
    initialize = function(x = NA,
                          na.rm = FALSE,
                          digits = 1,
                          probs = 0.5,
                          names = TRUE,
                          type = 7,
                          ...) {
      # ----------------------- check NA or NAN -------------------------
      if (missing(x) || is.null(x)) {
        stop("object 'x' not found")
      } else if (!missing(x)) {
        self$x <- x
      }
      if (!missing(na.rm)) {
        self$na.rm <- na.rm
      }
      if (na.rm == TRUE) {
        self$x <- x[!is.na(x)]
      } else if (anyNA(x) & self$na.rm == FALSE) {
        stop(
          "missing values and NaN's not allowed if 'na.rm' is FALSE"
        )
      }
      # ------------- stop if input x vector is not numeric -------------
      if (!is.numeric(x)) {
        stop("argument is not a numeric vector")
      }
      # -------------- check for probs being in range [0,1] -------------
      self$eps <- function(...) {
        100 * .Machine$double.eps
      }
      self$epsp1 <- function(...) {
        1 + 100 * .Machine$double.eps
      }
      self$epsm1 <- function(...) {
        -1 * (100 * .Machine$double.eps)
      }
      if (any(
        !missing(probs) && (
          probs < self$epsm1() | probs > self$epsp1())
      )
      ) {
        stop("probs outside [0,1]")
      } else if ((
        !missing(probs) && (probs <= self$epsp1() & probs >= 1)
      )) {
        self$probs <- 1
      } else if ((
        !missing(probs) && (probs >= self$epsm1() & probs <= 0)
      )) {
        self$probs <- 0
      } else if (
        (
          !missing(probs) && (probs >= 0 & probs <= 1)
        )
      ) {
        self$probs <- probs
      }
      # ------------------- set digits with user input ------------------
      if (!missing(digits)) {
        self$digits <- digits
      }
      # ------------------- set type with user input --------------------
      if (!missing(type)) {
        self$type <- type
      }
      # ------------------ set names with user input --------------------
      if (!missing(names)) {
        self$names <- names
      }
      self$qx() # initialize qx() i.e., sample quantile function
    },
    # --------- public function  qx() i.e., sample quantile function ------
    qx = function(...) {
      return(
        round(
          stats::quantile(
            self$x,
            probs = self$probs,
            na.rm = self$na.rm,
            type = self$type,
            names = self$names
          ),
          digits = self$digits
        )
      )
    }
  )
)

context("state_SampleQuantiles")
test_that(
  desc = "finds the correct quantiles",
  {
    x <- c(
      0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
      4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
    )
    expect_equal(
      SampleQuantiles$new(x, probs = 0.61, names = FALSE)$qx(), 5.4
    )
    expect_equal(
      SampleQuantiles$new(x, probs = 0.61, names = FALSE)$eps(),
      2.220446e-14,
      tolerance = 0.001
    )
    expect_equal(
      SampleQuantiles$new(x, names = FALSE)$qx(), 4.5
    )
    expect_equal(
      SampleQuantiles$new(
        x,
        na.rm = TRUE, digits = 2, probs = 0.95, names = FALSE
      )$qx(),
      7.14
    )
    expect_equal(
      SampleQuantiles$new(
        x,
        na.rm = TRUE, digits = 2, probs = 0.75, names = FALSE
      )$qx(),
      5.82
    )
  }
)
test_that(
  desc = "detect R6 class",
  {
    sample_quantile_example <- SampleQuantiles$new(
      x,
      na.rm = TRUE, digits = 2, probs = 0.75, names = FALSE
    )
    expect_true(
      R6::is.R6(sample_quantile_example)
    )
  }
)

context("state_R")
test_that(
  desc = "understands the default value of R = 1000",
  {
    x <- c(
      0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
      4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
    )
    expect_equal(
      (BootCoefQuartVar$new(x)$boot_cqv()$R), 1000
    )
    expect_equal(
      (BootCoefVar$new(x)$boot_cv()$R), 1000
    )
    expect_equal(
      (CoefQuartVarCI$new(x)$boot_cqv()$R), 1000
    )
    expect_equal(
      (CoefVarCI$new(x)$boot_cv()$R), 1000
    )
    expect_equal(BootCoefQuartVar$new(x)$R, 1000)
    expect_equal(BootCoefVar$new(x)$R, 1000)
    expect_equal(CoefQuartVarCI$new(x)$R, 1000)
    expect_equal(CoefVarCI$new(x)$R, 1000)
    expect_null(formals(cv_versatile)$R)
    expect_null(formals(cqv_versatile)$R)
  }
)
test_that(
  desc = "understands the user input value of R",
  {
    x <- c(
      0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
      4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
    )
    expect_equal(
      (BootCoefQuartVar$new(x, R = 100)$boot_cqv()$R), 100
    )
    expect_equal(
      (BootCoefVar$new(x, R = 100)$boot_cv()$R), 100
    )
    expect_equal(
      (CoefQuartVarCI$new(x, R = 100)$boot_cqv()$R), 100
    )
    expect_equal(
      (CoefVarCI$new(x, R = 100)$boot_cv()$R), 100
    )
    expect_equal(BootCoefQuartVar$new(x, R = 100)$R, 100)
    expect_equal(BootCoefVar$new(x, R = 100)$R, 100)
    expect_equal(CoefQuartVarCI$new(x, R = 100)$R, 100)
    expect_equal(CoefVarCI$new(x, R = 100)$R, 100)
  }
)

context("state_probs")
test_that(
  desc = "understands the default values",
  {
    x <- 1:100
    expect_equal(
      SampleQuantiles$new(x)$probs, 0.5
    )
  }
)
test_that(
  desc = "understands the user input of probs",
  {
    x <- 1:100
    expect_equal(
      SampleQuantiles$new(x, probs = 0.61)$probs, 0.61
    )
  }
)
test_that(
  desc = "understands the user input below 0",
  {
    x <- 1:100
    expect_error(
      SampleQuantiles$new(x, probs = -0.001)$qx(),
      "probs outside \\[0\\,1\\]"
    )
  }
)
test_that(
  desc = "understands the user input above 1",
  {
    x <- 1:100
    expect_error(
      SampleQuantiles$new(x, probs = 1.001)$qx(),
      "probs outside \\[0\\,1\\]"
    )
  }
)
test_that(
  desc = "understands the tolerance of below 0 and -2e-14",
  {
    x <- 1:100
    expect_equal(
      SampleQuantiles$new(
        x,
        probs = -2.220446e-14
      )$probs,
      0
    )
  }
)
test_that(
  desc = "understands the tolerance of above 1 and 1+2e-14",
  {
    x <- 1:100
    expect_equal(
      SampleQuantiles$new(
        x,
        probs = (1 + 2.220446e-14)
      )$probs,
      1
    )
  }
)

context("state_names")
test_that(
  desc = "understands the default value of names = TRUE",
  {
    x <- 1:100
    expect_equal(
      attr(SampleQuantiles$new(x)$qx(), "names"), "50%"
    )
  }
)
test_that(
  desc = "understands the user input value of names = TRUE",
  {
    x <- 1:100
    expect_equal(
      attr(
        SampleQuantiles$new(x, probs = 0.70, names = TRUE)$qx(),
        "names"
      ),
      "70%"
    )
  }
)
test_that(
  desc = "understands the user input value of names = FALSE",
  {
    x <- 1:100
    expect_null(
      attr(
        SampleQuantiles$new(x, probs = 0.70, names = FALSE)$qx(),
        "names"
      )
    )
  }
)

context("state_type")
test_that(
  desc = "understands the default value of type = 7",
  {
    x <- 1:100
    expect_equal(
      SampleQuantiles$new(x)$type, 7
    )
  }
)
test_that(
  desc = "understands the user input value of type",
  {
    x <- 1:100
    expect_equal(
      SampleQuantiles$new(x, type = 8)$type, 8
    )
  }
)

---
title: "Coefficient of Variation: cv_versatile"
author: "Maani Beigy"
date: "February 18, 2019"
output: 
    rmarkdown::html_vignette
bibliography: cvcqv.bib
csl: apa.csl
vignette: >
  %\VignetteIndexEntry{cv_versatile}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

# Coefficient of Variation   
Coefficient of variation *($CV$)* is a measure of relative dispersion representing 
the degree of variability relative to the mean [@Albatineh2014]. 
Since cv is unitless, it is useful for comparison of variables with different 
units [@Albatineh2014]. It is also a measure of homogeneity. The *population* coefficient of variation is:    
$$CV = \frac{\sigma}{\mu},$$
where $\sigma$ is the population standard deviation and $\mu$ is the population mean. Almost always, we analyze data from samples but want to generalize it 
as the population's parameter [@Albatineh2014]. Its sample's estimate is given 
as:    
$$cv = \frac{sd}{\bar{X}}$$   
where $sd$ is the sample standard deviation, the square root of the unbiased estimator of population variance, and $\bar{X}$ is the sample mean. The 
corrected *cv* to account for the sample size is:
$$
cv_{corr} = cv * \biggl(1 - \frac{1}{4(n-1)} 
+ \frac{1}{n}cv^2 
+ \frac{1}{2 (n-1)^2} \biggr)
$$
There are various methods for the calculation of **confidence intervals (CI)** 
for *cv*. All of them are fruitful and have particular use cases. Some of them 
are model-based hence their usage depends the assumptions regarding the distribution of data. For sake of versatility, we cover almost all of these methods in `cvcqv` package. Here, we explain them along with some examples:

### Kelley Confidence Interval        

Let us assume that *CV* follows a noncentral *t* distribution, when the parent  population of the scores is *normally-distributed*, with noncentrality 
($\lambda$) parameter:   
$$
\lambda = \frac{\sqrt{n}}{cv}
$$
with *v* degrees of freedom, where $v = n - 1$. 
Let $1 - \alpha$ be the CI coverage with $\alpha_L + \alpha_U = \alpha$ in which $\alpha_L$ is the the proportion of times that *cv* will be 
less than the lower confidence bound and $\alpha_U$ the proportion of times that
*cv* will be greater than the upper confidence bound in the CI procedure [@Kelley2007]. The lower confidence tile for $\lambda$ is is the noncentrality parameter that results in $t_{(1-\alpha_L,v,\lambda_L)}=\hat{\lambda}$ and the
upper confidence tile for $\lambda$ is is the noncentrality parameter that 
results in $t_{(\alpha_U,v,\lambda_U)}=\hat{\lambda}$, where $t_{(1-\alpha_L,v,\lambda_L)}=\hat{\lambda}$ is the value of noncentral *t* distribution at the $1-\alpha_L$ **quantile** with noncentrality parameter $\lambda_L$ and $t_{(\alpha_U,v,\lambda_U)}=\hat{\lambda}$ is the value of noncentral *t* distribution at the $\alpha_U$ **quantile** with noncentrality parameter $\lambda_U$, respectively [@Kelley2007].   
Afterwards, we transform the tiles of the confidence interval for $\lambda$, by
dividing the tiles by $\sqrt{n}$ and thereafter inverting them; the CI limits of 
$cv$ will be obtained:  
$$
p\left[\biggl(\frac{\lambda_U}{\sqrt{n}}\biggr)^{-1}
\le CV \le \biggl(\frac{\lambda_L}{\sqrt{n}}\biggr)^{-1}\right] = 1-\alpha
$$
where $p$ stands for *probability*. Thanks to package `MBESS` [@Kelley2018] for 
the computation of confidence limits for the noncentrality parameter from a *t* distribution (`conf.limits.nct`), $cv$ will be obtained as: 
```{r echo=FALSE, warning=FALSE, message=FALSE, include = FALSE}
library(dplyr)
library(boot)
x <- c(
    0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
    4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
)
cv_versatile <- function(
    x,  # Currently there are methods for numeric vectors
    na.rm = FALSE,  # indicating whether NA values should be stripped
    digits = 1,  # digits of output after rounding. default is 4
    method = NULL,  # method for the computation of confidence interval (CI)
    correction = FALSE,  # indicating whether to compute the unbiased statistics
    alpha = 0.05,  # The allowed type I error probability
    R = NULL,  # integer indicating the number of bootstrap replicates
    ...
) {
    # library(MBESS)
    # require(dplyr)
    # require(SciViews)
    # require(boot)
    if (missing(x) || is.null(x)) {
        stop("object 'x' not found")
    } else if (!missing(x)) {
        x <- x
    }
    if (!is.numeric(x)) {
        stop("argument is not a numeric vector: returning NA")
        return(NA_real_)
    }
    na.rm <- na.rm  # removes NAs if TRUE
    if (na.rm == TRUE) {
        x <- x[!is.na(x)]
    } else if (anyNA(x)) {
        stop(
            "missing values and NaN's not allowed if 'na.rm' is FALSE"
        )
    }

    if (is.null(digits)) {
        digits = 1
    }
    if (is.null(R)) {
        R = 1000
    }
    digits <- digits  # digits required for rounding
    shortest_length <- data.frame(  # "a" and "b" values for shortest-length CI
        v = c(  # degrees of freedom
            2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
            21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 40, 50, 60, 70, 80, 90,
            100, 150, 200, 250, 300, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13,
            14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
            30, 40, 50, 60, 70, 80, 90, 100, 150, 200, 250, 300, 2, 3, 4,
            5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21,
            22, 23, 24, 25, 26, 27, 28, 29, 30, 40, 50, 60, 70, 80, 90,
            100, 150, 200, 250, 300
        ),
        al = c(  # al is the allowed type I error probability
            0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,
            0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,
            0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,
            0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,
            0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
            0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
            0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
            0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
            0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
            0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01,
            0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01,
            0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01,
            0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01,
            0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01
        ),
        a = c(  # a is an attribute for the length of CI
            0.2065, 0.5654, 1.02, 1.5352, 2.093, 2.6828, 3.2981, 3.9343,
            4.5883, 5.2573, 5.9397, 6.6337, 7.3382, 8.0521, 8.7745,
            9.5047, 10.2421, 10.9861, 11.7362, 12.4919, 13.253,
            14.0191, 14.7899, 15.565, 16.3443, 17.1275, 17.9144,
            18.7049, 19.4987, 27.5919, 35.9012, 44.3661, 52.9501,
            61.629, 70.386, 79.2086, 124.0372, 169.6646, 215.8057,
            262.3132, 0.1015, 0.3449, 0.6918, 1.1092, 1.5776,
            2.0851, 2.6235, 3.1874, 3.7729, 4.3768, 4.9967,
            5.6308, 6.2776, 6.9357, 7.6042, 8.282, 8.9685,
            9.6629, 10.3647, 11.0733, 11.7882, 12.5092,
            13.2357, 13.9675, 14.7043, 15.4458, 16.1917,
            16.9419, 17.6961, 25.4233, 33.4085, 41.5794, 49.8923,
            58.3183, 66.8374, 75.4347, 119.2737, 164.0642,
            209.4667, 255.3057, 0.02, 0.114, 0.2937, 0.5461,
            0.8567, 1.2143, 1.6107, 2.0394, 2.4958, 2.976, 3.4771,
            3.9968, 4.5329, 5.084, 5.6487, 6.2256, 6.8139, 7.4126,
            8.0209, 8.6383, 9.264, 9.8976, 10.5385, 11.1864, 11.8408,
            12.5014, 13.1678, 13.8397, 14.517, 21.5331, 28.8879,
            36.4863, 44.2711, 52.2044, 60.2597, 68.4177, 110.3262,
            153.4834, 197.444, 241.9776
        ),
        b = c(  # b is an attribute for the length of CI
            12.5208, 13.1532, 14.18, 15.3498, 16.5807, 17.8391,
            19.1099, 20.3848, 21.6598, 22.9325, 24.2016,
            25.4666, 26.7269, 27.9825, 29.2334, 30.4796,
            31.7212, 32.9585, 34.1915, 35.4205, 36.6455,
            37.8668, 39.0844, 40.2986, 41.5095, 42.7171,
            43.9217, 45.1234, 46.3222, 58.1755, 69.8342,
            81.3479, 92.7487, 104.0584, 115.2925, 126.4628,
            181.6128, 235.9748, 289.8273, 343.3155, 15.1194,
            15.5897, 16.5735, 17.7432, 18.9954, 20.2863,
            21.5953, 22.9118, 24.2303, 25.5476, 26.8618,
            28.1717, 29.4769, 30.777, 32.072, 33.3619,
            34.6467, 35.9266, 37.2016, 38.472, 39.7379,
            40.9995, 42.257, 43.5105, 44.7601, 46.006,
            47.2483, 48.4872, 49.7229, 61.9217, 73.892,
            85.6914, 97.3573, 108.9153, 120.3839, 131.7767,
            187.9079, 243.1025, 297.691, 351.8461, 20.8264,
            20.9856, 21.8371, 22.9867, 24.2618, 25.6017,
            26.9749, 28.3643, 29.7602, 31.158, 32.5543,
            33.9474, 35.3358, 36.7192, 38.0968, 39.4688,
            40.8347, 42.1952, 43.5498, 44.8989, 46.2426,
            47.581, 48.9144, 50.2428, 51.5665, 52.8856,
            54.2002, 55.5107, 56.8169, 69.6808, 82.2534,
            94.6063, 106.7867, 118.8272, 130.7514, 142.5771,
            200.6194, 257.4375, 313.462, 368.9185
        )
    )
    if ("kelley" %in% method) {
        if (!require(MBESS)) {
            warning(
        "package 'MBESS' required to calculate Kelley's confidence interval"
            )
        }
    }
    cv <- (
        sd(x, na.rm = na.rm)/mean(x, na.rm = na.rm)  # coefficient of variation
    )
    cv_corr <- cv * (
        (1 - (1/(4 * (length(x) - 1))) +  # corrected coefficient of variation
             (1/length(x)) * cv^2) +
            (1/(2 * (length(x) - 1)^2))
    )
    if (is.null(method) == TRUE & correction == FALSE) {
        return(
            list(
                method = "cv = sd/mean (may be biased)",
                statistics = data.frame(
                    est = round(cv*100, digits = digits),
                    row.names = c(" ")
                    )
                )
            )
    } else if (is.null(method) == TRUE & correction == TRUE) {
        return(
            list(
                method = "Corrected (i.e., unbiased) cv",
                statistics = data.frame(
                    est = round(cv_corr*100, digits = digits),
                    row.names = c(" ")
                )
            )
        )
    } else if (!is.null(method)) {
        method <- tolower(method)  # convert user's input to lower-case
        method <- match.arg(  # match the user's input with available methods
            arg = method,
            choices = c(
                "kelley", "mckay", "miller", "vangel", "mahmoudvand_hassani",
                "equal_tailed", "shortest_length", "normal_approximation",
                "norm","basic", "perc", "bca", "all"
            ),
            several.ok = TRUE
        )
    }

# calculating cv's bootstrap CI
    boot.cv <- boot::boot(
        x,
        function(x, i) {  # coefficient of variation
            sd(x[i], na.rm = na.rm)/mean(x[i], na.rm = na.rm)
        },
        R = R
    )
    boot.cv_corr <- boot::boot(
        x,
        function(x, i) {  # corrected coefficient of variation
            sd(x[i], na.rm = na.rm)/mean(x[i], na.rm = na.rm) * (
                (1 - (1/(4 * (length(x[i]) - 1))) +
                     (1/length(x)) * (
                         sd(x[i], na.rm = na.rm)/mean(x[i], na.rm = na.rm)
                     )^2) +
                    (1/(2 * (length(x) - 1)^2))
            )
        },
        R = R
    )
# calculating cv and its CI attributes based on selected methods
    if ("kelley" %in% method && correction == FALSE) {
        ci <- MBESS::conf.limits.nct(
            ncp = sqrt(length(x))/cv_corr,
            df = length(x) - 1,
            conf.level = (1 - alpha)
        )
        est.kelley <- cv  # cv is an estimate of CV
        lower.tile.kelley <- unname(sqrt(length(x))/ci$Upper.Limit)
        upper.tile.kelley <- unname(sqrt(length(x))/ci$Lower.Limit)
    } else if ("kelley" %in% method && correction == TRUE) {
        ci <- MBESS::conf.limits.nct(
            ncp = sqrt(length(x))/cv_corr,
            df = length(x) - 1,
            conf.level = (1 - alpha)
        )
        est.kelley <- cv_corr  # corrected cv is an estimate of CV
        lower.tile.kelley <- unname(sqrt(length(x))/ci$Upper.Limit)
        upper.tile.kelley <- unname(sqrt(length(x))/ci$Lower.Limit)
    } else if ("mckay" %in% method && correction == FALSE) {
        if (cv > 0.33) {
            warning("Confidence interval may be very approximate")
        }
        v <- length(x) - 1
        t1 <- qchisq(1 - alpha/2,v)/v
        t2 <- qchisq(alpha/2,v)/v
        u1 <- v*t1
        u2 <- v*t2
        est.mckay <- cv  # cv is an estimate of CV
        lower.tile.mckay <- cv/sqrt((u1/(v + 1) - 1 )*(cv^2) + u1/v)
        upper.tile.mckay <- cv/sqrt((u2/(v + 1) - 1)*(cv^2) + u2/v)
    } else if ("mckay" %in% method && correction == TRUE) {
        if (cv_corr > 0.33) {
            warning("Confidence interval may be very approximate")
        }
        v <- length(x) - 1
        t1 <- qchisq(1 - alpha/2,v)/v
        t2 <- qchisq(alpha/2,v)/v
        u1 <- v*t1
        u2 <- v*t2
        est.mckay <- cv_corr  # corrected cv is an estimate of CV
        lower.tile.mckay <- cv_corr/sqrt((u1/(v + 1) - 1 )*(cv_corr^2) + u1/v)
        upper.tile.mckay <- cv_corr/sqrt((u2/(v + 1) - 1)*(cv_corr^2) + u2/v)
    } else if ("miller" %in% method && correction == FALSE) {
        v <- length(x) - 1
        z_alpha_over2 <- qnorm(1 - (alpha/2))
        u <- sqrt(
            (cv^2/v) * (0.5 + cv^2)
        )
        zu <- z_alpha_over2 * u
        est.miller <- cv  # cv is an estimate of CV
        lower.tile.miller <- cv - zu
        upper.tile.miller <- cv + zu
    } else if ("miller" %in% method && correction == TRUE) {
        v <- length(x) - 1
        z_alpha_over2 <- qnorm(1 - (alpha/2))
        u <- sqrt(
            (cv_corr^2/v) * (0.5 + cv_corr^2)
        )
        zu <- z_alpha_over2 * u
        est.miller <- cv_corr  # corrected cv is an estimate of CV
        lower.tile.miller <- cv_corr - zu
        upper.tile.miller <- cv_corr + zu
    }  else if ("vangel" %in% method && correction == FALSE) {
        if (cv > 0.33) {
            warning("Confidence interval may be very approximate")
        }
        v <- length(x) - 1
        t1 <- qchisq(1 - alpha/2,v)/v
        t2 <- qchisq(alpha/2,v)/v
        u1 <- v*t1
        u2 <- v*t2
        est.vangel <- cv  # cv is an estimate of CV
        lower.tile.vangel <- cv/sqrt(((u1 + 1)/(v + 1) - 1 )*(cv^2) + u1/v)
        upper.tile.vangel <- cv/sqrt(((u2 + 1)/(v + 1) - 1)*(cv^2) + u2/v)
    } else if ("vangel" %in% method && correction == TRUE) {
        if (cv_corr > 0.33) {
            warning("Confidence interval may be very approximate")
        }
        v <- length(x) - 1
        t1 <- qchisq(1 - alpha/2,v)/v
        t2 <- qchisq(alpha/2,v)/v
        u1 <- v*t1
        u2 <- v*t2
        est.vangel <- cv_corr  # corrected cv is an estimate of CV
        lower.tile.vangel <- cv_corr/sqrt(
            ((u1 + 1)/(v + 1) - 1 )*(cv_corr^2) + u1/v
            )
        upper.tile.vangel <- cv_corr/sqrt(
            ((u2 + 1)/(v + 1) - 1)*(cv_corr^2) + u2/v
            )
    } else if ("mahmoudvand_hassani" %in% method && correction == FALSE) {
        if (length(x) <= 340) {
            cn <- sqrt(2/(length(x) - 1)) * (
                (gamma(length(x)/2))/(gamma((length(x) - 1)/2))
                )
        } else {
            cn <- sqrt(2/(length(x) - 1)) * (
                (lgamma(length(x)/2))/(lgamma((length(x) - 1)/2))
            )
        }
        ul <- 2 - (cn + (qnorm((alpha/2)) * sqrt(1 - cn^2)))
        uu <- 2 - (cn - (qnorm((alpha/2)) * sqrt(1 - cn^2)))
        est.mahmoud <- cv  # cv is an estimate of CV
        lower.tile.mahmoud <- cv/ul
        upper.tile.mahmoud <- cv/uu
    } else if ("mahmoudvand_hassani" %in% method && correction == TRUE) {
        if (length(x) <= 340) {
            cn <- sqrt(2/(length(x) - 1)) * (
                (gamma(length(x)/2))/(gamma((length(x) - 1)/2))
            )
        } else {
            cn <- sqrt(2/(length(x) - 1)) * (
                (lgamma(length(x)/2))/(lgamma((length(x) - 1)/2))
            )
        }
        ul <- 2 - (cn + (qnorm((alpha/2)) * sqrt(1 - cn^2)))
        uu <- 2 - (cn - (qnorm((alpha/2)) * sqrt(1 - cn^2)))
        est.mahmoud <- cv_corr  # corrected cv is an estimate of CV
        lower.tile.mahmoud <- cv_corr/ul
        upper.tile.mahmoud <- cv_corr/uu
    } else if ("normal_approximation" %in% method && correction == FALSE) {
        cn <- sqrt(1 - (1/(2 * length(x))))
        ul <- cn + (qnorm(1 - (alpha/2)) * sqrt(1 - cn^2))
        uu <- cn - (qnorm(1 - (alpha/2)) * sqrt(1 - cn^2))
        est.normaapprox <- cv  # cv is an estimate of CV
        lower.tile.normaapprox <- cv/ul
        upper.tile.normaapprox <- cv/uu
    } else if ("normal_approximation" %in% method && correction == TRUE) {
        cn <- sqrt(1 - (1/(2 * length(x))))
        ul <- cn + (qnorm(1 - (alpha/2)) * sqrt(1 - cn^2))
        uu <- cn - (qnorm(1 - (alpha/2)) * sqrt(1 - cn^2))
        est.normaapprox <- cv_corr  # corrected cv is an estimate of CV
        lower.tile.normaapprox <- cv_corr/ul
        upper.tile.normaapprox <- cv_corr/uu
    } else if ("shortest_length" %in% method && correction == FALSE) {
        if (length(x) <= 300) {
            a_value <- shortest_length %>%
                subset(al == alpha & v == length(x) - 1) %>%
                dplyr::select(a)
            b_value <- shortest_length %>%
                subset(al == alpha & v == length(x) - 1) %>%
                dplyr::select(b)
        } else if (length(x) > 300) {
            a_value <- shortest_length %>%
                subset(al == alpha & v == 300) %>%
                dplyr::select(a)
            b_value <- shortest_length %>%
                subset(al == alpha & v == 300) %>%
                dplyr::select(b)
        }
        est.shortest <- cv  # cv is an estimate of CV
        lower.tile.shortest <- (cv*sqrt(length(x) - 1))/sqrt(b_value$b)
        upper.tile.shortest <- (cv*sqrt(length(x) - 1))/sqrt(a_value$a)
    } else if ("shortest_length" %in% method && correction == TRUE) {
        if (length(x) <= 300) {
            a_value <- shortest_length %>%
                subset(al == alpha & v == length(x) - 1) %>%
                dplyr::select(a)
            b_value <- shortest_length %>%
                subset(al == alpha & v == length(x) - 1) %>%
                dplyr::select(b)
        } else if (length(x) > 300) {
            a_value <- shortest_length %>%
                subset(al == alpha & v == 300) %>%
                dplyr::select(a)
            b_value <- shortest_length %>%
                subset(al == alpha & v == 300) %>%
                dplyr::select(b)
        }
        est.shortest <- cv_corr  # corrected cv is an estimate of CV
        lower.tile.shortest <- (cv_corr*sqrt(length(x) - 1))/sqrt(b_value$b)
        upper.tile.shortest <- (cv_corr*sqrt(length(x) - 1))/sqrt(a_value$a)
    } else if ("equal_tailed" %in% method && correction == FALSE) {
        v <- length(x) - 1
        tt1 <- qchisq(1 - alpha/2,v)
        tt2 <- qchisq(alpha/2,v)
        est.equal <- cv  # cv is an estimate of CV
        lower.tile.equal <- (cv*sqrt(v))/(sqrt(tt1))
        upper.tile.equal <- (cv*sqrt(v))/(sqrt(tt2))
    } else if ("equal_tailed" %in% method && correction == TRUE) {
        v <- length(x) - 1
        tt1 <- qchisq(1 - alpha/2,v)
        tt2 <- qchisq(alpha/2,v)
        est.equal <- cv_corr  # corrected cv is an estimate of CV
        lower.tile.equal <- (cv_corr*sqrt(v))/(sqrt(tt1))
        upper.tile.equal <- (cv_corr*sqrt(v))/(sqrt(tt2))
    } else if ("norm" %in% method && correction == FALSE) {
        boot.normcv.ci <- boot::boot.ci(
            boot.cv, conf = (1 - alpha), type = "norm"
        )
        est.norm <- cv  # cv is an estimate of CV
        lower.tile.norm <- boot.normcv.ci$normal[2]
        upper.tile.norm <- boot.normcv.ci$normal[3]
    } else if ("norm" %in% method && correction == TRUE) {
        boot.normcv_corr.ci <- boot::boot.ci(
            boot.cv_corr, conf = (1 - alpha), type = "norm"
        )
        est.norm <- cv_corr  # corrected cv is an estimate of CV
        lower.tile.norm <- boot.normcv_corr.ci$normal[2]
        upper.tile.norm <- boot.normcv_corr.ci$normal[3]
    } else if ("basic" %in% method && correction == FALSE) {
        boot.basiccv.ci <- boot::boot.ci(
            boot.cv, conf = (1 - alpha), type = "basic"
        )
        est.basic <- cv  # cv is an estimate of CV
        lower.tile.basic <- boot.basiccv.ci$basic[4]
        upper.tile.basic <- boot.basiccv.ci$basic[5]
    } else if ("basic" %in% method && correction == TRUE) {
        boot.basiccv_corr.ci <- boot::boot.ci(
            boot.cv_corr, conf = (1 - alpha), type = "basic"
        )
        est.basic <- cv_corr  # corrected cv is an estimate of CV
        lower.tile.basic <- boot.basiccv_corr.ci$basic[4]
        upper.tile.basic <- boot.basiccv_corr.ci$basic[5]
    } else if ("perc" %in% method && correction == FALSE) {
        boot.percentcv.ci <- boot::boot.ci(
            boot.cv, conf = (1 - alpha), type = "basic"
        )
        est.percent <- cv  # cv is an estimate of CV
        lower.tile.percent <- boot.percentcv.ci$percent[4]
        upper.tile.percent <- boot.percentcv.ci$percent[5]
    } else if ("perc" %in% method && correction == TRUE) {
        boot.percentcv_corr.ci <- boot::boot.ci(
            boot.cv_corr, conf = (1 - alpha), type = "basic"
        )
        est.percent <- cv_corr  # corrected cv is an estimate of CV
        lower.tile.percent <- boot.percentcv_corr.ci$percent[4]
        upper.tile.percent <- boot.percentcv_corr.ci$percent[5]
    } else if ("bca" %in% method && correction == FALSE) {
        boot.bcacv.ci <- boot::boot.ci(
            boot.cv, conf = (1 - alpha), type = "basic"
        )
        est.bca <- cv  # cv is an estimate of CV
        lower.tile.bca <- boot.bcacv.ci$bca[4]
        upper.tile.bca <- boot.bcacv.ci$bca[5]
    } else if ("bca" %in% method && correction == TRUE) {
        boot.bcacv_corr.ci <- boot::boot.ci(
            boot.cv_corr, conf = (1 - alpha), type = "basic"
        )
        est.bca <- cv_corr  # corrected cv is an estimate of CV
        lower.tile.bca <- boot.bcacv_corr.ci$percent[4]
        upper.tile.bca <- boot.bcacv_corr.ci$percent[5]
    } else if (method == "all" && correction == FALSE) {
        est.all <- cv  # cv is an estimate of CV
        ci <- MBESS::conf.limits.nct(
            ncp = sqrt(length(x))/cv_corr,
            df = length(x) - 1,
            conf.level = (1 - alpha)
        )
        lower.tile.kelley <- unname(sqrt(length(x))/ci$Upper.Limit)
        upper.tile.kelley <- unname(sqrt(length(x))/ci$Lower.Limit)
        if (cv > 0.33) {
            warning("Confidence interval may be very approximate")
        }
        v <- length(x) - 1
        t1 <- qchisq(1 - alpha/2,v)/v
        t2 <- qchisq(alpha/2,v)/v
        u1 <- v*t1
        u2 <- v*t2
        est.mckay <- cv  # cv is an estimate of CV
        lower.tile.mckay <- cv/sqrt((u1/(v + 1) - 1 )*(cv^2) + u1/v)
        upper.tile.mckay <- cv/sqrt((u2/(v + 1) - 1)*(cv^2) + u2/v)
        z_alpha_over2 <- qnorm(1 - (alpha/2))
        u <- sqrt(
            (cv^2/v) * (0.5 + cv^2)
        )
        zu <- z_alpha_over2 * u
        est.miller <- cv  # cv is an estimate of CV
        lower.tile.miller <- cv - zu
        upper.tile.miller <- cv + zu
        tt1 <- qchisq(1 - alpha/2,v)/v
        tt2 <- qchisq(alpha/2,v)/v
        uu1 <- v*tt1
        uu2 <- v*tt2
        est.vangel <- cv  # cv is an estimate of CV
        lower.tile.vangel <- cv/sqrt(((uu1 + 1)/(v + 1) - 1 )*(cv^2) + uu1/v)
        upper.tile.vangel <- cv/sqrt(((uu2 + 1)/(v + 1) - 1)*(cv^2) + uu2/v)
        if (length(x) <= 340) {
            cn <- sqrt(2/(length(x) - 1)) * (
                (gamma(length(x)/2))/(gamma((length(x) - 1)/2))
            )
        } else {
            cn <- sqrt(2/(length(x) - 1)) * (
                (lgamma(length(x)/2))/(lgamma((length(x) - 1)/2))
            )
        }
        ul <- 2 - (cn + (qnorm((alpha/2)) * sqrt(1 - cn^2)))
        uu <- 2 - (cn - (qnorm((alpha/2)) * sqrt(1 - cn^2)))
        est.mahmoud <- cv  # cv is an estimate of CV
        lower.tile.mahmoud <- cv/ul
        upper.tile.mahmoud <- cv/uu
        cnt <- sqrt(1 - (1/(2 * length(x))))
        ult <- cnt + (qnorm(1 - (alpha/2)) * sqrt(1 - cnt^2))
        uut <- cnt - (qnorm(1 - (alpha/2)) * sqrt(1 - cnt^2))
        est.normaapprox <- cv  # cv is an estimate of CV
        lower.tile.normaapprox <- cv/ult
        upper.tile.normaapprox <- cv/uut
        if (length(x) <= 300) {
            a_value <- shortest_length %>%
                subset(al == alpha & v == length(x) - 1) %>%
                dplyr::select(a)
            b_value <- shortest_length %>%
                subset(al == alpha & v == length(x) - 1) %>%
                dplyr::select(b)
        } else if (length(x) > 300) {
            a_value <- shortest_length %>%
                subset(al == alpha & v == 300) %>%
                dplyr::select(a)
            b_value <- shortest_length %>%
                subset(al == alpha & v == 300) %>%
                dplyr::select(b)
        }
        est.shortest <- cv  # cv is an estimate of CV
        lower.tile.shortest <- (cv*sqrt(length(x) - 1))/sqrt(b_value$b)
        upper.tile.shortest <- (cv*sqrt(length(x) - 1))/sqrt(a_value$a)
        ttt1 <- qchisq(1 - alpha/2,v)
        ttt2 <- qchisq(alpha/2,v)
        est.equal <- cv  # cv is an estimate of CV
        lower.tile.equal <- (cv*sqrt(v))/(sqrt(ttt1))
        upper.tile.equal <- (cv*sqrt(v))/(sqrt(ttt2))
        boot.normcv.ci <- boot::boot.ci(
            boot.cv, conf = (1 - alpha), type = "norm"
        )
        est.norm <- cv  # cv is an estimate of CV
        lower.tile.norm <- boot.normcv.ci$normal[2]
        upper.tile.norm <- boot.normcv.ci$normal[3]
        boot.basiccv.ci <- boot::boot.ci(
            boot.cv, conf = (1 - alpha), type = "basic"
        )
        est.basic <- cv  # cv is an estimate of CV
        lower.tile.basic <- boot.basiccv.ci$basic[4]
        upper.tile.basic <- boot.basiccv.ci$basic[5]
        boot.percentcv.ci <- boot::boot.ci(
            boot.cv, conf = (1 - alpha), type = "basic"
        )
        est.percent <- cv  # cv is an estimate of CV
        lower.tile.percent <- boot.percentcv.ci$percent[4]
        upper.tile.percent <- boot.percentcv.ci$percent[5]
        boot.bcacv.ci <- boot::boot.ci(
            boot.cv, conf = (1 - alpha), type = "basic"
        )
        est.bca <- cv  # cv is an estimate of CV
        lower.tile.bca <- boot.bcacv.ci$bca[4]
        upper.tile.bca <- boot.bcacv.ci$bca[5]
    } else if (method == "all" && correction == TRUE) {
        est.all <- cv_corr  # cv_corr is an estimate of CV
        ci <- MBESS::conf.limits.nct(
            ncp = sqrt(length(x))/cv_corr,
            df = length(x) - 1,
            conf.level = (1 - alpha)
        )
        lower.tile.kelley <- unname(sqrt(length(x))/ci$Upper.Limit)
        upper.tile.kelley <- unname(sqrt(length(x))/ci$Lower.Limit)
        if (cv_corr > 0.33) {
            warning("Confidence interval may be very approximate")
        }
        v <- length(x) - 1
        t1 <- qchisq(1 - alpha/2,v)/v
        t2 <- qchisq(alpha/2,v)/v
        u1 <- v*t1
        u2 <- v*t2
        est.mckay <- cv_corr  # cv_corr is an estimate of CV
        lower.tile.mckay <- cv_corr/sqrt((u1/(v + 1) - 1 )*(cv_corr^2) + u1/v)
        upper.tile.mckay <- cv_corr/sqrt((u2/(v + 1) - 1)*(cv_corr^2) + u2/v)
        z_alpha_over2 <- qnorm(1 - (alpha/2))
        u <- sqrt(
            (cv_corr^2/v) * (0.5 + cv_corr^2)
        )
        zu <- z_alpha_over2 * u
        est.miller <- cv_corr  # cv_corr is an estimate of CV
        lower.tile.miller <- cv_corr - zu
        upper.tile.miller <- cv_corr + zu
        tt1 <- qchisq(1 - alpha/2,v)/v
        tt2 <- qchisq(alpha/2,v)/v
        uu1 <- v*tt1
        uu2 <- v*tt2
        est.vangel <- cv_corr  # cv_corr is an estimate of CV
        lower.tile.vangel <- cv_corr/sqrt(((uu1 + 1)/(v + 1) - 1 )*(cv_corr^2) + uu1/v)
        upper.tile.vangel <- cv_corr/sqrt(((uu2 + 1)/(v + 1) - 1)*(cv_corr^2) + uu2/v)
        if (length(x) <= 340) {
            cn <- sqrt(2/(length(x) - 1)) * (
                (gamma(length(x)/2))/(gamma((length(x) - 1)/2))
            )
        } else {
            cn <- sqrt(2/(length(x) - 1)) * (
                (lgamma(length(x)/2))/(lgamma((length(x) - 1)/2))
            )
        }
        ul <- 2 - (cn + (qnorm((alpha/2)) * sqrt(1 - cn^2)))
        uu <- 2 - (cn - (qnorm((alpha/2)) * sqrt(1 - cn^2)))
        est.mahmoud <- cv_corr  # cv_corr is an estimate of CV
        lower.tile.mahmoud <- cv_corr/ul
        upper.tile.mahmoud <- cv_corr/uu
        cnt <- sqrt(1 - (1/(2 * length(x))))
        ult <- cnt + (qnorm(1 - (alpha/2)) * sqrt(1 - cnt^2))
        uut <- cnt - (qnorm(1 - (alpha/2)) * sqrt(1 - cnt^2))
        est.normaapprox <- cv_corr  # cv_corr is an estimate of CV
        lower.tile.normaapprox <- cv_corr/ult
        upper.tile.normaapprox <- cv_corr/uut
        if (length(x) <= 300) {
            a_value <- shortest_length %>%
                subset(al == alpha & v == length(x) - 1) %>%
                dplyr::select(a)
            b_value <- shortest_length %>%
                subset(al == alpha & v == length(x) - 1) %>%
                dplyr::select(b)
        } else if (length(x) > 300) {
            a_value <- shortest_length %>%
                subset(al == alpha & v == 300) %>%
                dplyr::select(a)
            b_value <- shortest_length %>%
                subset(al == alpha & v == 300) %>%
                dplyr::select(b)
        }
        est.shortest <- cv_corr  # cv_corr is an estimate of CV
        lower.tile.shortest <- (cv_corr*sqrt(length(x) - 1))/sqrt(b_value$b)
        upper.tile.shortest <- (cv_corr*sqrt(length(x) - 1))/sqrt(a_value$a)
        ttt1 <- qchisq(1 - alpha/2,v)
        ttt2 <- qchisq(alpha/2,v)
        est.equal <- cv_corr  # cv_corr is an estimate of CV
        lower.tile.equal <- (cv_corr*sqrt(v))/(sqrt(ttt1))
        upper.tile.equal <- (cv_corr*sqrt(v))/(sqrt(ttt2))
        boot.normcv_corr.ci <- boot::boot.ci(
            boot.cv_corr, conf = (1 - alpha), type = "norm"
        )
        est.norm <- cv_corr  # cv_corr is an estimate of CV
        lower.tile.norm <- boot.normcv_corr.ci$normal[2]
        upper.tile.norm <- boot.normcv_corr.ci$normal[3]
        boot.basiccv_corr.ci <- boot::boot.ci(
            boot.cv_corr, conf = (1 - alpha), type = "basic"
        )
        est.basic <- cv_corr  # cv_corr is an estimate of CV
        lower.tile.basic <- boot.basiccv_corr.ci$basic[4]
        upper.tile.basic <- boot.basiccv_corr.ci$basic[5]
        boot.percentcv_corr.ci <- boot::boot.ci(
            boot.cv_corr, conf = (1 - alpha), type = "basic"
        )
        est.percent <- cv_corr  # cv_corr is an estimate of CV
        lower.tile.percent <- boot.percentcv_corr.ci$percent[4]
        upper.tile.percent <- boot.percentcv_corr.ci$percent[5]
        boot.bcacv_corr.ci <- boot::boot.ci(
            boot.cv_corr, conf = (1 - alpha), type = "basic"
        )
        est.bca <- cv_corr  # cv_corr is an estimate of CV
        lower.tile.bca <- boot.bcacv_corr.ci$bca[4]
        upper.tile.bca <- boot.bcacv_corr.ci$bca[5]
    }
# preparing the output based on the selected methods
    if ("kelley" %in% method && correction == FALSE) {
        return(
            list(
                method = (
                    "cv with Kelley 95% CI"
                ),
                statistics = data.frame(
                    est = round(est.kelley * 100, digits = digits),
                    lower = round(lower.tile.kelley * 100, digits = digits),
                    upper = round(upper.tile.kelley * 100, digits = digits),
                    row.names = c(" ")
                )
            )
        )
    } else if ("kelley" %in% method && correction == TRUE) {
        return(
            list(
                method = (
                    "Corrected cv with Kelley 95% CI"
                ),
                statistics = data.frame(
                    est = round(est.kelley * 100, digits = digits),
                    lower = round(lower.tile.kelley * 100, digits = digits),
                    upper = round(upper.tile.kelley * 100, digits = digits),
                    row.names = c(" ")
                )
            )
        )
    } else if ("mckay" %in% method && correction == FALSE) {
        return(
            list(
                method = (
                    "cv with McKay 95% CI"
                ),
                statistics = data.frame(
                    est = round(est.mckay * 100, digits = digits),
                    lower = round(lower.tile.mckay * 100, digits = digits),
                    upper = round(upper.tile.mckay * 100, digits = digits),
                    row.names = c(" ")
                )
            )
        )
    } else if ("mckay" %in% method && correction == TRUE) {
        return(
            list(
                method = (
                    "Corrected cv with McKay 95% CI"
                ),
                statistics = data.frame(
                    est = round(est.mckay * 100, digits = digits),
                    lower = round(lower.tile.mckay * 100, digits = digits),
                    upper = round(upper.tile.mckay * 100, digits = digits),
                    row.names = c(" ")
                )
            )
        )
    } else if ("miller" %in% method && correction == FALSE) {
        return(
            list(
                method = (
                   "cv with Miller 95% CI"
                ),
                statistics = data.frame(
                    est = round(est.miller * 100, digits = digits),
                    lower = round(lower.tile.miller * 100, digits = digits),
                    upper = round(upper.tile.miller * 100, digits = digits),
                    row.names = c(" ")
                )
            )
        )
    } else if ("miller" %in% method && correction == TRUE) {
        return(
            list(
                method = (
                    "Corrected cv with Miller 95% CI"
                ),
                statistics = data.frame(
                    est = round(est.miller * 100, digits = digits),
                    lower = round(lower.tile.miller * 100, digits = digits),
                    upper = round(upper.tile.miller * 100, digits = digits),
                    row.names = c(" ")
                )
            )
        )
    } else if ("vangel" %in% method && correction == FALSE) {
        return(
            list(
                method = (
                    "cv with Vangel 95% CI"
                ),
                statistics = data.frame(
                    est = round(est.vangel * 100, digits = digits),
                    lower = round(lower.tile.vangel * 100, digits = digits),
                    upper = round(upper.tile.vangel * 100, digits = digits),
                    row.names = c(" ")
                )
            )
        )
    } else if ("vangel" %in% method && correction == TRUE) {
        return(
            list(
                method = (
                    "Corrected cv with Vangel 95% CI"
                ),
                statistics = data.frame(
                    est = round(est.vangel * 100, digits = digits),
                    lower = round(lower.tile.vangel * 100, digits = digits),
                    upper = round(upper.tile.vangel * 100, digits = digits),
                    row.names = c(" ")
                )
            )
        )
    } else if ("mahmoudvand_hassani" %in% method && correction == FALSE) {
        return(
            list(
                method = (
                    "cv with Mahmoudvand-Hassani 95% CI"
                ),
                statistics = data.frame(
                    est = round(est.mahmoud * 100, digits = digits),
                    lower = round(lower.tile.mahmoud * 100, digits = digits),
                    upper = round(upper.tile.mahmoud * 100, digits = digits),
                    row.names = c(" ")
                )
            )
        )
    } else if ("mahmoudvand_hassani" %in% method && correction == TRUE) {
        return(
            list(
                method = (
                    "Corrected cv with Mahmoudvand-Hassani 95% CI"
                ),
                statistics = data.frame(
                    est = round(est.mahmoud * 100, digits = digits),
                    lower = round(lower.tile.mahmoud * 100, digits = digits),
                    upper = round(upper.tile.mahmoud * 100, digits = digits),
                    row.names = c(" ")
                )
            )
        )
    } else if ("normal_approximation" %in% method && correction == FALSE) {
        return(
            list(
                method = (
                    "cv with Normal Approximation 95% CI"
                ),
                statistics = data.frame(
                    est = round(est.normaapprox * 100, digits = digits),
                    lower = round(
                        lower.tile.normaapprox * 100, digits = digits
                    ),
                    upper = round(
                        upper.tile.normaapprox * 100, digits = digits
                    ),
                    row.names = c(" ")
                )
            )
        )
    } else if ("normal_approximation" %in% method && correction == TRUE) {
        return(
            list(
                method = (
                    "Corrected cv with Normal Approximation 95% CI"
                ),
                statistics = data.frame(
                    est = round(est.normaapprox * 100, digits = digits),
                    lower = round(
                        lower.tile.normaapprox * 100, digits = digits
                        ),
                    upper = round(
                        upper.tile.normaapprox * 100, digits = digits
                    ),
                    row.names = c(" ")
                )
            )
        )
    } else if ("shortest_length" %in% method && correction == FALSE) {
        return(
            list(
                method = (
                    "cv with Shortest-Length 95% CI"
                ),
                statistics = data.frame(
                    est = round(est.shortest * 100, digits = digits),
                    lower = round(lower.tile.shortest * 100, digits = digits),
                    upper = round(upper.tile.shortest * 100, digits = digits),
                    row.names = c(" ")
                )
            )
        )
    } else if ("shortest_length" %in% method && correction == TRUE) {
        return(
            list(
                method = (
                    "Corrected cv with Shortest-Length 95% CI"
                ),
                statistics = data.frame(
                    est = round(est.shortest * 100, digits = digits),
                    lower = round(lower.tile.shortest * 100, digits = digits),
                    upper = round(upper.tile.shortest * 100, digits = digits),
                    row.names = c(" ")
                )
            )
        )
    } else if ("equal_tailed" %in% method && correction == FALSE) {
        return(
            list(
                method = (
                    "cv with Equal-Tailed 95% CI"
                ),
                statistics = data.frame(
                    est = round(est.equal * 100, digits = digits),
                    lower = round(lower.tile.equal * 100, digits = digits),
                    upper = round(upper.tile.equal * 100, digits = digits),
                    row.names = c(" ")
                )
            )
        )
    } else if ("equal_tailed" %in% method && correction == TRUE) {
        return(
            list(
                method = (
                    "Corrected cv with Equal-Tailed 95% CI"
                ),
                statistics = data.frame(
                    est = round(est.equal * 100, digits = digits),
                    lower = round(lower.tile.equal * 100, digits = digits),
                    upper = round(upper.tile.equal * 100, digits = digits),
                    row.names = c(" ")
                )
            )
        )
    } else if ("norm" %in% method && correction == FALSE) {
        return(
            list(
                method = (
                    "cv with Normal Approximation Bootstrap 95% CI"
                ),
                statistics = data.frame(
                    est = round(est.norm * 100, digits = digits),
                    lower = round(lower.tile.norm * 100, digits = digits),
                    upper = round(upper.tile.norm * 100, digits = digits),
                    row.names = c(" ")
                )
            )
        )
    } else if ("norm" %in% method && correction == TRUE) {
        return(
            list(
                method = (
                    "Corrected cv with Normal Approximation Bootstrap 95% CI"
                ),
                statistics = data.frame(
                    est = round(est.norm * 100, digits = digits),
                    lower = round(lower.tile.norm * 100, digits = digits),
                    upper = round(upper.tile.norm * 100, digits = digits),
                    row.names = c(" ")
                )
            )
        )
    } else if ("basic" %in% method && correction == FALSE) {
        return(
            list(
                method = (
                    "cv with Basic Bootstrap 95% CI"
                ),
                statistics = data.frame(
                    est = round(est.basic * 100, digits = digits),
                    lower = round(lower.tile.basic * 100, digits = digits),
                    upper = round(upper.tile.basic * 100, digits = digits),
                    row.names = c(" ")
                )
            )
            )
    } else if ("basic" %in% method && correction == TRUE) {
        return(
            list(
                method = (
                    "Corrected cv with Basic Bootstrap 95% CI"
                ),
                statistics = data.frame(
                    est = round(est.basic * 100, digits = digits),
                    lower = round(lower.tile.basic * 100, digits = digits),
                    upper = round(upper.tile.basic * 100, digits = digits),
                    row.names = c(" ")
                )
            )
        )
    } else if ("percent" %in% method && correction == FALSE) {
        stop("percent method is not developed yet")
        # return(
        #     list(
        #         method = (
        #             "cv with Bootstrap Percentile 95% CI"
        #         ),
        #         statistics = data.frame(
        #             est = round(est.percent * 100, digits = digits),
        #             lower = round(lower.tile.percent * 100, digits = digits),
        #             upper = round(upper.tile.percent * 100, digits = digits),
        #             row.names = c(" ")
        #         )
        #     )
        #     )
    } else if ("percent" %in% method && correction == TRUE) {
        stop("percent method is not developed yet")
        # return(
        #     list(
        #         method = (
        #             "Corrected cv with Bootstrap Percentile 95% CI"
        #         ),
        #         statistics = data.frame(
        #             est = round(est.percent * 100, digits = digits),
        #             lower = round(lower.tile.percent * 100, digits = digits),
        #             upper = round(upper.tile.percent * 100, digits = digits),
        #             row.names = c(" ")
        #         )
        #     )
        # )
    } else if ("bca" %in% method && correction == FALSE) {
        stop("BCa method is not developed yet")
        # return(
        #     list(
        #         method = (
        #             "cv with Adjusted Bootstrap Percentile (BCa) 95% CI"
        #         ),
        #         statistics = data.frame(
        #             est = round(est.bca * 100, digits = digits),
        #             lower = round(lower.tile.bca * 100, digits = digits),
        #             upper = round(upper.tile.bca * 100, digits = digits),
        #             row.names = c(" ")
        #         )
        #     )
        #     )
    } else if ("bca" %in% method && correction == TRUE) {
        stop("BCa method is not developed yet")
        # return(
        #     list(
        #         method = (
        #         "Corrected cv with Adjusted Bootstrap Percentile (BCa) 95% CI"
        #         ),
        #         statistics = data.frame(
        #             est = round(est.bca * 100, digits = digits),
        #             lower = round(lower.tile.bca * 100, digits = digits),
        #             upper = round(upper.tile.bca * 100, digits = digits),
        #             row.names = c(" ")
        #         )
        #     )
        # )
    } else if (method == "all" && correction == FALSE) {
        return(
            list(
                method = "All methods",
                statistics = data.frame(
                    row.names = c(
                        "kelley",  # 1
                        "mckay",  # 2
                        "miller",  # 3
                        "vangel",  # 4
                        "mahmoudvand_hassani",  # 5
                        "equal_tailed",  # 6
                        "shortest_length",  # 7
                        "normal_approximation",  # 8
                        "norm",  # 9
                        "basic"  # 10
                        # "perc"  # 11
                        # "bca"  # 12
                    ),
                    est = c(
                        round(cv * 100, digits = digits),
                        round(cv * 100, digits = digits),
                        round(cv * 100, digits = digits),
                        round(cv * 100, digits = digits),
                        round(cv * 100, digits = digits),
                        round(cv * 100, digits = digits),
                        round(cv * 100, digits = digits),
                        round(cv * 100, digits = digits),
                        round(cv * 100, digits = digits),
                        # round(cv * 100, digits = digits),
                        # round(cv * 100, digits = digits),
                        round(cv * 100, digits = digits)
                        ),
                    lower = c(
                        round(lower.tile.kelley * 100, digits = digits),
                        round(lower.tile.mckay * 100, digits = digits),
                        round(lower.tile.miller * 100, digits = digits),
                        round(lower.tile.vangel * 100, digits = digits),
                        round(lower.tile.mahmoud * 100, digits = digits),
                        round(lower.tile.equal * 100, digits = digits),
                        round(lower.tile.shortest * 100, digits = digits),
                        round(lower.tile.normaapprox * 100, digits = digits),
                        round(lower.tile.norm * 100, digits = digits),
                        round(lower.tile.basic * 100, digits = digits)
                        # ,
                        # round(lower.tile.percent * 100, digits = digits)
                        # ,
                        # round(lower.tile.bca * 100, digits = digits)
                    ),
                    upper = c(
                        round(upper.tile.kelley * 100, digits = digits),
                        round(upper.tile.mckay * 100, digits = digits),
                        round(upper.tile.miller * 100, digits = digits),
                        round(upper.tile.vangel * 100, digits = digits),
                        round(upper.tile.mahmoud * 100, digits = digits),
                        round(upper.tile.equal * 100, digits = digits),
                        round(upper.tile.shortest * 100, digits = digits),
                        round(upper.tile.normaapprox * 100, digits = digits),
                        round(upper.tile.norm * 100, digits = digits),
                        round(upper.tile.basic * 100, digits = digits)
                        # ,
                        # round(upper.tile.percent * 100, digits = digits)
                        # ,
                        # round(upper.tile.bca * 100, digits = digits)
                    ),
                    description = c(
                        "cv with Kelley 95% CI",
                        "cv with McKay 95% CI",
                        "cv with Miller 95% CI",
                        "cv with Vangel 95% CI",
                        "cv with Mahmoudvand-Hassani 95% CI",
                        "cv with Equal-Tailed 95% CI",
                        "cv with Shortest-Length 95% CI",
                        "cv with Normal Approximation 95% CI",
                        "cv with Normal Approximation Bootstrap 95% CI",
                        "cv with Basic Bootstrap 95% CI"
                        # ,
                        # "cv with Bootstrap Percentile 95% CI"
                        # ,
                        # "cv with Adjusted Bootstrap Percentile (BCa) 95% CI"
                    )
                )
            )
        )
    } else if (method == "all" && correction == TRUE) {
        return(
            list(
                method = "All methods",
                statistics = data.frame(
                    row.names = c(
                        "kelley",  # 1
                        "mckay",  # 2
                        "miller",  # 3
                        "vangel",  # 4
                        "mahmoudvand_hassani",  # 5
                        "equal_tailed",  # 6
                        "shortest_length",  # 7
                        "normal_approximation",  # 8
                        "norm",  # 9
                        "basic"  # 10
                        # "perc"  # 11
                        # "bca"  # 12
                    ),
                    est = c(
                        round(cv_corr * 100, digits = digits),
                        round(cv_corr * 100, digits = digits),
                        round(cv_corr * 100, digits = digits),
                        round(cv_corr * 100, digits = digits),
                        round(cv_corr * 100, digits = digits),
                        round(cv_corr * 100, digits = digits),
                        round(cv_corr * 100, digits = digits),
                        round(cv_corr * 100, digits = digits),
                        round(cv_corr * 100, digits = digits),
                        # round(cv_corr * 100, digits = digits),
                        # round(cv_corr * 100, digits = digits),
                        round(cv_corr * 100, digits = digits)
                    ),
                    lower = c(
                        round(lower.tile.kelley * 100, digits = digits),
                        round(lower.tile.mckay * 100, digits = digits),
                        round(lower.tile.miller * 100, digits = digits),
                        round(lower.tile.vangel * 100, digits = digits),
                        round(lower.tile.mahmoud * 100, digits = digits),
                        round(lower.tile.equal * 100, digits = digits),
                        round(lower.tile.shortest * 100, digits = digits),
                        round(lower.tile.normaapprox * 100, digits = digits),
                        round(lower.tile.norm * 100, digits = digits),
                        round(lower.tile.basic * 100, digits = digits)
                        # ,
                        # round(lower.tile.percent * 100, digits = digits)
                        # ,
                        # round(lower.tile.bca * 100, digits = digits)
                    ),
                    upper = c(
                        round(upper.tile.kelley * 100, digits = digits),
                        round(upper.tile.mckay * 100, digits = digits),
                        round(upper.tile.miller * 100, digits = digits),
                        round(upper.tile.vangel * 100, digits = digits),
                        round(upper.tile.mahmoud * 100, digits = digits),
                        round(upper.tile.equal * 100, digits = digits),
                        round(upper.tile.shortest * 100, digits = digits),
                        round(upper.tile.normaapprox * 100, digits = digits),
                        round(upper.tile.norm * 100, digits = digits),
                        round(upper.tile.basic * 100, digits = digits)
                        # ,
                        # round(upper.tile.percent * 100, digits = digits)
                        # ,
                        # round(upper.tile.bca * 100, digits = digits)
                    ),
                    description = c(
                "Corrected cv with Kelley 95% CI",
                "Corrected cv with McKay 95% CI",
                "Corrected cv with Miller 95% CI",
                "Corrected cv with Vangel 95% CI",
                "Corrected cv with Mahmoudvand-Hassani 95% CI",
                "Corrected cv with Equal-Tailed 95% CI",
                "Corrected cv with Shortest-Length 95% CI",
                "Corrected cv with Normal Approximation 95% CI",
                "Corrected cv with Normal Approximation Bootstrap 95% CI",
                "Corrected cv with Basic Bootstrap 95% CI"
                # ,
                # "Corrected cv with Bootstrap Percentile 95% CI"
                # ,
                # "Corrected cv with Adjusted Bootstrap Percentile (BCa) 95% CI"
                    )
                )
            )
        )
    }
}
```
```{r eval = TRUE, warning=FALSE, message=FALSE}
x <- c(
    0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
    4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
)
cv_versatile(
    x, 
    na.rm = TRUE, 
    digits = 3, 
    method = "kelley", 
    correction = TRUE, 
    alpha = 0.05
)
```
### McKay Confidence Interval
McKay [@McKay1932] introduced the following CI for $cv$; considering $u_1 = \chi_{v,1-\alpha/2}^2$ and $u_1 = \chi_{v,\alpha/2}^2$ being
the $100(1-\alpha/2)\%$ and $100(\alpha/2)\%$ percentile of the $\chi^2$ distribution with $v = n-1$ degrees of freedom, respectively [@Albatineh2014]:   
$$
\biggl(cv\left[\biggl(\frac{u_1}{v}-1\biggr)(cv)^{2}+\frac{u_1}{v}\right]^{-1/2}
\le CV \le cv
\left[\biggl(\frac{u_2}{v}-1\biggr)(cv)^{2}+\frac{u_2}{v}\right]^{-1/2}\biggr)
$$
Let us calculate the 95\% CI for our variable $x$ according to McKay's method [@McKay1932]:
```{r eval = TRUE, warning=FALSE, message=FALSE}
cv_versatile(
    x, 
    na.rm = TRUE, 
    digits = 3, 
    method = "mckay", 
    correction = TRUE, 
    alpha = 0.05
)
```
### Miller Confidence Interval    
Miller [@EdwardMiller1991] introduced the following CI for 
$cv$; considering $Z_{\alpha/2}$ being the $(1-\alpha/2)$ percentile of the standard normal distribution [@Albatineh2014]:
$$
\biggl(cv - Z_{\alpha/2}\sqrt{
\biggl(\frac{cv^2}{v}\biggr)\biggl(\frac{1}{2}+cv^2\biggr)} \le
CV \le cv + Z_{\alpha/2}\sqrt{
\biggl(\frac{cv^2}{v}\biggr)\biggl(\frac{1}{2}+cv^2\biggr)}
\biggr)
$$
where $v = n-1$ is the degree of freedom.   
Let us calculate the 95\% CI for $x$ according to Miller's method [@EdwardMiller1991]:   
```{r eval = TRUE, warning=FALSE, message=FALSE}
cv_versatile(
    x, 
    na.rm = TRUE, 
    digits = 3, 
    method = "miller", 
    correction = TRUE, 
    alpha = 0.05
)
```
### Vangel Confidence Interval      
Vangel [@Vangel1996] proposed the following CI for 
$cv$; which is a modification on McKay’s CI:
$$
\biggl(cv\left[\biggl(\frac{u_1+1}{v}-1\biggr)(cv)^{2}+\frac{u_1}{v}\right]^{-1/2}
\le CV \le cv
\left[\biggl(\frac{u_2+1}{v}-1\biggr)(cv)^{2}+\frac{u_2}{v}\right]^{-1/2}\biggr)
$$
Let us calculate the 95\% CI for $x$ according to Vangel's method [@Vangel1996]:   
```{r eval = TRUE, warning=FALSE, message=FALSE}
cv_versatile(
    x, 
    na.rm = TRUE, 
    digits = 3, 
    method = "vangel", 
    correction = TRUE, 
    alpha = 0.05
)
```
### Mahmoudvand-Hassani Confidence Interval   
Mahmoudvand and Hassani [@Mahmoudvand2009] proposed the following CI for 
$cv$; which is obtained using ranked set sampling *(RSS)*:    
$$
\biggl(\frac{cv}{2-C_n+Z_{1-\alpha/2}\sqrt{1-C_n^2}}
\le CV \le
\frac{cv}{2-C_n-Z_{1-\alpha/2}\sqrt{1-C_n^2}}
\biggr)
$$
where 
$$
C_n=\sqrt{\frac{2}{n-1}}\frac{\Gamma{(n/2)}}{\Gamma{((n-1)/2)}},
\Gamma(n)=(n-1)!
$$
Let us now calculate the 95\% CI for $x$ according to Mahmoudvand-Hassani's 
method [@Mahmoudvand2009]:   
```{r eval = TRUE, warning=FALSE, message=FALSE}
cv_versatile(
    x, 
    na.rm = TRUE, 
    digits = 3, 
    method = "mahmoudvand_hassani", 
    correction = TRUE, 
    alpha = 0.05
)
```
### Normal Approximation Confidence Interval     
Wararit Panichkitkosolkul [@Panichkitkosolkul2013] proposed the following CI for 
$cv$; which is a *normal approximation*: 
$$
\biggl(\frac{cv}{C_{n+1}+Z_{1-\alpha/2}\sqrt{1-C_{n+1}^2}}
\le CV \le
\frac{cv}{C_{n+1}-Z_{1-\alpha/2}\sqrt{1-C_{n+1}^2}}
\biggr)
$$
where $C_{n+1}=\sqrt{1-(1/2n)}$     
Now we calculate the normal approximation 95\% CI for $x$ according to Panichkitkosolkul [@Panichkitkosolkul2013]:   
```{r eval = TRUE, warning=FALSE, message=FALSE}
cv_versatile(
    x, 
    na.rm = TRUE, 
    digits = 3, 
    method = "normal_approximation", 
    correction = TRUE, 
    alpha = 0.05
)
```
### Shortest-Length Confidence Interval    
Panichkitkosolkul [@Panichkitkosolkul2013] has also introduced the following CI for $cv$:   
$$
\biggl(\frac{cv\sqrt{v}}{\sqrt{b}}
\le CV \le
\frac{cv\sqrt{v}}{\sqrt{a}}
\biggr)
$$
with $v = n-1$ degrees of freedom. Then, shortest-length 95\% CI for $x$ is:    
```{r eval = TRUE, warning=FALSE, message=FALSE}
cv_versatile(
    x, 
    na.rm = TRUE, 
    digits = 3, 
    method = "shortest_length", 
    correction = TRUE, 
    alpha = 0.05
)
```
### Equal-Tailed Confidence Interval     
The $100(1-\alpha)\%$ equal-tailed CI for $cv$ can be obtained as:
$$
\biggl(\frac{cv\sqrt{v}}{\sqrt{\chi_{v,1-\alpha/2}^2}}
\le CV \le
\frac{cv\sqrt{v}}{\sqrt{\chi_{v,\alpha/2}^2}}
\biggr)
$$
where $\chi_{v,\alpha/2}^2$ and $\chi_{v,1-\alpha/2}^2$ are the $100(\alpha/2)$ and $100(1-\alpha/2)$ percentiles of the central $\chi^2$ distribution with $v$ degrees of freedom, respectively [@Panichkitkosolkul2013].   
Then, equal-tailed 95\% CI for $x$ is:    
```{r eval = TRUE, warning=FALSE, message=FALSE}
cv_versatile(
    x, 
    na.rm = TRUE, 
    digits = 3, 
    method = "equal_tailed", 
    correction = TRUE, 
    alpha = 0.05
)
```
### Bootstrap Confidence Intervals      
Thanks to package `boot` [@Canty2017] we can obtain bootstrap CI around $cv$:    
```{r eval = TRUE, warning=FALSE, message=FALSE}
cv_versatile(
    x, 
    na.rm = TRUE, 
    digits = 3, 
    method = "basic", 
    correction = TRUE, 
    alpha = 0.05
)
```
### All Available Methods   
In conclusion, we can observe CIs calculated by all available methods:    
```{r eval = TRUE, warning=FALSE, message=FALSE}
cv_versatile(
    x, 
    na.rm = TRUE, 
    digits = 3, 
    method = "all", 
    correction = FALSE, 
    alpha = 0.05
)
```
# References   

---
title: "Coefficient of Quartile Variation: cqv_versatile"
author: "Maani Beigy"
date: "February 25, 2019"
output: rmarkdown::html_vignette
bibliography: cvcqv.bib
csl: apa.csl
vignette: >
  %\VignetteIndexEntry{cqv_versatile}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```
# Coefficient of Quartile Variation   
Coefficient of quartile variation *($CQV$)* is a measure of relative dispersion  that is based on interquartile range (`IQR`). Since *cqv* is unitless, it is 
useful for comparison of variables with different units. It is also a measure of homogeneity [@Bonett2006; @Altunkaynak2018].    
The *population* coefficient of quartile variation is:    
$$
CQV = \biggl(\frac{Q_3-Q_1}{Q_3+Q_1}\biggr)\times100
$$
where $q_3$ and $q_1$ are the population third quartile 
(*i.e.,* $75^{th}$ percentile) and first quartile (*i.e.,* $25^{th}$ percentile), respectively. Almost always, we analyze data from samples but want to 
generalize it as the population's parameter [@Albatineh2014]. Its sample's estimate is given as:    
$$
cqv = \biggl(\frac{q_3-q_1}{q_3+q_1}\biggr)\times100
$$
There are different methods for the calculation of **confidence intervals (CI)** 
for *CQV*. All of them are fruitful and have particular use cases. For sake of versatility, we cover almost all of these methods in `cvcqv` package. Here, we explain them along with some examples:     

### Bonett Confidence Interval    

Bonett [@Bonett2006] introduced the following confidence interval for *CQV*:   
$$
\exp\{\ln{(D/S)c\  \pm\ Z_{1-\alpha/2}\sqrt{v} }\}
$$
where $c = n/(n-1)$ is a centering adjustment which helps to equalize the tail error probabilities [@Bonett2006; @Altunkaynak2018]. $D = \hat{Q3}-\hat{Q1}$ and
$S = \hat{Q3}+\hat{Q1}$ are the sample $25^{th}$ and $75^{th}$ percentiles, respectively; $Z_{1-\alpha/2}$ is the $1-\alpha/2$ quantile of the standard 
normal distribution. Computation of $v$ which is $Var\{\ln{(D/S)}\}$ is long and 
a bit complicated, but has been implemented for `cqv` function:
```{r echo=FALSE, warning=FALSE, message=FALSE, include = FALSE}
library(dplyr)
library(boot)
library(SciViews)
x <- c(
  0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
  4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
)
cqv_versatile <- function(
  x,
  na.rm = FALSE,
  digits = 1,
  method = NULL,
  R = NULL,
  ...
) {
  # require(dplyr)
  # require(SciViews)
  # require(boot)
  if (missing(x) || is.null(x)) {
    stop("object 'x' not found")
  } else if (!missing(x)) {
    x <- x
  }
  if (!is.numeric(x)) {
    stop("argument is not a numeric vector: returning NA")
    return(NA_real_)
  }
  na.rm <- na.rm # removes NAs if TRUE
  if (na.rm == TRUE) {
    x <- x[!is.na(x)]
  } else if (anyNA(x)) {
    stop(
      "missing values and NaN's not allowed if 'na.rm' is FALSE"
    )
  }
  if (is.null(digits)) {
    digits <- 1
  }
  digits <- digits # digits required for rounding
  method <- method # returns 95% confidence interval
  if (is.null(R)) {
    R <- 1000
  }
  q3 <- unname(
    quantile(
      x,
      probs = 0.75, # third quartile (0.75 percentile)
      na.rm = na.rm
    )
  )
  q1 <- unname(
    quantile(
      x,
      probs = 0.25, # first quartile (0.25 percentile)
      na.rm = na.rm
    )
  )
  if (q3 == 0) { # to avoid NaNs when q3 and q1 are zero
    warning(
      "cqv is NaN because q3 and q1 are 0, max was used instead of q3"
    )
    q3 <- max(x, na.rm = na.rm)
  }
  a <- ceiling(
    (length(x) / 4) - (1.96 * (((3 * length(x)) / 16)^(0.5)))
  )
  b <- round(
    (length(x) / 4) + (1.96 * (((3 * length(x)) / 16)^(0.5))),
    digits = 0
  )
  c <- length(x) + 1 - b
  d <- length(x) + 1 - a
  Ya <- dplyr::nth(x, a, order_by = x)
  Yb <- dplyr::nth(x, b, order_by = x)
  Yc <- dplyr::nth(x, c, order_by = x)
  Yd <- dplyr::nth(x, d, order_by = x)
  star <- 0
  for (i in a:(b - 1)) {
    star[i] <- (
      (choose(length(x), i)) * (0.25^(i)) * (0.75^(length(x) - i))
    )
    alphastar <- 1 - sum(star[i], na.rm = na.rm)
  }
  zzz <- qnorm((1 - ((1 - alphastar) / 2)))
  f1square <- (3 * (zzz)^2) / (4 * length(x) * ((Yb - Ya)^2))
  f3square <- (3 * (zzz)^2) / (4 * length(x) * ((Yd - Yc)^2))
  D <- q3 - q1
  S <- q3 + q1
  v <- (
    (1 / (16 * length(x))) * (
      (((3 / f1square) + (3 / f3square) - (2 / sqrt(f1square * f3square))) / D^2) +
        (((3 / f1square) + (3 / f3square) + (2 / sqrt(f1square * f3square))) / S^2) -
        ((2 * ((3 / f3square) - (3 / f1square))) / (D * S))
    )
  )
  ccc <- length(x) / (length(x) - 1)
  upper.tile <- exp(((SciViews::ln((D / S)) * ccc)) + (zzz * (v^(0.5))))
  lower.tile <- exp(((SciViews::ln((D / S)) * ccc)) - (zzz * (v^(0.5))))
  if (
    unname(quantile(x, probs = 0.75, na.rm = na.rm)) != 0
  ) {
    boot.cqv <- boot::boot(
      x,
      function(x, i) {
        round(((
          unname(quantile(x[i], probs = 0.75, na.rm = na.rm)) -
            unname(quantile(x[i], probs = 0.25, na.rm = na.rm))
        ) / (
          unname(quantile(x[i], probs = 0.75, na.rm = na.rm)) +
            unname(quantile(x[i], probs = 0.25, na.rm = na.rm))
        )) * 100, digits = digits)
      },
      R = R
    )
  } else if (
    unname(quantile(x, probs = 0.75, na.rm = na.rm)) == 0
  ) {
    boot.cqv <- boot::boot(
      x,
      function(x, i) {
        round(((
          max(x[i], na.rm = na.rm) -
            unname(quantile(x[i], probs = 0.25, na.rm = na.rm))
        ) / (
          max(x[i], na.rm = na.rm) +
            unname(quantile(x[i], probs = 0.25, na.rm = na.rm))
        )) * 100, digits = digits)
      },
      R = R
    )
  }

  if (is.null(method)) {
    boot.cqv.ci <- NA
  } else if (method == "bonett") {
    boot.cqv.ci <- NA
  } else if (method == "norm") {
    boot.norm.ci <- boot::boot.ci(boot.cqv, conf = 0.95, type = "norm")
  } else if (method == "basic") {
    boot.basic.ci <- boot::boot.ci(boot.cqv, conf = 0.95, type = "basic")
  } else if (method == "perc") {
    boot.perc.ci <- boot::boot.ci(boot.cqv, conf = 0.95, type = "perc")
  } else if (method == "bca") {
    boot.bca.ci <- boot::boot.ci(boot.cqv, conf = 0.95, type = "bca")
  } else if (method == "all") {
    boot.norm.ci <- boot::boot.ci(boot.cqv, conf = 0.95, type = "norm")
    boot.basic.ci <- boot::boot.ci(boot.cqv, conf = 0.95, type = "basic")
    boot.perc.ci <- boot::boot.ci(boot.cqv, conf = 0.95, type = "perc")
    boot.bca.ci <- boot::boot.ci(boot.cqv, conf = 0.95, type = "bca")
  }

  if (is.null(method)) {
    cqv <- round(
      100 * ((q3 - q1) / (q3 + q1)),
      digits = digits
    )
  } else if (method == "bonett") {
    cqv <- round(
      100 * ((q3 - q1) / (q3 + q1)),
      digits = digits
    )
  } else if (method == "norm") {
    cqv <- round(
      100 * ((q3 - q1) / (q3 + q1)),
      digits = digits
    )
  } else if (method == "basic") {
    cqv <- round(
      100 * ((q3 - q1) / (q3 + q1)),
      digits = digits
    )
  } else if (method == "perc") {
    cqv <- round(
      100 * ((q3 - q1) / (q3 + q1)),
      digits = digits
    )
  } else if (method == "bca") {
    cqv <- round(
      100 * ((q3 - q1) / (q3 + q1)),
      digits = digits
    )
  } else if (method == "all") {
    cqv <- round(
      100 * ((q3 - q1) / (q3 + q1)),
      digits = digits
    )
  }

  if (is.null(method)) {
    lower <- NA
    upper <- NA
  } else if (method == "bonett") {
    lower <- round(lower.tile * 100, digits = digits)
    upper <- round(upper.tile * 100, digits = digits)
  } else if (method == "norm") {
    lower <- round(boot.norm.ci$normal[2], digits = digits)
    upper <- round(boot.norm.ci$normal[3], digits = digits)
  } else if (method == "basic") {
    lower <- round(boot.basic.ci$basic[4], digits = digits)
    upper <- round(boot.basic.ci$basic[5], digits = digits)
  } else if (method == "perc") {
    lower <- round(boot.perc.ci$percent[4], digits = digits)
    upper <- round(boot.perc.ci$percent[5], digits = digits)
  } else if (method == "bca") {
    lower <- round(boot.bca.ci$bca[4], digits = digits)
    upper <- round(boot.bca.ci$bca[5], digits = digits)
  }

  if (is.null(method)) {
    return(
      list(
        method = "cqv = (q3-q1)/(q3+q1)",
        statistics = data.frame(
          est = cqv,
          row.names = c(" ")
        )
      )
    )
  } else if (method == "bonett" && cqv != 100) {
    return(
      list(
        method = "cqv with Bonett 95% CI",
        statistics = data.frame(
          est = cqv,
          lower = lower,
          upper = upper,
          row.names = c(" ")
        )
      )
    )
  } else if (method == "norm" && cqv != 100) {
    return(
      list(
        method = "cqv with normal approximation 95% CI",
        statistics = data.frame(
          est = cqv,
          lower = lower,
          upper = upper,
          row.names = c(" ")
        )
      )
    )
  } else if (method == "basic" && cqv != 100) {
    return(
      list(
        method = "cqv with basic bootstrap 95% CI",
        statistics = data.frame(
          est = cqv,
          lower = lower,
          upper = upper,
          row.names = c(" ")
        )
      )
    )
  } else if (method == "perc" && cqv != 100) {
    return(
      list(
        method = "cqv with bootstrap percentile 95% CI",
        statistics = data.frame(
          est = cqv,
          lower = lower,
          upper = upper,
          row.names = c(" ")
        )
      )
    )
  } else if (method == "bca" && cqv != 100) {
    return(
      list(
        method = "cqv with adjusted bootstrap percentile (BCa) 95% CI",
        statistics = data.frame(
          est = cqv,
          lower = lower,
          upper = upper,
          row.names = c(" ")
        )
      )
    )
  } else if (
    (
      method == "norm" | method == "bonett" | method == "basic" | method == "perc" |
        method == "bca" | method == "all"
    ) && cqv == 100
  ) {
    warning(
      "All values of t are equal to  100 \n Cannot calculate confidence intervals \n"
    )
    return(
      list(
        method = "cqv with Bonett 95% CI",
        statistics = data.frame(
          est = cqv,
          lower = round(lower.tile * 100, digits = digits),
          upper = round(upper.tile * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if (method == "all" && cqv != 100) {
    return(
      list(
        method = "All methods",
        statistics = data.frame(
          row.names = c(
            "bonett",
            "norm",
            "basic",
            "percent",
            "bca"
          ),
          est = c(cqv, cqv, cqv, cqv, cqv),
          lower = c(
            round(lower.tile * 100, digits = digits),
            round(boot.norm.ci$normal[2], digits = digits),
            round(boot.basic.ci$basic[4], digits = digits),
            round(boot.perc.ci$percent[4], digits = digits),
            round(boot.bca.ci$bca[4], digits = digits)
          ),
          upper = c(
            round(upper.tile * 100, digits = digits),
            round(boot.norm.ci$normal[3], digits = digits),
            round(boot.basic.ci$basic[5], digits = digits),
            round(boot.perc.ci$percent[5], digits = digits),
            round(boot.bca.ci$bca[5], digits = digits)
          ),
          description = c(
            "cqv with Bonett 95% CI",
            "cqv with normal approximation 95% CI",
            "cqv with basic bootstrap 95% CI",
            "cqv with bootstrap percentile 95% CI",
            "cqv with adjusted bootstrap percentile (BCa) 95% CI"
          )
        )
      )
    )
  } else {
    stop("method for confidence interval is not available")
    return(NA_real_)
  }
}
```
```{r eval = TRUE, warning=FALSE, message=FALSE}
x <- c(
  0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
  4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
)
cqv_versatile(
  x,
  na.rm = TRUE,
  digits = 3,
  method = "bonett"
)
```
### Bootstrap Confidence Intervals      
Thanks to package `boot` [@Canty2017] we can obtain bootstrap CI around $cqv$:  
```{r eval = TRUE, warning=FALSE, message=FALSE}
cqv_versatile(
  x,
  na.rm = TRUE,
  digits = 3,
  method = "bca"
)
```
### All Available Methods   
In conclusion, we can observe CIs calculated by all available methods:    
```{r eval = TRUE, warning=FALSE, message=FALSE}
cqv_versatile(
  x,
  na.rm = TRUE,
  digits = 3,
  method = "all"
)
```
# References   

---
title: "Coefficient of Variation: cv_versatile"
author: "Maani Beigy"
date: "February 18, 2019"
output: 
    rmarkdown::html_vignette
bibliography: cvcqv.bib
csl: apa.csl
vignette: >
  %\VignetteIndexEntry{cv_versatile}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

# Coefficient of Variation   
Coefficient of variation *($CV$)* is a measure of relative dispersion representing 
the degree of variability relative to the mean [@Albatineh2014]. 
Since cv is unitless, it is useful for comparison of variables with different 
units [@Albatineh2014]. It is also a measure of homogeneity. The *population* coefficient of variation is:    
$$CV = \frac{\sigma}{\mu},$$
where $\sigma$ is the population standard deviation and $\mu$ is the population mean. Almost always, we analyze data from samples but want to generalize it 
as the population's parameter [@Albatineh2014]. Its sample's estimate is given 
as:    
$$cv = \frac{sd}{\bar{X}}$$   
where $sd$ is the sample standard deviation, the square root of the unbiased estimator of population variance, and $\bar{X}$ is the sample mean. The 
corrected *cv* to account for the sample size is:
$$
cv_{corr} = cv * \biggl(1 - \frac{1}{4(n-1)} 
+ \frac{1}{n}cv^2 
+ \frac{1}{2 (n-1)^2} \biggr)
$$
There are various methods for the calculation of **confidence intervals (CI)** 
for *cv*. All of them are fruitful and have particular use cases. Some of them 
are model-based hence their usage depends the assumptions regarding the distribution of data. For sake of versatility, we cover almost all of these methods in `cvcqv` package. Here, we explain them along with some examples:

### Kelley Confidence Interval        

Let us assume that *CV* follows a noncentral *t* distribution, when the parent  population of the scores is *normally-distributed*, with noncentrality 
($\lambda$) parameter:   
$$
\lambda = \frac{\sqrt{n}}{cv}
$$
with *v* degrees of freedom, where $v = n - 1$. 
Let $1 - \alpha$ be the CI coverage with $\alpha_L + \alpha_U = \alpha$ in which $\alpha_L$ is the the proportion of times that *cv* will be 
less than the lower confidence bound and $\alpha_U$ the proportion of times that
*cv* will be greater than the upper confidence bound in the CI procedure [@Kelley2007]. The lower confidence tile for $\lambda$ is is the noncentrality parameter that results in $t_{(1-\alpha_L,v,\lambda_L)}=\hat{\lambda}$ and the
upper confidence tile for $\lambda$ is is the noncentrality parameter that 
results in $t_{(\alpha_U,v,\lambda_U)}=\hat{\lambda}$, where $t_{(1-\alpha_L,v,\lambda_L)}=\hat{\lambda}$ is the value of noncentral *t* distribution at the $1-\alpha_L$ **quantile** with noncentrality parameter $\lambda_L$ and $t_{(\alpha_U,v,\lambda_U)}=\hat{\lambda}$ is the value of noncentral *t* distribution at the $\alpha_U$ **quantile** with noncentrality parameter $\lambda_U$, respectively [@Kelley2007].   
Afterwards, we transform the tiles of the confidence interval for $\lambda$, by
dividing the tiles by $\sqrt{n}$ and thereafter inverting them; the CI limits of 
$cv$ will be obtained:  
$$
p\left[\biggl(\frac{\lambda_U}{\sqrt{n}}\biggr)^{-1}
\le CV \le \biggl(\frac{\lambda_L}{\sqrt{n}}\biggr)^{-1}\right] = 1-\alpha
$$
where $p$ stands for *probability*. Thanks to package `MBESS` [@Kelley2018] for 
the computation of confidence limits for the noncentrality parameter from a *t* distribution (`conf.limits.nct`), $cv$ will be obtained as: 
```{r echo=FALSE, warning=FALSE, message=FALSE, include = FALSE}
library(dplyr)
library(boot)
x <- c(
  0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
  4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
)
cv_versatile <- function(
  x, # Currently there are methods for numeric vectors
  na.rm = FALSE, # indicating whether NA values should be stripped
  digits = 1, # digits of output after rounding. default is 4
  method = NULL, # method for the computation of confidence interval (CI)
  correction = FALSE, # indicating whether to compute the unbiased statistics
  alpha = 0.05, # The allowed type I error probability
  R = NULL, # integer indicating the number of bootstrap replicates
  ...
) {
  # library(MBESS)
  # require(dplyr)
  # require(SciViews)
  # require(boot)
  if (missing(x) || is.null(x)) {
    stop("object 'x' not found")
  } else if (!missing(x)) {
    x <- x
  }
  if (!is.numeric(x)) {
    stop("argument is not a numeric vector: returning NA")
    return(NA_real_)
  }
  na.rm <- na.rm # removes NAs if TRUE
  if (na.rm == TRUE) {
    x <- x[!is.na(x)]
  } else if (anyNA(x)) {
    stop(
      "missing values and NaN's not allowed if 'na.rm' is FALSE"
    )
  }

  if (is.null(digits)) {
    digits <- 1
  }
  if (is.null(R)) {
    R <- 1000
  }
  digits <- digits # digits required for rounding
  shortest_length <- data.frame( # "a" and "b" values for shortest-length CI
    v = c( # degrees of freedom
      2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
      21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 40, 50, 60, 70, 80, 90,
      100, 150, 200, 250, 300, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13,
      14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
      30, 40, 50, 60, 70, 80, 90, 100, 150, 200, 250, 300, 2, 3, 4,
      5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21,
      22, 23, 24, 25, 26, 27, 28, 29, 30, 40, 50, 60, 70, 80, 90,
      100, 150, 200, 250, 300
    ),
    al = c( # al is the allowed type I error probability
      0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,
      0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,
      0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,
      0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1, 0.1,
      0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
      0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
      0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
      0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
      0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05, 0.05,
      0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01,
      0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01,
      0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01,
      0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01,
      0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01
    ),
    a = c( # a is an attribute for the length of CI
      0.2065, 0.5654, 1.02, 1.5352, 2.093, 2.6828, 3.2981, 3.9343,
      4.5883, 5.2573, 5.9397, 6.6337, 7.3382, 8.0521, 8.7745,
      9.5047, 10.2421, 10.9861, 11.7362, 12.4919, 13.253,
      14.0191, 14.7899, 15.565, 16.3443, 17.1275, 17.9144,
      18.7049, 19.4987, 27.5919, 35.9012, 44.3661, 52.9501,
      61.629, 70.386, 79.2086, 124.0372, 169.6646, 215.8057,
      262.3132, 0.1015, 0.3449, 0.6918, 1.1092, 1.5776,
      2.0851, 2.6235, 3.1874, 3.7729, 4.3768, 4.9967,
      5.6308, 6.2776, 6.9357, 7.6042, 8.282, 8.9685,
      9.6629, 10.3647, 11.0733, 11.7882, 12.5092,
      13.2357, 13.9675, 14.7043, 15.4458, 16.1917,
      16.9419, 17.6961, 25.4233, 33.4085, 41.5794, 49.8923,
      58.3183, 66.8374, 75.4347, 119.2737, 164.0642,
      209.4667, 255.3057, 0.02, 0.114, 0.2937, 0.5461,
      0.8567, 1.2143, 1.6107, 2.0394, 2.4958, 2.976, 3.4771,
      3.9968, 4.5329, 5.084, 5.6487, 6.2256, 6.8139, 7.4126,
      8.0209, 8.6383, 9.264, 9.8976, 10.5385, 11.1864, 11.8408,
      12.5014, 13.1678, 13.8397, 14.517, 21.5331, 28.8879,
      36.4863, 44.2711, 52.2044, 60.2597, 68.4177, 110.3262,
      153.4834, 197.444, 241.9776
    ),
    b = c( # b is an attribute for the length of CI
      12.5208, 13.1532, 14.18, 15.3498, 16.5807, 17.8391,
      19.1099, 20.3848, 21.6598, 22.9325, 24.2016,
      25.4666, 26.7269, 27.9825, 29.2334, 30.4796,
      31.7212, 32.9585, 34.1915, 35.4205, 36.6455,
      37.8668, 39.0844, 40.2986, 41.5095, 42.7171,
      43.9217, 45.1234, 46.3222, 58.1755, 69.8342,
      81.3479, 92.7487, 104.0584, 115.2925, 126.4628,
      181.6128, 235.9748, 289.8273, 343.3155, 15.1194,
      15.5897, 16.5735, 17.7432, 18.9954, 20.2863,
      21.5953, 22.9118, 24.2303, 25.5476, 26.8618,
      28.1717, 29.4769, 30.777, 32.072, 33.3619,
      34.6467, 35.9266, 37.2016, 38.472, 39.7379,
      40.9995, 42.257, 43.5105, 44.7601, 46.006,
      47.2483, 48.4872, 49.7229, 61.9217, 73.892,
      85.6914, 97.3573, 108.9153, 120.3839, 131.7767,
      187.9079, 243.1025, 297.691, 351.8461, 20.8264,
      20.9856, 21.8371, 22.9867, 24.2618, 25.6017,
      26.9749, 28.3643, 29.7602, 31.158, 32.5543,
      33.9474, 35.3358, 36.7192, 38.0968, 39.4688,
      40.8347, 42.1952, 43.5498, 44.8989, 46.2426,
      47.581, 48.9144, 50.2428, 51.5665, 52.8856,
      54.2002, 55.5107, 56.8169, 69.6808, 82.2534,
      94.6063, 106.7867, 118.8272, 130.7514, 142.5771,
      200.6194, 257.4375, 313.462, 368.9185
    )
  )
  if ("kelley" %in% method) {
    if (!require(MBESS)) {
      warning(
        "package 'MBESS' required to calculate Kelley's confidence interval"
      )
    }
  }
  cv <- (
    sd(x, na.rm = na.rm) / mean(x, na.rm = na.rm) # coefficient of variation
  )
  cv_corr <- cv * (
    (1 - (1 / (4 * (length(x) - 1))) + # corrected coefficient of variation
      (1 / length(x)) * cv^2) +
      (1 / (2 * (length(x) - 1)^2))
  )
  if (is.null(method) == TRUE & correction == FALSE) {
    return(
      list(
        method = "cv = sd/mean (may be biased)",
        statistics = data.frame(
          est = round(cv * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if (is.null(method) == TRUE & correction == TRUE) {
    return(
      list(
        method = "Corrected (i.e., unbiased) cv",
        statistics = data.frame(
          est = round(cv_corr * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if (!is.null(method)) {
    method <- tolower(method) # convert user's input to lower-case
    method <- match.arg( # match the user's input with available methods
      arg = method,
      choices = c(
        "kelley", "mckay", "miller", "vangel", "mahmoudvand_hassani",
        "equal_tailed", "shortest_length", "normal_approximation",
        "norm", "basic", "perc", "bca", "all"
      ),
      several.ok = TRUE
    )
  }

  # calculating cv's bootstrap CI
  boot.cv <- boot::boot(
    x,
    function(x, i) { # coefficient of variation
      sd(x[i], na.rm = na.rm) / mean(x[i], na.rm = na.rm)
    },
    R = R
  )
  boot.cv_corr <- boot::boot(
    x,
    function(x, i) { # corrected coefficient of variation
      sd(x[i], na.rm = na.rm) / mean(x[i], na.rm = na.rm) * (
        (1 - (1 / (4 * (length(x[i]) - 1))) +
          (1 / length(x)) * (
            sd(x[i], na.rm = na.rm) / mean(x[i], na.rm = na.rm)
          )^2) +
          (1 / (2 * (length(x) - 1)^2))
      )
    },
    R = R
  )
  # calculating cv and its CI attributes based on selected methods
  if ("kelley" %in% method && correction == FALSE) {
    ci <- MBESS::conf.limits.nct(
      ncp = sqrt(length(x)) / cv_corr,
      df = length(x) - 1,
      conf.level = (1 - alpha)
    )
    est.kelley <- cv # cv is an estimate of CV
    lower.tile.kelley <- unname(sqrt(length(x)) / ci$Upper.Limit)
    upper.tile.kelley <- unname(sqrt(length(x)) / ci$Lower.Limit)
  } else if ("kelley" %in% method && correction == TRUE) {
    ci <- MBESS::conf.limits.nct(
      ncp = sqrt(length(x)) / cv_corr,
      df = length(x) - 1,
      conf.level = (1 - alpha)
    )
    est.kelley <- cv_corr # corrected cv is an estimate of CV
    lower.tile.kelley <- unname(sqrt(length(x)) / ci$Upper.Limit)
    upper.tile.kelley <- unname(sqrt(length(x)) / ci$Lower.Limit)
  } else if ("mckay" %in% method && correction == FALSE) {
    if (cv > 0.33) {
      warning("Confidence interval may be very approximate")
    }
    v <- length(x) - 1
    t1 <- qchisq(1 - alpha / 2, v) / v
    t2 <- qchisq(alpha / 2, v) / v
    u1 <- v * t1
    u2 <- v * t2
    est.mckay <- cv # cv is an estimate of CV
    lower.tile.mckay <- cv / sqrt((u1 / (v + 1) - 1) * (cv^2) + u1 / v)
    upper.tile.mckay <- cv / sqrt((u2 / (v + 1) - 1) * (cv^2) + u2 / v)
  } else if ("mckay" %in% method && correction == TRUE) {
    if (cv_corr > 0.33) {
      warning("Confidence interval may be very approximate")
    }
    v <- length(x) - 1
    t1 <- qchisq(1 - alpha / 2, v) / v
    t2 <- qchisq(alpha / 2, v) / v
    u1 <- v * t1
    u2 <- v * t2
    est.mckay <- cv_corr # corrected cv is an estimate of CV
    lower.tile.mckay <- cv_corr / sqrt((u1 / (v + 1) - 1) * (cv_corr^2) + u1 / v)
    upper.tile.mckay <- cv_corr / sqrt((u2 / (v + 1) - 1) * (cv_corr^2) + u2 / v)
  } else if ("miller" %in% method && correction == FALSE) {
    v <- length(x) - 1
    z_alpha_over2 <- qnorm(1 - (alpha / 2))
    u <- sqrt(
      (cv^2 / v) * (0.5 + cv^2)
    )
    zu <- z_alpha_over2 * u
    est.miller <- cv # cv is an estimate of CV
    lower.tile.miller <- cv - zu
    upper.tile.miller <- cv + zu
  } else if ("miller" %in% method && correction == TRUE) {
    v <- length(x) - 1
    z_alpha_over2 <- qnorm(1 - (alpha / 2))
    u <- sqrt(
      (cv_corr^2 / v) * (0.5 + cv_corr^2)
    )
    zu <- z_alpha_over2 * u
    est.miller <- cv_corr # corrected cv is an estimate of CV
    lower.tile.miller <- cv_corr - zu
    upper.tile.miller <- cv_corr + zu
  } else if ("vangel" %in% method && correction == FALSE) {
    if (cv > 0.33) {
      warning("Confidence interval may be very approximate")
    }
    v <- length(x) - 1
    t1 <- qchisq(1 - alpha / 2, v) / v
    t2 <- qchisq(alpha / 2, v) / v
    u1 <- v * t1
    u2 <- v * t2
    est.vangel <- cv # cv is an estimate of CV
    lower.tile.vangel <- cv / sqrt(((u1 + 1) / (v + 1) - 1) * (cv^2) + u1 / v)
    upper.tile.vangel <- cv / sqrt(((u2 + 1) / (v + 1) - 1) * (cv^2) + u2 / v)
  } else if ("vangel" %in% method && correction == TRUE) {
    if (cv_corr > 0.33) {
      warning("Confidence interval may be very approximate")
    }
    v <- length(x) - 1
    t1 <- qchisq(1 - alpha / 2, v) / v
    t2 <- qchisq(alpha / 2, v) / v
    u1 <- v * t1
    u2 <- v * t2
    est.vangel <- cv_corr # corrected cv is an estimate of CV
    lower.tile.vangel <- cv_corr / sqrt(
      ((u1 + 1) / (v + 1) - 1) * (cv_corr^2) + u1 / v
    )
    upper.tile.vangel <- cv_corr / sqrt(
      ((u2 + 1) / (v + 1) - 1) * (cv_corr^2) + u2 / v
    )
  } else if ("mahmoudvand_hassani" %in% method && correction == FALSE) {
    if (length(x) <= 340) {
      cn <- sqrt(2 / (length(x) - 1)) * (
        (gamma(length(x) / 2)) / (gamma((length(x) - 1) / 2))
      )
    } else {
      cn <- sqrt(2 / (length(x) - 1)) * (
        (lgamma(length(x) / 2)) / (lgamma((length(x) - 1) / 2))
      )
    }
    ul <- 2 - (cn + (qnorm((alpha / 2)) * sqrt(1 - cn^2)))
    uu <- 2 - (cn - (qnorm((alpha / 2)) * sqrt(1 - cn^2)))
    est.mahmoud <- cv # cv is an estimate of CV
    lower.tile.mahmoud <- cv / ul
    upper.tile.mahmoud <- cv / uu
  } else if ("mahmoudvand_hassani" %in% method && correction == TRUE) {
    if (length(x) <= 340) {
      cn <- sqrt(2 / (length(x) - 1)) * (
        (gamma(length(x) / 2)) / (gamma((length(x) - 1) / 2))
      )
    } else {
      cn <- sqrt(2 / (length(x) - 1)) * (
        (lgamma(length(x) / 2)) / (lgamma((length(x) - 1) / 2))
      )
    }
    ul <- 2 - (cn + (qnorm((alpha / 2)) * sqrt(1 - cn^2)))
    uu <- 2 - (cn - (qnorm((alpha / 2)) * sqrt(1 - cn^2)))
    est.mahmoud <- cv_corr # corrected cv is an estimate of CV
    lower.tile.mahmoud <- cv_corr / ul
    upper.tile.mahmoud <- cv_corr / uu
  } else if ("normal_approximation" %in% method && correction == FALSE) {
    cn <- sqrt(1 - (1 / (2 * length(x))))
    ul <- cn + (qnorm(1 - (alpha / 2)) * sqrt(1 - cn^2))
    uu <- cn - (qnorm(1 - (alpha / 2)) * sqrt(1 - cn^2))
    est.normaapprox <- cv # cv is an estimate of CV
    lower.tile.normaapprox <- cv / ul
    upper.tile.normaapprox <- cv / uu
  } else if ("normal_approximation" %in% method && correction == TRUE) {
    cn <- sqrt(1 - (1 / (2 * length(x))))
    ul <- cn + (qnorm(1 - (alpha / 2)) * sqrt(1 - cn^2))
    uu <- cn - (qnorm(1 - (alpha / 2)) * sqrt(1 - cn^2))
    est.normaapprox <- cv_corr # corrected cv is an estimate of CV
    lower.tile.normaapprox <- cv_corr / ul
    upper.tile.normaapprox <- cv_corr / uu
  } else if ("shortest_length" %in% method && correction == FALSE) {
    if (length(x) <= 300) {
      a_value <- shortest_length %>%
        subset(al == alpha & v == length(x) - 1) %>%
        dplyr::select(a)
      b_value <- shortest_length %>%
        subset(al == alpha & v == length(x) - 1) %>%
        dplyr::select(b)
    } else if (length(x) > 300) {
      a_value <- shortest_length %>%
        subset(al == alpha & v == 300) %>%
        dplyr::select(a)
      b_value <- shortest_length %>%
        subset(al == alpha & v == 300) %>%
        dplyr::select(b)
    }
    est.shortest <- cv # cv is an estimate of CV
    lower.tile.shortest <- (cv * sqrt(length(x) - 1)) / sqrt(b_value$b)
    upper.tile.shortest <- (cv * sqrt(length(x) - 1)) / sqrt(a_value$a)
  } else if ("shortest_length" %in% method && correction == TRUE) {
    if (length(x) <= 300) {
      a_value <- shortest_length %>%
        subset(al == alpha & v == length(x) - 1) %>%
        dplyr::select(a)
      b_value <- shortest_length %>%
        subset(al == alpha & v == length(x) - 1) %>%
        dplyr::select(b)
    } else if (length(x) > 300) {
      a_value <- shortest_length %>%
        subset(al == alpha & v == 300) %>%
        dplyr::select(a)
      b_value <- shortest_length %>%
        subset(al == alpha & v == 300) %>%
        dplyr::select(b)
    }
    est.shortest <- cv_corr # corrected cv is an estimate of CV
    lower.tile.shortest <- (cv_corr * sqrt(length(x) - 1)) / sqrt(b_value$b)
    upper.tile.shortest <- (cv_corr * sqrt(length(x) - 1)) / sqrt(a_value$a)
  } else if ("equal_tailed" %in% method && correction == FALSE) {
    v <- length(x) - 1
    tt1 <- qchisq(1 - alpha / 2, v)
    tt2 <- qchisq(alpha / 2, v)
    est.equal <- cv # cv is an estimate of CV
    lower.tile.equal <- (cv * sqrt(v)) / (sqrt(tt1))
    upper.tile.equal <- (cv * sqrt(v)) / (sqrt(tt2))
  } else if ("equal_tailed" %in% method && correction == TRUE) {
    v <- length(x) - 1
    tt1 <- qchisq(1 - alpha / 2, v)
    tt2 <- qchisq(alpha / 2, v)
    est.equal <- cv_corr # corrected cv is an estimate of CV
    lower.tile.equal <- (cv_corr * sqrt(v)) / (sqrt(tt1))
    upper.tile.equal <- (cv_corr * sqrt(v)) / (sqrt(tt2))
  } else if ("norm" %in% method && correction == FALSE) {
    boot.normcv.ci <- boot::boot.ci(
      boot.cv,
      conf = (1 - alpha), type = "norm"
    )
    est.norm <- cv # cv is an estimate of CV
    lower.tile.norm <- boot.normcv.ci$normal[2]
    upper.tile.norm <- boot.normcv.ci$normal[3]
  } else if ("norm" %in% method && correction == TRUE) {
    boot.normcv_corr.ci <- boot::boot.ci(
      boot.cv_corr,
      conf = (1 - alpha), type = "norm"
    )
    est.norm <- cv_corr # corrected cv is an estimate of CV
    lower.tile.norm <- boot.normcv_corr.ci$normal[2]
    upper.tile.norm <- boot.normcv_corr.ci$normal[3]
  } else if ("basic" %in% method && correction == FALSE) {
    boot.basiccv.ci <- boot::boot.ci(
      boot.cv,
      conf = (1 - alpha), type = "basic"
    )
    est.basic <- cv # cv is an estimate of CV
    lower.tile.basic <- boot.basiccv.ci$basic[4]
    upper.tile.basic <- boot.basiccv.ci$basic[5]
  } else if ("basic" %in% method && correction == TRUE) {
    boot.basiccv_corr.ci <- boot::boot.ci(
      boot.cv_corr,
      conf = (1 - alpha), type = "basic"
    )
    est.basic <- cv_corr # corrected cv is an estimate of CV
    lower.tile.basic <- boot.basiccv_corr.ci$basic[4]
    upper.tile.basic <- boot.basiccv_corr.ci$basic[5]
  } else if ("perc" %in% method && correction == FALSE) {
    boot.percentcv.ci <- boot::boot.ci(
      boot.cv,
      conf = (1 - alpha), type = "basic"
    )
    est.percent <- cv # cv is an estimate of CV
    lower.tile.percent <- boot.percentcv.ci$percent[4]
    upper.tile.percent <- boot.percentcv.ci$percent[5]
  } else if ("perc" %in% method && correction == TRUE) {
    boot.percentcv_corr.ci <- boot::boot.ci(
      boot.cv_corr,
      conf = (1 - alpha), type = "basic"
    )
    est.percent <- cv_corr # corrected cv is an estimate of CV
    lower.tile.percent <- boot.percentcv_corr.ci$percent[4]
    upper.tile.percent <- boot.percentcv_corr.ci$percent[5]
  } else if ("bca" %in% method && correction == FALSE) {
    boot.bcacv.ci <- boot::boot.ci(
      boot.cv,
      conf = (1 - alpha), type = "basic"
    )
    est.bca <- cv # cv is an estimate of CV
    lower.tile.bca <- boot.bcacv.ci$bca[4]
    upper.tile.bca <- boot.bcacv.ci$bca[5]
  } else if ("bca" %in% method && correction == TRUE) {
    boot.bcacv_corr.ci <- boot::boot.ci(
      boot.cv_corr,
      conf = (1 - alpha), type = "basic"
    )
    est.bca <- cv_corr # corrected cv is an estimate of CV
    lower.tile.bca <- boot.bcacv_corr.ci$percent[4]
    upper.tile.bca <- boot.bcacv_corr.ci$percent[5]
  } else if (method == "all" && correction == FALSE) {
    est.all <- cv # cv is an estimate of CV
    ci <- MBESS::conf.limits.nct(
      ncp = sqrt(length(x)) / cv_corr,
      df = length(x) - 1,
      conf.level = (1 - alpha)
    )
    lower.tile.kelley <- unname(sqrt(length(x)) / ci$Upper.Limit)
    upper.tile.kelley <- unname(sqrt(length(x)) / ci$Lower.Limit)
    if (cv > 0.33) {
      warning("Confidence interval may be very approximate")
    }
    v <- length(x) - 1
    t1 <- qchisq(1 - alpha / 2, v) / v
    t2 <- qchisq(alpha / 2, v) / v
    u1 <- v * t1
    u2 <- v * t2
    est.mckay <- cv # cv is an estimate of CV
    lower.tile.mckay <- cv / sqrt((u1 / (v + 1) - 1) * (cv^2) + u1 / v)
    upper.tile.mckay <- cv / sqrt((u2 / (v + 1) - 1) * (cv^2) + u2 / v)
    z_alpha_over2 <- qnorm(1 - (alpha / 2))
    u <- sqrt(
      (cv^2 / v) * (0.5 + cv^2)
    )
    zu <- z_alpha_over2 * u
    est.miller <- cv # cv is an estimate of CV
    lower.tile.miller <- cv - zu
    upper.tile.miller <- cv + zu
    tt1 <- qchisq(1 - alpha / 2, v) / v
    tt2 <- qchisq(alpha / 2, v) / v
    uu1 <- v * tt1
    uu2 <- v * tt2
    est.vangel <- cv # cv is an estimate of CV
    lower.tile.vangel <- cv / sqrt(((uu1 + 1) / (v + 1) - 1) * (cv^2) + uu1 / v)
    upper.tile.vangel <- cv / sqrt(((uu2 + 1) / (v + 1) - 1) * (cv^2) + uu2 / v)
    if (length(x) <= 340) {
      cn <- sqrt(2 / (length(x) - 1)) * (
        (gamma(length(x) / 2)) / (gamma((length(x) - 1) / 2))
      )
    } else {
      cn <- sqrt(2 / (length(x) - 1)) * (
        (lgamma(length(x) / 2)) / (lgamma((length(x) - 1) / 2))
      )
    }
    ul <- 2 - (cn + (qnorm((alpha / 2)) * sqrt(1 - cn^2)))
    uu <- 2 - (cn - (qnorm((alpha / 2)) * sqrt(1 - cn^2)))
    est.mahmoud <- cv # cv is an estimate of CV
    lower.tile.mahmoud <- cv / ul
    upper.tile.mahmoud <- cv / uu
    cnt <- sqrt(1 - (1 / (2 * length(x))))
    ult <- cnt + (qnorm(1 - (alpha / 2)) * sqrt(1 - cnt^2))
    uut <- cnt - (qnorm(1 - (alpha / 2)) * sqrt(1 - cnt^2))
    est.normaapprox <- cv # cv is an estimate of CV
    lower.tile.normaapprox <- cv / ult
    upper.tile.normaapprox <- cv / uut
    if (length(x) <= 300) {
      a_value <- shortest_length %>%
        subset(al == alpha & v == length(x) - 1) %>%
        dplyr::select(a)
      b_value <- shortest_length %>%
        subset(al == alpha & v == length(x) - 1) %>%
        dplyr::select(b)
    } else if (length(x) > 300) {
      a_value <- shortest_length %>%
        subset(al == alpha & v == 300) %>%
        dplyr::select(a)
      b_value <- shortest_length %>%
        subset(al == alpha & v == 300) %>%
        dplyr::select(b)
    }
    est.shortest <- cv # cv is an estimate of CV
    lower.tile.shortest <- (cv * sqrt(length(x) - 1)) / sqrt(b_value$b)
    upper.tile.shortest <- (cv * sqrt(length(x) - 1)) / sqrt(a_value$a)
    ttt1 <- qchisq(1 - alpha / 2, v)
    ttt2 <- qchisq(alpha / 2, v)
    est.equal <- cv # cv is an estimate of CV
    lower.tile.equal <- (cv * sqrt(v)) / (sqrt(ttt1))
    upper.tile.equal <- (cv * sqrt(v)) / (sqrt(ttt2))
    boot.normcv.ci <- boot::boot.ci(
      boot.cv,
      conf = (1 - alpha), type = "norm"
    )
    est.norm <- cv # cv is an estimate of CV
    lower.tile.norm <- boot.normcv.ci$normal[2]
    upper.tile.norm <- boot.normcv.ci$normal[3]
    boot.basiccv.ci <- boot::boot.ci(
      boot.cv,
      conf = (1 - alpha), type = "basic"
    )
    est.basic <- cv # cv is an estimate of CV
    lower.tile.basic <- boot.basiccv.ci$basic[4]
    upper.tile.basic <- boot.basiccv.ci$basic[5]
    boot.percentcv.ci <- boot::boot.ci(
      boot.cv,
      conf = (1 - alpha), type = "basic"
    )
    est.percent <- cv # cv is an estimate of CV
    lower.tile.percent <- boot.percentcv.ci$percent[4]
    upper.tile.percent <- boot.percentcv.ci$percent[5]
    boot.bcacv.ci <- boot::boot.ci(
      boot.cv,
      conf = (1 - alpha), type = "basic"
    )
    est.bca <- cv # cv is an estimate of CV
    lower.tile.bca <- boot.bcacv.ci$bca[4]
    upper.tile.bca <- boot.bcacv.ci$bca[5]
  } else if (method == "all" && correction == TRUE) {
    est.all <- cv_corr # cv_corr is an estimate of CV
    ci <- MBESS::conf.limits.nct(
      ncp = sqrt(length(x)) / cv_corr,
      df = length(x) - 1,
      conf.level = (1 - alpha)
    )
    lower.tile.kelley <- unname(sqrt(length(x)) / ci$Upper.Limit)
    upper.tile.kelley <- unname(sqrt(length(x)) / ci$Lower.Limit)
    if (cv_corr > 0.33) {
      warning("Confidence interval may be very approximate")
    }
    v <- length(x) - 1
    t1 <- qchisq(1 - alpha / 2, v) / v
    t2 <- qchisq(alpha / 2, v) / v
    u1 <- v * t1
    u2 <- v * t2
    est.mckay <- cv_corr # cv_corr is an estimate of CV
    lower.tile.mckay <- cv_corr / sqrt((u1 / (v + 1) - 1) * (cv_corr^2) + u1 / v)
    upper.tile.mckay <- cv_corr / sqrt((u2 / (v + 1) - 1) * (cv_corr^2) + u2 / v)
    z_alpha_over2 <- qnorm(1 - (alpha / 2))
    u <- sqrt(
      (cv_corr^2 / v) * (0.5 + cv_corr^2)
    )
    zu <- z_alpha_over2 * u
    est.miller <- cv_corr # cv_corr is an estimate of CV
    lower.tile.miller <- cv_corr - zu
    upper.tile.miller <- cv_corr + zu
    tt1 <- qchisq(1 - alpha / 2, v) / v
    tt2 <- qchisq(alpha / 2, v) / v
    uu1 <- v * tt1
    uu2 <- v * tt2
    est.vangel <- cv_corr # cv_corr is an estimate of CV
    lower.tile.vangel <- cv_corr / sqrt(((uu1 + 1) / (v + 1) - 1) * (cv_corr^2) + uu1 / v)
    upper.tile.vangel <- cv_corr / sqrt(((uu2 + 1) / (v + 1) - 1) * (cv_corr^2) + uu2 / v)
    if (length(x) <= 340) {
      cn <- sqrt(2 / (length(x) - 1)) * (
        (gamma(length(x) / 2)) / (gamma((length(x) - 1) / 2))
      )
    } else {
      cn <- sqrt(2 / (length(x) - 1)) * (
        (lgamma(length(x) / 2)) / (lgamma((length(x) - 1) / 2))
      )
    }
    ul <- 2 - (cn + (qnorm((alpha / 2)) * sqrt(1 - cn^2)))
    uu <- 2 - (cn - (qnorm((alpha / 2)) * sqrt(1 - cn^2)))
    est.mahmoud <- cv_corr # cv_corr is an estimate of CV
    lower.tile.mahmoud <- cv_corr / ul
    upper.tile.mahmoud <- cv_corr / uu
    cnt <- sqrt(1 - (1 / (2 * length(x))))
    ult <- cnt + (qnorm(1 - (alpha / 2)) * sqrt(1 - cnt^2))
    uut <- cnt - (qnorm(1 - (alpha / 2)) * sqrt(1 - cnt^2))
    est.normaapprox <- cv_corr # cv_corr is an estimate of CV
    lower.tile.normaapprox <- cv_corr / ult
    upper.tile.normaapprox <- cv_corr / uut
    if (length(x) <= 300) {
      a_value <- shortest_length %>%
        subset(al == alpha & v == length(x) - 1) %>%
        dplyr::select(a)
      b_value <- shortest_length %>%
        subset(al == alpha & v == length(x) - 1) %>%
        dplyr::select(b)
    } else if (length(x) > 300) {
      a_value <- shortest_length %>%
        subset(al == alpha & v == 300) %>%
        dplyr::select(a)
      b_value <- shortest_length %>%
        subset(al == alpha & v == 300) %>%
        dplyr::select(b)
    }
    est.shortest <- cv_corr # cv_corr is an estimate of CV
    lower.tile.shortest <- (cv_corr * sqrt(length(x) - 1)) / sqrt(b_value$b)
    upper.tile.shortest <- (cv_corr * sqrt(length(x) - 1)) / sqrt(a_value$a)
    ttt1 <- qchisq(1 - alpha / 2, v)
    ttt2 <- qchisq(alpha / 2, v)
    est.equal <- cv_corr # cv_corr is an estimate of CV
    lower.tile.equal <- (cv_corr * sqrt(v)) / (sqrt(ttt1))
    upper.tile.equal <- (cv_corr * sqrt(v)) / (sqrt(ttt2))
    boot.normcv_corr.ci <- boot::boot.ci(
      boot.cv_corr,
      conf = (1 - alpha), type = "norm"
    )
    est.norm <- cv_corr # cv_corr is an estimate of CV
    lower.tile.norm <- boot.normcv_corr.ci$normal[2]
    upper.tile.norm <- boot.normcv_corr.ci$normal[3]
    boot.basiccv_corr.ci <- boot::boot.ci(
      boot.cv_corr,
      conf = (1 - alpha), type = "basic"
    )
    est.basic <- cv_corr # cv_corr is an estimate of CV
    lower.tile.basic <- boot.basiccv_corr.ci$basic[4]
    upper.tile.basic <- boot.basiccv_corr.ci$basic[5]
    boot.percentcv_corr.ci <- boot::boot.ci(
      boot.cv_corr,
      conf = (1 - alpha), type = "basic"
    )
    est.percent <- cv_corr # cv_corr is an estimate of CV
    lower.tile.percent <- boot.percentcv_corr.ci$percent[4]
    upper.tile.percent <- boot.percentcv_corr.ci$percent[5]
    boot.bcacv_corr.ci <- boot::boot.ci(
      boot.cv_corr,
      conf = (1 - alpha), type = "basic"
    )
    est.bca <- cv_corr # cv_corr is an estimate of CV
    lower.tile.bca <- boot.bcacv_corr.ci$bca[4]
    upper.tile.bca <- boot.bcacv_corr.ci$bca[5]
  }
  # preparing the output based on the selected methods
  if ("kelley" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Kelley 95% CI"
        ),
        statistics = data.frame(
          est = round(est.kelley * 100, digits = digits),
          lower = round(lower.tile.kelley * 100, digits = digits),
          upper = round(upper.tile.kelley * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("kelley" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Kelley 95% CI"
        ),
        statistics = data.frame(
          est = round(est.kelley * 100, digits = digits),
          lower = round(lower.tile.kelley * 100, digits = digits),
          upper = round(upper.tile.kelley * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("mckay" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with McKay 95% CI"
        ),
        statistics = data.frame(
          est = round(est.mckay * 100, digits = digits),
          lower = round(lower.tile.mckay * 100, digits = digits),
          upper = round(upper.tile.mckay * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("mckay" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with McKay 95% CI"
        ),
        statistics = data.frame(
          est = round(est.mckay * 100, digits = digits),
          lower = round(lower.tile.mckay * 100, digits = digits),
          upper = round(upper.tile.mckay * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("miller" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Miller 95% CI"
        ),
        statistics = data.frame(
          est = round(est.miller * 100, digits = digits),
          lower = round(lower.tile.miller * 100, digits = digits),
          upper = round(upper.tile.miller * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("miller" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Miller 95% CI"
        ),
        statistics = data.frame(
          est = round(est.miller * 100, digits = digits),
          lower = round(lower.tile.miller * 100, digits = digits),
          upper = round(upper.tile.miller * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("vangel" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Vangel 95% CI"
        ),
        statistics = data.frame(
          est = round(est.vangel * 100, digits = digits),
          lower = round(lower.tile.vangel * 100, digits = digits),
          upper = round(upper.tile.vangel * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("vangel" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Vangel 95% CI"
        ),
        statistics = data.frame(
          est = round(est.vangel * 100, digits = digits),
          lower = round(lower.tile.vangel * 100, digits = digits),
          upper = round(upper.tile.vangel * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("mahmoudvand_hassani" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Mahmoudvand-Hassani 95% CI"
        ),
        statistics = data.frame(
          est = round(est.mahmoud * 100, digits = digits),
          lower = round(lower.tile.mahmoud * 100, digits = digits),
          upper = round(upper.tile.mahmoud * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("mahmoudvand_hassani" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Mahmoudvand-Hassani 95% CI"
        ),
        statistics = data.frame(
          est = round(est.mahmoud * 100, digits = digits),
          lower = round(lower.tile.mahmoud * 100, digits = digits),
          upper = round(upper.tile.mahmoud * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("normal_approximation" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Normal Approximation 95% CI"
        ),
        statistics = data.frame(
          est = round(est.normaapprox * 100, digits = digits),
          lower = round(
            lower.tile.normaapprox * 100,
            digits = digits
          ),
          upper = round(
            upper.tile.normaapprox * 100,
            digits = digits
          ),
          row.names = c(" ")
        )
      )
    )
  } else if ("normal_approximation" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Normal Approximation 95% CI"
        ),
        statistics = data.frame(
          est = round(est.normaapprox * 100, digits = digits),
          lower = round(
            lower.tile.normaapprox * 100,
            digits = digits
          ),
          upper = round(
            upper.tile.normaapprox * 100,
            digits = digits
          ),
          row.names = c(" ")
        )
      )
    )
  } else if ("shortest_length" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Shortest-Length 95% CI"
        ),
        statistics = data.frame(
          est = round(est.shortest * 100, digits = digits),
          lower = round(lower.tile.shortest * 100, digits = digits),
          upper = round(upper.tile.shortest * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("shortest_length" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Shortest-Length 95% CI"
        ),
        statistics = data.frame(
          est = round(est.shortest * 100, digits = digits),
          lower = round(lower.tile.shortest * 100, digits = digits),
          upper = round(upper.tile.shortest * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("equal_tailed" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Equal-Tailed 95% CI"
        ),
        statistics = data.frame(
          est = round(est.equal * 100, digits = digits),
          lower = round(lower.tile.equal * 100, digits = digits),
          upper = round(upper.tile.equal * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("equal_tailed" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Equal-Tailed 95% CI"
        ),
        statistics = data.frame(
          est = round(est.equal * 100, digits = digits),
          lower = round(lower.tile.equal * 100, digits = digits),
          upper = round(upper.tile.equal * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("norm" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Normal Approximation Bootstrap 95% CI"
        ),
        statistics = data.frame(
          est = round(est.norm * 100, digits = digits),
          lower = round(lower.tile.norm * 100, digits = digits),
          upper = round(upper.tile.norm * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("norm" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Normal Approximation Bootstrap 95% CI"
        ),
        statistics = data.frame(
          est = round(est.norm * 100, digits = digits),
          lower = round(lower.tile.norm * 100, digits = digits),
          upper = round(upper.tile.norm * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("basic" %in% method && correction == FALSE) {
    return(
      list(
        method = (
          "cv with Basic Bootstrap 95% CI"
        ),
        statistics = data.frame(
          est = round(est.basic * 100, digits = digits),
          lower = round(lower.tile.basic * 100, digits = digits),
          upper = round(upper.tile.basic * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("basic" %in% method && correction == TRUE) {
    return(
      list(
        method = (
          "Corrected cv with Basic Bootstrap 95% CI"
        ),
        statistics = data.frame(
          est = round(est.basic * 100, digits = digits),
          lower = round(lower.tile.basic * 100, digits = digits),
          upper = round(upper.tile.basic * 100, digits = digits),
          row.names = c(" ")
        )
      )
    )
  } else if ("percent" %in% method && correction == FALSE) {
    stop("percent method is not developed yet")
    # return(
    #     list(
    #         method = (
    #             "cv with Bootstrap Percentile 95% CI"
    #         ),
    #         statistics = data.frame(
    #             est = round(est.percent * 100, digits = digits),
    #             lower = round(lower.tile.percent * 100, digits = digits),
    #             upper = round(upper.tile.percent * 100, digits = digits),
    #             row.names = c(" ")
    #         )
    #     )
    #     )
  } else if ("percent" %in% method && correction == TRUE) {
    stop("percent method is not developed yet")
    # return(
    #     list(
    #         method = (
    #             "Corrected cv with Bootstrap Percentile 95% CI"
    #         ),
    #         statistics = data.frame(
    #             est = round(est.percent * 100, digits = digits),
    #             lower = round(lower.tile.percent * 100, digits = digits),
    #             upper = round(upper.tile.percent * 100, digits = digits),
    #             row.names = c(" ")
    #         )
    #     )
    # )
  } else if ("bca" %in% method && correction == FALSE) {
    stop("BCa method is not developed yet")
    # return(
    #     list(
    #         method = (
    #             "cv with Adjusted Bootstrap Percentile (BCa) 95% CI"
    #         ),
    #         statistics = data.frame(
    #             est = round(est.bca * 100, digits = digits),
    #             lower = round(lower.tile.bca * 100, digits = digits),
    #             upper = round(upper.tile.bca * 100, digits = digits),
    #             row.names = c(" ")
    #         )
    #     )
    #     )
  } else if ("bca" %in% method && correction == TRUE) {
    stop("BCa method is not developed yet")
    # return(
    #     list(
    #         method = (
    #         "Corrected cv with Adjusted Bootstrap Percentile (BCa) 95% CI"
    #         ),
    #         statistics = data.frame(
    #             est = round(est.bca * 100, digits = digits),
    #             lower = round(lower.tile.bca * 100, digits = digits),
    #             upper = round(upper.tile.bca * 100, digits = digits),
    #             row.names = c(" ")
    #         )
    #     )
    # )
  } else if (method == "all" && correction == FALSE) {
    return(
      list(
        method = "All methods",
        statistics = data.frame(
          row.names = c(
            "kelley", # 1
            "mckay", # 2
            "miller", # 3
            "vangel", # 4
            "mahmoudvand_hassani", # 5
            "equal_tailed", # 6
            "shortest_length", # 7
            "normal_approximation", # 8
            "norm", # 9
            "basic" # 10
            # "perc"  # 11
            # "bca"  # 12
          ),
          est = c(
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            round(cv * 100, digits = digits),
            # round(cv * 100, digits = digits),
            # round(cv * 100, digits = digits),
            round(cv * 100, digits = digits)
          ),
          lower = c(
            round(lower.tile.kelley * 100, digits = digits),
            round(lower.tile.mckay * 100, digits = digits),
            round(lower.tile.miller * 100, digits = digits),
            round(lower.tile.vangel * 100, digits = digits),
            round(lower.tile.mahmoud * 100, digits = digits),
            round(lower.tile.equal * 100, digits = digits),
            round(lower.tile.shortest * 100, digits = digits),
            round(lower.tile.normaapprox * 100, digits = digits),
            round(lower.tile.norm * 100, digits = digits),
            round(lower.tile.basic * 100, digits = digits)
            # ,
            # round(lower.tile.percent * 100, digits = digits)
            # ,
            # round(lower.tile.bca * 100, digits = digits)
          ),
          upper = c(
            round(upper.tile.kelley * 100, digits = digits),
            round(upper.tile.mckay * 100, digits = digits),
            round(upper.tile.miller * 100, digits = digits),
            round(upper.tile.vangel * 100, digits = digits),
            round(upper.tile.mahmoud * 100, digits = digits),
            round(upper.tile.equal * 100, digits = digits),
            round(upper.tile.shortest * 100, digits = digits),
            round(upper.tile.normaapprox * 100, digits = digits),
            round(upper.tile.norm * 100, digits = digits),
            round(upper.tile.basic * 100, digits = digits)
            # ,
            # round(upper.tile.percent * 100, digits = digits)
            # ,
            # round(upper.tile.bca * 100, digits = digits)
          ),
          description = c(
            "cv with Kelley 95% CI",
            "cv with McKay 95% CI",
            "cv with Miller 95% CI",
            "cv with Vangel 95% CI",
            "cv with Mahmoudvand-Hassani 95% CI",
            "cv with Equal-Tailed 95% CI",
            "cv with Shortest-Length 95% CI",
            "cv with Normal Approximation 95% CI",
            "cv with Normal Approximation Bootstrap 95% CI",
            "cv with Basic Bootstrap 95% CI"
            # ,
            # "cv with Bootstrap Percentile 95% CI"
            # ,
            # "cv with Adjusted Bootstrap Percentile (BCa) 95% CI"
          )
        )
      )
    )
  } else if (method == "all" && correction == TRUE) {
    return(
      list(
        method = "All methods",
        statistics = data.frame(
          row.names = c(
            "kelley", # 1
            "mckay", # 2
            "miller", # 3
            "vangel", # 4
            "mahmoudvand_hassani", # 5
            "equal_tailed", # 6
            "shortest_length", # 7
            "normal_approximation", # 8
            "norm", # 9
            "basic" # 10
            # "perc"  # 11
            # "bca"  # 12
          ),
          est = c(
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits),
            # round(cv_corr * 100, digits = digits),
            # round(cv_corr * 100, digits = digits),
            round(cv_corr * 100, digits = digits)
          ),
          lower = c(
            round(lower.tile.kelley * 100, digits = digits),
            round(lower.tile.mckay * 100, digits = digits),
            round(lower.tile.miller * 100, digits = digits),
            round(lower.tile.vangel * 100, digits = digits),
            round(lower.tile.mahmoud * 100, digits = digits),
            round(lower.tile.equal * 100, digits = digits),
            round(lower.tile.shortest * 100, digits = digits),
            round(lower.tile.normaapprox * 100, digits = digits),
            round(lower.tile.norm * 100, digits = digits),
            round(lower.tile.basic * 100, digits = digits)
            # ,
            # round(lower.tile.percent * 100, digits = digits)
            # ,
            # round(lower.tile.bca * 100, digits = digits)
          ),
          upper = c(
            round(upper.tile.kelley * 100, digits = digits),
            round(upper.tile.mckay * 100, digits = digits),
            round(upper.tile.miller * 100, digits = digits),
            round(upper.tile.vangel * 100, digits = digits),
            round(upper.tile.mahmoud * 100, digits = digits),
            round(upper.tile.equal * 100, digits = digits),
            round(upper.tile.shortest * 100, digits = digits),
            round(upper.tile.normaapprox * 100, digits = digits),
            round(upper.tile.norm * 100, digits = digits),
            round(upper.tile.basic * 100, digits = digits)
            # ,
            # round(upper.tile.percent * 100, digits = digits)
            # ,
            # round(upper.tile.bca * 100, digits = digits)
          ),
          description = c(
            "Corrected cv with Kelley 95% CI",
            "Corrected cv with McKay 95% CI",
            "Corrected cv with Miller 95% CI",
            "Corrected cv with Vangel 95% CI",
            "Corrected cv with Mahmoudvand-Hassani 95% CI",
            "Corrected cv with Equal-Tailed 95% CI",
            "Corrected cv with Shortest-Length 95% CI",
            "Corrected cv with Normal Approximation 95% CI",
            "Corrected cv with Normal Approximation Bootstrap 95% CI",
            "Corrected cv with Basic Bootstrap 95% CI"
            # ,
            # "Corrected cv with Bootstrap Percentile 95% CI"
            # ,
            # "Corrected cv with Adjusted Bootstrap Percentile (BCa) 95% CI"
          )
        )
      )
    )
  }
}
```
```{r eval = TRUE, warning=FALSE, message=FALSE}
x <- c(
  0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
  4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
)
cv_versatile(
  x,
  na.rm = TRUE,
  digits = 3,
  method = "kelley",
  correction = TRUE,
  alpha = 0.05
)
```
### McKay Confidence Interval
McKay [@McKay1932] introduced the following CI for $cv$; considering $u_1 = \chi_{v,1-\alpha/2}^2$ and $u_1 = \chi_{v,\alpha/2}^2$ being
the $100(1-\alpha/2)\%$ and $100(\alpha/2)\%$ percentile of the $\chi^2$ distribution with $v = n-1$ degrees of freedom, respectively [@Albatineh2014]:   
$$
\biggl(cv\left[\biggl(\frac{u_1}{v}-1\biggr)(cv)^{2}+\frac{u_1}{v}\right]^{-1/2}
\le CV \le cv
\left[\biggl(\frac{u_2}{v}-1\biggr)(cv)^{2}+\frac{u_2}{v}\right]^{-1/2}\biggr)
$$
Let us calculate the 95\% CI for our variable $x$ according to McKay's method [@McKay1932]:
```{r eval = TRUE, warning=FALSE, message=FALSE}
cv_versatile(
  x,
  na.rm = TRUE,
  digits = 3,
  method = "mckay",
  correction = TRUE,
  alpha = 0.05
)
```
### Miller Confidence Interval    
Miller [@EdwardMiller1991] introduced the following CI for 
$cv$; considering $Z_{\alpha/2}$ being the $(1-\alpha/2)$ percentile of the standard normal distribution [@Albatineh2014]:
$$
\biggl(cv - Z_{\alpha/2}\sqrt{
\biggl(\frac{cv^2}{v}\biggr)\biggl(\frac{1}{2}+cv^2\biggr)} \le
CV \le cv + Z_{\alpha/2}\sqrt{
\biggl(\frac{cv^2}{v}\biggr)\biggl(\frac{1}{2}+cv^2\biggr)}
\biggr)
$$
where $v = n-1$ is the degree of freedom.   
Let us calculate the 95\% CI for $x$ according to Miller's method [@EdwardMiller1991]:   
```{r eval = TRUE, warning=FALSE, message=FALSE}
cv_versatile(
  x,
  na.rm = TRUE,
  digits = 3,
  method = "miller",
  correction = TRUE,
  alpha = 0.05
)
```
### Vangel Confidence Interval      
Vangel [@Vangel1996] proposed the following CI for 
$cv$; which is a modification on McKay’s CI:
$$
\biggl(cv\left[\biggl(\frac{u_1+1}{v}-1\biggr)(cv)^{2}+\frac{u_1}{v}\right]^{-1/2}
\le CV \le cv
\left[\biggl(\frac{u_2+1}{v}-1\biggr)(cv)^{2}+\frac{u_2}{v}\right]^{-1/2}\biggr)
$$
Let us calculate the 95\% CI for $x$ according to Vangel's method [@Vangel1996]:   
```{r eval = TRUE, warning=FALSE, message=FALSE}
cv_versatile(
  x,
  na.rm = TRUE,
  digits = 3,
  method = "vangel",
  correction = TRUE,
  alpha = 0.05
)
```
### Mahmoudvand-Hassani Confidence Interval   
Mahmoudvand and Hassani [@Mahmoudvand2009] proposed the following CI for 
$cv$; which is obtained using ranked set sampling *(RSS)*:    
$$
\biggl(\frac{cv}{2-C_n+Z_{1-\alpha/2}\sqrt{1-C_n^2}}
\le CV \le
\frac{cv}{2-C_n-Z_{1-\alpha/2}\sqrt{1-C_n^2}}
\biggr)
$$
where 
$$
C_n=\sqrt{\frac{2}{n-1}}\frac{\Gamma{(n/2)}}{\Gamma{((n-1)/2)}},
\Gamma(n)=(n-1)!
$$
Let us now calculate the 95\% CI for $x$ according to Mahmoudvand-Hassani's 
method [@Mahmoudvand2009]:   
```{r eval = TRUE, warning=FALSE, message=FALSE}
cv_versatile(
  x,
  na.rm = TRUE,
  digits = 3,
  method = "mahmoudvand_hassani",
  correction = TRUE,
  alpha = 0.05
)
```
### Normal Approximation Confidence Interval     
Wararit Panichkitkosolkul [@Panichkitkosolkul2013] proposed the following CI for 
$cv$; which is a *normal approximation*: 
$$
\biggl(\frac{cv}{C_{n+1}+Z_{1-\alpha/2}\sqrt{1-C_{n+1}^2}}
\le CV \le
\frac{cv}{C_{n+1}-Z_{1-\alpha/2}\sqrt{1-C_{n+1}^2}}
\biggr)
$$
where $C_{n+1}=\sqrt{1-(1/2n)}$     
Now we calculate the normal approximation 95\% CI for $x$ according to Panichkitkosolkul [@Panichkitkosolkul2013]:   
```{r eval = TRUE, warning=FALSE, message=FALSE}
cv_versatile(
  x,
  na.rm = TRUE,
  digits = 3,
  method = "normal_approximation",
  correction = TRUE,
  alpha = 0.05
)
```
### Shortest-Length Confidence Interval    
Panichkitkosolkul [@Panichkitkosolkul2013] has also introduced the following CI for $cv$:   
$$
\biggl(\frac{cv\sqrt{v}}{\sqrt{b}}
\le CV \le
\frac{cv\sqrt{v}}{\sqrt{a}}
\biggr)
$$
with $v = n-1$ degrees of freedom. Then, shortest-length 95\% CI for $x$ is:    
```{r eval = TRUE, warning=FALSE, message=FALSE}
cv_versatile(
  x,
  na.rm = TRUE,
  digits = 3,
  method = "shortest_length",
  correction = TRUE,
  alpha = 0.05
)
```
### Equal-Tailed Confidence Interval     
The $100(1-\alpha)\%$ equal-tailed CI for $cv$ can be obtained as:
$$
\biggl(\frac{cv\sqrt{v}}{\sqrt{\chi_{v,1-\alpha/2}^2}}
\le CV \le
\frac{cv\sqrt{v}}{\sqrt{\chi_{v,\alpha/2}^2}}
\biggr)
$$
where $\chi_{v,\alpha/2}^2$ and $\chi_{v,1-\alpha/2}^2$ are the $100(\alpha/2)$ and $100(1-\alpha/2)$ percentiles of the central $\chi^2$ distribution with $v$ degrees of freedom, respectively [@Panichkitkosolkul2013].   
Then, equal-tailed 95\% CI for $x$ is:    
```{r eval = TRUE, warning=FALSE, message=FALSE}
cv_versatile(
  x,
  na.rm = TRUE,
  digits = 3,
  method = "equal_tailed",
  correction = TRUE,
  alpha = 0.05
)
```
### Bootstrap Confidence Intervals      
Thanks to package `boot` [@Canty2017] we can obtain bootstrap CI around $cv$:    
```{r eval = TRUE, warning=FALSE, message=FALSE}
cv_versatile(
  x,
  na.rm = TRUE,
  digits = 3,
  method = "basic",
  correction = TRUE,
  alpha = 0.05
)
```
### All Available Methods   
In conclusion, we can observe CIs calculated by all available methods:    
```{r eval = TRUE, warning=FALSE, message=FALSE}
cv_versatile(
  x,
  na.rm = TRUE,
  digits = 3,
  method = "all",
  correction = FALSE,
  alpha = 0.05
)
```
# References   

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cv_versatile.R
\name{cv_versatile}
\alias{cv_versatile}
\title{Coefficient of Variation (cv)}
\arguments{
\item{x}{An \code{R} object. Currently there are methods for numeric vectors}

\item{na.rm}{a logical value indicating whether \code{NA} values should be
stripped before the computation proceeds.}

\item{digits}{integer indicating the number of decimal places to be used.}

\item{method}{a scalar representing the type of confidence intervals
required. The value should be any of the values "kelley", "mckay",
"miller", "vangel", "mahmoudvand_hassani", "equal_tailed",
"shortest_length", "normal_approximation", "norm","basic", "aak_adj",
"aak_ls", "aak_als", or "all".}

\item{correction}{returns the unbiased estimate of the coefficient of
variation}

\item{alpha}{The allowed type I error probability}

\item{R}{integer indicating the number of bootstrap replicates.}
}
\value{
An object of type "list" which contains the estimate, the intervals,
  and the computation method. It has two main components:

\describe{ \item{$method}{ A description of statistical method used
  for the computations. } \item{$statistics}{ A data frame representing three
  vectors: est, lower and upper limits of confidence interval \code{(CI)};
  additional description vector is provided when "all" is selected: \cr \cr
  \strong{est:} \code{cv*100}  \cr \cr \strong{Kelley Confidence
  Interval:} Thanks to package \pkg{MBESS} \code{[2]} for the
  computation of confidence limits for the noncentrality parameter from a
  \emph{t} distribution \link[MBESS]{conf.limits.nct} \code{[3]}.  \cr \cr
  \strong{McKay Confidence Interval:} The intervals calculated by the method
  introduced by McKay \code{[4]}, using chi-square distribution.  \cr \cr
  \strong{Miller Confidence Interval:} The intervals calculated by the
  method introduced by Miller \code{[5]}, using the standard normal
  distribution.  \cr \cr \strong{Vangel Confidence Interval:} Vangel
  \code{[6]} proposed a method for the calculation of CI for \emph{cv}; which
  is a modification on McKay’s CI.  \cr \cr \strong{Mahmoudvand-Hassani
  Confidence Interval:} Mahmoudvand and Hassani \code{[7]} proposed a new CI
  for \emph{cv}; which is obtained using ranked set sampling \emph{(RSS)}
  \cr \cr \strong{Normal Approximation Confidence Interval:} Wararit
  Panichkitkosolkul \code{[8]} proposed another CI for \emph{cv}; which is a
  normal approximation.  \cr \cr \strong{Shortest-Length Confidence
  Interval:} Wararit Panichkitkosolkul \code{[8]} proposed another CI for
  \emph{cv}; which is obtained through minimizing the length of CI.  \cr \cr
  \strong{Equal-Tailed Confidence Interval:} Wararit Panichkitkosolkul
  \code{[8]} proposed another CI for \emph{cv}; which is obtained using
  chi-square distribution.  \cr \cr \strong{Bootstrap Confidence
  Intervals:} Thanks to package \pkg{boot} by Canty & Ripley \code{[9]} we
  can obtain bootstrap CI around \emph{cv} using \link[boot]{boot.ci}.  \cr
  \cr \strong{Abu-Shawiesh-Akyuz-Kibria Confidence Intervals:}
  Abu-Shawiesh, Akyuz, & Kibria \code{[10]} proposed three CIs for the
  population \emph{cv} that adjust for non-normality through the sample
  kurtosis: the adjusted-degrees-of-freedom CI (\code{aak_adj}), the
  large-sample CI (\code{aak_ls}), and the augmented-large-sample CI
  (\code{aak_als}).  \cr \cr } }
}
\description{
Versatile function for the coefficient of variation (cv)
}
\details{
\describe{ \item{\strong{Coefficient of Variation}}{ The \emph{cv}
  is a measure of relative dispersion representing the degree of variability
  relative to the mean \code{[1]}. Since \eqn{cv} is unitless, it is useful
  for comparison of variables with different units. It is also a measure of
  homogeneity \code{[1]}. } }
}
\examples{
x <- c(
    0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
    4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
)
cv_versatile(x)
cv_versatile(x, correction = TRUE)
cv_versatile(x, na.rm = TRUE, digits = 3, method = "kelley", correction = TRUE)
cv_versatile(x, na.rm = TRUE, method = "mahmoudvand_hassani", correction = TRUE)
}
\references{
\code{[1]} Albatineh, AN., Kibria, BM., Wilcox, ML., & Zogheib,
  B, 2014, Confidence interval estimation for the population coefficient of
  variation using ranked set sampling: A simulation study, Journal of Applied
  Statistics, 41(4), 733–751, DOI:
  \doi{10.1080/02664763.2013.847405}

\code{[2]} Kelley, K., 2018, MBESS: The MBESS R Package. R
  package version 4.4. 3.

\code{[3]} Kelley, K., 2007, Sample size planning for the
  coefficient of variation from the accuracy in parameter estimation
  approach, Behavior Research Methods, 39(4), 755–766, DOI:
  \doi{10.3758/BF03192966}

\code{[4]} McKay, AT., 1932, Distribution of the Coefficient of
  Variation and the Extended“ t” Distribution, Journal of the Royal
  Statistical Society, 95(4), 695–698

\code{[5]} Miller, E., 1991, Asymptotic test statistics for
  coefficients of variation, Communications in Statistics-Theory and Methods,
  20(10), 3351–3363

\code{[6]} Vangel, MG., 1996, Confidence intervals for a normal
  coefficient of variation, The American Statistician, 50(1), 21–26

\code{[7]} Mahmoudvand, R., & Hassani, H., 2009, Two new
  confidence intervals for the coefficient of variation in a normal
  distribution, Journal of Applied Statistics, 36(4), 429–442

\code{[8]} Panichkitkosolkul, W., 2013, Confidence Intervals for
  the Coefficient of Variation in a Normal Distribution with a Known
  Population Mean, Journal of Probability and Statistics, 2013, 1–11,
  \doi{10.1155/2013/324940}

\code{[9]} Canty, A., & Ripley, B., 2017, boot: Bootstrap R
  (S-Plus) Functions, R package version 1.3-20

\code{[10]} Abu-Shawiesh, MOA., Akyuz, HE., & Kibria, BG.,
  2019, Performance of Some Confidence Intervals for Estimating the
  Population Coefficient of Variation under both Symmetric and Skewed
  Distributions, Statistics, Optimization & Information Computing, 7(2),
  277-290, DOI: \doi{10.19139/soic.v7i2.630}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CoefQuartVarCI.R
\name{CoefQuartVarCI}
\alias{CoefQuartVarCI}
\title{R6 Confidence Intervals for the Coefficient of Quartile Variation
  (cqv)}
\arguments{
\item{x}{An \code{R} object. Currently there are methods for numeric vectors}

\item{na.rm}{a logical value indicating whether \code{NA} values should be
stripped before the computation proceeds.}

\item{digits}{integer indicating the number of decimal places to be used.}

\item{methods}{the available computation methods of confidence intervals are:
"bonett_ci", "norm_ci", "basic_ci", "perc_ci", "bca_ci" or "all_ci".}

\item{R}{integer indicating the number of bootstrap replicates.}
}
\value{
An object of type "list" which contains the estimate, the intervals,
  and the computation method. It has two components:

\describe{ \item{$method}{ A description of statistical method used
  for the computations. } \item{$statistics}{ A data frame representing three
  vectors: est, lower and upper limits of 95\% confidence interval
  \code{(CI)}: \cr \cr \strong{est:} \code{cqv*100}  \cr \cr \strong{Bonett
  95\% CI:} It uses a centering adjustment which helps to equalize the tail
  error probabilities \code{[1, 2]}.  \cr \cr \strong{Normal approximation
  95\% CI:} The intervals calculated by the normal approximation \code{[3,
  4]}, using \link[boot]{boot.ci}.  \cr \cr \strong{Basic bootstrap 95\%
  CI:} The intervals calculated by the basic bootstrap method \code{[3, 4]},
  using \link[boot]{boot.ci}.  \cr \cr \strong{Bootstrap percentile 95\%
  CI:} The intervals calculated by the bootstrap percentile method \code{[3,
  4]}, using \link[boot]{boot.ci}.  \cr \cr \strong{Adjusted bootstrap
  percentile (BCa) 95\% CI:} The intervals calculated by the adjusted
  bootstrap percentile (BCa) method \code{[3, 4]}, using
  \link[boot]{boot.ci}.  } }
}
\description{
The R6 class \code{CoefQuartVarCI} for the confidence intervals
  of coefficient of quartile variation (cqv)
}
\details{
\describe{ \item{\strong{Coefficient of Quartile Variation}}{ The
  \emph{cqv} is a measure of relative dispersion that is based on
  interquartile range \emph{(iqr)}. Since \eqn{cqv} is unitless, it is useful
  for comparison of variables with different units. It is also a measure of
  homogeneity \code{[1, 2]}. } }
}
\examples{
y <- c(
0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
)
CoefQuartVarCI$new(x = y)$bonett_ci()
cqv_y <- CoefQuartVarCI$new(
    x = y,
    alpha = 0.05,
    R = 1000,
    digits = 2
)
cqv_y$bonett_ci()
R6::is.R6(cqv_y)
}
\references{
\code{[1]} Bonett, DG., 2006, Confidence interval for a
  coefficient of quartile variation, Computational Statistics & Data
  Analysis, 50(11), 2953-7, DOI:
  \doi{10.1016/j.csda.2005.05.007}

\code{[2]} Altunkaynak, B., Gamgam, H., 2018, Bootstrap
  confidence intervals for the coefficient of quartile variation, Simulation
  and Computation, 1-9, DOI:
  \doi{10.1080/03610918.2018.1435800}

\code{[3]} Canty, A., & Ripley, B, 2017, boot: Bootstrap R
  (S-Plus) Functions. R package version 1.3-20.

\code{[4]} Davison, AC., & Hinkley, DV., 1997, Bootstrap Methods
  and Their Applications. Cambridge University Press, Cambridge. ISBN
  0-521-57391-2
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CoefQuartVar.R
\name{CoefQuartVar}
\alias{CoefQuartVar}
\title{R6 Coefficient of Quartile Variation (cqv)}
\arguments{
\item{x}{An \code{R} object. Currently there are methods for numeric vectors}

\item{na.rm}{a logical value indicating whether \code{NA} values should be
stripped before the computation proceeds.}

\item{digits}{integer indicating the number of decimal places to be used.}
}
\description{
The R6 class \code{CoefQuartVar} for the coefficient of quartile
  variation (cqv)
}
\details{
\describe{ \item{\strong{Coefficient of Quartile Variation}}{
  \emph{cqv} is a measure of relative dispersion that is based on
  interquartile range \emph{(iqr)}. Since \eqn{cqv} is unitless, it is useful
  for comparison of variables with different units. It is also a measure of
  homogeneity \code{[1]}. } }
}
\examples{
x <- c(
   0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
   4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
)
CoefQuartVar$new(x)$est()
cqv_x <- CoefQuartVar$new(x, digits = 2)
cqv_x$est()
R6::is.R6(cqv_x)
}
\references{
\code{[1]} Bonett, DG., 2006, Confidence interval for a
  coefficient of quartile variation, Computational Statistics & Data
  Analysis, 50(11), 2953-7, DOI:
  \doi{10.1016/j.csda.2005.05.007}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BootCoefVar.R
\name{BootCoefVar}
\alias{BootCoefVar}
\title{R6 Bootstrap Resampling for Coefficient of Variation}
\arguments{
\item{x}{An \code{R} object. Currently there are methods for numeric vectors}

\item{na.rm}{a logical value indicating whether \code{NA} values should be
stripped before the computation proceeds.}

\item{alpha}{The allowed type I error probability}

\item{R}{integer indicating the number of bootstrap replicates.}
}
\description{
The R6 class \code{BootCoefVar} produces the bootstrap
  resampling for the coefficient of variation (cv) of the given numeric
  vectors. It uses \link[boot]{boot} and \link[boot]{boot.ci} from the
  package \pkg{boot}.
}
\examples{
x <- c(
    0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
    4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
)
cv_x <- BootCoefVar$new(x)
cv_x$boot_cv()
cv_x$boot_cv_corr()
cv_x$boot_basic_ci_cv()
cv_x$boot_norm_ci_cv()
cv_x$boot_perc_ci_cv()
cv_x$boot_bca_ci_cv()
cv_x$boot_basic_ci_cv_corr()
cv_x$boot_norm_ci_cv_corr()
cv_x$boot_perc_ci_cv_corr()
cv_x$boot_bca_ci_cv_corr()
R6::is.R6(cv_x)
}
\references{
Canty, A., & Ripley, B, 2017, boot: Bootstrap R (S-Plus)
  Functions. R package version 1.3-20.

Davison, AC., & Hinkley, DV., 1997, Bootstrap Methods and Their
  Applications. Cambridge University Press, Cambridge. ISBN 0-521-57391-2
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SampleQuantiles.R
\name{SampleQuantiles}
\alias{SampleQuantiles}
\title{R6 Sample Quantiles}
\arguments{
\item{x}{An \code{R} object. Currently there are methods for numeric vectors}

\item{na.rm}{a logical value indicating whether \code{NA} values should be
stripped before the computation proceeds.}

\item{digits}{integer indicating the number of decimal places to be used.}

\item{probs}{numeric vector of probabilities with values in \code{[0,1]}.}

\item{names}{logical; if TRUE, the result has a \link[base]{names} attribute
regarding the percentiles.}

\item{type}{an integer between 1 and 9 selecting one of the nine quantile
algorithms explained in \link[stats]{quantile} to be used.}
}
\description{
The R6 class \code{SampleQuantiles} produces the sample
  quantiles corresponding to the given probabilities. It uses
  \link[stats]{quantile} from the package \pkg{stats}.
}
\examples{
x <- c(
   0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
   4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
)
SampleQuantiles$new(x)$qx()
percentile_95 <-  SampleQuantiles$new(x, na.rm = TRUE, digits = 2, probs = 0.95)
percentile_95$qx()
percentile_75 <-  SampleQuantiles$new(x, na.rm = TRUE, digits = 3, probs = 0.75)
percentile_75$qx()
R6::is.R6(percentile_95)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CoefVar.R
\name{CoefVar}
\alias{CoefVar}
\title{R6 Coefficient of Variation (cv)}
\arguments{
\item{x}{An \code{R} object. Currently there are methods for numeric vectors}

\item{na.rm}{a logical value indicating whether \code{NA} values should be
stripped before the computation proceeds.}

\item{digits}{integer indicating the number of decimal places to be used.}
}
\description{
The R6 class \code{CoefVar} for the coefficient of variation
  (cv)
}
\details{
\describe{ \item{\strong{Coefficient of Variation}}{ The \emph{cv}
  is a measure of relative dispersion representing the degree of variability
  relative to the mean \code{[1]}. Since \eqn{cv} is unitless, it is useful
  for comparison of variables with different units. It is also a measure of
  homogeneity \code{[1]}. } }
}
\examples{
x <- c(
    0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
    4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
)
CoefVar$new(x)$est()
cv_x <- CoefVar$new(x, digits = 2)
cv_x$est()
cv_x$est_corr()
R6::is.R6(cv_x)
}
\references{
\code{[1]} Albatineh, AN., Kibria, BM., Wilcox, ML., & Zogheib,
  B, 2014, Confidence interval estimation for the population coefficient of
  variation using ranked set sampling: A simulation study, Journal of Applied
  Statistics, 41(4), 733–751, DOI:
  \doi{10.1080/02664763.2013.847405}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BootCoefQuartVar.R
\name{BootCoefQuartVar}
\alias{BootCoefQuartVar}
\title{R6 Bootstrap Resampling for Coefficient of Quartile Variation}
\arguments{
\item{x}{An \code{R} object. Currently there are methods for numeric vectors}

\item{na.rm}{a logical value indicating whether \code{NA} values should be
stripped before the computation proceeds.}

\item{alpha}{The allowed type I error probability}

\item{R}{integer indicating the number of bootstrap replicates.}
}
\description{
The R6 class \code{BootCoefQuartVar} produces the bootstrap
  resampling for the coefficient of quartile variation (cqv) of the given
  numeric vectors. It uses \link[boot]{boot} from the package \pkg{boot}.
  Also, it produces the bootstrap confidence intervals for the cqv based on
  the \link[boot]{boot.ci} from the package \pkg{boot}.
}
\examples{
x <- c(
    0.2, 0.5, 1.1, 1.4, 1.8, 2.3, 2.5, 2.7, 3.5, 4.4,
    4.6, 5.4, 5.4, 5.7, 5.8, 5.9, 6.0, 6.6, 7.1, 7.9
)
cqv_x <- BootCoefQuartVar$new(x)
cqv_x$boot_cqv()
cqv_x$boot_basic_ci()
cqv_x$boot_norm_ci()
cqv_x$boot_perc_ci()
cqv_x$boot_bca_ci()
R6::is.R6(cqv_x)
}
\references{
Canty, A., & Ripley, B, 2017, boot: Bootstrap R (S-Plus)
  Functions. R package version 1.3-20.

Davison, AC., & Hinkley, DV., 1997, Bootstrap Methods and Their
  Applications. Cambridge University Press, Cambridge. ISBN 0-521-57391-2

Altunkaynak, B., Gamgam, H., 2018, Bootstrap confidence intervals
  for the coefficient of quartile variation, Simulation and Computation, 1-9,
  DOI:
  \doi{10.1080/03610918.2018.1435800}
}
